/* Generated by Cython 3.1.1 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "pyteomics/compat.h"
        ],
        "include_dirs": [
            "./pyteomics"
        ],
        "name": "pyteomics.cparser",
        "sources": [
            "pyteomics/cparser.pyx"
        ]
    },
    "module_name": "pyteomics.cparser"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
/* InitLimitedAPI */
#if defined(Py_LIMITED_API) && !defined(CYTHON_LIMITED_API)
  #define CYTHON_LIMITED_API 1
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x03080000
    #error Cython requires Python 3.8+.
#else
#define __PYX_ABI_VERSION "3_1_1"
#define CYTHON_HEX_VERSION 0x030101F0
#define CYTHON_FUTURE_DIVISION 1
/* CModulePreamble */
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #define HAVE_LONG_LONG
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 1
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 1
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PYPY_VERSION_NUM >= 0x07030C00)
  #endif
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_NUM >= 0x07031100)
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_CLINE_IN_TRACEBACK
  #define CYTHON_CLINE_IN_TRACEBACK 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND (__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #ifdef Py_GIL_DISABLED
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 1
  #else
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #endif
  #if PY_VERSION_HEX < 0x030A0000
    #undef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #elif !defined(CYTHON_USE_TYPE_SLOTS)
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLIST_INTERNALS)
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #elif !defined(CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL 0
  #elif !defined(CYTHON_FAST_GIL)
    #define CYTHON_FAST_GIL (PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_SYS_MONITORING
    #define CYTHON_USE_SYS_MONITORING (PY_VERSION_HEX >= 0x030d00B1)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5 && !CYTHON_USE_MODULE_STATE)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS (!CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
#endif
#ifndef CYTHON_FAST_PYCCALL
#define CYTHON_FAST_PYCCALL  CYTHON_FAST_PYCALL
#endif
#ifndef CYTHON_VECTORCALL
#if CYTHON_COMPILING_IN_LIMITED_API
#define CYTHON_VECTORCALL  (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
#else
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL && PY_VERSION_HEX >= 0x030800B1)
#endif
#endif
#define CYTHON_BACKPORT_VECTORCALL (CYTHON_METH_FASTCALL && PY_VERSION_HEX < 0x030800B1)
#if CYTHON_USE_PYLONG_INTERNALS
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
            typedef unsigned char     uint8_t;
            typedef unsigned short    uint16_t;
            typedef unsigned int      uint32_t;
        #else
            typedef unsigned __int8   uint8_t;
            typedef unsigned __int16  uint16_t;
            typedef unsigned __int32  uint32_t;
        #endif
    #endif
    #if _MSC_VER < 1300
        #ifdef _WIN64
            typedef unsigned long long  __pyx_uintptr_t;
        #else
            typedef unsigned int        __pyx_uintptr_t;
        #endif
    #else
        #ifdef _WIN64
            typedef unsigned __int64    __pyx_uintptr_t;
        #else
            typedef unsigned __int32    __pyx_uintptr_t;
        #endif
    #endif
#else
    #include <stdint.h>
    typedef uintptr_t  __pyx_uintptr_t;
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifndef Py_UNREACHABLE
  #define Py_UNREACHABLE()  assert(0); abort()
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

/* CInitCode */
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

/* PythonCompatibility */
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#define __Pyx_BUILTIN_MODULE_NAME "builtins"
#define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    #ifndef CO_OPTIMIZED
    static int CO_OPTIMIZED;
    #endif
    #ifndef CO_NEWLOCALS
    static int CO_NEWLOCALS;
    #endif
    #ifndef CO_VARARGS
    static int CO_VARARGS;
    #endif
    #ifndef CO_VARKEYWORDS
    static int CO_VARKEYWORDS;
    #endif
    #ifndef CO_ASYNC_GENERATOR
    static int CO_ASYNC_GENERATOR;
    #endif
    #ifndef CO_GENERATOR
    static int CO_GENERATOR;
    #endif
    #ifndef CO_COROUTINE
    static int CO_COROUTINE;
    #endif
#else
    #ifndef CO_COROUTINE
      #define CO_COROUTINE 0x80
    #endif
    #ifndef CO_ASYNC_GENERATOR
      #define CO_ASYNC_GENERATOR 0x200
    #endif
#endif
static int __Pyx_init_co_variables(void);
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#ifndef METH_FASTCALL
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#elif CYTHON_BACKPORT_VECTORCALL
  typedef PyObject *(*__pyx_vectorcallfunc)(PyObject *callable, PyObject *const *args,
                                            size_t nargsf, PyObject *kwnames);
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  ((size_t)1 << (8 * sizeof(size_t) - 1))
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(((size_t)(n)) & ~__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void (*cfunc)(void)) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if __PYX_LIMITED_VERSION_HEX < 0x03090000
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#elif CYTHON_COMPILING_IN_GRAAL
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) _PyFrame_SetLineNumber((frame), (lineno))
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#endif
#if CYTHON_USE_MODULE_STATE
static CYTHON_INLINE void *__Pyx__PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#define __Pyx_PyModule_GetState(o) (__pyx_mstatetype *)__Pyx__PyModule_GetState(o)
#else
#define __Pyx_PyModule_GetState(op) ((void)op,__pyx_mstate_global)
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE((PyObject *) obj), name, func_ctype)
#define __Pyx_PyObject_TryGetSlot(obj, name, func_ctype) __Pyx_PyType_TryGetSlot(Py_TYPE(obj), name, func_ctype)
#define __Pyx_PyObject_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#define __Pyx_PyObject_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype) __Pyx_PyType_GetSlot(type, name, func_ctype)
  #define __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype) (((type)->sub) ? ((type)->sub->name) : NULL)
  #define __Pyx_PyType_TryGetSubSlot(type, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype)
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype)\
    ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000 ||\
     (PyType_GetFlags(type) & Py_TPFLAGS_HEAPTYPE) || __Pyx_get_runtime_version() >= 0x030A0000) ?\
     __Pyx_PyType_GetSlot(type, name, func_ctype) : NULL)
  #define __Pyx_PyType_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSlot(obj, name, func_ctype)
  #define __Pyx_PyType_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSlot(obj, name, func_ctype)
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
#define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
#endif
#define __Pyx_PyObject_GetIterNextFunc(iterator)  __Pyx_PyObject_GetSlot(iterator, tp_iternext, iternextfunc)
#if CYTHON_USE_TYPE_SPECS && PY_VERSION_HEX >= 0x03080000
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#else
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains)
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #else
    #define __Pyx_PyList_GetItemRef(o, i) PySequence_GetItem(o, i)
  #endif
#else
  #define __Pyx_PyList_GetItemRef(o, i) __Pyx_NewRef(PyList_GET_ITEM(o, i))
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyDict_GetItemRef(dict, key, result) PyDict_GetItemRef(dict, key, result)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyObject_GetItem(dict, key);
  if (*result == NULL) {
    if (PyErr_ExceptionMatches(PyExc_KeyError)) {
      PyErr_Clear();
      return 0;
    }
    return -1;
  }
  return 1;
}
#else
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyDict_GetItemWithError(dict, key);
  if (*result == NULL) {
    return PyErr_Occurred() ? -1 : 0;
  }
  Py_INCREF(*result);
  return 1;
}
#endif
#if defined(CYTHON_DEBUG_VISIT_CONST) && CYTHON_DEBUG_VISIT_CONST
  #define __Pyx_VISIT_CONST(obj)  Py_VISIT(obj)
#else
  #define __Pyx_VISIT_CONST(obj)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GET_ITEM(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GET_ITEM(o, i)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GetItem(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GetItem(o, i)
#endif
#if CYTHON_ASSUME_SAFE_SIZE
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GET_LENGTH(o)
#else
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GetLength(o)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static CYTHON_INLINE PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_InternFromString)
  #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
#endif
#define __Pyx_PyLong_FromHash_t PyLong_FromSsize_t
#define __Pyx_PyLong_AsHash_t   __Pyx_PyIndex_AsSsize_t
#if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
    #define __Pyx_PySendResult PySendResult
#else
    typedef enum {
        PYGEN_RETURN = 0,
        PYGEN_ERROR = -1,
        PYGEN_NEXT = 1,
    } __Pyx_PySendResult;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030A00A3
  typedef __Pyx_PySendResult (*__Pyx_pyiter_sendfunc)(PyObject *iter, PyObject *value, PyObject **result);
#else
  #define __Pyx_pyiter_sendfunc sendfunc
#endif
#if !CYTHON_USE_AM_SEND
#define __PYX_HAS_PY_AM_SEND 0
#elif __PYX_LIMITED_VERSION_HEX >= 0x030A0000
#define __PYX_HAS_PY_AM_SEND 1
#else
#define __PYX_HAS_PY_AM_SEND 2  // our own backported implementation
#endif
#if __PYX_HAS_PY_AM_SEND < 2
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
        __Pyx_pyiter_sendfunc am_send;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_SlotTpAsAsync(s) ((PyAsyncMethods*)(s))
#endif
#if CYTHON_USE_AM_SEND && PY_VERSION_HEX < 0x030A00F0
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (1UL << 21)
#else
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (0)
#endif
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyInterpreterState_Get() PyInterpreterState_Get()
#else
#define __Pyx_PyInterpreterState_Get() PyThreadState_Get()->interp
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030A0000
#ifdef __cplusplus
extern "C"
#endif
PyAPI_FUNC(void *) PyMem_Calloc(size_t nelem, size_t elsize);
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_init_co_variable(PyObject *inspect, const char* name, int *write_to) {
    int value;
    PyObject *py_value = PyObject_GetAttrString(inspect, name);
    if (!py_value) return 0;
    value = (int) PyLong_AsLong(py_value);
    Py_DECREF(py_value);
    *write_to = value;
    return value != -1 || !PyErr_Occurred();
}
static int __Pyx_init_co_variables(void) {
    PyObject *inspect;
    int result;
    inspect = PyImport_ImportModule("inspect");
    result =
#if !defined(CO_OPTIMIZED)
        __Pyx_init_co_variable(inspect, "CO_OPTIMIZED", &CO_OPTIMIZED) &&
#endif
#if !defined(CO_NEWLOCALS)
        __Pyx_init_co_variable(inspect, "CO_NEWLOCALS", &CO_NEWLOCALS) &&
#endif
#if !defined(CO_VARARGS)
        __Pyx_init_co_variable(inspect, "CO_VARARGS", &CO_VARARGS) &&
#endif
#if !defined(CO_VARKEYWORDS)
        __Pyx_init_co_variable(inspect, "CO_VARKEYWORDS", &CO_VARKEYWORDS) &&
#endif
#if !defined(CO_ASYNC_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_ASYNC_GENERATOR", &CO_ASYNC_GENERATOR) &&
#endif
#if !defined(CO_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_GENERATOR", &CO_GENERATOR) &&
#endif
#if !defined(CO_COROUTINE)
        __Pyx_init_co_variable(inspect, "CO_COROUTINE", &CO_COROUTINE) &&
#endif
        1;
    Py_DECREF(inspect);
    return result ? 0 : -1;
}
#else
static int __Pyx_init_co_variables(void) {
    return 0;  // It's a limited API-only feature
}
#endif

/* MathInitCode */
#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #ifndef _USE_MATH_DEFINES
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#ifndef CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#define CYTHON_CLINE_IN_TRACEBACK_RUNTIME 0
#endif
#ifndef CYTHON_CLINE_IN_TRACEBACK
#define CYTHON_CLINE_IN_TRACEBACK CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#endif
#if CYTHON_CLINE_IN_TRACEBACK
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; __pyx_clineno = __LINE__; (void) __pyx_clineno; }
#else
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; (void) __pyx_clineno; }
#endif
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pyteomics__cparser
#define __PYX_HAVE_API__pyteomics__cparser
/* Early includes */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

    #if PY_MAJOR_VERSION <= 2
    #define PyDict_GetItemWithError _PyDict_GetItemWithError
    #endif
    
#include "compat.h"

    #if PY_MAJOR_VERSION >= 3
      #define __Pyx_PyFloat_FromString(obj)  PyFloat_FromString(obj)
    #else
      #define __Pyx_PyFloat_FromString(obj)  PyFloat_FromString(obj, NULL)
    #endif
    
#include <ctype.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AS_STRING(s)
#else
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AsString(s)
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
static CYTHON_INLINE PyObject *__Pyx_NewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_NewRef)
    return Py_NewRef(obj);
#else
    Py_INCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_XNewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_XNewRef)
    return Py_XNewRef(obj);
#else
    Py_XINCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b);
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __Pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AS_DOUBLE(x)
#else
#define __Pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AsDouble(x)
#endif
#define __Pyx_PyFloat_AsFloat(x) ((float) __Pyx_PyFloat_AsDouble(x))
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#elif __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeASCII(c_str, size, NULL)
#else
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
/* PretendToInitialize */
#ifdef __cplusplus
#include <type_traits>
template <typename T>
static void __Pyx_pretend_to_initialize(T* ptr) {
#if __cplusplus > 201103L
    if ((std::is_trivially_default_constructible<T>::value))
#endif
        *ptr = T();
    (void)ptr;
}
#else
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }
#endif


#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * const __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char* const __pyx_f[] = {
  "pyteomics/cparser.pyx",
  "<stringsource>",
  "cpython/type.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* Atomics.proto */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __PYX_CYTHON_ATOMICS_ENABLED() CYTHON_ATOMICS
#define __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __pyx_atomic_int_type int
#define __pyx_nonatomic_int_type int
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__))
    #include <stdatomic.h>
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)))
    #include <atomic>
#endif
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__) &&\
                       ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type atomic_int
    #define __pyx_atomic_ptr_type atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) atomic_fetch_add_explicit(value, 1, memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) atomic_fetch_add_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) atomic_fetch_sub_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) atomic_load(value)
    #define __pyx_atomic_store(value, new_value) atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) atomic_load_explicit(value, memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) atomic_load_explicit(value, memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)) &&\
                    ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type std::atomic_int
    #define __pyx_atomic_ptr_type std::atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) std::atomic_fetch_sub_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) std::atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) std::atomic_load(value)
    #define __pyx_atomic_store(value, new_value) std::atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) std::atomic_load_explicit(value, std::memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) std::atomic_load_explicit(value, std::memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) std::atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C++ atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C++ atomics"
    #endif
#elif CYTHON_ATOMICS && (__GNUC__ >= 5 || (__GNUC__ == 4 &&\
                    (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ >= 2))))
    #define __pyx_atomic_ptr_type void*
    #define __pyx_atomic_incr_relaxed(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) __sync_fetch_and_sub(value, 1)
    #define __pyx_atomic_sub(value, arg) __sync_fetch_and_sub(value, arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_store(value, new_value) __sync_lock_test_and_set(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_load_acquire(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) __sync_lock_test_and_set(value, (__pyx_atomic_ptr_type)new_value)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER)
    #include <intrin.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type long
    #define __pyx_atomic_ptr_type void*
    #undef __pyx_nonatomic_int_type
    #define __pyx_nonatomic_int_type long
    #pragma intrinsic (_InterlockedExchangeAdd, _InterlockedExchange, _InterlockedCompareExchange, _InterlockedCompareExchangePointer, _InterlockedExchangePointer)
    #define __pyx_atomic_incr_relaxed(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) _InterlockedExchangeAdd(value, -1)
    #define __pyx_atomic_sub(value, arg) _InterlockedExchangeAdd(value, -arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = _InterlockedCompareExchange(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) _InterlockedExchangeAdd(value, 0)
    #define __pyx_atomic_store(value, new_value) _InterlockedExchange(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) *(void * volatile *)value
    #define __pyx_atomic_pointer_load_acquire(value) _InterlockedCompareExchangePointer(value, 0, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) _InterlockedExchangePointer(value, (__pyx_atomic_ptr_type)new_value)
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif
#if CYTHON_ATOMICS
    #define __pyx_add_acquisition_count(memview)\
             __pyx_atomic_incr_relaxed(__pyx_get_slice_count_pointer(memview))
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_atomic_decr_acq_rel(__pyx_get_slice_count_pointer(memview))
#else
    #define __pyx_add_acquisition_count(memview)\
            __pyx_add_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_sub_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
#endif

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* CriticalSections.proto */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection void*
#define __Pyx_PyCriticalSection2 void*
#define __Pyx_PyCriticalSection_Begin1(cs, arg) (void)cs
#define __Pyx_PyCriticalSection_Begin2(cs, arg1, arg2) (void)cs
#define __Pyx_PyCriticalSection_End1(cs)
#define __Pyx_PyCriticalSection_End2(cs)
#else
#define __Pyx_PyCriticalSection PyCriticalSection
#define __Pyx_PyCriticalSection2 PyCriticalSection2
#define __Pyx_PyCriticalSection_Begin1 PyCriticalSection_Begin
#define __Pyx_PyCriticalSection_Begin2 PyCriticalSection2_Begin
#define __Pyx_PyCriticalSection_End1 PyCriticalSection_End
#define __Pyx_PyCriticalSection_End2 PyCriticalSection2_End
#endif
#if PY_VERSION_HEX < 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_BEGIN_CRITICAL_SECTION(o) {
#define __Pyx_END_CRITICAL_SECTION() }
#else
#define __Pyx_BEGIN_CRITICAL_SECTION Py_BEGIN_CRITICAL_SECTION
#define __Pyx_END_CRITICAL_SECTION Py_END_CRITICAL_SECTION
#endif

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_9pyteomics_7cparser_SequencePosition;
struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator;
struct __pyx_obj_9pyteomics_7cparser_ModificationRule;
struct __pyx_obj_9pyteomics_7cparser_ModificationTarget;
struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate;
struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms;
struct __pyx_opt_args_9pyteomics_7cparser_parse;
struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition;
struct __pyx_opt_args_9pyteomics_7cparser_tostring;
struct __pyx_opt_args_9pyteomics_7cparser_length;
struct __pyx_opt_args_9pyteomics_7cparser__tokenize_modx_sequence;
struct __pyx_opt_args_9pyteomics_7cparser__sequence_tostring;
struct __pyx_opt_args_9pyteomics_7cparser_16IsoformGenerator_build_site_map;

/* "pyteomics/cparser.pxd":17
 * 
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=*, bint split=*,             # <<<<<<<<<<<<<<
 *                  bint allow_unknown_modifications=*, object  labels=*)
 * 
*/
struct __pyx_opt_args_9pyteomics_7cparser_parse {
  int __pyx_n;
  int show_unmodified_termini;
  int split;
  int allow_unknown_modifications;
  PyObject *labels;
};

/* "pyteomics/cparser.pxd":20
 *                  bint allow_unknown_modifications=*, object  labels=*)
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=*, bint term_aa=*,             # <<<<<<<<<<<<<<
 *                                   bint allow_unknown_modifications=*, object labels=*)
 * 
*/
struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition {
  int __pyx_n;
  int show_unmodified_termini;
  int term_aa;
  int allow_unknown_modifications;
  PyObject *labels;
};

/* "pyteomics/cparser.pxd":25
 * cpdef tuple _split_label(str label)
 * 
 * cpdef str tostring(object parsed_sequence, bint show_unmodified_termini=*)             # <<<<<<<<<<<<<<
*/
struct __pyx_opt_args_9pyteomics_7cparser_tostring {
  int __pyx_n;
  int show_unmodified_termini;
};

/* "pyteomics/cparser.pyx":676
 * 
 * 
 * cpdef enum PositionType:             # <<<<<<<<<<<<<<
 *     n_term = 1
 *     c_term = 2
*/
enum __pyx_t_9pyteomics_7cparser_PositionType {
  __pyx_e_9pyteomics_7cparser_n_term = 1,
  __pyx_e_9pyteomics_7cparser_c_term = 2,
  __pyx_e_9pyteomics_7cparser_internal = 3,
  __pyx_e_9pyteomics_7cparser_universal = 4
};

/* "pyteomics/cparser.pyx":118
 *     return bool(match_modX(label))
 * 
 * cpdef int length(object sequence,             # <<<<<<<<<<<<<<
 *                  int show_unmodified_termini=0, int split=0,
 *                  int allow_unknown_modifications=0, object labels=std_labels):
*/
struct __pyx_opt_args_9pyteomics_7cparser_length {
  int __pyx_n;
  int show_unmodified_termini;
  int split;
  int allow_unknown_modifications;
  PyObject *labels;
};

/* "pyteomics/cparser.pyx":375
 * 
 * 
 * cdef list _tokenize_modx_sequence(str sequence, bint split=True):             # <<<<<<<<<<<<<<
 *     cdef:
 *         char* csequence
*/
struct __pyx_opt_args_9pyteomics_7cparser__tokenize_modx_sequence {
  int __pyx_n;
  int split;
};

/* "pyteomics/cparser.pyx":906
 * 
 * 
 * cpdef str _sequence_tostring(list sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n, acc, j
*/
struct __pyx_opt_args_9pyteomics_7cparser__sequence_tostring {
  int __pyx_n;
  int show_unmodified_termini;
};

/* "pyteomics/cparser.pyx":1067
 *         return sequence
 * 
 *     cpdef dict build_site_map(self, list sequence, bint include_unmodified=True):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/
struct __pyx_opt_args_9pyteomics_7cparser_16IsoformGenerator_build_site_map {
  int __pyx_n;
  int include_unmodified;
};

/* "pyteomics/cparser.pyx":697
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * @cython.freelist(5000)
 * cdef class SequencePosition(object):
*/
struct __pyx_obj_9pyteomics_7cparser_SequencePosition {
  PyObject_HEAD
  struct __pyx_vtabstruct_9pyteomics_7cparser_SequencePosition *__pyx_vtab;
  PyObject *modification;
  enum __pyx_t_9pyteomics_7cparser_PositionType position_type;
  PyObject *amino_acid;
  PyObject *terminal;
};


/* "pyteomics/cparser.pyx":987
 * 
 * 
 * @cython.freelist(100)             # <<<<<<<<<<<<<<
 * cdef class IsoformGenerator(object):
 *     cdef:
*/
struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator {
  PyObject_HEAD
  struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *__pyx_vtab;
  PyObject *variable_mods;
  PyObject *fixed_mods;
  PyObject *labels;
  long max_mods;
  int override;
  int show_unmodified_termini;
  int persist;
  PyObject *sequence;
  PyObject *original_sequence;
  PyObject *last_indices;
};


/* "pyteomics/cparser.pyx":1209
 * 
 * 
 * @cython.freelist(100)             # <<<<<<<<<<<<<<
 * cdef class ModificationRule(object):
 *     cdef:
*/
struct __pyx_obj_9pyteomics_7cparser_ModificationRule {
  PyObject_HEAD
  struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationRule *__pyx_vtab;
  PyObject *modification;
  PyObject *targets;
};


/* "pyteomics/cparser.pyx":1240
 * 
 * 
 * @cython.freelist(1000)             # <<<<<<<<<<<<<<
 * cdef class ModificationTarget(object):
 *     cdef:
*/
struct __pyx_obj_9pyteomics_7cparser_ModificationTarget {
  PyObject_HEAD
  struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationTarget *__pyx_vtab;
  PyObject *target;
  enum __pyx_t_9pyteomics_7cparser_PositionType position_type;
};


/* "pyteomics/cparser.pyx":1105
 *         return site_map
 * 
 *     @cython.boundscheck(False)             # <<<<<<<<<<<<<<
 *     def generate(self, sequence_spec):
 *         cdef:
*/
struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate {
  PyObject_HEAD
  PyObject *__pyx_v_a;
  PyObject *__pyx_v_assignment;
  PyObject *__pyx_v_assignments;
  PyObject *__pyx_v_assignments_product;
  size_t __pyx_v_i;
  PyObject *__pyx_v_indices;
  size_t __pyx_v_j;
  size_t __pyx_v_k;
  size_t __pyx_v_n;
  size_t __pyx_v_n_mods;
  long *__pyx_v_pindices;
  struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self;
  PyObject *__pyx_v_sequence;
  PyObject *__pyx_v_sequence_spec;
  PyObject *__pyx_v_subassignments;
  PyObject *__pyx_v_subindices;
  PyObject *__pyx_v_temp1;
  PyObject *__pyx_v_temp2;
  PyObject *__pyx_v_variable_mod_sites;
  Py_ssize_t __pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
  size_t __pyx_t_3;
  long __pyx_t_4;
  long __pyx_t_5;
  PyObject *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
};


/* "pyteomics/cparser.pyx":1308
 * 
 * 
 * def isoforms(str sequence, **kwargs):             # <<<<<<<<<<<<<<
 *     cdef:
 *         IsoformGenerator isoform_generator
*/
struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms {
  PyObject_HEAD
  PyObject *__pyx_v__format;
  PyObject *__pyx_v_fixed_mods;
  PyObject *__pyx_v_form;
  PyObject *__pyx_v_gen;
  struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_isoform_generator;
  PyObject *__pyx_v_kwargs;
  PyObject *__pyx_v_labels;
  PyObject *__pyx_v_max_mods;
  PyObject *__pyx_v_override;
  PyObject *__pyx_v_sequence;
  PyObject *__pyx_v_show_unmodified_termini;
  PyObject *__pyx_v_variable_mods;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};



/* "pyteomics/cparser.pyx":697
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * @cython.freelist(5000)
 * cdef class SequencePosition(object):
*/

struct __pyx_vtabstruct_9pyteomics_7cparser_SequencePosition {
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *(*_create)(PyObject *, PyObject *, PyObject *, enum __pyx_t_9pyteomics_7cparser_PositionType);
  PyObject *(*get_index)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, long, int __pyx_skip_dispatch);
  size_t (*get_size)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *);
  int (*is_modified)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *);
  PyObject *(*add_modification)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*as_tuple)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, int __pyx_skip_dispatch);
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *(*copy)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *);
  int (*_eq)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *);
  int (*_eq_tuple)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, PyObject *);
};
static struct __pyx_vtabstruct_9pyteomics_7cparser_SequencePosition *__pyx_vtabptr_9pyteomics_7cparser_SequencePosition;
static struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_f_9pyteomics_7cparser_16SequencePosition__create(PyObject *, PyObject *, PyObject *, enum __pyx_t_9pyteomics_7cparser_PositionType);
static PyObject *__pyx_f_9pyteomics_7cparser_16SequencePosition_get_index(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, long, int __pyx_skip_dispatch);
static size_t __pyx_f_9pyteomics_7cparser_16SequencePosition_get_size(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *);
static int __pyx_f_9pyteomics_7cparser_16SequencePosition_is_modified(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *);
static PyObject *__pyx_f_9pyteomics_7cparser_16SequencePosition_add_modification(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
static PyObject *__pyx_f_9pyteomics_7cparser_16SequencePosition_as_tuple(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, int __pyx_skip_dispatch);
static struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_f_9pyteomics_7cparser_16SequencePosition_copy(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *);
static int __pyx_f_9pyteomics_7cparser_16SequencePosition__eq(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *);
static int __pyx_f_9pyteomics_7cparser_16SequencePosition__eq_tuple(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, PyObject *);


/* "pyteomics/cparser.pyx":987
 * 
 * 
 * @cython.freelist(100)             # <<<<<<<<<<<<<<
 * cdef class IsoformGenerator(object):
 *     cdef:
*/

struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator {
  PyObject *(*coerce_modification_dict)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*initialize_from_str)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*initialize_from_iterable)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*apply_fixed_mods)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*build_site_map)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_16IsoformGenerator_build_site_map *__pyx_optional_args);
  void (*_reset_sequence)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *);
  PyObject *(*__pyx_fuse_0_apply_variable_assignment)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, long *, PyObject *, size_t);
  PyObject *(*__pyx_fuse_1_apply_variable_assignment)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, long *, PyObject *, size_t);
};
static struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *__pyx_vtabptr_9pyteomics_7cparser_IsoformGenerator;


/* "pyteomics/cparser.pyx":1209
 * 
 * 
 * @cython.freelist(100)             # <<<<<<<<<<<<<<
 * cdef class ModificationRule(object):
 *     cdef:
*/

struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationRule {
  int (*test)(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationRule *__pyx_vtabptr_9pyteomics_7cparser_ModificationRule;


/* "pyteomics/cparser.pyx":1240
 * 
 * 
 * @cython.freelist(1000)             # <<<<<<<<<<<<<<
 * cdef class ModificationTarget(object):
 *     cdef:
*/

struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationTarget {
  int (*test)(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, int __pyx_skip_dispatch);
  struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *(*_create)(PyObject *, enum __pyx_t_9pyteomics_7cparser_PositionType);
  struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *(*_from_string)(PyObject *);
};
static struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationTarget *__pyx_vtabptr_9pyteomics_7cparser_ModificationTarget;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_mstate_global->__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#if !CYTHON_VECTORCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject *const *args, Py_ssize_t nargs, PyObject *kwargs);
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if !CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03080000
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()
  #define __Pyx_PyFrame_GetLocalsplus(frame)  ((frame)->f_localsplus)
#else
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs);

/* unicode_tailmatch.proto */
static int __Pyx_PyUnicode_Tailmatch(
    PyObject* s, PyObject* substr, Py_ssize_t start, Py_ssize_t end, int direction);

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyLongBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE PyObject* __Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyLong_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectCallNoArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
    PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple);
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* TupleAndListFromArray.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
#endif
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_PySequence_ITEM(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(args, i))
#else
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_XNewRef(PyTuple_GetItem(args, i))
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_ArgRef_FASTCALL(args, i) __Pyx_NewRef(args[i])
    #define __Pyx_NumKwargs_FASTCALL(kwds) __Pyx_PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
#else
    #define __Pyx_ArgRef_FASTCALL __Pyx_ArgRef_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#if CYTHON_METH_FASTCALL || (CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(args + start, stop - start)
#else
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static CYTHON_INLINE int __Pyx_ParseKeywords(
    PyObject *kwds, PyObject *const *kwvalues, PyObject ** const argnames[],
    PyObject *kwds2, PyObject *values[],
    Py_ssize_t num_pos_args, Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* CallCFunction.proto */
#define __Pyx_CallCFunction(cfunc, self, args)\
    ((PyCFunction)(void(*)(void))(cfunc)->func)(self, args)
#define __Pyx_CallCFunctionWithKeywords(cfunc, self, args, kwargs)\
    ((PyCFunctionWithKeywords)(void(*)(void))(cfunc)->func)(self, args, kwargs)
#define __Pyx_CallCFunctionFast(cfunc, self, args, nargs)\
    ((__Pyx_PyCFunctionFast)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs)
#define __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, nargs, kwnames)\
    ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs, kwnames)

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && CYTHON_ATOMICS
    __pyx_atomic_int_type initialized;
#endif
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
static CYTHON_INLINE int __Pyx_CachedCFunction_GetAndSetInitializing(__Pyx_CachedCFunction *cfunc) {
#if !CYTHON_ATOMICS
    return 1;
#else
    __pyx_nonatomic_int_type expected = 0;
    if (__pyx_atomic_int_cmp_exchange(&cfunc->initialized, &expected, 1)) {
        return 0;
    }
    return expected;
#endif
}
static CYTHON_INLINE void __Pyx_CachedCFunction_SetFinishedInitializing(__Pyx_CachedCFunction *cfunc) {
#if CYTHON_ATOMICS
    __pyx_atomic_store(&cfunc->initialized, 2);
#endif
}
#else
#define __Pyx_CachedCFunction_GetAndSetInitializing(cfunc) 2
#define __Pyx_CachedCFunction_SetFinishedInitializing(cfunc)
#endif

/* CallUnboundCMethod2.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyUnicode_Substring.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
            PyObject* text, Py_ssize_t start, Py_ssize_t stop);

/* PyObjectFastCallMethod.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyObject_FastCallMethod(name, args, nargsf) PyObject_VectorcallMethod(name, args, nargsf, NULL)
#else
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf);
#endif

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* PyObjectCall2Args.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* pop_index.proto */
static PyObject* __Pyx__PyObject_PopNewIndex(PyObject* L, PyObject* py_ix);
static PyObject* __Pyx__PyObject_PopIndex(PyObject* L, PyObject* py_ix);
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
static PyObject* __Pyx__PyList_PopIndex(PyObject* L, PyObject* py_ix, Py_ssize_t ix);
#define __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    (likely(PyList_CheckExact(L) && __Pyx_fits_Py_ssize_t(ix, type, is_signed))) ?\
        __Pyx__PyList_PopIndex(L, py_ix, ix) : (\
        (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
            __Pyx__PyObject_PopIndex(L, py_ix)))
#define __Pyx_PyList_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    __Pyx_fits_Py_ssize_t(ix, type, is_signed) ?\
        __Pyx__PyList_PopIndex(L, py_ix, ix) : (\
        (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
            __Pyx__PyObject_PopIndex(L, py_ix)))
#else
#define __Pyx_PyList_PopIndex(L, py_ix, ix, is_signed, type, to_py_func)\
    __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func)
#define __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
        __Pyx__PyObject_PopIndex(L, py_ix))
#endif

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyObject_Unicode.proto */
#define __Pyx_PyObject_Unicode(obj)\
    (likely(PyUnicode_CheckExact(obj)) ? __Pyx_NewRef(obj) : PyObject_Str(obj))

/* RejectKeywords.proto */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds);

/* BuildPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* COrdinalToPyUnicode.proto */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value);
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t width, char padding_char);

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* IncludeStdlibH.proto */
#include <stdlib.h>

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_enum____pyx_t_9pyteomics_7cparser_PositionType(enum __pyx_t_9pyteomics_7cparser_PositionType value, Py_ssize_t width, char padding_char, char format_char);

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* MergeKeywords.proto */
static int __Pyx_MergeKeywords(PyObject *kwdict, PyObject *source_mapping);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PyObjectVectorCallKwBuilder.proto */
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#if CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_Vectorcall_CallFromBuilder PyObject_Vectorcall
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder _PyObject_Vectorcall
#endif
#define __Pyx_MakeVectorcallBuilderKwds(n) PyTuple_New(n)
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder __Pyx_PyObject_FastCallDict
#define __Pyx_MakeVectorcallBuilderKwds(n) __Pyx_PyDict_NewPresized(n)
#define __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n) PyDict_SetItem(builder, key, value)
#define __Pyx_VectorcallBuilder_AddArgStr(key, value, builder, args, n) PyDict_SetItemString(builder, key, value)
#endif

/* PyObjectVectorCallMethodKwBuilder.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_VectorcallMethod_CallFromBuilder PyObject_VectorcallMethod
#else
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* pep479.proto */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyUnicode_Unicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj);

/* GetItemIntUnicode.proto */
#define __Pyx_GetItemInt_Unicode(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Unicode_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "string index out of range"), (Py_UCS4)-1))
static CYTHON_INLINE Py_UCS4 __Pyx_GetItemInt_Unicode_Fast(PyObject* ustring, Py_ssize_t i,
                                                           int wraparound, int boundscheck);

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* CallUnboundCMethod1.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_HasAttr(o, n)  PyObject_HasAttrWithError(o, n)
#else
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);
#endif

/* CallTypeTraverse.proto */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#define __Pyx_call_type_traverse(o, always_call, visit, arg) 0
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg);
#endif

/* FixUpExtensionType.proto */
static CYTHON_INLINE int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);

/* ValidateBasesTuple.proto */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
static int __Pyx_MergeVtables(PyTypeObject *type);

/* LimitedApiGetTypeDict.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp);
#endif

/* DelItemOnTypeDict.proto */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k);
#define __Pyx_DelItemOnTypeDict(tp, k) __Pyx__DelItemOnTypeDict((PyTypeObject*)tp, k)

/* SetItemOnTypeDict.proto */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v);
#define __Pyx_SetItemOnTypeDict(tp, k, v) __Pyx__SetItemOnTypeDict((PyTypeObject*)tp, k, v)

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_1_1
#define __PYX_HAVE_RT_ImportType_proto_3_1_1
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_1_1(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_1_1(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_1_1 {
   __Pyx_ImportType_CheckSize_Error_3_1_1 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_1_1 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_1_1 = 2
};
static PyTypeObject *__Pyx_ImportType_3_1_1(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_1_1 check_size);
#endif

/* ListPack.proto */
static PyObject *__Pyx_PyList_Pack(Py_ssize_t n, ...);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* Globals.proto */
static PyObject* __Pyx_Globals(void);

/* PyObjectDelAttr.proto */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
#define __Pyx_PyObject_DelAttr(o, n) PyObject_SetAttr(o, n, NULL)
#else
#define __Pyx_PyObject_DelAttr(o, n) PyObject_DelAttr(o, n)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   __Pyx_PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* ImportDottedModule.proto */
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple);
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple);

/* FetchSharedCythonModule.proto */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* dict_setdefault.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value, int is_safe_type);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases);

/* PyMethodNew.proto */
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ);

/* PyVectorcallFastCallDict.proto */
#if CYTHON_METH_FASTCALL && (CYTHON_VECTORCALL || CYTHON_BACKPORT_VECTORCALL)
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_BACKPORT_VECTORCALL ||\
        (CYTHON_COMPILING_IN_LIMITED_API && CYTHON_METH_FASTCALL)
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    PyObject *defaults;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_mstate_global->__pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_mstate_global->__pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void));
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func,
                                                         PyTypeObject *defaults_type);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_BACKPORT_VECTORCALL || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* ClassMethod.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
#include "descrobject.h"
#endif
CYTHON_UNUSED static PyObject* __Pyx_Method_ClassMethod(PyObject *method);

/* GetNameInClass.proto */
#define __Pyx_GetNameInClass(var, nmspace, name)  (var) = __Pyx__GetNameInClass(nmspace, name)
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name);

/* CLineInTraceback.proto */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#else
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#endif

/* CodeObjectCache.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject __Pyx_CachedCodeObjectType;
#else
typedef PyCodeObject __Pyx_CachedCodeObjectType;
#endif
typedef struct {
    __Pyx_CachedCodeObjectType* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_int_type accessor_count;
  #endif
};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntFromPy.proto */
static CYTHON_INLINE enum __pyx_t_9pyteomics_7cparser_PositionType __Pyx_PyLong_As_enum____pyx_t_9pyteomics_7cparser_PositionType(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(enum __pyx_t_9pyteomics_7cparser_PositionType value);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyType_GetFullyQualifiedName PyType_GetFullyQualifiedName
#else
static __Pyx_TypeName __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp);
#endif
#else  // !LIMITED_API
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetFullyQualifiedName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2) {
    return PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2);
}
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#ifdef PyExceptionInstance_Check
  #define __Pyx_PyBaseException_Check(obj) PyExceptionInstance_Check(obj)
#else
  #define __Pyx_PyBaseException_Check(obj) __Pyx_TypeCheck(obj, PyExc_BaseException)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* IterNextPlain.proto */
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator);
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void);
#endif

/* ReturnWithStopIteration.proto */
static CYTHON_INLINE void __Pyx_ReturnWithStopIteration(PyObject* value, int async, int iternext);

/* CoroutineBase.proto */
struct __pyx_CoroutineObject;
typedef PyObject *(*__pyx_coroutine_body_t)(struct __pyx_CoroutineObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct __pyx_CoroutineObject {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    __Pyx_pyiter_sendfunc yieldfrom_am_send;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
#if CYTHON_USE_SYS_MONITORING && (CYTHON_PROFILE || CYTHON_TRACE)
    PyMonitoringState __pyx_pymonitoring_state[__Pyx_MonitoringEventTypes_CyGen_count];
    uint64_t __pyx_pymonitoring_version;
#endif
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static __Pyx_PySendResult __Pyx_Coroutine_AmSend(PyObject *self, PyObject *value, PyObject **retval);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static __Pyx_PySendResult __Pyx_Coroutine_Close(PyObject *self, PyObject **retval);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);
static char __Pyx_Coroutine_test_and_set_is_running(__pyx_CoroutineObject *gen);
static void __Pyx_Coroutine_unset_is_running(__pyx_CoroutineObject *gen);
static char __Pyx_Coroutine_get_is_running(__pyx_CoroutineObject *gen);
static PyObject *__Pyx_Coroutine_get_is_running_getter(PyObject *gen, void *closure);
#if __PYX_HAS_PY_AM_SEND == 2
static void __Pyx_SetBackportTypeAmSend(PyTypeObject *type, __Pyx_PyAsyncMethodsStruct *static_amsend_methods, __Pyx_pyiter_sendfunc am_send);
#endif

/* Generator.proto */
#define __Pyx_Generator_USED
#define __Pyx_Generator_CheckExact(obj) __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_mstate_global->__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(PyObject *module);
static CYTHON_INLINE PyObject *__Pyx_Generator_GetInlinedResult(PyObject *self);

/* GetRuntimeVersion.proto */
static unsigned long __Pyx_get_runtime_version(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* MultiPhaseInitModuleState.proto */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
static PyObject *__Pyx_State_FindModule(void*);
static int __Pyx_State_AddModule(PyObject* module, void*);
static int __Pyx_State_RemoveModule(void*);
#elif CYTHON_USE_MODULE_STATE
#define __Pyx_State_FindModule PyState_FindModule
#define __Pyx_State_AddModule PyState_AddModule
#define __Pyx_State_RemoveModule PyState_RemoveModule
#endif

/* #### Code section: module_declarations ### */
/* CythonABIVersion.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
    #if CYTHON_METH_FASTCALL
        #define __PYX_FASTCALL_ABI_SUFFIX  "_fastcall"
    #else
        #define __PYX_FASTCALL_ABI_SUFFIX
    #endif
    #define __PYX_LIMITED_ABI_SUFFIX "limited" __PYX_FASTCALL_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#else
    #define __PYX_LIMITED_ABI_SUFFIX
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    #define __PYX_AM_SEND_ABI_SUFFIX
#elif __PYX_HAS_PY_AM_SEND == 2
    #define __PYX_AM_SEND_ABI_SUFFIX "amsendbackport"
#else
    #define __PYX_AM_SEND_ABI_SUFFIX "noamsend"
#endif
#ifndef __PYX_MONITORING_ABI_SUFFIX
    #define __PYX_MONITORING_ABI_SUFFIX
#endif
#if CYTHON_USE_TP_FINALIZE
    #define __PYX_TP_FINALIZE_ABI_SUFFIX
#else
    #define __PYX_TP_FINALIZE_ABI_SUFFIX "nofinalize"
#endif
#if CYTHON_USE_FREELISTS || !defined(__Pyx_AsyncGen_USED)
    #define __PYX_FREELISTS_ABI_SUFFIX
#else
    #define __PYX_FREELISTS_ABI_SUFFIX "nofreelists"
#endif
#define CYTHON_ABI  __PYX_ABI_VERSION __PYX_LIMITED_ABI_SUFFIX __PYX_MONITORING_ABI_SUFFIX __PYX_TP_FINALIZE_ABI_SUFFIX __PYX_FREELISTS_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."

static struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_f_9pyteomics_7cparser_16SequencePosition__create(PyObject *__pyx_v_amino_acid, PyObject *__pyx_v_modification, PyObject *__pyx_v_terminal, enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_v_position_type); /* proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16SequencePosition_get_index(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, long __pyx_v_i, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_9pyteomics_7cparser_16SequencePosition_get_size(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto*/
static int __pyx_f_9pyteomics_7cparser_16SequencePosition_is_modified(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16SequencePosition_add_modification(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_mod, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16SequencePosition_as_tuple(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_f_9pyteomics_7cparser_16SequencePosition_copy(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto*/
static int __pyx_f_9pyteomics_7cparser_16SequencePosition__eq(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_other); /* proto*/
static int __pyx_f_9pyteomics_7cparser_16SequencePosition__eq_tuple(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_other); /* proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16IsoformGenerator_coerce_modification_dict(CYTHON_UNUSED struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_source, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16IsoformGenerator_initialize_from_str(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16IsoformGenerator_initialize_from_iterable(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16IsoformGenerator_apply_fixed_mods(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16IsoformGenerator_build_site_map(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_16IsoformGenerator_build_site_map *__pyx_optional_args); /* proto*/
static PyObject *__pyx_fuse_0__pyx_f_9pyteomics_7cparser_16IsoformGenerator__apply_variable_assignment(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, long *__pyx_v_indices, PyObject *__pyx_v_assignment, size_t __pyx_v_n); /* proto*/
static PyObject *__pyx_fuse_1__pyx_f_9pyteomics_7cparser_16IsoformGenerator__apply_variable_assignment(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, long *__pyx_v_indices, PyObject *__pyx_v_assignment, size_t __pyx_v_n); /* proto*/
static void __pyx_f_9pyteomics_7cparser_16IsoformGenerator__reset_sequence(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto*/
static int __pyx_f_9pyteomics_7cparser_16ModificationRule_test(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9pyteomics_7cparser_18ModificationTarget_test(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_f_9pyteomics_7cparser_18ModificationTarget__create(PyObject *__pyx_v_target, enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_v_position_type); /* proto*/
static struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_f_9pyteomics_7cparser_18ModificationTarget__from_string(PyObject *__pyx_v_string); /* proto*/

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdlib" */

/* Module declarations from "cython" */

/* Module declarations from "libc.stdio" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.type" */

/* Module declarations from "cpython" */

/* Module declarations from "cpython.object" */

/* Module declarations from "cpython.ref" */

/* Module declarations from "cpython.iterator" */

/* Module declarations from "cpython.pyport" */

/* Module declarations from "cpython.dict" */

/* Module declarations from "pyteomics.ccompat" */

/* Module declarations from "cpython.float" */

/* Module declarations from "cpython.list" */

/* Module declarations from "cpython.tuple" */

/* Module declarations from "cpython.sequence" */

/* Module declarations from "cpython.exc" */

/* Module declarations from "pyteomics.cparser" */
static PyObject *__pyx_v_9pyteomics_7cparser_std_amino_acids = 0;
static PyObject *__pyx_v_9pyteomics_7cparser_std_labels = 0;
static PyObject *__pyx_v_9pyteomics_7cparser_std_nterm = 0;
static PyObject *__pyx_v_9pyteomics_7cparser_std_cterm = 0;
static PyObject *__pyx_v_9pyteomics_7cparser__modX_sequence = 0;
static PyObject *__pyx_v_9pyteomics_7cparser__modX_group = 0;
static PyObject *__pyx_v_9pyteomics_7cparser__modX_split = 0;
static PyObject *__pyx_v_9pyteomics_7cparser__modX_split_findall = 0;
static PyObject *__Pyx_EnumBase = 0;
static PyObject *__Pyx_FlagBase = 0;
static PyObject *__pyx_f_9pyteomics_7cparser_parse(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_parse *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_amino_acid_composition(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition *__pyx_optional_args); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser__split_label(PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_tostring(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_tostring *__pyx_optional_args); /*proto*/
static int __pyx_f_9pyteomics_7cparser_is_term_mod(PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser_match_modX(PyObject *); /*proto*/
static int __pyx_f_9pyteomics_7cparser_length(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_length *__pyx_optional_args); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser_interpolate_labels(PyObject *); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser__tokenize_modx_sequence(PyObject *, struct __pyx_opt_args_9pyteomics_7cparser__tokenize_modx_sequence *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser__sequence_tostring(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser__sequence_tostring *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_copy_sequence(PyObject *, int __pyx_skip_dispatch); /*proto*/
static int __pyx_f_9pyteomics_7cparser__is_nterm_mod(PyObject *); /*proto*/
static int __pyx_f_9pyteomics_7cparser__is_cterm_mod(PyObject *); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser___pyx_unpickle_SequencePosition__set_state(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, PyObject *); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser___pyx_unpickle_IsoformGenerator__set_state(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser___pyx_unpickle_ModificationRule__set_state(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *, PyObject *); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser___pyx_unpickle_ModificationTarget__set_state(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *, PyObject *); /*proto*/
static PyObject *__Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(enum __pyx_t_9pyteomics_7cparser_PositionType); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "pyteomics.cparser"
extern int __pyx_module_is_main_pyteomics__cparser;
int __pyx_module_is_main_pyteomics__cparser = 0;

/* Implementation of "pyteomics.cparser" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_staticmethod;
static PyObject *__pyx_builtin_sum;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_IndexError;
static PyObject *__pyx_builtin_ValueError;
/* #### Code section: string_decls ### */
static const char __pyx_k_[] = "-";
static const char __pyx_k_A[] = "A";
static const char __pyx_k_C[] = "C";
static const char __pyx_k_D[] = "D";
static const char __pyx_k_E[] = "E";
static const char __pyx_k_F[] = "F";
static const char __pyx_k_G[] = "G";
static const char __pyx_k_H[] = "H";
static const char __pyx_k_I[] = "I";
static const char __pyx_k_K[] = "K";
static const char __pyx_k_L[] = "L";
static const char __pyx_k_M[] = "M";
static const char __pyx_k_N[] = "N";
static const char __pyx_k_P[] = "P";
static const char __pyx_k_Q[] = "Q";
static const char __pyx_k_R[] = "R";
static const char __pyx_k_S[] = "S";
static const char __pyx_k_T[] = "T";
static const char __pyx_k_V[] = "V";
static const char __pyx_k_W[] = "W";
static const char __pyx_k_Y[] = "Y";
static const char __pyx_k_a[] = "a";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_j[] = "j";
static const char __pyx_k_k[] = "k";
static const char __pyx_k_n[] = "n";
static const char __pyx_k_OH[] = "-OH";
static const char __pyx_k__3[] = "";
static const char __pyx_k__5[] = ".";
static const char __pyx_k__6[] = "?";
static const char __pyx_k__7[] = "\200\001";
static const char __pyx_k_gc[] = "gc";
static const char __pyx_k_it[] = "it";
static const char __pyx_k_re[] = "re";
static const char __pyx_k_0_q[] = "\200\001\330\0040\260\001\260\026\260q";
static const char __pyx_k_A_2[] = "\200A";
static const char __pyx_k_A_A[] = "\200A\330\010\014\320\014\034\230A";
static const char __pyx_k_H_2[] = "H-";
static const char __pyx_k_avQ[] = "\200\001\330\004.\250a\250v\260Q";
static const char __pyx_k_cls[] = "cls";
static const char __pyx_k_gen[] = "gen";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_mod[] = "mod";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_obj[] = "obj";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_str[] = "str";
static const char __pyx_k_sum[] = "sum";
static const char __pyx_k_A_aq[] = "\200A\340\010!\240\035\250a\250q";
static const char __pyx_k_None[] = "None";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_enum[] = "enum";
static const char __pyx_k_form[] = "form";
static const char __pyx_k_func[] = "__func__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_next[] = "next";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_spec[] = "__spec__";
static const char __pyx_k_test[] = "test";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_cterm[] = "cterm";
static const char __pyx_k_deque[] = "deque";
static const char __pyx_k_entry[] = "entry";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_label[] = "label";
static const char __pyx_k_match[] = "match";
static const char __pyx_k_nterm[] = "nterm";
static const char __pyx_k_parse[] = "parse";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_state[] = "state";
static const char __pyx_k_temp1[] = "temp1";
static const char __pyx_k_temp2[] = "temp2";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_c_term[] = "c_term";
static const char __pyx_k_dict_2[] = "_dict";
static const char __pyx_k_enable[] = "enable";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_groups[] = "groups";
static const char __pyx_k_kwargs[] = "kwargs";
static const char __pyx_k_labels[] = "labels";
static const char __pyx_k_length[] = "length";
static const char __pyx_k_module[] = "module";
static const char __pyx_k_n_mods[] = "n_mods";
static const char __pyx_k_n_term[] = "n_term";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_result[] = "result";
static const char __pyx_k_source[] = "source";
static const char __pyx_k_string[] = "string";
static const char __pyx_k_target[] = "target";
static const char __pyx_k_test_2[] = "__test__";
static const char __pyx_k_tokens[] = "tokens";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_A_Z_A_Z[] = "^([^-]+-)?((?:[^A-Z-]*[A-Z])+)(-[^-]+)?$";
static const char __pyx_k_IntEnum[] = "IntEnum";
static const char __pyx_k_IntFlag[] = "IntFlag";
static const char __pyx_k_compile[] = "compile";
static const char __pyx_k_disable[] = "disable";
static const char __pyx_k_findall[] = "findall";
static const char __pyx_k_indices[] = "indices";
static const char __pyx_k_members[] = "__members__";
static const char __pyx_k_memoize[] = "memoize";
static const char __pyx_k_persist[] = "persist";
static const char __pyx_k_product[] = "product";
static const char __pyx_k_targets[] = "targets";
static const char __pyx_k_term_aa[] = "term_aa";
static const char __pyx_k_add_note[] = "add_note";
static const char __pyx_k_as_tuple[] = "as_tuple";
static const char __pyx_k_format_2[] = "_format";
static const char __pyx_k_generate[] = "generate";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_internal[] = "internal";
static const char __pyx_k_isoforms[] = "isoforms";
static const char __pyx_k_iterable[] = "iterable";
static const char __pyx_k_max_mods[] = "max_mods";
static const char __pyx_k_module_2[] = "__module__";
static const char __pyx_k_override[] = "override";
static const char __pyx_k_pindices[] = "pindices";
static const char __pyx_k_position[] = "position";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_sequence[] = "sequence";
static const char __pyx_k_set_name[] = "__set_name__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_terminal[] = "terminal";
static const char __pyx_k_tostring[] = "tostring";
static const char __pyx_k_A_L_aq_t1[] = "\200A\330\010\014\210L\320\030(\250\016\260a\260q\330\010\017\210t\2201";
static const char __pyx_k_A_Z_A_Z_2[] = "[^A-Z-]*[A-Z]";
static const char __pyx_k_A_Z_A_Z_3[] = "([^A-Z-]*)([A-Z])";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_converter[] = "converter";
static const char __pyx_k_get_index[] = "get_index";
static const char __pyx_k_isenabled[] = "isenabled";
static const char __pyx_k_itertools[] = "itertools";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_universal[] = "universal";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_amino_acid[] = "amino_acid";
static const char __pyx_k_assignment[] = "assignment";
static const char __pyx_k_fixed_mods[] = "fixed_mods";
static const char __pyx_k_from_tuple[] = "from_tuple";
static const char __pyx_k_last_entry[] = "last_entry";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_subindices[] = "subindices";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_assignments[] = "assignments";
static const char __pyx_k_collections[] = "collections";
static const char __pyx_k_from_string[] = "from_string";
static const char __pyx_k_split_label[] = "_split_label";
static const char __pyx_k_PositionType[] = "PositionType";
static const char __pyx_k_combinations[] = "combinations";
static const char __pyx_k_initializing[] = "_initializing";
static const char __pyx_k_is_coroutine[] = "_is_coroutine";
static const char __pyx_k_member_names[] = "_member_names_";
static const char __pyx_k_modification[] = "modification";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_staticmethod[] = "staticmethod";
static const char __pyx_k_stringsource[] = "<stringsource>";
static const char __pyx_k_use_setstate[] = "use_setstate";
static const char __pyx_k_Unknown_label[] = "Unknown label: {}";
static const char __pyx_k_class_getitem[] = "__class_getitem__";
static const char __pyx_k_copy_sequence[] = "copy_sequence";
static const char __pyx_k_from_iterable[] = "from_iterable";
static const char __pyx_k_position_type[] = "position_type";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_sequence_spec[] = "sequence_spec";
static const char __pyx_k_variable_mods[] = "variable_mods";
static const char __pyx_k_PyteomicsError[] = "PyteomicsError";
static const char __pyx_k_build_site_map[] = "build_site_map";
static const char __pyx_k_parse_line_211[] = "parse (line 211)";
static const char __pyx_k_subassignments[] = "subassignments";
static const char __pyx_k_A_XRt1D_L_az_t1[] = "\200A\330\010\021\220\024\220X\230R\230t\2401\240D\250\001\330\010\014\210L\320\030(\250\006\250a\250z\270\027\300\001\330\010\017\210t\2201";
static const char __pyx_k_length_line_118[] = "length (line 118)";
static const char __pyx_k_parsed_sequence[] = "parsed_sequence";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_A_G_fF_1G4q_87_q[] = "\200A\360\014\000\t\022\220\021\330\010\014\210G\220;\230f\240F\250!\330\014\023\320\023#\2401\240G\2504\250q\260\001\330\014\032\230!\2308\2407\250!\330\010\017\210q";
static const char __pyx_k_BasicComposition[] = "BasicComposition";
static const char __pyx_k_IsoformGenerator[] = "IsoformGenerator";
static const char __pyx_k_ModificationRule[] = "ModificationRule";
static const char __pyx_k_SequencePosition[] = "SequencePosition";
static const char __pyx_k_add_modification[] = "add_modification";
static const char __pyx_k_apply_fixed_mods[] = "apply_fixed_mods";
static const char __pyx_k_isoform_generator[] = "isoform_generator";
static const char __pyx_k_pyteomics_cparser[] = "pyteomics.cparser";
static const char __pyx_k_sequence_tostring[] = "_sequence_tostring";
static const char __pyx_k_ModificationTarget[] = "ModificationTarget";
static const char __pyx_k_asyncio_coroutines[] = "asyncio.coroutines";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_include_unmodified[] = "include_unmodified";
static const char __pyx_k_variable_mod_sites[] = "variable_mod_sites";
static const char __pyx_k_assignments_product[] = "assignments_product";
static const char __pyx_k_initialize_from_str[] = "initialize_from_str";
static const char __pyx_k_pyteomics_auxiliary[] = "pyteomics.auxiliary";
static const char __pyx_k_second_type_provided[] = ", second type provided ";
static const char __pyx_k_ModificationRule_test[] = "ModificationRule.test";
static const char __pyx_k_pyteomics_cparser_pyx[] = "pyteomics/cparser.pyx";
static const char __pyx_k_SequencePosition_parse[] = "SequencePosition.parse";
static const char __pyx_k_amino_acid_composition[] = "amino_acid_composition";
static const char __pyx_k_1A_Zq_U_1__AZq_85_q_1_1[] = "\200\001\360\n\000\005\t\210\013\2201\220A\330\004\r\210Z\220q\230\001\330\004\010\210\005\210U\220!\2201\330\010\023\320\023%\240_\260A\260Z\270q\330\010\023\2208\2305\240\001\330\010\021\220\021\220!\330\010\027\220q\230\010\240\003\2401\330\004\013\2101";
static const char __pyx_k_ModificationTarget_test[] = "ModificationTarget.test";
static const char __pyx_k_show_unmodified_termini[] = "show_unmodified_termini";
static const char __pyx_k_A_Kq_A_E_aq_O1D_q_Qa_q_q[] = "\200A\360\010\000\t\r\210K\220q\230\004\230A\330\010\014\210E\220\025\220a\220q\330\014\020\320\020$\240O\2601\260D\270\n\300!\330\014\017\210q\220\005\220Q\220a\330\020\027\220q\330\010\017\210q";
static const char __pyx_k_coerce_modification_dict[] = "coerce_modification_dict";
static const char __pyx_k_initialize_from_iterable[] = "initialize_from_iterable";
static const char __pyx_k_A_Cq_G1_z_q_gR_1_U_gQiq_q[] = "\200A\360\016\000\t\025\220C\220q\330\010\021\220\021\330\010\014\210G\2201\330\014\017\210z\230\021\230%\230q\330\020\026\220g\230R\320\0371\260\024\260U\270!\340\020\026\220g\230Q\230i\240q\250\010\260\001\330\010\017\210q";
static const char __pyx_k_IsoformGenerator_generate[] = "IsoformGenerator.generate";
static const char __pyx_k_Not_a_valid_modX_sequence[] = "Not a valid modX sequence: ";
static const char __pyx_k_SequencePosition_as_tuple[] = "SequencePosition.as_tuple";
static const char __pyx_k_SequencePosition_get_index[] = "SequencePosition.get_index";
static const char __pyx_k_hk_A_1_____81A_7_2_3FnTU_1[] = "\200\001\360\006\000\005\010\200\220h\230k\250\033\260A\330\010\r\210^\2301\330\010\016\320\016!\360\000\000\"]\002\360\000\000]\002_\002\360\000\000_\002`\002\330\004\023\320\023#\2408\2501\250A\330\004\007\200|\2207\230!\330\0102\260!\3203F\300n\320TU\330\004\013\2101";
static const char __pyx_k_SequencePosition_from_tuple[] = "SequencePosition.from_tuple";
static const char __pyx_k_Unsupported_value_of_format[] = "Unsupported value of \"format\": {}";
static const char __pyx_k_allow_unknown_modifications[] = "allow_unknown_modifications";
static const char __pyx_k_Unsupported_type_of_sequence[] = "Unsupported type of sequence.";
static const char __pyx_k_7_AZq_s_nA_A_1_q_vQ_QgV1_t1_q[] = "\200\001\360\014\000\005\014\2107\320\022,\250A\250Z\260q\270\010\300\n\310!\330\004\007\200{\220%\220s\230!\330\010\016\210n\230A\320\035A\300\022\3001\330\004\n\210%\320\017\037\230q\240\007\240v\250Q\330\004\010\210\005\320\r\035\230Q\230g\240V\2501\330\004\007\200t\2101\330\010\020\220\001\340\010\017\210q";
static const char __pyx_k_A_Cq_az_9_vXQ_Kq_E_aq_1G_81_q[] = "\200A\360\014\000\t\025\220C\220q\330\010\021\220\025\220a\220z\320!9\270\026\270v\300X\310Q\330\010\014\210K\220q\230\001\330\010\021\220\021\330\010\014\210E\220\025\220a\220q\330\014\022\220'\230\021\230)\2401\240G\250?\270!\2708\3001\330\010\017\210q";
static const char __pyx_k_pyx_unpickle_IsoformGenerator[] = "__pyx_unpickle_IsoformGenerator";
static const char __pyx_k_pyx_unpickle_ModificationRule[] = "__pyx_unpickle_ModificationRule";
static const char __pyx_k_pyx_unpickle_SequencePosition[] = "__pyx_unpickle_SequencePosition";
static const char __pyx_k_K1_q_Kq_E_aq_1_Qd_2_3avQa_4uAQ[] = "\320\004K\3101\360\030\000\t\030\220q\330\010\014\210K\220q\230\001\330\010\014\210E\220\025\220a\220q\330\014\027\320\027)\250\037\270\001\270\032\3001\330\014\020\220\001\330\014\022\220+\230Q\230d\320\"2\260!\2603\260a\260v\270Q\270a\330\020\027\320\027)\250\021\330\020\023\2204\220u\230A\230Q\330\024\027\220t\2308\240<\250q\330\030!\240\036\250q\260\n\270!\330\030\033\2307\240#\240Q\330\034$\240A\330\034*\250!\250:\260S\270\001\340\034$\240F\250!\330\030#\2408\2505\260\001\330\030 \320 0\260\004\260A\330\030\035\230W\240A\240Q\330\010\013\2101\330\014\020\220\001\330\014\022\220+\230Q\230j\250\001\250\023\250A\250V\2601\260A\330\020\033\320\033-\250_\270A\270Z\300|\320ST\320T\\\320\\]\330\020\033\2308\2405\250\001\330\021\027\220w\230g\240Q\240a\330\010\017\210q";
static const char __pyx_k_ModificationTarget_from_string[] = "ModificationTarget.from_string";
static const char __pyx_k_N_2C1P_wc_a_a_7y_UV_vS_nA_2Q_7[] = "\320\000<\270N\310!\3302C\3001\360P\002\000\005\010\200w\210c\220\021\330\010&\240a\330\004\016\320\016 \240\001\240\021\340\004\014\320\014&\240a\320'7\260y\300\013\310:\320UV\330\004\007\200v\210S\220\001\330\010\016\210n\230A\320\035;\2702\270Q\330\004\013\2107\320\022,\250A\250W\260J\270a\330\004\010\210\005\320\r\035\230Q\230f\240A\330\004\013\2105\320\020 \240\001\240\026\240q\330\004\010\210\005\320\r\035\230Q\230f\240A\360\006\000\005\010\200r\210\023\210E\220\024\220R\220w\230a\340\010\013\2102\210S\220\n\230$\230b\240\007\240q\340\014\020\220\004\220B\220a\330\014\023\2201\220C\220q\330\014\020\220\001\360\022\000\005\027\320\026-\250Q\250f\260A\340\004\013\2109\220B\220c\230\035\240b\250\003\2501\340\004\007\200t\2101\330\010\013\2106\220\027\230\005\230T\240\026\240w\250a\330\014\022\220.\240\001\320!4\260G\2701\270A\330\010\013\2106\220\027\230\005\230T\240\026\240w\250a\330\014\022\220.\240\001\320!4\260G\2701\270A\340\004\007\200q\330\010\014\210A\330\010\014\210E\220\025\220a\220{\240!\2401\330\014\025\220W\230O\2501\320,=\270Q\330\014\017\210s\220!\2208\2303\230a\330\020\026\220e\320\033+\2501\250H\260A\330\020\024\220E\320\031)\250\021\250(\260!\340\020\026\220a\330\020\024\220E\320\031)\250\021\250(\260!\330\014\021\220\024\220U\230$\230b\240\007\240y\260\001\330\020\026\220c\230\021\230\"\230C\230y\250\001\330\021\023\2203\220h\230e\2404\240s\250(\260#\260Q\330\020\026\220n\240A\320%8\270\007\270q\300\001\360\006\000\t\r\210A\330\010\014\210E\220\025\220a\220{\240!\2401\330\014\025\220U\230/\250\021\320*;\2701\340\014\024\320\024.\250a\250}\270I\300[\320PX\320XY\330\014\023\2207\320\0324\260A\260W\270J\300a\330\014\022\220%\320\027'\240q\250\006\250a\330\014\020\220\005\320\025%\240Q\240f\250A\330\014\025\220_\240A\240Q\330\014\030\230\002\230'\240\021\330\014\021\220\024\220X\230T\240\033\250A\330\020\026\220c\230\021\230\"\230C\230y\250\001\330\021\023\2203\220h\230e\2404\240s\250(\260#\260Q\330\020\026\220n\240A\320%8\270\007\270q\300\001\360""\006\000\005\010\320\007\037\230s\240&\250\003\2501\330\010\013\2101\330\014\023\2207\230/\250\021\320*;\2701\330\014\024\220E\230\031\240\"\240A\330\014\025\220Q\220a\330\014\033\2301\230F\320\"3\2603\260e\2701\270A\340\014\031\230\021\320\032+\2503\250a\330\004\007\320\007\037\230s\240&\250\003\2501\330\010\013\2101\330\014\023\2207\230/\250\021\320*;\2703\270a\320?P\320PR\320RS\330\014\023\2205\230\003\2301\330\014\025\220Q\220a\330\014\033\2301\320\034-\250S\260\001\3201B\300\"\300C\300q\340\014\031\230\021\320\032+\2501\360\006\000\005\014\2101";
static const char __pyx_k_SequencePosition_from_iterable[] = "SequencePosition.from_iterable";
static const char __pyx_k_T_A_G1F_a_vWA_q_t_S_IWA_q_1_Qg[] = "\200\001\360\010\000\005\016\210T\220\037\240\004\240A\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220A\330\010\022\220!\330\010\027\220q\340\010\027\220t\230>\250\027\260\005\260S\270\004\270I\300W\310A\330\004\007\200q\330\010\017\320\0171\260\024\260Q\260g\270[\310\007\310q\340\010\017\320\0171\260\024\260Q\260g\270[\310\001";
static const char __pyx_k_hk_A_1_A_A_C_C_D_81A_7_2_3FnTU[] = "\200\001\360\006\000\005\010\200\220h\230k\250\033\260A\330\010\r\210^\2301\330\010\016\320\016!\360\000\000\"A\002\360\000\000A\002C\002\360\000\000C\002D\002\330\004\023\320\023#\2408\2501\250A\330\004\007\200|\2207\230!\330\0102\260!\3203F\300n\320TU\330\004\013\2101";
static const char __pyx_k_hk_A_1_A_A_C_C_D_XQa_7_4A5J_XY[] = "\200\001\360\006\000\005\010\200\220h\230k\250\033\260A\330\010\r\210^\2301\330\010\016\320\016!\360\000\000\"A\002\360\000\000A\002C\002\360\000\000C\002D\002\330\004\023\320\023%\240X\250Q\250a\330\004\007\200|\2207\230!\330\0104\260A\3205J\310.\320XY\330\004\013\2101";
static const char __pyx_k_hk_A_1_n_n_p_p_q_81A_7_2_3FnTU[] = "\200\001\360\006\000\005\010\200\220h\230k\250\033\260A\330\010\r\210^\2301\330\010\016\320\016!\360\000\000\"n\003\360\000\000n\003p\003\360\000\000p\003q\003\330\004\023\320\023#\2408\2501\250A\330\004\007\200|\2207\230!\330\0102\260!\3203F\300n\320TU\330\004\013\2101";
static const char __pyx_k_pyx_unpickle_ModificationTarge[] = "__pyx_unpickle_ModificationTarget";
static const char __pyx_k_1B_t_WA_z_E_Jaz_Qj_e1Ja_H_a_a_a[] = "\320\000\020\220\001\330-:\270!\3301B\300!\360&\000\005\010\200t\210:\220W\230A\340\004\007\200z\220\021\220*\230E\240\023\240J\250a\250z\270\021\330\010\013\210:\220Q\220j\240\001\330\014\036\230e\2401\240J\250a\330$+\320+H\310\001\340\014\036\230a\330\010\032\230!\330\010\013\210;\220a\220\240a\240q\330\014\037\230q\330\010\013\210;\220a\220\240b\250\001\330\014\037\230q\330\010\017\210s\220!\320\023$\240B\240a\330\t\023\2201\220J\230a\330\010\017\210s\220!\2201\220G\2304\230t\240:\250X\260V\2701\330\024\027\220t\230;\240a\240q\340\004\n\210.\230\001\230\021";
static const char __pyx_k_A_A_q_1A_Q_1_E_aq_E_uIQa_A_7q_A[] = "\200A\360\026\000\t%\240A\330\010\027\220q\330\010\017\210|\2301\230A\330\010\025\220Q\330\010\023\2201\330\010\014\210E\220\025\220a\220q\330\014\024\220E\320\031)\250\021\250(\260!\330\014\017\210u\220I\230Q\230a\330\020\023\220>\240\037\260\001\330\024\032\230*\240A\330\030\031\330\0347\260q\330\020,\250A\330\020\033\2301\330\021\026\220k\240\021\240!\330\020\023\220>\240\037\260\001\330\024\032\230*\240A\330\030\031\330\0347\260q\330\020,\250A\330\020\033\2301\340\020\023\220;\230g\240Q\330\024#\2401\330\020\035\230Q\330\010\025\220Q\330\010\017\210s\220!\220<\230~\250Z\260q";
static const char __pyx_k_A_Kq_E_aq_1_Qd_q_1F_1_4uAQ_t8_q[] = "\200A\360\022\000\t\r\210K\220q\230\001\330\010\014\210E\220\025\220a\220q\330\014\027\320\027)\250\037\270\001\270\032\3001\330\014\020\220\001\330\014\022\220+\230Q\230d\240-\250q\260\003\2601\260F\270!\2701\330\020\027\320\027)\250\021\330\020\023\2204\220u\230A\230Q\330\024\027\220t\2308\240<\250q\330\030 \320 1\260\021\260$\260a\330\010\017\210q";
static const char __pyx_k_A_t9A_7_2S_AQ_r_uBb_j_Rq_4_oQ_r[] = "\200A\360\006\000\t\020\210t\2209\230A\330\010\021\220\024\220^\2407\250!\330\010\013\2102\210S\220\001\330\014\022\220*\230A\230Q\330\r\017\210r\220\021\330\014\017\210u\220B\220b\230\002\230!\330\020\026\220j\240\001\240\021\330\014\020\220\005\220R\220q\330\010\013\2104\210\230o\250Q\330\014\017\210r\220\023\220A\330\020\027\220t\2301\330\022\024\220B\220c\230\023\230A\330\020\027\220t\2301\340\020\027\220t\2301\330\r\021\220\037\240\017\250q\330\014\017\210r\220\023\220A\330\020\027\220t\2301\340\020\027\220t\2301\340\014\017\210r\220\022\2207\230#\230Q\330\020\027\220t\2301\330\021\023\2202\220W\230C\230q\330\020\027\220t\2301\340\020\027\220t\2301";
static const char __pyx_k_Cannot_split_a_non_modX_label_s[] = "Cannot split a non-modX label: %s";
static const char __pyx_k_H_O1_5Q_U_1_0_QgQ_AQ_U_1_awa_1I[] = "\320\000H\310\001\360@\001\000\005\022\220\021\330\004\t\210\021\330\004\n\210!\330\004\024\220O\2401\320$5\260Q\330\004\010\320\010 \240\001\240\021\330\004\010\210\005\210U\220!\2201\330\010\020\220\010\320\0300\260\001\260\037\300\001\330\010\013\210:\220Q\220g\230Q\330\014\020\220\014\230A\230Q\330\014\020\220\005\220U\230!\2301\330\020\032\230%\230\250a\250w\260a\330\020#\2401\240I\250Q\250a\330\020\027\220q\330\r\027\220q\230\007\230q\330\014\037\230q\240\007\240q\250\001\330\014\023\2201\330\r\027\220q\230\007\230q\330\014\022\320\022$\240A\330\014\017\210s\220*\230G\2401\330\020\027\320\027*\250!\2503\250k\270\021\270!\330\020\027\220q\330\014\017\210s\220.\240\007\240q\330\020\027\320\027*\250!\2503\250o\270Q\270a\330\020\027\220q\330\014\017\210s\220,\230g\240Q\330\020\027\320\027*\250!\2503\250m\2701\270A\330\020\027\220q\340\014\022\220)\2301\230D\240\001\240\021\340\004\007\200t\2102\210Q\330\010\030\230\001\340\010\030\230\007\230v\240Q\240n\260B\260a\330\010\025\220Q\330\004\n\210!\330\004\010\210\005\210U\220!\2201\330\010\020\220\010\320\0300\260\001\260\037\300\001\330\010\013\210:\220Q\220g\230Q\330\014\020\220\014\230A\230Q\330\014\020\220\005\220U\230!\2301\330\020\032\230%\230\250a\250w\260a\330\020\024\220H\230C\230z\250\023\250H\260C\260{\300$\300d\310!\330\024\025\340\020\027\320\027*\250!\2509\260A\260Q\330\020\024\220E\230\025\230a\230q\330\024!\240\021\240'\250\024\250Q\250a\330\024\033\2301\340\r\027\220q\230\007\230q\330\014\026\220e\2301\330\014\020\220\010\230\003\230:\240S\250\010\260\003\260;\270d\300$\300a\330\020\021\340\014\037\230q\240\007\240q\250\001\330\014\023\320\023&\240a\240w\250a\250q\330\014\020\220\005\220U\230!\2301\330\020\035\230Q\230g\240T\250\021\250!\330\020\027\220q\340\r\027\220q\230\007\230q\330\014\022\320\022$\240A\330\014\017\210s\220/\240\037\260\001\330\020\023\2203\220j\240\007\240q\330\024\027\320\027/\250s\260#\260Z\270s\300!\330\030\037\320\0372\260!\2603\260k\300\021\300!\330\030\034\230E\240\025""\240a\240q\330\034)\250\021\250'\260\024\260Q\260a\330\034#\2401\340\014\017\210s\220.\240\007\240q\330\020\027\320\027*\250!\2503\250o\270Q\270a\330\020\024\220E\230\025\230a\230q\330\024!\240\021\240'\250\024\250Q\250a\330\024\033\2301\340\014\017\210s\220,\230g\240Q\330\020\027\320\027*\250!\2503\250m\2701\270A\330\020\024\220E\230\025\230a\230q\330\024!\240\021\240'\250\024\250Q\250a\330\024\033\2301\340\014\017\210s\220/\240\037\260\001\330\020\023\2203\220j\240\007\240q\330\024\027\320\027/\250s\260#\260Z\270s\300!\330\030\037\320\0372\260!\2603\260k\300\021\300!\330\030\034\230E\240\025\240a\240q\330\034)\250\021\250'\260\024\260Q\260a\330\034#\2401\340\004\r\320\r$\240A\240_\260A\330\004\007\200q\330\010\014\210A\210Q\330\004\013\2101";
static const char __pyx_k_I_1A_U_1_1_3j_q_as_Qa_1_3nG1_as[] = "\320\000I\310\021\360\026\000\005\022\220\021\330\004\t\210\021\330\004\010\210\013\2201\220A\330\004\n\210!\330\004\010\210\005\210U\220!\2201\330\010\016\320\016!\240\037\260\001\260\032\2701\330\010\013\2103\210j\230\007\230q\330\014\023\320\023&\240a\240s\250+\260Q\260a\330\014\023\2201\330\010\013\2103\210n\230G\2401\330\014\023\320\023&\240a\240s\250/\270\021\270!\330\014\023\2201\330\010\013\2103\210l\230'\240\021\330\014\023\320\023&\240a\240s\250-\260q\270\001\330\014\023\2201\330\004\007\200t\2102\210Q\330\010\030\230\001\340\010\030\230\007\230v\240Q\240n\260B\260a\330\010\025\220Q\330\004\n\210!\330\004\010\210\005\210U\220!\2201\330\010\016\320\016!\240\037\260\001\260\032\2701\330\010\013\2103\210o\230_\250A\330\014\017\210s\220*\230G\2401\330\020\027\320\027*\250!\2503\250k\270\021\270!\330\020\024\220E\230\025\230a\230q\330\024!\240\021\240'\250\024\250Q\250a\330\024\033\2301\340\010\013\2103\210n\230G\2401\330\014\023\320\023&\240a\240s\250/\270\021\270!\330\014\020\220\005\220U\230!\2301\330\020\035\230Q\230g\240T\250\021\250!\330\020\027\220q\340\010\013\2103\210l\230'\240\021\330\014\023\320\023&\240a\240s\250-\260q\270\001\330\014\020\220\005\220U\230!\2301\330\020\035\230Q\230g\240T\250\021\250!\330\020\027\220q\340\010\013\2103\210o\230_\250A\330\014\017\210s\220*\230G\2401\330\020\027\320\027*\250!\2503\250k\270\021\270!\330\020\024\220E\230\025\230a\230q\330\024!\240\021\240'\250\024\250Q\250a\330\024\033\2301\340\004\r\320\r$\240A\240_\260A\330\004\007\200q\330\010\014\210A\210Q\330\004\013\2101";
static const char __pyx_k_IsoformGenerator_build_site_map[] = "IsoformGenerator.build_site_map";
static const char __pyx_k_PP_aCTTU_wc_a_z_A_q_Z_Na_1Ja_9D[] = "\320\000P\320P`\320`a\330CT\320TU\360`\001\000\005\010\200w\210c\220\021\330\010&\240a\340\004\016\320\016 \240\001\240\021\340\004\007\200z\220\021\220*\230A\330\010\032\230%\230q\240\005\240Z\320/N\310a\330<=\330'(\330\t\023\2201\220J\230a\330\010\013\2109\220D\230\n\240!\2408\250>\270\021\270*\300D\310\001\330\014\036\230e\2401\240H\250A\250Z\260q\330\020\021\330\026\027\330,-\330\027\030\340\014\036\230a\340\010\016\210n\230A\230Q\330.0\260\004\260A\260Q\340\004\016\210a\340\004\007\200q\330\010\013\210;\220a\220x\230~\250Q\320.?\270q\330\014 \240\001\340\014 \240\001\330\010\013\210;\220a\220\240b\250\001\330\014 \240\013\2501\320,=\270R\270q\340\014 \240\013\2501\320,=\270R\270q\330\010\013\210;\220a\320\027(\250\002\250!\360\006\000\r\024\220?\240$\240a\240q\330\014\032\230!\2309\240H\250B\250f\260A\360\006\000\t\020\210\230d\240!\2401\330\010\026\220a\220y\240\010\250\002\250&\260\001\340\004\010\210\001\330\004\010\210\005\210U\220!\220;\230a\230q\330\010\r\210W\220O\2401\320$5\260Q\330\010\021\220\036\230q\240\t\250\021\330\010\013\2107\220#\220Q\330\014\027\220q\340\014\027\220|\2401\240H\250A\330\010\024\220A\330\010\026\220a\220y\240\004\240A\340\004\013\2101";
static const char __pyx_k_Parse_a_sequence_string_written[] = "Parse a sequence string written in modX notation into a list of\n    labels or (if `split` argument is :py:const:`True`) into a list of\n    tuples representing amino acid residues and their modifications.\n\n    Parameters\n    ----------\n    sequence : str\n        The sequence of a polypeptide.\n    show_unmodified_termini : bool, optional\n        If :py:const:`True` then the unmodified N- and C-termini are explicitly\n        shown in the returned list. Default value is :py:const:`False`.\n    split : bool, optional\n        If :py:const:`True` then the result will be a list of tuples with 1 to 4\n        elements: terminal modification, modification, residue. Default value is\n        :py:const:`False`.\n    allow_unknown_modifications : bool, optional\n        If :py:const:`True` then do not raise an exception when an unknown\n        modification of a known amino acid residue is found in the sequence.\n        This also includes terminal groups.\n        Default value is :py:const:`False`.\n\n        .. note::\n            Since version 2.5, this parameter has effect only if `labels`\n            are provided.\n    labels : container, optional\n        A container of allowed labels for amino acids,\n        modifications and terminal modifications.\n        If not provided, no checks will be done.\n        Separate labels for modifications (such as 'p' or 'ox')\n        can be supplied, which means they are applicable to all residues.\n\n        .. warning::\n            If `show_unmodified_termini` is set to :py:const:`True`, standard\n            terminal groups need to be present in `labels`.\n\n        .. warning::\n            Avoid using sequences with only one terminal group, as they are\n            ambiguous. If you provide one, `labels` (or :py:const:`std_labels`)\n            will be used to resolve the ambiguity.\n\n    Returns\n    -------\n    out : list\n        List of tuples with labels of modifications and amino acid residues.\n\n    Examp""les\n    --------\n    >>> parse('PEPTIDE', split=True)\n    [('P',), ('E',), ('P',), ('T',), ('I',), ('D',), ('E',)]\n    >>> parse('H-PEPTIDE')\n    ['P', 'E', 'P', 'T', 'I', 'D', 'E']\n    >>> parse('PEPTIDE', show_unmodified_termini=True)\n    ['H-', 'P', 'E', 'P', 'T', 'I', 'D', 'E', '-OH']\n    >>> parse('TEpSToxM', labels=std_labels + ['pS', 'oxM'])\n    ['T', 'E', 'pS', 'T', 'oxM']\n    >>> parse('zPEPzTIDzE', True, True, labels=std_labels+['z'])\n    [('H-', 'z', 'P'), ('E',), ('P',), ('z', 'T'), ('I',), ('D',), ('z', 'E', '-OH')]\n    ";
static const char __pyx_k_T_Q_G1F_a_vWA_q_t87_q_34q_QR_34[] = "\200\001\360\010\000\005\016\210T\320\021!\240\024\240Q\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220A\330\010\022\220!\330\010\027\220q\340\010\027\220t\2308\2407\250!\330\004\007\200q\330\010\017\320\0173\2604\260q\270\007\270{\310'\320QR\340\010\017\320\0173\2604\260q\270\007\270{\310!";
static const char __pyx_k_T_d_4_d_TQeeiittx_y_C_C_G_G_R_R[] = "\200\001\360\010\000\005\016\210T\220\035\230d\240)\2504\250\270d\300+\310T\320Qe\320ei\320it\320tx\360\000\000y\001C\002\360\000\000C\002G\002\360\000\000G\002R\002\360\000\000R\002V\002\360\000\000V\002p\002\360\000\000p\002t\002\360\000\000t\002u\002\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220A\330\010\022\220!\330\010\027\220q\340\010\027\220t\230<\240w\250e\2603\260d\270(\300'\310\025\310c\320QU\320Uc\320cj\320jo\320or\320rv\360\000\000w\001J\002\360\000\000J\002Q\002\360\000\000Q\002V\002\360\000\000V\002Y\002\360\000\000Y\002]\002\360\000\000]\002g\002\360\000\000g\002n\002\360\000\000n\002s\002\360\000\000s\002v\002\360\000\000v\002z\002\360\000\000z\002I\003\360\000\000I\003P\003\360\000\000P\003Q\003\330\004\007\200q\330\010\017\320\0171\260\024\260Q\260g\270[\310\007\310q\340\010\017\320\0171\260\024\260Q\260g\270[\310\001";
static const char __pyx_k_T_d_5ET_G1F_a_vWA_q_t_we3d_uTWW[] = "\200\001\360\010\000\005\016\210T\220\035\230d\240/\260\024\3205E\300T\310\021\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220A\330\010\022\220!\330\010\027\220q\340\010\027\220t\230<\240w\250e\2603\260d\270.\310\007\310u\320TW\320W[\320[e\320el\320lm\330\004\007\200q\330\010\017\320\0171\260\024\260Q\260g\270[\310\007\310q\340\010\017\320\0171\260\024\260Q\260g\270[\310\001";
static const char __pyx_k_amino_acid_composition_line_565[] = "amino_acid_composition (line 565)";
static const char __pyx_k_self___class_____name___self_mo[] = "{self.__class__.__name__}({self.modification!r}, {self.targets})";
static const char __pyx_k_self___class_____name___self_ta[] = "{self.__class__.__name__}({self.target!r}, {self.position_type!r})";
static const char __pyx_k_A_4_oQ_1_4_oQ_x_c_Q_q_4xwa_t83ha[] = "\200A\330\010\013\2104\210\230o\250Q\330\014\023\2201\330\010\013\2104\210\230o\250Q\330\014\017\210x\220\240c\250\024\250Q\330\020\027\220q\330\010\013\2104\210x\220w\230a\330\014\017\210t\2208\2303\230h\240a\330\020\027\220q\330\010\017\210q";
static const char __pyx_k_A_t9A_AQ_A_4_oQ_Qd_AXS_A_4_1_a_Q[] = "\200A\360\n\000\t\020\210t\2209\230A\330\010\021\220\033\230A\230Q\330\010\014\210A\330\010\013\2104\210\230o\250Q\330\014\025\220Q\220d\230!\330\014\034\230A\230X\240S\250\004\250A\330\014\021\220\021\330\010\013\2104\210|\2301\330\014\022\220$\220a\330\014\025\220Q\220a\330\014\034\230A\230X\240S\250\001\330\014\021\220\021\330\010\021\220\021\220$\220a\330\010\030\230\001\230\030\240\023\240D\250\001\330\010\r\210Q\330\010\013\2104\210\230o\250Q\330\014\025\220Q\220d\230!\330\014\034\230A\230X\240S\250\004\250A\330\014\021\220\021\330\010\017\210q";
static const char __pyx_k_Calculate_amino_acid_composition[] = "Calculate amino acid composition of a polypeptide.\n\n    Parameters\n    ----------\n    sequence : str or list\n        The sequence of a polypeptide or a list with a parsed sequence.\n    show_unmodified_termini : bool, optional\n        If :py:const:`True` then the unmodified N- and C-terminus are explicitly\n        shown in the returned dict. Default value is :py:const:`False`.\n    term_aa : bool, optional\n        If :py:const:`True` then the terminal amino acid residues are\n        artificially modified with `nterm` or `cterm` modification.\n        Default value is :py:const:`False`.\n    allow_unknown_modifications : bool, optional\n        If :py:const:`True` then do not raise an exception when an unknown\n        modification of a known amino acid residue is found in the sequence.\n        Default value is :py:const:`False`.\n    labels : list, optional\n        A list of allowed labels for amino acids and terminal modifications.\n\n    Returns\n    -------\n    out : dict\n        A dictionary of amino acid composition.\n\n    Examples\n    --------\n    >>> amino_acid_composition('PEPTIDE') ==     {'I': 1, 'P': 2, 'E': 2, 'T': 1, 'D': 1}\n    True\n    >>> amino_acid_composition('PEPTDE', term_aa=True) ==     {'ctermE': 1, 'E': 1, 'D': 1, 'P': 1, 'T': 1, 'ntermP': 1}\n    True\n    >>> amino_acid_composition('PEPpTIDE', labels=std_labels+['pT']) ==     {'I': 1, 'P': 2, 'E': 2, 'D': 1, 'pT': 1}\n    True\n    ";
static const char __pyx_k_Calculate_the_number_of_amino_ac[] = "Calculate the number of amino acid residues in a polypeptide\n    written in modX notation.\n\n    Parameters\n    ----------\n    sequence : str or list or dict\n        A string with a polypeptide sequence, a list with a parsed sequence or\n        a dict of amino acid composition.\n    labels : list, optional\n        A list of allowed labels for amino acids and terminal modifications.\n\n    Examples\n    --------\n    >>> length('PEPTIDE')\n    7\n    >>> length('H-PEPTIDE-OH')\n    7\n    ";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0[] = "Incompatible checksums (0x%x vs (0xe2ca0e1, 0xf7d828e, 0x96edc61) = (amino_acid, modification, position_type, terminal))";
static const char __pyx_k_IsoformGenerator___reduce_cython[] = "IsoformGenerator.__reduce_cython__";
static const char __pyx_k_IsoformGenerator___setstate_cyth[] = "IsoformGenerator.__setstate_cython__";
static const char __pyx_k_IsoformGenerator_apply_fixed_mod[] = "IsoformGenerator.apply_fixed_mods";
static const char __pyx_k_IsoformGenerator_coerce_modifica[] = "IsoformGenerator.coerce_modification_dict";
static const char __pyx_k_IsoformGenerator_initialize_from[] = "IsoformGenerator.initialize_from_str";
static const char __pyx_k_ModificationRule___reduce_cython[] = "ModificationRule.__reduce_cython__";
static const char __pyx_k_ModificationRule___setstate_cyth[] = "ModificationRule.__setstate_cython__";
static const char __pyx_k_ModificationTarget___reduce_cyth[] = "ModificationTarget.__reduce_cython__";
static const char __pyx_k_ModificationTarget___setstate_cy[] = "ModificationTarget.__setstate_cython__";
static const char __pyx_k_Note_that_Cython_is_deliberately[] = "Note that Cython is deliberately stricter than PEP-484 and rejects subclasses of builtin types. If you need to pass subclasses then set the 'annotation_typing' directive to False.";
static const char __pyx_k_Position_already_has_position_ty[] = "Position already has position type ";
static const char __pyx_k_SequencePosition___reduce_cython[] = "SequencePosition.__reduce_cython__";
static const char __pyx_k_SequencePosition___setstate_cyth[] = "SequencePosition.__setstate_cython__";
static const char __pyx_k_SequencePosition_add_modificatio[] = "SequencePosition.add_modification";
static const char __pyx_k_Target_must_be_a_single_Amino_Ac[] = "Target must be a single Amino Acid, got ";
static const char __pyx_k_Terminal_Modifications_with_Targ[] = "Terminal Modifications with Target must be a single Amino Acid, got ";
static const char __pyx_k_Unsupported_type_of_a_sequence_M[] = "Unsupported type of a sequence.Must be str or list, not %s";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0_2[] = "Incompatible checksums (0x%x vs (0x09daa66, 0x1139f77, 0x2fc4213) = (fixed_mods, labels, last_indices, max_mods, original_sequence, override, persist, sequence, show_unmodified_termini, variable_mods))";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0_3[] = "Incompatible checksums (0x%x vs (0xccc7aa9, 0xe851a62, 0xc83ff59) = (modification, targets))";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0_4[] = "Incompatible checksums (0x%x vs (0x0f5dd8b, 0xa1a722a, 0x5a11ef3) = (position_type, target))";
static const char __pyx_k_IsoformGenerator_initialize_from_2[] = "IsoformGenerator.initialize_from_iterable";
/* #### Code section: decls ### */
static PyObject *__pyx_pf_9pyteomics_7cparser_length(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_split, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_2_split_label(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_label); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_4parse(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_split, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_6tostring(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_parsed_sequence, int __pyx_v_show_unmodified_termini); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_8amino_acid_composition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_term_aa, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16SequencePosition___init__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_amino_acid, PyObject *__pyx_v_modification, PyObject *__pyx_v_terminal, PyObject *__pyx_v_position_type); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_2__getitem__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_i); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_4get_index(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, long __pyx_v_i); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_6add_modification(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_mod); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_8__repr__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_9pyteomics_7cparser_16SequencePosition_10__len__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_12as_tuple(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_14from_tuple(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_source); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_16parse(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_sequence, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_18from_iterable(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_iterable); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_20__eq__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_22__ne__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_12modification___get__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_12modification_2__set__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_12modification_4__del__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_13position_type___get__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_13position_type_2__set__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_10amino_acid___get__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_10amino_acid_2__set__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_10amino_acid_4__del__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_8terminal___get__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_8terminal_2__set__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_8terminal_4__del__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_24__reduce_cython__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_26__setstate_cython__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_10_sequence_tostring(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_12copy_sequence(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator___init__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_variable_mods, PyObject *__pyx_v_fixed_mods, PyObject *__pyx_v_labels, PyObject *__pyx_v_max_mods, PyObject *__pyx_v_override, PyObject *__pyx_v_show_unmodified_termini, PyObject *__pyx_v_persist); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_2coerce_modification_dict(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_source); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_4initialize_from_str(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6initialize_from_iterable(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8apply_fixed_mods(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10build_site_map(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, int __pyx_v_include_unmodified); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12generate(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence_spec); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_15__call__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence_spec); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_13variable_mods___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6labels___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6labels_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6labels_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8max_mods___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8max_mods_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8override___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8override_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_7persist___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_7persist_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8sequence___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8sequence_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8sequence_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17original_sequence___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12last_indices___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12last_indices_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12last_indices_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17__reduce_cython__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_19__setstate_cython__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16ModificationRule___init__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, PyObject *__pyx_v_label, PyObject *__pyx_v_targets); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_2test(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_4__call__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, PyObject *__pyx_v_position); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_6__repr__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_12modification___get__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16ModificationRule_12modification_2__set__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16ModificationRule_12modification_4__del__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_7targets___get__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16ModificationRule_7targets_2__set__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_16ModificationRule_7targets_4__del__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_8__reduce_cython__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_10__setstate_cython__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9pyteomics_7cparser_18ModificationTarget___init__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, PyObject *__pyx_v_target, PyObject *__pyx_v_position_type); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_2test(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_4__call__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, PyObject *__pyx_v_position); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_6__repr__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_8from_string(PyObject *__pyx_v_string); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_6target___get__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_18ModificationTarget_6target_2__set__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9pyteomics_7cparser_18ModificationTarget_6target_4__del__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_13position_type___get__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self); /* proto */
static int __pyx_pf_9pyteomics_7cparser_18ModificationTarget_13position_type_2__set__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_10__reduce_cython__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_12__setstate_cython__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_14isoforms(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_17__pyx_unpickle_SequencePosition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_19__pyx_unpickle_IsoformGenerator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_21__pyx_unpickle_ModificationRule(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_23__pyx_unpickle_ModificationTarget(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_9pyteomics_7cparser_SequencePosition(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9pyteomics_7cparser_IsoformGenerator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9pyteomics_7cparser_ModificationRule(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9pyteomics_7cparser_ModificationTarget(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9pyteomics_7cparser___pyx_scope_struct__generate(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9pyteomics_7cparser___pyx_scope_struct_1_isoforms(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
/* SmallCodeConfig */
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  #ifdef __Pyx_CyFunction_USED
  PyTypeObject *__pyx_CyFunctionType;
  #endif
  #ifdef __Pyx_FusedFunction_USED
  PyTypeObject *__pyx_FusedFunctionType;
  #endif
  #ifdef __Pyx_Generator_USED
  PyTypeObject *__pyx_GeneratorType;
  #endif
  #ifdef __Pyx_IterableCoroutine_USED
  PyTypeObject *__pyx_IterableCoroutineType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineAwaitType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineType;
  #endif
  PyTypeObject *__pyx_ptype_7cpython_4type_type;
  PyObject *__pyx_type_9pyteomics_7cparser_SequencePosition;
  PyObject *__pyx_type_9pyteomics_7cparser_IsoformGenerator;
  PyObject *__pyx_type_9pyteomics_7cparser_ModificationRule;
  PyObject *__pyx_type_9pyteomics_7cparser_ModificationTarget;
  PyObject *__pyx_type_9pyteomics_7cparser___pyx_scope_struct__generate;
  PyObject *__pyx_type_9pyteomics_7cparser___pyx_scope_struct_1_isoforms;
  PyTypeObject *__pyx_ptype_9pyteomics_7cparser_SequencePosition;
  PyTypeObject *__pyx_ptype_9pyteomics_7cparser_IsoformGenerator;
  PyTypeObject *__pyx_ptype_9pyteomics_7cparser_ModificationRule;
  PyTypeObject *__pyx_ptype_9pyteomics_7cparser_ModificationTarget;
  PyTypeObject *__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate;
  PyTypeObject *__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop;
  PyObject *__pyx_k__2;
  PyObject *__pyx_k__4;
  PyObject *__pyx_tuple[9];
  PyObject *__pyx_codeobj_tab[35];
  PyObject *__pyx_string_tab[247];
  PyObject *__pyx_int_0;
  PyObject *__pyx_int_1;
  PyObject *__pyx_int_10332774;
  PyObject *__pyx_int_16113035;
  PyObject *__pyx_int_18063223;
  PyObject *__pyx_int_50086419;
  PyObject *__pyx_int_94445299;
  PyObject *__pyx_int_158260321;
  PyObject *__pyx_int_169505322;
  PyObject *__pyx_int_209977177;
  PyObject *__pyx_int_214727337;
  PyObject *__pyx_int_237805793;
  PyObject *__pyx_int_243604066;
  PyObject *__pyx_int_259883662;
  PyObject *__pyx_int_neg_1;
/* #### Code section: module_state_contents ### */

#if CYTHON_USE_FREELISTS
struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_freelist_9pyteomics_7cparser_SequencePosition[5000];
int __pyx_freecount_9pyteomics_7cparser_SequencePosition;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_freelist_9pyteomics_7cparser_IsoformGenerator[100];
int __pyx_freecount_9pyteomics_7cparser_IsoformGenerator;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_freelist_9pyteomics_7cparser_ModificationRule[100];
int __pyx_freecount_9pyteomics_7cparser_ModificationRule;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_freelist_9pyteomics_7cparser_ModificationTarget[1000];
int __pyx_freecount_9pyteomics_7cparser_ModificationTarget;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *__pyx_freelist_9pyteomics_7cparser___pyx_scope_struct__generate[8];
int __pyx_freecount_9pyteomics_7cparser___pyx_scope_struct__generate;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *__pyx_freelist_9pyteomics_7cparser___pyx_scope_struct_1_isoforms[8];
int __pyx_freecount_9pyteomics_7cparser___pyx_scope_struct_1_isoforms;
#endif
/* CachedMethodType.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API
PyObject *__Pyx_CachedMethodType;
#endif

/* CodeObjectCache.module_state_decls */
struct __Pyx_CodeObjectCache __pyx_code_cache;

/* IterNextPlain.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
PyObject *__Pyx_GetBuiltinNext_LimitedAPI_cache;
#endif

/* #### Code section: module_state_end ### */
} __pyx_mstatetype;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate_global (__Pyx_PyModule_GetState(__Pyx_State_FindModule(&__pyx_moduledef)))

#define __pyx_m (__Pyx_State_FindModule(&__pyx_moduledef))
#else
static __pyx_mstatetype __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstatetype * const __pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: constant_name_defines ### */
#define __pyx_kp_u_ __pyx_string_tab[0]
#define __pyx_n_u_A __pyx_string_tab[1]
#define __pyx_kp_u_A_Z_A_Z __pyx_string_tab[2]
#define __pyx_kp_u_A_Z_A_Z_2 __pyx_string_tab[3]
#define __pyx_kp_u_A_Z_A_Z_3 __pyx_string_tab[4]
#define __pyx_n_u_BasicComposition __pyx_string_tab[5]
#define __pyx_n_u_C __pyx_string_tab[6]
#define __pyx_kp_u_Calculate_amino_acid_composition __pyx_string_tab[7]
#define __pyx_kp_u_Calculate_the_number_of_amino_ac __pyx_string_tab[8]
#define __pyx_kp_u_Cannot_split_a_non_modX_label_s __pyx_string_tab[9]
#define __pyx_n_u_D __pyx_string_tab[10]
#define __pyx_n_u_E __pyx_string_tab[11]
#define __pyx_n_u_F __pyx_string_tab[12]
#define __pyx_n_u_G __pyx_string_tab[13]
#define __pyx_n_u_H __pyx_string_tab[14]
#define __pyx_kp_u_H_2 __pyx_string_tab[15]
#define __pyx_n_u_I __pyx_string_tab[16]
#define __pyx_kp_u_Incompatible_checksums_0x_x_vs_0 __pyx_string_tab[17]
#define __pyx_kp_u_Incompatible_checksums_0x_x_vs_0_2 __pyx_string_tab[18]
#define __pyx_kp_u_Incompatible_checksums_0x_x_vs_0_3 __pyx_string_tab[19]
#define __pyx_kp_u_Incompatible_checksums_0x_x_vs_0_4 __pyx_string_tab[20]
#define __pyx_n_u_IndexError __pyx_string_tab[21]
#define __pyx_n_u_IntEnum __pyx_string_tab[22]
#define __pyx_n_u_IntFlag __pyx_string_tab[23]
#define __pyx_n_u_IsoformGenerator __pyx_string_tab[24]
#define __pyx_n_u_IsoformGenerator___reduce_cython __pyx_string_tab[25]
#define __pyx_n_u_IsoformGenerator___setstate_cyth __pyx_string_tab[26]
#define __pyx_n_u_IsoformGenerator_apply_fixed_mod __pyx_string_tab[27]
#define __pyx_n_u_IsoformGenerator_build_site_map __pyx_string_tab[28]
#define __pyx_n_u_IsoformGenerator_coerce_modifica __pyx_string_tab[29]
#define __pyx_n_u_IsoformGenerator_generate __pyx_string_tab[30]
#define __pyx_n_u_IsoformGenerator_initialize_from __pyx_string_tab[31]
#define __pyx_n_u_IsoformGenerator_initialize_from_2 __pyx_string_tab[32]
#define __pyx_n_u_K __pyx_string_tab[33]
#define __pyx_n_u_L __pyx_string_tab[34]
#define __pyx_n_u_M __pyx_string_tab[35]
#define __pyx_n_u_ModificationRule __pyx_string_tab[36]
#define __pyx_n_u_ModificationRule___reduce_cython __pyx_string_tab[37]
#define __pyx_n_u_ModificationRule___setstate_cyth __pyx_string_tab[38]
#define __pyx_n_u_ModificationRule_test __pyx_string_tab[39]
#define __pyx_n_u_ModificationTarget __pyx_string_tab[40]
#define __pyx_n_u_ModificationTarget___reduce_cyth __pyx_string_tab[41]
#define __pyx_n_u_ModificationTarget___setstate_cy __pyx_string_tab[42]
#define __pyx_n_u_ModificationTarget_from_string __pyx_string_tab[43]
#define __pyx_n_u_ModificationTarget_test __pyx_string_tab[44]
#define __pyx_n_u_N __pyx_string_tab[45]
#define __pyx_kp_u_None __pyx_string_tab[46]
#define __pyx_kp_u_Not_a_valid_modX_sequence __pyx_string_tab[47]
#define __pyx_kp_u_Note_that_Cython_is_deliberately __pyx_string_tab[48]
#define __pyx_kp_u_OH __pyx_string_tab[49]
#define __pyx_n_u_P __pyx_string_tab[50]
#define __pyx_kp_u_Parse_a_sequence_string_written __pyx_string_tab[51]
#define __pyx_n_u_PickleError __pyx_string_tab[52]
#define __pyx_n_u_PositionType __pyx_string_tab[53]
#define __pyx_kp_u_Position_already_has_position_ty __pyx_string_tab[54]
#define __pyx_n_u_PyteomicsError __pyx_string_tab[55]
#define __pyx_n_u_Q __pyx_string_tab[56]
#define __pyx_n_u_R __pyx_string_tab[57]
#define __pyx_n_u_S __pyx_string_tab[58]
#define __pyx_n_u_SequencePosition __pyx_string_tab[59]
#define __pyx_n_u_SequencePosition___reduce_cython __pyx_string_tab[60]
#define __pyx_n_u_SequencePosition___setstate_cyth __pyx_string_tab[61]
#define __pyx_n_u_SequencePosition_add_modificatio __pyx_string_tab[62]
#define __pyx_n_u_SequencePosition_as_tuple __pyx_string_tab[63]
#define __pyx_n_u_SequencePosition_from_iterable __pyx_string_tab[64]
#define __pyx_n_u_SequencePosition_from_tuple __pyx_string_tab[65]
#define __pyx_n_u_SequencePosition_get_index __pyx_string_tab[66]
#define __pyx_n_u_SequencePosition_parse __pyx_string_tab[67]
#define __pyx_n_u_T __pyx_string_tab[68]
#define __pyx_kp_u_Target_must_be_a_single_Amino_Ac __pyx_string_tab[69]
#define __pyx_kp_u_Terminal_Modifications_with_Targ __pyx_string_tab[70]
#define __pyx_n_u_TypeError __pyx_string_tab[71]
#define __pyx_kp_u_Unknown_label __pyx_string_tab[72]
#define __pyx_kp_u_Unsupported_type_of_a_sequence_M __pyx_string_tab[73]
#define __pyx_kp_u_Unsupported_type_of_sequence __pyx_string_tab[74]
#define __pyx_kp_u_Unsupported_value_of_format __pyx_string_tab[75]
#define __pyx_n_u_V __pyx_string_tab[76]
#define __pyx_n_u_ValueError __pyx_string_tab[77]
#define __pyx_n_u_W __pyx_string_tab[78]
#define __pyx_n_u_Y __pyx_string_tab[79]
#define __pyx_kp_u__3 __pyx_string_tab[80]
#define __pyx_kp_u__5 __pyx_string_tab[81]
#define __pyx_kp_u__6 __pyx_string_tab[82]
#define __pyx_n_u_a __pyx_string_tab[83]
#define __pyx_n_u_add_modification __pyx_string_tab[84]
#define __pyx_kp_u_add_note __pyx_string_tab[85]
#define __pyx_n_u_allow_unknown_modifications __pyx_string_tab[86]
#define __pyx_n_u_amino_acid __pyx_string_tab[87]
#define __pyx_n_u_amino_acid_composition __pyx_string_tab[88]
#define __pyx_kp_u_amino_acid_composition_line_565 __pyx_string_tab[89]
#define __pyx_n_u_apply_fixed_mods __pyx_string_tab[90]
#define __pyx_n_u_as_tuple __pyx_string_tab[91]
#define __pyx_n_u_assignment __pyx_string_tab[92]
#define __pyx_n_u_assignments __pyx_string_tab[93]
#define __pyx_n_u_assignments_product __pyx_string_tab[94]
#define __pyx_n_u_asyncio_coroutines __pyx_string_tab[95]
#define __pyx_n_u_build_site_map __pyx_string_tab[96]
#define __pyx_n_u_c_term __pyx_string_tab[97]
#define __pyx_n_u_class_getitem __pyx_string_tab[98]
#define __pyx_n_u_cline_in_traceback __pyx_string_tab[99]
#define __pyx_n_u_close __pyx_string_tab[100]
#define __pyx_n_u_cls __pyx_string_tab[101]
#define __pyx_n_u_coerce_modification_dict __pyx_string_tab[102]
#define __pyx_n_u_collections __pyx_string_tab[103]
#define __pyx_n_u_combinations __pyx_string_tab[104]
#define __pyx_n_u_compile __pyx_string_tab[105]
#define __pyx_n_u_converter __pyx_string_tab[106]
#define __pyx_n_u_copy_sequence __pyx_string_tab[107]
#define __pyx_n_u_cterm __pyx_string_tab[108]
#define __pyx_n_u_deque __pyx_string_tab[109]
#define __pyx_n_u_dict __pyx_string_tab[110]
#define __pyx_n_u_dict_2 __pyx_string_tab[111]
#define __pyx_kp_u_disable __pyx_string_tab[112]
#define __pyx_kp_u_enable __pyx_string_tab[113]
#define __pyx_n_u_entry __pyx_string_tab[114]
#define __pyx_n_u_enum __pyx_string_tab[115]
#define __pyx_n_u_findall __pyx_string_tab[116]
#define __pyx_n_u_fixed_mods __pyx_string_tab[117]
#define __pyx_n_u_form __pyx_string_tab[118]
#define __pyx_n_u_format __pyx_string_tab[119]
#define __pyx_n_u_format_2 __pyx_string_tab[120]
#define __pyx_n_u_from_iterable __pyx_string_tab[121]
#define __pyx_n_u_from_string __pyx_string_tab[122]
#define __pyx_n_u_from_tuple __pyx_string_tab[123]
#define __pyx_n_u_func __pyx_string_tab[124]
#define __pyx_kp_u_gc __pyx_string_tab[125]
#define __pyx_n_u_gen __pyx_string_tab[126]
#define __pyx_n_u_generate __pyx_string_tab[127]
#define __pyx_n_u_get __pyx_string_tab[128]
#define __pyx_n_u_get_index __pyx_string_tab[129]
#define __pyx_n_u_getstate __pyx_string_tab[130]
#define __pyx_n_u_groups __pyx_string_tab[131]
#define __pyx_n_u_i __pyx_string_tab[132]
#define __pyx_n_u_include_unmodified __pyx_string_tab[133]
#define __pyx_n_u_indices __pyx_string_tab[134]
#define __pyx_n_u_initialize_from_iterable __pyx_string_tab[135]
#define __pyx_n_u_initialize_from_str __pyx_string_tab[136]
#define __pyx_n_u_initializing __pyx_string_tab[137]
#define __pyx_n_u_internal __pyx_string_tab[138]
#define __pyx_n_u_is_coroutine __pyx_string_tab[139]
#define __pyx_kp_u_isenabled __pyx_string_tab[140]
#define __pyx_n_u_isoform_generator __pyx_string_tab[141]
#define __pyx_n_u_isoforms __pyx_string_tab[142]
#define __pyx_n_u_it __pyx_string_tab[143]
#define __pyx_n_u_items __pyx_string_tab[144]
#define __pyx_n_u_iterable __pyx_string_tab[145]
#define __pyx_n_u_itertools __pyx_string_tab[146]
#define __pyx_n_u_j __pyx_string_tab[147]
#define __pyx_n_u_k __pyx_string_tab[148]
#define __pyx_n_u_kwargs __pyx_string_tab[149]
#define __pyx_n_u_label __pyx_string_tab[150]
#define __pyx_n_u_labels __pyx_string_tab[151]
#define __pyx_n_u_last_entry __pyx_string_tab[152]
#define __pyx_n_u_length __pyx_string_tab[153]
#define __pyx_kp_u_length_line_118 __pyx_string_tab[154]
#define __pyx_n_u_main __pyx_string_tab[155]
#define __pyx_n_u_match __pyx_string_tab[156]
#define __pyx_n_u_max_mods __pyx_string_tab[157]
#define __pyx_n_u_member_names __pyx_string_tab[158]
#define __pyx_n_u_members __pyx_string_tab[159]
#define __pyx_n_u_memoize __pyx_string_tab[160]
#define __pyx_n_u_mod __pyx_string_tab[161]
#define __pyx_n_u_modification __pyx_string_tab[162]
#define __pyx_n_u_module __pyx_string_tab[163]
#define __pyx_n_u_module_2 __pyx_string_tab[164]
#define __pyx_n_u_n __pyx_string_tab[165]
#define __pyx_n_u_n_mods __pyx_string_tab[166]
#define __pyx_n_u_n_term __pyx_string_tab[167]
#define __pyx_n_u_name __pyx_string_tab[168]
#define __pyx_n_u_new __pyx_string_tab[169]
#define __pyx_n_u_next __pyx_string_tab[170]
#define __pyx_n_u_nterm __pyx_string_tab[171]
#define __pyx_n_u_obj __pyx_string_tab[172]
#define __pyx_n_u_override __pyx_string_tab[173]
#define __pyx_n_u_parse __pyx_string_tab[174]
#define __pyx_kp_u_parse_line_211 __pyx_string_tab[175]
#define __pyx_n_u_parsed_sequence __pyx_string_tab[176]
#define __pyx_n_u_persist __pyx_string_tab[177]
#define __pyx_n_u_pickle __pyx_string_tab[178]
#define __pyx_n_u_pindices __pyx_string_tab[179]
#define __pyx_n_u_pop __pyx_string_tab[180]
#define __pyx_n_u_position __pyx_string_tab[181]
#define __pyx_n_u_position_type __pyx_string_tab[182]
#define __pyx_n_u_product __pyx_string_tab[183]
#define __pyx_n_u_pyteomics_auxiliary __pyx_string_tab[184]
#define __pyx_n_u_pyteomics_cparser __pyx_string_tab[185]
#define __pyx_kp_u_pyteomics_cparser __pyx_string_tab[186]
#define __pyx_kp_u_pyteomics_cparser_pyx __pyx_string_tab[187]
#define __pyx_n_u_pyx_PickleError __pyx_string_tab[188]
#define __pyx_n_u_pyx_checksum __pyx_string_tab[189]
#define __pyx_n_u_pyx_result __pyx_string_tab[190]
#define __pyx_n_u_pyx_state __pyx_string_tab[191]
#define __pyx_n_u_pyx_type __pyx_string_tab[192]
#define __pyx_n_u_pyx_unpickle_IsoformGenerator __pyx_string_tab[193]
#define __pyx_n_u_pyx_unpickle_ModificationRule __pyx_string_tab[194]
#define __pyx_n_u_pyx_unpickle_ModificationTarge __pyx_string_tab[195]
#define __pyx_n_u_pyx_unpickle_SequencePosition __pyx_string_tab[196]
#define __pyx_n_u_pyx_vtable __pyx_string_tab[197]
#define __pyx_n_u_qualname __pyx_string_tab[198]
#define __pyx_n_u_range __pyx_string_tab[199]
#define __pyx_n_u_re __pyx_string_tab[200]
#define __pyx_n_u_reduce __pyx_string_tab[201]
#define __pyx_n_u_reduce_cython __pyx_string_tab[202]
#define __pyx_n_u_reduce_ex __pyx_string_tab[203]
#define __pyx_n_u_result __pyx_string_tab[204]
#define __pyx_kp_u_second_type_provided __pyx_string_tab[205]
#define __pyx_n_u_self __pyx_string_tab[206]
#define __pyx_kp_u_self___class_____name___self_mo __pyx_string_tab[207]
#define __pyx_kp_u_self___class_____name___self_ta __pyx_string_tab[208]
#define __pyx_n_u_send __pyx_string_tab[209]
#define __pyx_n_u_sequence __pyx_string_tab[210]
#define __pyx_n_u_sequence_spec __pyx_string_tab[211]
#define __pyx_n_u_sequence_tostring __pyx_string_tab[212]
#define __pyx_n_u_set_name __pyx_string_tab[213]
#define __pyx_n_u_setstate __pyx_string_tab[214]
#define __pyx_n_u_setstate_cython __pyx_string_tab[215]
#define __pyx_n_u_show_unmodified_termini __pyx_string_tab[216]
#define __pyx_n_u_size __pyx_string_tab[217]
#define __pyx_n_u_source __pyx_string_tab[218]
#define __pyx_n_u_spec __pyx_string_tab[219]
#define __pyx_n_u_split __pyx_string_tab[220]
#define __pyx_n_u_split_label __pyx_string_tab[221]
#define __pyx_n_u_state __pyx_string_tab[222]
#define __pyx_n_u_staticmethod __pyx_string_tab[223]
#define __pyx_n_u_str __pyx_string_tab[224]
#define __pyx_n_u_string __pyx_string_tab[225]
#define __pyx_kp_u_stringsource __pyx_string_tab[226]
#define __pyx_n_u_subassignments __pyx_string_tab[227]
#define __pyx_n_u_subindices __pyx_string_tab[228]
#define __pyx_n_u_sum __pyx_string_tab[229]
#define __pyx_n_u_target __pyx_string_tab[230]
#define __pyx_n_u_targets __pyx_string_tab[231]
#define __pyx_n_u_temp1 __pyx_string_tab[232]
#define __pyx_n_u_temp2 __pyx_string_tab[233]
#define __pyx_n_u_term_aa __pyx_string_tab[234]
#define __pyx_n_u_terminal __pyx_string_tab[235]
#define __pyx_n_u_test __pyx_string_tab[236]
#define __pyx_n_u_test_2 __pyx_string_tab[237]
#define __pyx_n_u_throw __pyx_string_tab[238]
#define __pyx_n_u_tokens __pyx_string_tab[239]
#define __pyx_n_u_tostring __pyx_string_tab[240]
#define __pyx_n_u_universal __pyx_string_tab[241]
#define __pyx_n_u_update __pyx_string_tab[242]
#define __pyx_n_u_use_setstate __pyx_string_tab[243]
#define __pyx_n_u_value __pyx_string_tab[244]
#define __pyx_n_u_variable_mod_sites __pyx_string_tab[245]
#define __pyx_n_u_variable_mods __pyx_string_tab[246]
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_clear(PyObject *m) {
  __pyx_mstatetype *clear_module_state = __Pyx_PyModule_GetState(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_CLEAR(clear_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_CLEAR(clear_module_state->__pyx_FusedFunctionType);
  #endif
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __Pyx_State_RemoveModule(NULL);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4type_type);
  Py_CLEAR(clear_module_state->__pyx_ptype_9pyteomics_7cparser_SequencePosition);
  Py_CLEAR(clear_module_state->__pyx_type_9pyteomics_7cparser_SequencePosition);
  Py_CLEAR(clear_module_state->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator);
  Py_CLEAR(clear_module_state->__pyx_type_9pyteomics_7cparser_IsoformGenerator);
  Py_CLEAR(clear_module_state->__pyx_ptype_9pyteomics_7cparser_ModificationRule);
  Py_CLEAR(clear_module_state->__pyx_type_9pyteomics_7cparser_ModificationRule);
  Py_CLEAR(clear_module_state->__pyx_ptype_9pyteomics_7cparser_ModificationTarget);
  Py_CLEAR(clear_module_state->__pyx_type_9pyteomics_7cparser_ModificationTarget);
  Py_CLEAR(clear_module_state->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate);
  Py_CLEAR(clear_module_state->__pyx_type_9pyteomics_7cparser___pyx_scope_struct__generate);
  Py_CLEAR(clear_module_state->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms);
  Py_CLEAR(clear_module_state->__pyx_type_9pyteomics_7cparser___pyx_scope_struct_1_isoforms);
  Py_CLEAR(clear_module_state->__pyx_k__2);
  Py_CLEAR(clear_module_state->__pyx_k__4);
  for (int i=0; i<9; ++i) { Py_CLEAR(clear_module_state->__pyx_tuple[i]); }
  for (int i=0; i<35; ++i) { Py_CLEAR(clear_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<247; ++i) { Py_CLEAR(clear_module_state->__pyx_string_tab[i]); }
  Py_CLEAR(clear_module_state->__pyx_int_0);
  Py_CLEAR(clear_module_state->__pyx_int_1);
  Py_CLEAR(clear_module_state->__pyx_int_10332774);
  Py_CLEAR(clear_module_state->__pyx_int_16113035);
  Py_CLEAR(clear_module_state->__pyx_int_18063223);
  Py_CLEAR(clear_module_state->__pyx_int_50086419);
  Py_CLEAR(clear_module_state->__pyx_int_94445299);
  Py_CLEAR(clear_module_state->__pyx_int_158260321);
  Py_CLEAR(clear_module_state->__pyx_int_169505322);
  Py_CLEAR(clear_module_state->__pyx_int_209977177);
  Py_CLEAR(clear_module_state->__pyx_int_214727337);
  Py_CLEAR(clear_module_state->__pyx_int_237805793);
  Py_CLEAR(clear_module_state->__pyx_int_243604066);
  Py_CLEAR(clear_module_state->__pyx_int_259883662);
  Py_CLEAR(clear_module_state->__pyx_int_neg_1);
  return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstatetype *traverse_module_state = __Pyx_PyModule_GetState(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_tuple);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_bytes);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_VISIT(traverse_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_VISIT(traverse_module_state->__pyx_FusedFunctionType);
  #endif
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4type_type);
  Py_VISIT(traverse_module_state->__pyx_ptype_9pyteomics_7cparser_SequencePosition);
  Py_VISIT(traverse_module_state->__pyx_type_9pyteomics_7cparser_SequencePosition);
  Py_VISIT(traverse_module_state->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator);
  Py_VISIT(traverse_module_state->__pyx_type_9pyteomics_7cparser_IsoformGenerator);
  Py_VISIT(traverse_module_state->__pyx_ptype_9pyteomics_7cparser_ModificationRule);
  Py_VISIT(traverse_module_state->__pyx_type_9pyteomics_7cparser_ModificationRule);
  Py_VISIT(traverse_module_state->__pyx_ptype_9pyteomics_7cparser_ModificationTarget);
  Py_VISIT(traverse_module_state->__pyx_type_9pyteomics_7cparser_ModificationTarget);
  Py_VISIT(traverse_module_state->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate);
  Py_VISIT(traverse_module_state->__pyx_type_9pyteomics_7cparser___pyx_scope_struct__generate);
  Py_VISIT(traverse_module_state->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms);
  Py_VISIT(traverse_module_state->__pyx_type_9pyteomics_7cparser___pyx_scope_struct_1_isoforms);
  Py_VISIT(traverse_module_state->__pyx_k__2);
  Py_VISIT(traverse_module_state->__pyx_k__4);
  for (int i=0; i<9; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_tuple[i]); }
  for (int i=0; i<35; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<247; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_string_tab[i]); }
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_0);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_1);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_10332774);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_16113035);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_18063223);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_50086419);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_94445299);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_158260321);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_169505322);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_209977177);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_214727337);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_237805793);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_243604066);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_259883662);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_int_neg_1);
  return 0;
}
#endif
/* #### Code section: module_code ### */

/* "EnumTypeToPy":2
 * 
 * @cname("__Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py")             # <<<<<<<<<<<<<<
 * cdef __Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(PositionType c_val):
 *     cdef object __pyx_enum
*/

static PyObject *__Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_v_c_val) {
  PyObject *__pyx_v___pyx_enum = 0;
  int __pyx_v_underlying_c_val;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py", 0);

  /* "EnumTypeToPy":5
 * cdef __Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(PositionType c_val):
 *     cdef object __pyx_enum
 *     __pyx_enum = PositionType             # <<<<<<<<<<<<<<
 * 
 *     if 0:
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_PositionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v___pyx_enum = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "EnumTypeToPy":7
 *     __pyx_enum = PositionType
 * 
 *     if 0:             # <<<<<<<<<<<<<<
 *         pass
 *     elif c_val == PositionType.n_term:
*/
  switch (__pyx_v_c_val) {
    case __pyx_e_9pyteomics_7cparser_n_term:

    /* "EnumTypeToPy":10
 *         pass
 *     elif c_val == PositionType.n_term:
 *         return __pyx_enum.n_term             # <<<<<<<<<<<<<<
 *     elif c_val == PositionType.c_term:
 *         return __pyx_enum.c_term
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v___pyx_enum, __pyx_mstate_global->__pyx_n_u_n_term); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "EnumTypeToPy":9
 *     if 0:
 *         pass
 *     elif c_val == PositionType.n_term:             # <<<<<<<<<<<<<<
 *         return __pyx_enum.n_term
 *     elif c_val == PositionType.c_term:
*/
    break;
    case __pyx_e_9pyteomics_7cparser_c_term:

    /* "EnumTypeToPy":12
 *         return __pyx_enum.n_term
 *     elif c_val == PositionType.c_term:
 *         return __pyx_enum.c_term             # <<<<<<<<<<<<<<
 *     elif c_val == PositionType.internal:
 *         return __pyx_enum.internal
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v___pyx_enum, __pyx_mstate_global->__pyx_n_u_c_term); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "EnumTypeToPy":11
 *     elif c_val == PositionType.n_term:
 *         return __pyx_enum.n_term
 *     elif c_val == PositionType.c_term:             # <<<<<<<<<<<<<<
 *         return __pyx_enum.c_term
 *     elif c_val == PositionType.internal:
*/
    break;
    case __pyx_e_9pyteomics_7cparser_internal:

    /* "EnumTypeToPy":14
 *         return __pyx_enum.c_term
 *     elif c_val == PositionType.internal:
 *         return __pyx_enum.internal             # <<<<<<<<<<<<<<
 *     elif c_val == PositionType.universal:
 *         return __pyx_enum.universal
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v___pyx_enum, __pyx_mstate_global->__pyx_n_u_internal); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "EnumTypeToPy":13
 *     elif c_val == PositionType.c_term:
 *         return __pyx_enum.c_term
 *     elif c_val == PositionType.internal:             # <<<<<<<<<<<<<<
 *         return __pyx_enum.internal
 *     elif c_val == PositionType.universal:
*/
    break;
    case __pyx_e_9pyteomics_7cparser_universal:

    /* "EnumTypeToPy":16
 *         return __pyx_enum.internal
 *     elif c_val == PositionType.universal:
 *         return __pyx_enum.universal             # <<<<<<<<<<<<<<
 *     else:
 *         underlying_c_val = <int>c_val
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v___pyx_enum, __pyx_mstate_global->__pyx_n_u_universal); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 16, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "EnumTypeToPy":15
 *     elif c_val == PositionType.internal:
 *         return __pyx_enum.internal
 *     elif c_val == PositionType.universal:             # <<<<<<<<<<<<<<
 *         return __pyx_enum.universal
 *     else:
*/
    break;
    default:

    /* "EnumTypeToPy":18
 *         return __pyx_enum.universal
 *     else:
 *         underlying_c_val = <int>c_val             # <<<<<<<<<<<<<<
 *         return __pyx_enum(underlying_c_val)
 * 
*/
    __pyx_v_underlying_c_val = ((int)__pyx_v_c_val);

    /* "EnumTypeToPy":19
 *     else:
 *         underlying_c_val = <int>c_val
 *         return __pyx_enum(underlying_c_val)             # <<<<<<<<<<<<<<
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = NULL;
    __Pyx_INCREF(__pyx_v___pyx_enum);
    __pyx_t_3 = __pyx_v___pyx_enum; 
    __pyx_t_4 = __Pyx_PyLong_From_int(__pyx_v_underlying_c_val); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 19, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 19, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
    break;
  }

  /* "EnumTypeToPy":2
 * 
 * @cname("__Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py")             # <<<<<<<<<<<<<<
 * cdef __Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(PositionType c_val):
 *     cdef object __pyx_enum
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("EnumTypeToPy.__Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_enum);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":71
 * 
 * 
 * @cython.nonecheck(False)             # <<<<<<<<<<<<<<
 * cdef bint is_term_mod(str label):
 *     """Check if `label` corresponds to a terminal modification.
*/

static int __pyx_f_9pyteomics_7cparser_is_term_mod(PyObject *__pyx_v_label) {
  int __pyx_v_result;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "pyteomics/cparser.pyx":85
 *     cdef:
 *         bint result
 *     result = label.startswith("-")             # <<<<<<<<<<<<<<
 *     if result:
 *         return result
*/
  if (unlikely(__pyx_v_label == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "startswith");
    __PYX_ERR(0, 85, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyUnicode_Tailmatch(__pyx_v_label, __pyx_mstate_global->__pyx_kp_u_, 0, PY_SSIZE_T_MAX, -1); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 85, __pyx_L1_error)
  __pyx_v_result = __pyx_t_1;

  /* "pyteomics/cparser.pyx":86
 *         bint result
 *     result = label.startswith("-")
 *     if result:             # <<<<<<<<<<<<<<
 *         return result
 *     else:
*/
  if (__pyx_v_result) {

    /* "pyteomics/cparser.pyx":87
 *     result = label.startswith("-")
 *     if result:
 *         return result             # <<<<<<<<<<<<<<
 *     else:
 *         result = label.endswith("-")
*/
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":86
 *         bint result
 *     result = label.startswith("-")
 *     if result:             # <<<<<<<<<<<<<<
 *         return result
 *     else:
*/
  }

  /* "pyteomics/cparser.pyx":89
 *         return result
 *     else:
 *         result = label.endswith("-")             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
  /*else*/ {
    if (unlikely(__pyx_v_label == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "endswith");
      __PYX_ERR(0, 89, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyUnicode_Tailmatch(__pyx_v_label, __pyx_mstate_global->__pyx_kp_u_, 0, PY_SSIZE_T_MAX, 1); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 89, __pyx_L1_error)
    __pyx_v_result = __pyx_t_1;

    /* "pyteomics/cparser.pyx":90
 *     else:
 *         result = label.endswith("-")
 *         return result             # <<<<<<<<<<<<<<
 * 
 * cdef inline object match_modX(str label):
*/
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;
  }

  /* "pyteomics/cparser.pyx":71
 * 
 * 
 * @cython.nonecheck(False)             # <<<<<<<<<<<<<<
 * cdef bint is_term_mod(str label):
 *     """Check if `label` corresponds to a terminal modification.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.is_term_mod", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":92
 *         return result
 * 
 * cdef inline object match_modX(str label):             # <<<<<<<<<<<<<<
 *     """Check if `label` is a valid 'modX' label.
 * 
*/

static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser_match_modX(PyObject *__pyx_v_label) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("match_modX", 0);

  /* "pyteomics/cparser.pyx":103
 *     out : re.match or None
 *     """
 *     return PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>label, NULL)             # <<<<<<<<<<<<<<
 * 
 * cdef inline int is_modX(str label):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_9pyteomics_7cparser__modX_split;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_CallMethodObjArgs(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_match, ((PyObject *)__pyx_v_label), NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":92
 *         return result
 * 
 * cdef inline object match_modX(str label):             # <<<<<<<<<<<<<<
 *     """Check if `label` is a valid 'modX' label.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.match_modX", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":105
 *     return PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>label, NULL)
 * 
 * cdef inline int is_modX(str label):             # <<<<<<<<<<<<<<
 *     """Check if `label` is a valid 'modX' label.
 * 
*/

static CYTHON_INLINE int __pyx_f_9pyteomics_7cparser_is_modX(PyObject *__pyx_v_label) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_modX", 0);

  /* "pyteomics/cparser.pyx":116
 *     out : bool
 *     """
 *     return bool(match_modX(label))             # <<<<<<<<<<<<<<
 * 
 * cpdef int length(object sequence,
*/
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_match_modX(__pyx_v_label); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = (!(!__pyx_t_2));
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":105
 *     return PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>label, NULL)
 * 
 * cdef inline int is_modX(str label):             # <<<<<<<<<<<<<<
 *     """Check if `label` is a valid 'modX' label.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.is_modX", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":118
 *     return bool(match_modX(label))
 * 
 * cpdef int length(object sequence,             # <<<<<<<<<<<<<<
 *                  int show_unmodified_termini=0, int split=0,
 *                  int allow_unknown_modifications=0, object labels=std_labels):
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_1length(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static int __pyx_f_9pyteomics_7cparser_length(PyObject *__pyx_v_sequence, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_length *__pyx_optional_args) {
  int __pyx_v_show_unmodified_termini = ((int)0);
  int __pyx_v_split = ((int)0);
  int __pyx_v_allow_unknown_modifications = ((int)0);
  PyObject *__pyx_v_labels = __pyx_mstate_global->__pyx_k__2;
  PyObject *__pyx_v_parsed_sequence = NULL;
  PyObject *__pyx_v_num_term_groups = NULL;
  PyObject *__pyx_7genexpr__pyx_v_aa = NULL;
  PyObject *__pyx_7genexpr__pyx_v_amount = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_9pyteomics_7cparser_parse __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  size_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("length", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_show_unmodified_termini = __pyx_optional_args->show_unmodified_termini;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_split = __pyx_optional_args->split;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_allow_unknown_modifications = __pyx_optional_args->allow_unknown_modifications;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_labels = __pyx_optional_args->labels;
          }
        }
      }
    }
  }

  /* "pyteomics/cparser.pyx":139
 *     7
 *     """
 *     if not sequence: return 0             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(sequence, str) or isinstance(sequence, list):
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_sequence); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 139, __pyx_L1_error)
  __pyx_t_2 = (!__pyx_t_1);
  if (__pyx_t_2) {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "pyteomics/cparser.pyx":141
 *     if not sequence: return 0
 * 
 *     if isinstance(sequence, str) or isinstance(sequence, list):             # <<<<<<<<<<<<<<
 *         if isinstance(sequence, str):
 *             parsed_sequence = parse(sequence, show_unmodified_termini,
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_v_sequence); 
  if (!__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_1 = PyList_Check(__pyx_v_sequence); 
  __pyx_t_2 = __pyx_t_1;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":142
 * 
 *     if isinstance(sequence, str) or isinstance(sequence, list):
 *         if isinstance(sequence, str):             # <<<<<<<<<<<<<<
 *             parsed_sequence = parse(sequence, show_unmodified_termini,
 *                                     split, allow_unknown_modifications, labels)
*/
    __pyx_t_2 = PyUnicode_Check(__pyx_v_sequence); 
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":143
 *     if isinstance(sequence, str) or isinstance(sequence, list):
 *         if isinstance(sequence, str):
 *             parsed_sequence = parse(sequence, show_unmodified_termini,             # <<<<<<<<<<<<<<
 *                                     split, allow_unknown_modifications, labels)
 *         else:
*/
      if (!(likely(PyUnicode_CheckExact(__pyx_v_sequence))||((__pyx_v_sequence) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_sequence))) __PYX_ERR(0, 143, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":144
 *         if isinstance(sequence, str):
 *             parsed_sequence = parse(sequence, show_unmodified_termini,
 *                                     split, allow_unknown_modifications, labels)             # <<<<<<<<<<<<<<
 *         else:
 *             parsed_sequence = sequence
*/
      __pyx_t_4.__pyx_n = 4;
      __pyx_t_4.show_unmodified_termini = __pyx_v_show_unmodified_termini;
      __pyx_t_4.split = __pyx_v_split;
      __pyx_t_4.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
      __pyx_t_4.labels = __pyx_v_labels;
      __pyx_t_3 = __pyx_f_9pyteomics_7cparser_parse(((PyObject*)__pyx_v_sequence), 0, &__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 143, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_v_parsed_sequence = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":142
 * 
 *     if isinstance(sequence, str) or isinstance(sequence, list):
 *         if isinstance(sequence, str):             # <<<<<<<<<<<<<<
 *             parsed_sequence = parse(sequence, show_unmodified_termini,
 *                                     split, allow_unknown_modifications, labels)
*/
      goto __pyx_L7;
    }

    /* "pyteomics/cparser.pyx":146
 *                                     split, allow_unknown_modifications, labels)
 *         else:
 *             parsed_sequence = sequence             # <<<<<<<<<<<<<<
 *         num_term_groups = 0
 *         if is_term_mod(parsed_sequence[0]):
*/
    /*else*/ {
      __Pyx_INCREF(__pyx_v_sequence);
      __pyx_v_parsed_sequence = __pyx_v_sequence;
    }
    __pyx_L7:;

    /* "pyteomics/cparser.pyx":147
 *         else:
 *             parsed_sequence = sequence
 *         num_term_groups = 0             # <<<<<<<<<<<<<<
 *         if is_term_mod(parsed_sequence[0]):
 *             num_term_groups += 1
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
    __pyx_v_num_term_groups = __pyx_mstate_global->__pyx_int_0;

    /* "pyteomics/cparser.pyx":148
 *             parsed_sequence = sequence
 *         num_term_groups = 0
 *         if is_term_mod(parsed_sequence[0]):             # <<<<<<<<<<<<<<
 *             num_term_groups += 1
 *         if is_term_mod(parsed_sequence[-1]):
*/
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_parsed_sequence, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_3))) __PYX_ERR(0, 148, __pyx_L1_error)
    __pyx_t_2 = __pyx_f_9pyteomics_7cparser_is_term_mod(((PyObject*)__pyx_t_3)); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":149
 *         num_term_groups = 0
 *         if is_term_mod(parsed_sequence[0]):
 *             num_term_groups += 1             # <<<<<<<<<<<<<<
 *         if is_term_mod(parsed_sequence[-1]):
 *             num_term_groups += 1
*/
      __pyx_t_3 = __Pyx_PyLong_AddObjC(__pyx_v_num_term_groups, __pyx_mstate_global->__pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 149, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF_SET(__pyx_v_num_term_groups, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":148
 *             parsed_sequence = sequence
 *         num_term_groups = 0
 *         if is_term_mod(parsed_sequence[0]):             # <<<<<<<<<<<<<<
 *             num_term_groups += 1
 *         if is_term_mod(parsed_sequence[-1]):
*/
    }

    /* "pyteomics/cparser.pyx":150
 *         if is_term_mod(parsed_sequence[0]):
 *             num_term_groups += 1
 *         if is_term_mod(parsed_sequence[-1]):             # <<<<<<<<<<<<<<
 *             num_term_groups += 1
 *         return len(parsed_sequence) - num_term_groups
*/
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_parsed_sequence, -1L, long, 1, __Pyx_PyLong_From_long, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_3))) __PYX_ERR(0, 150, __pyx_L1_error)
    __pyx_t_2 = __pyx_f_9pyteomics_7cparser_is_term_mod(((PyObject*)__pyx_t_3)); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 150, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":151
 *             num_term_groups += 1
 *         if is_term_mod(parsed_sequence[-1]):
 *             num_term_groups += 1             # <<<<<<<<<<<<<<
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):
*/
      __pyx_t_3 = __Pyx_PyLong_AddObjC(__pyx_v_num_term_groups, __pyx_mstate_global->__pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 151, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF_SET(__pyx_v_num_term_groups, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":150
 *         if is_term_mod(parsed_sequence[0]):
 *             num_term_groups += 1
 *         if is_term_mod(parsed_sequence[-1]):             # <<<<<<<<<<<<<<
 *             num_term_groups += 1
 *         return len(parsed_sequence) - num_term_groups
*/
    }

    /* "pyteomics/cparser.pyx":152
 *         if is_term_mod(parsed_sequence[-1]):
 *             num_term_groups += 1
 *         return len(parsed_sequence) - num_term_groups             # <<<<<<<<<<<<<<
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()
*/
    __pyx_t_5 = PyObject_Length(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 152, __pyx_L1_error)
    __pyx_t_3 = PyLong_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 152, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = PyNumber_Subtract(__pyx_t_3, __pyx_v_num_term_groups); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 152, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_7 = __Pyx_PyLong_As_int(__pyx_t_6); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 152, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_r = __pyx_t_7;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":141
 *     if not sequence: return 0
 * 
 *     if isinstance(sequence, str) or isinstance(sequence, list):             # <<<<<<<<<<<<<<
 *         if isinstance(sequence, str):
 *             parsed_sequence = parse(sequence, show_unmodified_termini,
*/
  }

  /* "pyteomics/cparser.pyx":153
 *             num_term_groups += 1
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):             # <<<<<<<<<<<<<<
 *         return sum([amount for aa, amount in sequence.items()
 *                     if not is_term_mod(aa)])
*/
  __pyx_t_2 = PyDict_Check(__pyx_v_sequence); 
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":154
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()             # <<<<<<<<<<<<<<
 *                     if not is_term_mod(aa)])
 * 
*/
    __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_builtin_sum);
    __pyx_t_8 = __pyx_builtin_sum; 
    { /* enter inner scope */
      __pyx_t_9 = PyList_New(0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 154, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_5 = 0;
      if (unlikely(__pyx_v_sequence == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
        __PYX_ERR(0, 154, __pyx_L12_error)
      }
      __pyx_t_12 = __Pyx_dict_iterator(__pyx_v_sequence, 0, __pyx_mstate_global->__pyx_n_u_items, (&__pyx_t_11), (&__pyx_t_7)); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 154, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_XDECREF(__pyx_t_10);
      __pyx_t_10 = __pyx_t_12;
      __pyx_t_12 = 0;
      while (1) {
        __pyx_t_14 = __Pyx_dict_iter_next(__pyx_t_10, __pyx_t_11, &__pyx_t_5, &__pyx_t_12, &__pyx_t_13, NULL, __pyx_t_7);
        if (unlikely(__pyx_t_14 == 0)) break;
        if (unlikely(__pyx_t_14 == -1)) __PYX_ERR(0, 154, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_aa, __pyx_t_12);
        __pyx_t_12 = 0;
        __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_amount, __pyx_t_13);
        __pyx_t_13 = 0;

        /* "pyteomics/cparser.pyx":155
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()
 *                     if not is_term_mod(aa)])             # <<<<<<<<<<<<<<
 * 
 *     raise PyteomicsError('Unsupported type of sequence.')
*/
        if (!(likely(PyUnicode_CheckExact(__pyx_7genexpr__pyx_v_aa))||((__pyx_7genexpr__pyx_v_aa) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_7genexpr__pyx_v_aa))) __PYX_ERR(0, 155, __pyx_L12_error)
        __pyx_t_2 = __pyx_f_9pyteomics_7cparser_is_term_mod(((PyObject*)__pyx_7genexpr__pyx_v_aa)); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 155, __pyx_L12_error)
        __pyx_t_1 = (!__pyx_t_2);
        if (__pyx_t_1) {

          /* "pyteomics/cparser.pyx":154
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()             # <<<<<<<<<<<<<<
 *                     if not is_term_mod(aa)])
 * 
*/
          if (unlikely(__Pyx_ListComp_Append(__pyx_t_9, (PyObject*)__pyx_7genexpr__pyx_v_amount))) __PYX_ERR(0, 154, __pyx_L12_error)

          /* "pyteomics/cparser.pyx":155
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()
 *                     if not is_term_mod(aa)])             # <<<<<<<<<<<<<<
 * 
 *     raise PyteomicsError('Unsupported type of sequence.')
*/
        }
      }
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_aa); __pyx_7genexpr__pyx_v_aa = 0;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_amount); __pyx_7genexpr__pyx_v_amount = 0;
      goto __pyx_L16_exit_scope;
      __pyx_L12_error:;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_aa); __pyx_7genexpr__pyx_v_aa = 0;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_amount); __pyx_7genexpr__pyx_v_amount = 0;
      goto __pyx_L1_error;
      __pyx_L16_exit_scope:;
    } /* exit inner scope */
    __pyx_t_15 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_9};
      __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+__pyx_t_15, (2-__pyx_t_15) | (__pyx_t_15*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 154, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }

    /* "pyteomics/cparser.pyx":154
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()             # <<<<<<<<<<<<<<
 *                     if not is_term_mod(aa)])
 * 
*/
    __pyx_t_7 = __Pyx_PyLong_As_int(__pyx_t_6); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_r = __pyx_t_7;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":153
 *             num_term_groups += 1
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):             # <<<<<<<<<<<<<<
 *         return sum([amount for aa, amount in sequence.items()
 *                     if not is_term_mod(aa)])
*/
  }

  /* "pyteomics/cparser.pyx":157
 *                     if not is_term_mod(aa)])
 * 
 *     raise PyteomicsError('Unsupported type of sequence.')             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_PyteomicsError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_15 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
    assert(__pyx_t_8);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
    __pyx_t_15 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_mstate_global->__pyx_kp_u_Unsupported_type_of_sequence};
    __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+__pyx_t_15, (2-__pyx_t_15) | (__pyx_t_15*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 157, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
  }
  __Pyx_Raise(__pyx_t_6, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __PYX_ERR(0, 157, __pyx_L1_error)

  /* "pyteomics/cparser.pyx":118
 *     return bool(match_modX(label))
 * 
 * cpdef int length(object sequence,             # <<<<<<<<<<<<<<
 *                  int show_unmodified_termini=0, int split=0,
 *                  int allow_unknown_modifications=0, object labels=std_labels):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyteomics.cparser.length", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_parsed_sequence);
  __Pyx_XDECREF(__pyx_v_num_term_groups);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_aa);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_amount);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_1length(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_length, "length(sequence, int show_unmodified_termini=0, int split=0, int allow_unknown_modifications=0, labels=std_labels) -> int\nCalculate the number of amino acid residues in a polypeptide\nwritten in modX notation.\n\nParameters\n----------\nsequence : str or list or dict\n    A string with a polypeptide sequence, a list with a parsed sequence or\n    a dict of amino acid composition.\nlabels : list, optional\n    A list of allowed labels for amino acids and terminal modifications.\n\nExamples\n--------\n>>> length('PEPTIDE')\n7\n>>> length('H-PEPTIDE-OH')\n7");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_1length = {"length", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_1length, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_length};
static PyObject *__pyx_pw_9pyteomics_7cparser_1length(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  int __pyx_v_show_unmodified_termini;
  int __pyx_v_split;
  int __pyx_v_allow_unknown_modifications;
  PyObject *__pyx_v_labels = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("length (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,&__pyx_mstate_global->__pyx_n_u_show_unmodified_termini,&__pyx_mstate_global->__pyx_n_u_split,&__pyx_mstate_global->__pyx_n_u_allow_unknown_modifications,&__pyx_mstate_global->__pyx_n_u_labels,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 118, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 118, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 118, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 118, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 118, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 118, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "length", 0) < 0) __PYX_ERR(0, 118, __pyx_L3_error)
      if (!values[4]) values[4] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__2);
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("length", 0, 1, 5, i); __PYX_ERR(0, 118, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 118, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 118, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 118, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 118, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 118, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[4]) values[4] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__2);
    }
    __pyx_v_sequence = values[0];
    if (values[1]) {
      __pyx_v_show_unmodified_termini = __Pyx_PyLong_As_int(values[1]); if (unlikely((__pyx_v_show_unmodified_termini == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 119, __pyx_L3_error)
    } else {
      __pyx_v_show_unmodified_termini = ((int)0);
    }
    if (values[2]) {
      __pyx_v_split = __Pyx_PyLong_As_int(values[2]); if (unlikely((__pyx_v_split == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 119, __pyx_L3_error)
    } else {
      __pyx_v_split = ((int)0);
    }
    if (values[3]) {
      __pyx_v_allow_unknown_modifications = __Pyx_PyLong_As_int(values[3]); if (unlikely((__pyx_v_allow_unknown_modifications == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 120, __pyx_L3_error)
    } else {
      __pyx_v_allow_unknown_modifications = ((int)0);
    }
    __pyx_v_labels = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("length", 0, 1, 5, __pyx_nargs); __PYX_ERR(0, 118, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.length", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_length(__pyx_self, __pyx_v_sequence, __pyx_v_show_unmodified_termini, __pyx_v_split, __pyx_v_allow_unknown_modifications, __pyx_v_labels);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_length(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_split, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  struct __pyx_opt_args_9pyteomics_7cparser_length __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("length", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 4;
  __pyx_t_2.show_unmodified_termini = __pyx_v_show_unmodified_termini;
  __pyx_t_2.split = __pyx_v_split;
  __pyx_t_2.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
  __pyx_t_2.labels = __pyx_v_labels;
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_length(__pyx_v_sequence, 1, &__pyx_t_2); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 118, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyteomics.cparser.length", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":160
 * 
 * 
 * cdef inline list interpolate_labels(object labels):             # <<<<<<<<<<<<<<
 *     cdef list _labels
 *     if labels is None:
*/

static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser_interpolate_labels(PyObject *__pyx_v_labels) {
  PyObject *__pyx_v__labels = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("interpolate_labels", 0);

  /* "pyteomics/cparser.pyx":162
 * cdef inline list interpolate_labels(object labels):
 *     cdef list _labels
 *     if labels is None:             # <<<<<<<<<<<<<<
 *         _labels = list(std_labels)
 *     else:
*/
  __pyx_t_1 = (__pyx_v_labels == Py_None);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":163
 *     cdef list _labels
 *     if labels is None:
 *         _labels = list(std_labels)             # <<<<<<<<<<<<<<
 *     else:
 *         _labels = list(labels)
*/
    __pyx_t_2 = PySequence_List(__pyx_v_9pyteomics_7cparser_std_labels); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v__labels = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":162
 * cdef inline list interpolate_labels(object labels):
 *     cdef list _labels
 *     if labels is None:             # <<<<<<<<<<<<<<
 *         _labels = list(std_labels)
 *     else:
*/
    goto __pyx_L3;
  }

  /* "pyteomics/cparser.pyx":165
 *         _labels = list(std_labels)
 *     else:
 *         _labels = list(labels)             # <<<<<<<<<<<<<<
 *     _labels.extend((std_cterm, std_nterm))
 *     return _labels
*/
  /*else*/ {
    __pyx_t_2 = PySequence_List(__pyx_v_labels); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v__labels = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "pyteomics/cparser.pyx":166
 *     else:
 *         _labels = list(labels)
 *     _labels.extend((std_cterm, std_nterm))             # <<<<<<<<<<<<<<
 *     return _labels
 * 
*/
  __pyx_t_3 = __Pyx_ListComp_Append(__pyx_v__labels, __pyx_v_9pyteomics_7cparser_std_cterm); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 166, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyList_Append(__pyx_v__labels, __pyx_v_9pyteomics_7cparser_std_nterm); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 166, __pyx_L1_error)
  (void)((__pyx_t_3 | __pyx_t_4));

  /* "pyteomics/cparser.pyx":167
 *         _labels = list(labels)
 *     _labels.extend((std_cterm, std_nterm))
 *     return _labels             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v__labels);
  __pyx_r = __pyx_v__labels;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":160
 * 
 * 
 * cdef inline list interpolate_labels(object labels):             # <<<<<<<<<<<<<<
 *     cdef list _labels
 *     if labels is None:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.interpolate_labels", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__labels);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":170
 * 
 * 
 * cpdef inline tuple _split_label(str label):             # <<<<<<<<<<<<<<
 *     cdef:
 *         str mod, X
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_3_split_label(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser__split_label(PyObject *__pyx_v_label, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_mod = 0;
  PyObject *__pyx_v_X = 0;
  PyObject *__pyx_v_temp = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_split_label", 0);

  /* "pyteomics/cparser.pyx":176
 * 
 * 
 *     temp = <tuple>PyObject_CallMethodObjArgs(match_modX(label), "groups", NULL)             # <<<<<<<<<<<<<<
 *     if <PyObject*>temp == NULL:
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)
*/
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_match_modX(__pyx_v_label); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_CallMethodObjArgs(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_groups, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_temp = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":177
 * 
 *     temp = <tuple>PyObject_CallMethodObjArgs(match_modX(label), "groups", NULL)
 *     if <PyObject*>temp == NULL:             # <<<<<<<<<<<<<<
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
*/
  __pyx_t_3 = (((PyObject *)__pyx_v_temp) == NULL);
  if (unlikely(__pyx_t_3)) {

    /* "pyteomics/cparser.pyx":178
 *     temp = <tuple>PyObject_CallMethodObjArgs(match_modX(label), "groups", NULL)
 *     if <PyObject*>temp == NULL:
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)             # <<<<<<<<<<<<<<
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_PyteomicsError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Cannot_split_a_non_modX_label_s, __pyx_v_label); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_5};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 178, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 178, __pyx_L1_error)

    /* "pyteomics/cparser.pyx":177
 * 
 *     temp = <tuple>PyObject_CallMethodObjArgs(match_modX(label), "groups", NULL)
 *     if <PyObject*>temp == NULL:             # <<<<<<<<<<<<<<
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
*/
  }

  /* "pyteomics/cparser.pyx":179
 *     if <PyObject*>temp == NULL:
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)             # <<<<<<<<<<<<<<
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)
 *     if not mod:
*/
  __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_temp, 0);
  __pyx_t_1 = ((PyObject *)__pyx_t_7);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_mod = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":180
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)             # <<<<<<<<<<<<<<
 *     if not mod:
 *         return (X,)
*/
  __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_temp, 1);
  __pyx_t_1 = ((PyObject *)__pyx_t_7);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_X = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":181
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)
 *     if not mod:             # <<<<<<<<<<<<<<
 *         return (X,)
 *     else:
*/
  __pyx_t_3 = (__pyx_v_mod != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_mod) != 0);
  if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_3 < 0))) __PYX_ERR(0, 181, __pyx_L1_error)
  __pyx_t_8 = (!__pyx_t_3);
  if (__pyx_t_8) {

    /* "pyteomics/cparser.pyx":182
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)
 *     if not mod:
 *         return (X,)             # <<<<<<<<<<<<<<
 *     else:
 *         return temp
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_X);
    __Pyx_GIVEREF(__pyx_v_X);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_X) != (0)) __PYX_ERR(0, 182, __pyx_L1_error);
    __pyx_r = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":181
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)
 *     if not mod:             # <<<<<<<<<<<<<<
 *         return (X,)
 *     else:
*/
  }

  /* "pyteomics/cparser.pyx":184
 *         return (X,)
 *     else:
 *         return temp             # <<<<<<<<<<<<<<
 * 
 * 
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_temp);
    __pyx_r = __pyx_v_temp;
    goto __pyx_L0;
  }

  /* "pyteomics/cparser.pyx":170
 * 
 * 
 * cpdef inline tuple _split_label(str label):             # <<<<<<<<<<<<<<
 *     cdef:
 *         str mod, X
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyteomics.cparser._split_label", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_mod);
  __Pyx_XDECREF(__pyx_v_X);
  __Pyx_XDECREF(__pyx_v_temp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_3_split_label(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_2_split_label, "_split_label(str label) -> tuple");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_3_split_label = {"_split_label", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_3_split_label, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_2_split_label};
static PyObject *__pyx_pw_9pyteomics_7cparser_3_split_label(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_label = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_split_label (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_label,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 170, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 170, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_split_label", 0) < 0) __PYX_ERR(0, 170, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_split_label", 1, 1, 1, i); __PYX_ERR(0, 170, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 170, __pyx_L3_error)
    }
    __pyx_v_label = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_split_label", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 170, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser._split_label", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_label), (&PyUnicode_Type), 1, "label", 1))) __PYX_ERR(0, 170, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_2_split_label(__pyx_self, __pyx_v_label);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_2_split_label(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_label) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_split_label", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser__split_label(__pyx_v_label, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser._split_label", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":190
 * 
 * 
 * cdef list _parse_modx_sequence_split(str sequence):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list parsed_sequence
*/

static PyObject *__pyx_f_9pyteomics_7cparser__parse_modx_sequence_split(PyObject *__pyx_v_sequence) {
  PyObject *__pyx_v_parsed_sequence = 0;
  PyObject *__pyx_v_parts = 0;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_n;
  PyObject *__pyx_v_g = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *__pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_parse_modx_sequence_split", 0);

  /* "pyteomics/cparser.pyx":197
 *         object g
 * 
 *     parsed_sequence = []             # <<<<<<<<<<<<<<
 *     parts = tuple(_modX_split_findall(sequence))
 *     n = len(parts)
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":198
 * 
 *     parsed_sequence = []
 *     parts = tuple(_modX_split_findall(sequence))             # <<<<<<<<<<<<<<
 *     n = len(parts)
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser__modX_split_findall);
  __pyx_t_3 = __pyx_v_9pyteomics_7cparser__modX_split_findall; 
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_sequence};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_3 = __Pyx_PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_parts = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyteomics/cparser.pyx":199
 *     parsed_sequence = []
 *     parts = tuple(_modX_split_findall(sequence))
 *     n = len(parts)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(n):
*/
  __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(__pyx_v_parts); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 199, __pyx_L1_error)
  __pyx_v_n = __pyx_t_5;

  /* "pyteomics/cparser.pyx":201
 *     n = len(parts)
 * 
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         g = <object>PyTuple_GET_ITEM(parts, i)
 * 
*/
  __pyx_t_5 = __pyx_v_n;
  __pyx_t_6 = __pyx_t_5;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "pyteomics/cparser.pyx":202
 * 
 *     for i in range(n):
 *         g = <object>PyTuple_GET_ITEM(parts, i)             # <<<<<<<<<<<<<<
 * 
 *         if g[0]:
*/
    __pyx_t_8 = PyTuple_GET_ITEM(__pyx_v_parts, __pyx_v_i);
    __pyx_t_3 = ((PyObject *)__pyx_t_8);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_g, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyteomics/cparser.pyx":204
 *         g = <object>PyTuple_GET_ITEM(parts, i)
 * 
 *         if g[0]:             # <<<<<<<<<<<<<<
 *             parsed_sequence.append(g)
 *         else:
*/
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_g, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 204, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "pyteomics/cparser.pyx":205
 * 
 *         if g[0]:
 *             parsed_sequence.append(g)             # <<<<<<<<<<<<<<
 *         else:
 *             parsed_sequence.append((g[1],))
*/
      __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_parsed_sequence, __pyx_v_g); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 205, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":204
 *         g = <object>PyTuple_GET_ITEM(parts, i)
 * 
 *         if g[0]:             # <<<<<<<<<<<<<<
 *             parsed_sequence.append(g)
 *         else:
*/
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":207
 *             parsed_sequence.append(g)
 *         else:
 *             parsed_sequence.append((g[1],))             # <<<<<<<<<<<<<<
 *     return parsed_sequence
 * 
*/
    /*else*/ {
      __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_g, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_3);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 207, __pyx_L1_error);
      __pyx_t_3 = 0;
      __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_parsed_sequence, __pyx_t_1); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L5:;
  }

  /* "pyteomics/cparser.pyx":208
 *         else:
 *             parsed_sequence.append((g[1],))
 *     return parsed_sequence             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_parsed_sequence);
  __pyx_r = __pyx_v_parsed_sequence;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":190
 * 
 * 
 * cdef list _parse_modx_sequence_split(str sequence):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list parsed_sequence
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyteomics.cparser._parse_modx_sequence_split", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_parsed_sequence);
  __Pyx_XDECREF(__pyx_v_parts);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":211
 * 
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,             # <<<<<<<<<<<<<<
 *                  bint allow_unknown_modifications=0, object labels=None):
 *     """Parse a sequence string written in modX notation into a list of
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_5parse(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_parse(PyObject *__pyx_v_sequence, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_parse *__pyx_optional_args) {
  int __pyx_v_show_unmodified_termini = ((int)0);
  int __pyx_v_split = ((int)0);
  int __pyx_v_allow_unknown_modifications = ((int)0);

  /* "pyteomics/cparser.pyx":212
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,
 *                  bint allow_unknown_modifications=0, object labels=None):             # <<<<<<<<<<<<<<
 *     """Parse a sequence string written in modX notation into a list of
 *     labels or (if `split` argument is :py:const:`True`) into a list of
*/
  PyObject *__pyx_v_labels = ((PyObject *)Py_None);
  PyObject *__pyx_v_match = 0;
  PyObject *__pyx_v_temp = 0;
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_body = 0;
  PyObject *__pyx_v_c = 0;
  PyObject *__pyx_v_nterm = 0;
  PyObject *__pyx_v_cterm = 0;
  PyObject *__pyx_v_mod = 0;
  PyObject *__pyx_v_X = 0;
  PyObject *__pyx_v_parsed_sequence = 0;
  PyObject *__pyx_v_tgroup = 0;
  PyObject *__pyx_v_sgroup = 0;
  PyObject *__pyx_v__labels = 0;
  Py_ssize_t __pyx_v_i;
  int __pyx_v_is_mod;
  int __pyx_v_in_labels;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7;
  int __pyx_t_8;
  struct __pyx_opt_args_9pyteomics_7cparser__tokenize_modx_sequence __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("parse", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_show_unmodified_termini = __pyx_optional_args->show_unmodified_termini;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_split = __pyx_optional_args->split;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_allow_unknown_modifications = __pyx_optional_args->allow_unknown_modifications;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_labels = __pyx_optional_args->labels;
          }
        }
      }
    }
  }

  /* "pyteomics/cparser.pyx":284
 *         int is_mod, is_residue, in_labels
 * 
 *     if labels is None:             # <<<<<<<<<<<<<<
 *         allow_unknown_modifications = True
 *     _labels = interpolate_labels(labels)
*/
  __pyx_t_1 = (__pyx_v_labels == Py_None);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":285
 * 
 *     if labels is None:
 *         allow_unknown_modifications = True             # <<<<<<<<<<<<<<
 *     _labels = interpolate_labels(labels)
 * 
*/
    __pyx_v_allow_unknown_modifications = 1;

    /* "pyteomics/cparser.pyx":284
 *         int is_mod, is_residue, in_labels
 * 
 *     if labels is None:             # <<<<<<<<<<<<<<
 *         allow_unknown_modifications = True
 *     _labels = interpolate_labels(labels)
*/
  }

  /* "pyteomics/cparser.pyx":286
 *     if labels is None:
 *         allow_unknown_modifications = True
 *     _labels = interpolate_labels(labels)             # <<<<<<<<<<<<<<
 * 
 *     match = PyObject_CallMethodObjArgs(_modX_sequence, "match", <PyObject*>sequence, NULL)
*/
  __pyx_t_2 = __pyx_f_9pyteomics_7cparser_interpolate_labels(__pyx_v_labels); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 286, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__labels = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":288
 *     _labels = interpolate_labels(labels)
 * 
 *     match = PyObject_CallMethodObjArgs(_modX_sequence, "match", <PyObject*>sequence, NULL)             # <<<<<<<<<<<<<<
 *     if match is None:
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)
*/
  __pyx_t_2 = __pyx_v_9pyteomics_7cparser__modX_sequence;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = PyObject_CallMethodObjArgs(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_match, ((PyObject *)__pyx_v_sequence), NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_match = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyteomics/cparser.pyx":289
 * 
 *     match = PyObject_CallMethodObjArgs(_modX_sequence, "match", <PyObject*>sequence, NULL)
 *     if match is None:             # <<<<<<<<<<<<<<
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
*/
  __pyx_t_1 = (__pyx_v_match == Py_None);
  if (unlikely(__pyx_t_1)) {

    /* "pyteomics/cparser.pyx":290
 *     match = PyObject_CallMethodObjArgs(_modX_sequence, "match", <PyObject*>sequence, NULL)
 *     if match is None:
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)             # <<<<<<<<<<<<<<
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *     n = <str>PyTuple_GET_ITEM(temp, 0)
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_PyteomicsError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyUnicode_ConcatSafe(__pyx_mstate_global->__pyx_kp_u_Not_a_valid_modX_sequence, __pyx_v_sequence); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 290, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 290, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 290, __pyx_L1_error)

    /* "pyteomics/cparser.pyx":289
 * 
 *     match = PyObject_CallMethodObjArgs(_modX_sequence, "match", <PyObject*>sequence, NULL)
 *     if match is None:             # <<<<<<<<<<<<<<
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
*/
  }

  /* "pyteomics/cparser.pyx":291
 *     if match is None:
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)             # <<<<<<<<<<<<<<
 *     n = <str>PyTuple_GET_ITEM(temp, 0)
 *     body = <str>PyTuple_GET_ITEM(temp, 1)
*/
  __pyx_t_3 = PyObject_CallMethodObjArgs(__pyx_v_match, __pyx_mstate_global->__pyx_n_u_groups, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_temp = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":292
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *     n = <str>PyTuple_GET_ITEM(temp, 0)             # <<<<<<<<<<<<<<
 *     body = <str>PyTuple_GET_ITEM(temp, 1)
 *     c = <str>PyTuple_GET_ITEM(temp, 2)
*/
  __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_temp, 0);
  __pyx_t_4 = ((PyObject *)__pyx_t_7);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_n = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":293
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *     n = <str>PyTuple_GET_ITEM(temp, 0)
 *     body = <str>PyTuple_GET_ITEM(temp, 1)             # <<<<<<<<<<<<<<
 *     c = <str>PyTuple_GET_ITEM(temp, 2)
 *     # slabels = set(_labels)
*/
  __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_temp, 1);
  __pyx_t_4 = ((PyObject *)__pyx_t_7);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_body = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":294
 *     n = <str>PyTuple_GET_ITEM(temp, 0)
 *     body = <str>PyTuple_GET_ITEM(temp, 1)
 *     c = <str>PyTuple_GET_ITEM(temp, 2)             # <<<<<<<<<<<<<<
 *     # slabels = set(_labels)
 *     # labels help save the day when only one terminal group is given
*/
  __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_temp, 2);
  __pyx_t_4 = ((PyObject *)__pyx_t_7);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_c = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":297
 *     # slabels = set(_labels)
 *     # labels help save the day when only one terminal group is given
 *     if c is None and n is not None:             # <<<<<<<<<<<<<<
 *         # we can try to resolve the ambiguity
 *         if n != std_nterm and n not in _labels:
*/
  __pyx_t_8 = (__pyx_v_c == ((PyObject*)Py_None));
  if (__pyx_t_8) {
  } else {
    __pyx_t_1 = __pyx_t_8;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_8 = (__pyx_v_n != ((PyObject*)Py_None));
  __pyx_t_1 = __pyx_t_8;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":299
 *     if c is None and n is not None:
 *         # we can try to resolve the ambiguity
 *         if n != std_nterm and n not in _labels:             # <<<<<<<<<<<<<<
 *             # n is the body then
 *             c = '-' + body
*/
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_n, __pyx_v_9pyteomics_7cparser_std_nterm, Py_NE)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 299, __pyx_L1_error)
    if (__pyx_t_8) {
    } else {
      __pyx_t_1 = __pyx_t_8;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_n, __pyx_v__labels, Py_NE)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 299, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_8;
    __pyx_L9_bool_binop_done:;
    if (__pyx_t_1) {

      /* "pyteomics/cparser.pyx":301
 *         if n != std_nterm and n not in _labels:
 *             # n is the body then
 *             c = '-' + body             # <<<<<<<<<<<<<<
 *             body = n[:-1]
 *             n = None
*/
      __pyx_t_4 = __Pyx_PyUnicode_ConcatSafe(__pyx_mstate_global->__pyx_kp_u_, __pyx_v_body); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 301, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF_SET(__pyx_v_c, ((PyObject*)__pyx_t_4));
      __pyx_t_4 = 0;

      /* "pyteomics/cparser.pyx":302
 *             # n is the body then
 *             c = '-' + body
 *             body = n[:-1]             # <<<<<<<<<<<<<<
 *             n = None
 * 
*/
      if (unlikely(__pyx_v_n == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 302, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_PyUnicode_Substring(__pyx_v_n, 0, -1L); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 302, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF_SET(__pyx_v_body, ((PyObject*)__pyx_t_4));
      __pyx_t_4 = 0;

      /* "pyteomics/cparser.pyx":303
 *             c = '-' + body
 *             body = n[:-1]
 *             n = None             # <<<<<<<<<<<<<<
 * 
 *     # Actual parsing
*/
      __Pyx_INCREF(Py_None);
      __Pyx_DECREF_SET(__pyx_v_n, ((PyObject*)Py_None));

      /* "pyteomics/cparser.pyx":299
 *     if c is None and n is not None:
 *         # we can try to resolve the ambiguity
 *         if n != std_nterm and n not in _labels:             # <<<<<<<<<<<<<<
 *             # n is the body then
 *             c = '-' + body
*/
    }

    /* "pyteomics/cparser.pyx":297
 *     # slabels = set(_labels)
 *     # labels help save the day when only one terminal group is given
 *     if c is None and n is not None:             # <<<<<<<<<<<<<<
 *         # we can try to resolve the ambiguity
 *         if n != std_nterm and n not in _labels:
*/
  }

  /* "pyteomics/cparser.pyx":312
 *     # else:
 *     #     parsed_sequence = <list>PyObject_CallMethodObjArgs(_modX_group, "findall", <PyObject*>body, NULL)
 *     parsed_sequence = _tokenize_modx_sequence(body, split)             # <<<<<<<<<<<<<<
 * 
 *     nterm, cterm = (n or std_nterm), (c or std_cterm)
*/
  __pyx_t_9.__pyx_n = 1;
  __pyx_t_9.split = __pyx_v_split;
  __pyx_t_4 = __pyx_f_9pyteomics_7cparser__tokenize_modx_sequence(__pyx_v_body, &__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":314
 *     parsed_sequence = _tokenize_modx_sequence(body, split)
 * 
 *     nterm, cterm = (n or std_nterm), (c or std_cterm)             # <<<<<<<<<<<<<<
 * 
 *     if not allow_unknown_modifications:
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_n); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 314, __pyx_L1_error)
  if (!__pyx_t_1) {
  } else {
    __Pyx_INCREF(__pyx_v_n);
    __pyx_t_4 = __pyx_v_n;
    goto __pyx_L11_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_nterm);
  __pyx_t_4 = __pyx_v_9pyteomics_7cparser_std_nterm;
  __pyx_L11_bool_binop_done:;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_c); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 314, __pyx_L1_error)
  if (!__pyx_t_1) {
  } else {
    __Pyx_INCREF(__pyx_v_c);
    __pyx_t_3 = __pyx_v_c;
    goto __pyx_L13_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_cterm);
  __pyx_t_3 = __pyx_v_9pyteomics_7cparser_std_cterm;
  __pyx_L13_bool_binop_done:;
  __pyx_v_nterm = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_v_cterm = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyteomics/cparser.pyx":316
 *     nterm, cterm = (n or std_nterm), (c or std_cterm)
 * 
 *     if not allow_unknown_modifications:             # <<<<<<<<<<<<<<
 *         if nterm is not None and nterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
*/
  __pyx_t_1 = (!__pyx_v_allow_unknown_modifications);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":317
 * 
 *     if not allow_unknown_modifications:
 *         if nterm is not None and nterm not in _labels:             # <<<<<<<<<<<<<<
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 *         if cterm is not None and cterm not in _labels:
*/
    __pyx_t_8 = (__pyx_v_nterm != ((PyObject*)Py_None));
    if (__pyx_t_8) {
    } else {
      __pyx_t_1 = __pyx_t_8;
      goto __pyx_L17_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_nterm, __pyx_v__labels, Py_NE)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 317, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_8;
    __pyx_L17_bool_binop_done:;
    if (unlikely(__pyx_t_1)) {

      /* "pyteomics/cparser.pyx":318
 *     if not allow_unknown_modifications:
 *         if nterm is not None and nterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))             # <<<<<<<<<<<<<<
 *         if cterm is not None and cterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))
*/
      __pyx_t_4 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_PyteomicsError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 318, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_10 = __pyx_mstate_global->__pyx_kp_u_Unknown_label;
      __Pyx_INCREF(__pyx_t_10);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_nterm};
        __pyx_t_2 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
        assert(__pyx_t_4);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_2};
        __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 318, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":317
 * 
 *     if not allow_unknown_modifications:
 *         if nterm is not None and nterm not in _labels:             # <<<<<<<<<<<<<<
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 *         if cterm is not None and cterm not in _labels:
*/
    }

    /* "pyteomics/cparser.pyx":319
 *         if nterm is not None and nterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 *         if cterm is not None and cterm not in _labels:             # <<<<<<<<<<<<<<
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))
 * 
*/
    __pyx_t_8 = (__pyx_v_cterm != ((PyObject*)Py_None));
    if (__pyx_t_8) {
    } else {
      __pyx_t_1 = __pyx_t_8;
      goto __pyx_L20_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_cterm, __pyx_v__labels, Py_NE)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 319, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_8;
    __pyx_L20_bool_binop_done:;
    if (unlikely(__pyx_t_1)) {

      /* "pyteomics/cparser.pyx":320
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 *         if cterm is not None and cterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))             # <<<<<<<<<<<<<<
 * 
 *     if split:
*/
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_PyteomicsError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 320, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_10 = __pyx_mstate_global->__pyx_kp_u_Unknown_label;
      __Pyx_INCREF(__pyx_t_10);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_cterm};
        __pyx_t_4 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 320, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
      }
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 320, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 320, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":319
 *         if nterm is not None and nterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 *         if cterm is not None and cterm not in _labels:             # <<<<<<<<<<<<<<
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))
 * 
*/
    }

    /* "pyteomics/cparser.pyx":316
 *     nterm, cterm = (n or std_nterm), (c or std_cterm)
 * 
 *     if not allow_unknown_modifications:             # <<<<<<<<<<<<<<
 *         if nterm is not None and nterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
*/
  }

  /* "pyteomics/cparser.pyx":322
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))
 * 
 *     if split:             # <<<<<<<<<<<<<<
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):
*/
  if (__pyx_v_split) {

    /* "pyteomics/cparser.pyx":323
 * 
 *     if split:
 *         i = 0             # <<<<<<<<<<<<<<
 *         for i in range(PyList_Size(parsed_sequence)):
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)
*/
    __pyx_v_i = 0;

    /* "pyteomics/cparser.pyx":324
 *     if split:
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):             # <<<<<<<<<<<<<<
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *             if len(tgroup) == 2:
*/
    __pyx_t_11 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 324, __pyx_L1_error)
    __pyx_t_12 = __pyx_t_11;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_i = __pyx_t_13;

      /* "pyteomics/cparser.pyx":325
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)             # <<<<<<<<<<<<<<
 *             if len(tgroup) == 2:
 *                 mod = <str>PyTuple_GET_ITEM(tgroup, 0)
*/
      __pyx_t_7 = PyList_GET_ITEM(__pyx_v_parsed_sequence, __pyx_v_i);
      __pyx_t_3 = ((PyObject *)__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_tgroup, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":326
 *         for i in range(PyList_Size(parsed_sequence)):
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *             if len(tgroup) == 2:             # <<<<<<<<<<<<<<
 *                 mod = <str>PyTuple_GET_ITEM(tgroup, 0)
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 1)
*/
      if (unlikely(__pyx_v_tgroup == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 326, __pyx_L1_error)
      }
      __pyx_t_14 = __Pyx_PyTuple_GET_SIZE(__pyx_v_tgroup); if (unlikely(__pyx_t_14 == ((Py_ssize_t)-1))) __PYX_ERR(0, 326, __pyx_L1_error)
      __pyx_t_1 = (__pyx_t_14 == 2);
      if (__pyx_t_1) {

        /* "pyteomics/cparser.pyx":327
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *             if len(tgroup) == 2:
 *                 mod = <str>PyTuple_GET_ITEM(tgroup, 0)             # <<<<<<<<<<<<<<
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 1)
 *             else:
*/
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_tgroup, 0);
        __pyx_t_3 = ((PyObject *)__pyx_t_7);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_XDECREF_SET(__pyx_v_mod, ((PyObject*)__pyx_t_3));
        __pyx_t_3 = 0;

        /* "pyteomics/cparser.pyx":328
 *             if len(tgroup) == 2:
 *                 mod = <str>PyTuple_GET_ITEM(tgroup, 0)
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 1)             # <<<<<<<<<<<<<<
 *             else:
 *                 mod = ''
*/
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_tgroup, 1);
        __pyx_t_3 = ((PyObject *)__pyx_t_7);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_XDECREF_SET(__pyx_v_X, ((PyObject*)__pyx_t_3));
        __pyx_t_3 = 0;

        /* "pyteomics/cparser.pyx":326
 *         for i in range(PyList_Size(parsed_sequence)):
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *             if len(tgroup) == 2:             # <<<<<<<<<<<<<<
 *                 mod = <str>PyTuple_GET_ITEM(tgroup, 0)
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 1)
*/
        goto __pyx_L25;
      }

      /* "pyteomics/cparser.pyx":330
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 1)
 *             else:
 *                 mod = ''             # <<<<<<<<<<<<<<
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\
*/
      /*else*/ {
        __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u__3);
        __Pyx_XDECREF_SET(__pyx_v_mod, __pyx_mstate_global->__pyx_kp_u__3);

        /* "pyteomics/cparser.pyx":331
 *             else:
 *                 mod = ''
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)             # <<<<<<<<<<<<<<
 *             if ((not mod) and X not in _labels) or\
 *                 not ((mod+X in _labels) or\
*/
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_tgroup, 0);
        __pyx_t_3 = ((PyObject *)__pyx_t_7);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_XDECREF_SET(__pyx_v_X, ((PyObject*)__pyx_t_3));
        __pyx_t_3 = 0;
      }
      __pyx_L25:;

      /* "pyteomics/cparser.pyx":332
 *                 mod = ''
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
*/
      __pyx_t_8 = (__pyx_v_mod != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_mod) != 0);
      if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_8 < 0))) __PYX_ERR(0, 332, __pyx_L1_error)
      __pyx_t_15 = (!__pyx_t_8);
      if (!__pyx_t_15) {
        goto __pyx_L28_next_or;
      } else {
      }
      __pyx_t_15 = (__Pyx_PySequence_ContainsTF(__pyx_v_X, __pyx_v__labels, Py_NE)); if (unlikely((__pyx_t_15 < 0))) __PYX_ERR(0, 332, __pyx_L1_error)
      if (!__pyx_t_15) {
      } else {
        __pyx_t_1 = __pyx_t_15;
        goto __pyx_L27_bool_binop_done;
      }
      __pyx_L28_next_or:;

      /* "pyteomics/cparser.pyx":333
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\
 *                 not ((mod+X in _labels) or\             # <<<<<<<<<<<<<<
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(tgroup))
*/
      __pyx_t_3 = __Pyx_PyUnicode_ConcatSafe(__pyx_v_mod, __pyx_v_X); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 333, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_t_3, __pyx_v__labels, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 333, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!__pyx_t_8) {
      } else {
        __pyx_t_15 = __pyx_t_8;
        goto __pyx_L30_bool_binop_done;
      }

      /* "pyteomics/cparser.pyx":334
 *             if ((not mod) and X not in _labels) or\
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):             # <<<<<<<<<<<<<<
 *                 raise PyteomicsError('Unknown label: {}'.format(tgroup))
 * 
*/
      __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_X, __pyx_v__labels, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 334, __pyx_L1_error)
      if (__pyx_t_8) {
      } else {
        __pyx_t_15 = __pyx_t_8;
        goto __pyx_L30_bool_binop_done;
      }
      __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_mod, __pyx_v__labels, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 334, __pyx_L1_error)
      if (!__pyx_t_8) {
      } else {
        __pyx_t_15 = __pyx_t_8;
        goto __pyx_L30_bool_binop_done;
      }
      __pyx_t_15 = __pyx_v_allow_unknown_modifications;
      __pyx_L30_bool_binop_done:;

      /* "pyteomics/cparser.pyx":333
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\
 *                 not ((mod+X in _labels) or\             # <<<<<<<<<<<<<<
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(tgroup))
*/
      __pyx_t_8 = (!__pyx_t_15);
      __pyx_t_1 = __pyx_t_8;
      __pyx_L27_bool_binop_done:;

      /* "pyteomics/cparser.pyx":332
 *                 mod = ''
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
*/
      if (unlikely(__pyx_t_1)) {

        /* "pyteomics/cparser.pyx":335
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(tgroup))             # <<<<<<<<<<<<<<
 * 
 *     else:
*/
        __pyx_t_2 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_PyteomicsError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 335, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_10 = __pyx_mstate_global->__pyx_kp_u_Unknown_label;
        __Pyx_INCREF(__pyx_t_10);
        __pyx_t_6 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_tgroup};
          __pyx_t_5 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 335, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
        }
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_2);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_5};
          __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 335, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 335, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":332
 *                 mod = ''
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
*/
      }
    }

    /* "pyteomics/cparser.pyx":322
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))
 * 
 *     if split:             # <<<<<<<<<<<<<<
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):
*/
    goto __pyx_L22;
  }

  /* "pyteomics/cparser.pyx":338
 * 
 *     else:
 *         i = 0             # <<<<<<<<<<<<<<
 *         for i in range(PyList_Size(parsed_sequence)):
 *             sgroup = <str>PyList_GET_ITEM(parsed_sequence, i)
*/
  /*else*/ {
    __pyx_v_i = 0;

    /* "pyteomics/cparser.pyx":339
 *     else:
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):             # <<<<<<<<<<<<<<
 *             sgroup = <str>PyList_GET_ITEM(parsed_sequence, i)
 * 
*/
    __pyx_t_11 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 339, __pyx_L1_error)
    __pyx_t_12 = __pyx_t_11;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_i = __pyx_t_13;

      /* "pyteomics/cparser.pyx":340
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):
 *             sgroup = <str>PyList_GET_ITEM(parsed_sequence, i)             # <<<<<<<<<<<<<<
 * 
 *             match = PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>sgroup, NULL)
*/
      __pyx_t_7 = PyList_GET_ITEM(__pyx_v_parsed_sequence, __pyx_v_i);
      __pyx_t_3 = ((PyObject *)__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_sgroup, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":342
 *             sgroup = <str>PyList_GET_ITEM(parsed_sequence, i)
 * 
 *             match = PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>sgroup, NULL)             # <<<<<<<<<<<<<<
 *             temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *             mod = <str>PyTuple_GET_ITEM(temp, 0)
*/
      __pyx_t_3 = __pyx_v_9pyteomics_7cparser__modX_split;
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_4 = PyObject_CallMethodObjArgs(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_match, ((PyObject *)__pyx_v_sgroup), NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 342, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF_SET(__pyx_v_match, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyteomics/cparser.pyx":343
 * 
 *             match = PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>sgroup, NULL)
 *             temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)             # <<<<<<<<<<<<<<
 *             mod = <str>PyTuple_GET_ITEM(temp, 0)
 *             X = <str>PyTuple_GET_ITEM(temp, 1)
*/
      __pyx_t_4 = PyObject_CallMethodObjArgs(__pyx_v_match, __pyx_mstate_global->__pyx_n_u_groups, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 343, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __pyx_t_4;
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF_SET(__pyx_v_temp, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":344
 *             match = PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>sgroup, NULL)
 *             temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *             mod = <str>PyTuple_GET_ITEM(temp, 0)             # <<<<<<<<<<<<<<
 *             X = <str>PyTuple_GET_ITEM(temp, 1)
 *             is_mod = PyObject_IsTrue(mod)
*/
      __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_temp, 0);
      __pyx_t_3 = ((PyObject *)__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_mod, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":345
 *             temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *             mod = <str>PyTuple_GET_ITEM(temp, 0)
 *             X = <str>PyTuple_GET_ITEM(temp, 1)             # <<<<<<<<<<<<<<
 *             is_mod = PyObject_IsTrue(mod)
 *             in_labels = X not in _labels
*/
      __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_temp, 1);
      __pyx_t_3 = ((PyObject *)__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_X, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":346
 *             mod = <str>PyTuple_GET_ITEM(temp, 0)
 *             X = <str>PyTuple_GET_ITEM(temp, 1)
 *             is_mod = PyObject_IsTrue(mod)             # <<<<<<<<<<<<<<
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\
*/
      __pyx_t_1 = PyObject_IsTrue(__pyx_v_mod); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 346, __pyx_L1_error)
      __pyx_v_is_mod = __pyx_t_1;

      /* "pyteomics/cparser.pyx":347
 *             X = <str>PyTuple_GET_ITEM(temp, 1)
 *             is_mod = PyObject_IsTrue(mod)
 *             in_labels = X not in _labels             # <<<<<<<<<<<<<<
 *             if ((not is_mod) and in_labels) or\
 *                 not ((mod+X in _labels) or\
*/
      __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_v_X, __pyx_v__labels, Py_NE)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 347, __pyx_L1_error)
      __pyx_v_in_labels = __pyx_t_1;

      /* "pyteomics/cparser.pyx":348
 *             is_mod = PyObject_IsTrue(mod)
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
*/
      __pyx_t_8 = (!(__pyx_v_is_mod != 0));
      if (!__pyx_t_8) {
        goto __pyx_L38_next_or;
      } else {
      }
      __pyx_t_8 = (__pyx_v_in_labels != 0);
      if (!__pyx_t_8) {
      } else {
        __pyx_t_1 = __pyx_t_8;
        goto __pyx_L37_bool_binop_done;
      }
      __pyx_L38_next_or:;

      /* "pyteomics/cparser.pyx":349
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\
 *                 not ((mod+X in _labels) or\             # <<<<<<<<<<<<<<
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(sgroup))
*/
      __pyx_t_3 = __Pyx_PyUnicode_ConcatSafe(__pyx_v_mod, __pyx_v_X); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 349, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_15 = (__Pyx_PySequence_ContainsTF(__pyx_t_3, __pyx_v__labels, Py_EQ)); if (unlikely((__pyx_t_15 < 0))) __PYX_ERR(0, 349, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!__pyx_t_15) {
      } else {
        __pyx_t_8 = __pyx_t_15;
        goto __pyx_L40_bool_binop_done;
      }

      /* "pyteomics/cparser.pyx":350
 *             if ((not is_mod) and in_labels) or\
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):             # <<<<<<<<<<<<<<
 *                 raise PyteomicsError('Unknown label: {}'.format(sgroup))
 * 
*/
      __pyx_t_15 = (__Pyx_PySequence_ContainsTF(__pyx_v_X, __pyx_v__labels, Py_EQ)); if (unlikely((__pyx_t_15 < 0))) __PYX_ERR(0, 350, __pyx_L1_error)
      if (__pyx_t_15) {
      } else {
        __pyx_t_8 = __pyx_t_15;
        goto __pyx_L40_bool_binop_done;
      }
      __pyx_t_15 = (__Pyx_PySequence_ContainsTF(__pyx_v_mod, __pyx_v__labels, Py_EQ)); if (unlikely((__pyx_t_15 < 0))) __PYX_ERR(0, 350, __pyx_L1_error)
      if (!__pyx_t_15) {
      } else {
        __pyx_t_8 = __pyx_t_15;
        goto __pyx_L40_bool_binop_done;
      }
      __pyx_t_8 = __pyx_v_allow_unknown_modifications;
      __pyx_L40_bool_binop_done:;

      /* "pyteomics/cparser.pyx":349
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\
 *                 not ((mod+X in _labels) or\             # <<<<<<<<<<<<<<
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(sgroup))
*/
      __pyx_t_15 = (!__pyx_t_8);
      __pyx_t_1 = __pyx_t_15;
      __pyx_L37_bool_binop_done:;

      /* "pyteomics/cparser.pyx":348
 *             is_mod = PyObject_IsTrue(mod)
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
*/
      if (unlikely(__pyx_t_1)) {

        /* "pyteomics/cparser.pyx":351
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(sgroup))             # <<<<<<<<<<<<<<
 * 
 *     # Append terminal labels
*/
        __pyx_t_4 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_PyteomicsError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 351, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_10 = __pyx_mstate_global->__pyx_kp_u_Unknown_label;
        __Pyx_INCREF(__pyx_t_10);
        __pyx_t_6 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_sgroup};
          __pyx_t_2 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
          assert(__pyx_t_4);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_2};
          __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 351, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 351, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":348
 *             is_mod = PyObject_IsTrue(mod)
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
*/
      }
    }
  }
  __pyx_L22:;

  /* "pyteomics/cparser.pyx":354
 * 
 *     # Append terminal labels
 *     if show_unmodified_termini or nterm != std_nterm:             # <<<<<<<<<<<<<<
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
*/
  if (!__pyx_v_show_unmodified_termini) {
  } else {
    __pyx_t_1 = __pyx_v_show_unmodified_termini;
    goto __pyx_L45_bool_binop_done;
  }
  __pyx_t_15 = (__Pyx_PyUnicode_Equals(__pyx_v_nterm, __pyx_v_9pyteomics_7cparser_std_nterm, Py_NE)); if (unlikely((__pyx_t_15 < 0))) __PYX_ERR(0, 354, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_15;
  __pyx_L45_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":355
 *     # Append terminal labels
 *     if show_unmodified_termini or nterm != std_nterm:
 *         if split:             # <<<<<<<<<<<<<<
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
 *             temp = (<str>nterm, ) + temp
*/
    if (__pyx_v_split) {

      /* "pyteomics/cparser.pyx":356
 *     if show_unmodified_termini or nterm != std_nterm:
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)             # <<<<<<<<<<<<<<
 *             temp = (<str>nterm, ) + temp
 *             Py_INCREF(temp)
*/
      __pyx_t_7 = PyList_GET_ITEM(__pyx_v_parsed_sequence, 0);
      __pyx_t_3 = ((PyObject *)__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_DECREF_SET(__pyx_v_temp, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":357
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
 *             temp = (<str>nterm, ) + temp             # <<<<<<<<<<<<<<
 *             Py_INCREF(temp)
 *             PyList_SET_ITEM(<list>parsed_sequence, 0, tuple(temp))
*/
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(((PyObject*)__pyx_v_nterm));
      __Pyx_GIVEREF(((PyObject*)__pyx_v_nterm));
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_nterm) != (0)) __PYX_ERR(0, 357, __pyx_L1_error);
      __pyx_t_5 = PyNumber_Add(__pyx_t_3, __pyx_v_temp); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF_SET(__pyx_v_temp, ((PyObject*)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "pyteomics/cparser.pyx":358
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
 *             temp = (<str>nterm, ) + temp
 *             Py_INCREF(temp)             # <<<<<<<<<<<<<<
 *             PyList_SET_ITEM(<list>parsed_sequence, 0, tuple(temp))
 *         else:
*/
      Py_INCREF(__pyx_v_temp);

      /* "pyteomics/cparser.pyx":359
 *             temp = (<str>nterm, ) + temp
 *             Py_INCREF(temp)
 *             PyList_SET_ITEM(<list>parsed_sequence, 0, tuple(temp))             # <<<<<<<<<<<<<<
 *         else:
 *             PyList_Insert(parsed_sequence, 0, nterm)
*/
      PyList_SET_ITEM(__pyx_v_parsed_sequence, 0, __pyx_v_temp);

      /* "pyteomics/cparser.pyx":355
 *     # Append terminal labels
 *     if show_unmodified_termini or nterm != std_nterm:
 *         if split:             # <<<<<<<<<<<<<<
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
 *             temp = (<str>nterm, ) + temp
*/
      goto __pyx_L47;
    }

    /* "pyteomics/cparser.pyx":361
 *             PyList_SET_ITEM(<list>parsed_sequence, 0, tuple(temp))
 *         else:
 *             PyList_Insert(parsed_sequence, 0, nterm)             # <<<<<<<<<<<<<<
 *     if show_unmodified_termini or cterm != std_cterm:
 *         if split:
*/
    /*else*/ {
      __pyx_t_16 = PyList_Insert(__pyx_v_parsed_sequence, 0, __pyx_v_nterm); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 361, __pyx_L1_error)
    }
    __pyx_L47:;

    /* "pyteomics/cparser.pyx":354
 * 
 *     # Append terminal labels
 *     if show_unmodified_termini or nterm != std_nterm:             # <<<<<<<<<<<<<<
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
*/
  }

  /* "pyteomics/cparser.pyx":362
 *         else:
 *             PyList_Insert(parsed_sequence, 0, nterm)
 *     if show_unmodified_termini or cterm != std_cterm:             # <<<<<<<<<<<<<<
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
*/
  if (!__pyx_v_show_unmodified_termini) {
  } else {
    __pyx_t_1 = __pyx_v_show_unmodified_termini;
    goto __pyx_L49_bool_binop_done;
  }
  __pyx_t_15 = (__Pyx_PyUnicode_Equals(__pyx_v_cterm, __pyx_v_9pyteomics_7cparser_std_cterm, Py_NE)); if (unlikely((__pyx_t_15 < 0))) __PYX_ERR(0, 362, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_15;
  __pyx_L49_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":363
 *             PyList_Insert(parsed_sequence, 0, nterm)
 *     if show_unmodified_termini or cterm != std_cterm:
 *         if split:             # <<<<<<<<<<<<<<
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
 *             temp = temp + (cterm,)
*/
    if (__pyx_v_split) {

      /* "pyteomics/cparser.pyx":364
 *     if show_unmodified_termini or cterm != std_cterm:
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)             # <<<<<<<<<<<<<<
 *             temp = temp + (cterm,)
 *             Py_INCREF(temp)
*/
      if (unlikely(__pyx_v_parsed_sequence == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 364, __pyx_L1_error)
      }
      __pyx_t_11 = __Pyx_PyList_GET_SIZE(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 364, __pyx_L1_error)
      __pyx_t_7 = PyList_GET_ITEM(__pyx_v_parsed_sequence, (__pyx_t_11 - 1));
      __pyx_t_5 = ((PyObject *)__pyx_t_7);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_DECREF_SET(__pyx_v_temp, ((PyObject*)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "pyteomics/cparser.pyx":365
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
 *             temp = temp + (cterm,)             # <<<<<<<<<<<<<<
 *             Py_INCREF(temp)
 *             PyList_SET_ITEM(parsed_sequence, len(parsed_sequence) - 1, temp)
*/
      __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 365, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_v_cterm);
      __Pyx_GIVEREF(__pyx_v_cterm);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_cterm) != (0)) __PYX_ERR(0, 365, __pyx_L1_error);
      __pyx_t_3 = PyNumber_Add(__pyx_v_temp, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF_SET(__pyx_v_temp, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":366
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
 *             temp = temp + (cterm,)
 *             Py_INCREF(temp)             # <<<<<<<<<<<<<<
 *             PyList_SET_ITEM(parsed_sequence, len(parsed_sequence) - 1, temp)
 *         else:
*/
      Py_INCREF(__pyx_v_temp);

      /* "pyteomics/cparser.pyx":367
 *             temp = temp + (cterm,)
 *             Py_INCREF(temp)
 *             PyList_SET_ITEM(parsed_sequence, len(parsed_sequence) - 1, temp)             # <<<<<<<<<<<<<<
 *         else:
 *             PyList_Append(parsed_sequence, cterm)
*/
      if (unlikely(__pyx_v_parsed_sequence == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 367, __pyx_L1_error)
      }
      __pyx_t_11 = __Pyx_PyList_GET_SIZE(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 367, __pyx_L1_error)
      PyList_SET_ITEM(__pyx_v_parsed_sequence, (__pyx_t_11 - 1), __pyx_v_temp);

      /* "pyteomics/cparser.pyx":363
 *             PyList_Insert(parsed_sequence, 0, nterm)
 *     if show_unmodified_termini or cterm != std_cterm:
 *         if split:             # <<<<<<<<<<<<<<
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
 *             temp = temp + (cterm,)
*/
      goto __pyx_L51;
    }

    /* "pyteomics/cparser.pyx":369
 *             PyList_SET_ITEM(parsed_sequence, len(parsed_sequence) - 1, temp)
 *         else:
 *             PyList_Append(parsed_sequence, cterm)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    /*else*/ {
      __pyx_t_16 = PyList_Append(__pyx_v_parsed_sequence, __pyx_v_cterm); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 369, __pyx_L1_error)
    }
    __pyx_L51:;

    /* "pyteomics/cparser.pyx":362
 *         else:
 *             PyList_Insert(parsed_sequence, 0, nterm)
 *     if show_unmodified_termini or cterm != std_cterm:             # <<<<<<<<<<<<<<
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
*/
  }

  /* "pyteomics/cparser.pyx":372
 * 
 * 
 *     return parsed_sequence             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_parsed_sequence);
  __pyx_r = __pyx_v_parsed_sequence;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":211
 * 
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,             # <<<<<<<<<<<<<<
 *                  bint allow_unknown_modifications=0, object labels=None):
 *     """Parse a sequence string written in modX notation into a list of
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pyteomics.cparser.parse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_match);
  __Pyx_XDECREF(__pyx_v_temp);
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_body);
  __Pyx_XDECREF(__pyx_v_c);
  __Pyx_XDECREF(__pyx_v_nterm);
  __Pyx_XDECREF(__pyx_v_cterm);
  __Pyx_XDECREF(__pyx_v_mod);
  __Pyx_XDECREF(__pyx_v_X);
  __Pyx_XDECREF(__pyx_v_parsed_sequence);
  __Pyx_XDECREF(__pyx_v_tgroup);
  __Pyx_XDECREF(__pyx_v_sgroup);
  __Pyx_XDECREF(__pyx_v__labels);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_5parse(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_4parse, "parse(str sequence, bool show_unmodified_termini=0, bool split=0, bool allow_unknown_modifications=0, labels=None) -> list\nParse a sequence string written in modX notation into a list of\nlabels or (if `split` argument is :py:const:`True`) into a list of\ntuples representing amino acid residues and their modifications.\n\nParameters\n----------\nsequence : str\n    The sequence of a polypeptide.\nshow_unmodified_termini : bool, optional\n    If :py:const:`True` then the unmodified N- and C-termini are explicitly\n    shown in the returned list. Default value is :py:const:`False`.\nsplit : bool, optional\n    If :py:const:`True` then the result will be a list of tuples with 1 to 4\n    elements: terminal modification, modification, residue. Default value is\n    :py:const:`False`.\nallow_unknown_modifications : bool, optional\n    If :py:const:`True` then do not raise an exception when an unknown\n    modification of a known amino acid residue is found in the sequence.\n    This also includes terminal groups.\n    Default value is :py:const:`False`.\n\n    .. note::\n        Since version 2.5, this parameter has effect only if `labels`\n        are provided.\nlabels : container, optional\n    A container of allowed labels for amino acids,\n    modifications and terminal modifications.\n    If not provided, no checks will be done.\n    Separate labels for modifications (such as 'p' or 'ox')\n    can be supplied, which means they are applicable to all residues.\n\n    .. warning::\n        If `show_unmodified_termini` is set to :py:const:`True`, standard\n        terminal groups need to be present in `labels`.\n\n    .. warning::\n        Avoid using sequences with only one terminal group, as they are\n        ambiguous. If you provide one, `labels` (or :py:const:`std_labels`)\n        will be used to resolve the ambiguity.\n\nReturns\n-------\nout : list\n    List of tuples with labels of modifications and amino acid residues.\n\nExamples\n--------\n>>> parse('PEPTID""E', split=True)\n[('P',), ('E',), ('P',), ('T',), ('I',), ('D',), ('E',)]\n>>> parse('H-PEPTIDE')\n['P', 'E', 'P', 'T', 'I', 'D', 'E']\n>>> parse('PEPTIDE', show_unmodified_termini=True)\n['H-', 'P', 'E', 'P', 'T', 'I', 'D', 'E', '-OH']\n>>> parse('TEpSToxM', labels=std_labels + ['pS', 'oxM'])\n['T', 'E', 'pS', 'T', 'oxM']\n>>> parse('zPEPzTIDzE', True, True, labels=std_labels+['z'])\n[('H-', 'z', 'P'), ('E',), ('P',), ('z', 'T'), ('I',), ('D',), ('z', 'E', '-OH')]");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_5parse = {"parse", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_5parse, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_4parse};
static PyObject *__pyx_pw_9pyteomics_7cparser_5parse(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  int __pyx_v_show_unmodified_termini;
  int __pyx_v_split;
  int __pyx_v_allow_unknown_modifications;
  PyObject *__pyx_v_labels = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("parse (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,&__pyx_mstate_global->__pyx_n_u_show_unmodified_termini,&__pyx_mstate_global->__pyx_n_u_split,&__pyx_mstate_global->__pyx_n_u_allow_unknown_modifications,&__pyx_mstate_global->__pyx_n_u_labels,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 211, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "parse", 0) < 0) __PYX_ERR(0, 211, __pyx_L3_error)

      /* "pyteomics/cparser.pyx":212
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,
 *                  bint allow_unknown_modifications=0, object labels=None):             # <<<<<<<<<<<<<<
 *     """Parse a sequence string written in modX notation into a list of
 *     labels or (if `split` argument is :py:const:`True`) into a list of
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("parse", 0, 1, 5, i); __PYX_ERR(0, 211, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 211, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_sequence = ((PyObject*)values[0]);
    if (values[1]) {
      __pyx_v_show_unmodified_termini = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_show_unmodified_termini == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 211, __pyx_L3_error)
    } else {
      __pyx_v_show_unmodified_termini = ((int)0);
    }
    if (values[2]) {
      __pyx_v_split = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_split == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 211, __pyx_L3_error)
    } else {
      __pyx_v_split = ((int)0);
    }
    if (values[3]) {
      __pyx_v_allow_unknown_modifications = __Pyx_PyObject_IsTrue(values[3]); if (unlikely((__pyx_v_allow_unknown_modifications == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 212, __pyx_L3_error)
    } else {
      __pyx_v_allow_unknown_modifications = ((int)0);
    }
    __pyx_v_labels = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("parse", 0, 1, 5, __pyx_nargs); __PYX_ERR(0, 211, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.parse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sequence), (&PyUnicode_Type), 1, "sequence", 1))) __PYX_ERR(0, 211, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_4parse(__pyx_self, __pyx_v_sequence, __pyx_v_show_unmodified_termini, __pyx_v_split, __pyx_v_allow_unknown_modifications, __pyx_v_labels);

  /* "pyteomics/cparser.pyx":211
 * 
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,             # <<<<<<<<<<<<<<
 *                  bint allow_unknown_modifications=0, object labels=None):
 *     """Parse a sequence string written in modX notation into a list of
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_4parse(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_split, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_9pyteomics_7cparser_parse __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("parse", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 4;
  __pyx_t_2.show_unmodified_termini = __pyx_v_show_unmodified_termini;
  __pyx_t_2.split = __pyx_v_split;
  __pyx_t_2.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
  __pyx_t_2.labels = __pyx_v_labels;
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_parse(__pyx_v_sequence, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.parse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":375
 * 
 * 
 * cdef list _tokenize_modx_sequence(str sequence, bint split=True):             # <<<<<<<<<<<<<<
 *     cdef:
 *         char* csequence
*/

static PyObject *__pyx_f_9pyteomics_7cparser__tokenize_modx_sequence(PyObject *__pyx_v_sequence, struct __pyx_opt_args_9pyteomics_7cparser__tokenize_modx_sequence *__pyx_optional_args) {
  int __pyx_v_split = ((int)1);
  char *__pyx_v_csequence;
  char __pyx_v_c;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_n;
  Py_ssize_t __pyx_v_parsed_n;
  Py_ssize_t __pyx_v_start;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_tokens = 0;
  PyObject *__pyx_v_nterm = 0;
  PyObject *__pyx_v_cterm = 0;
  PyObject *__pyx_v_mod = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_tokenize_modx_sequence", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_split = __pyx_optional_args->split;
    }
  }

  /* "pyteomics/cparser.pyx":383
 *         str nterm, cterm
 * 
 *     start = 0             # <<<<<<<<<<<<<<
 *     end = 0
 *     tokens = []
*/
  __pyx_v_start = 0;

  /* "pyteomics/cparser.pyx":384
 * 
 *     start = 0
 *     end = 0             # <<<<<<<<<<<<<<
 *     tokens = []
 *     i = 0
*/
  __pyx_v_end = 0;

  /* "pyteomics/cparser.pyx":385
 *     start = 0
 *     end = 0
 *     tokens = []             # <<<<<<<<<<<<<<
 *     i = 0
 *     parsed_n = 0
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_tokens = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":386
 *     end = 0
 *     tokens = []
 *     i = 0             # <<<<<<<<<<<<<<
 *     parsed_n = 0
 *     csequence = PyStr_AsUTF8AndSize(sequence, &n)
*/
  __pyx_v_i = 0;

  /* "pyteomics/cparser.pyx":387
 *     tokens = []
 *     i = 0
 *     parsed_n = 0             # <<<<<<<<<<<<<<
 *     csequence = PyStr_AsUTF8AndSize(sequence, &n)
 *     nterm = None
*/
  __pyx_v_parsed_n = 0;

  /* "pyteomics/cparser.pyx":388
 *     i = 0
 *     parsed_n = 0
 *     csequence = PyStr_AsUTF8AndSize(sequence, &n)             # <<<<<<<<<<<<<<
 *     nterm = None
 *     cterm = None
*/
  __pyx_v_csequence = PyStr_AsUTF8AndSize(__pyx_v_sequence, (&__pyx_v_n));

  /* "pyteomics/cparser.pyx":389
 *     parsed_n = 0
 *     csequence = PyStr_AsUTF8AndSize(sequence, &n)
 *     nterm = None             # <<<<<<<<<<<<<<
 *     cterm = None
 * 
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_nterm = ((PyObject*)Py_None);

  /* "pyteomics/cparser.pyx":390
 *     csequence = PyStr_AsUTF8AndSize(sequence, &n)
 *     nterm = None
 *     cterm = None             # <<<<<<<<<<<<<<
 * 
 *     while i < n:
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_cterm = ((PyObject*)Py_None);

  /* "pyteomics/cparser.pyx":392
 *     cterm = None
 * 
 *     while i < n:             # <<<<<<<<<<<<<<
 *         c = csequence[i]
 *         # We've reached an amino acid. Process the current modX block
*/
  while (1) {
    __pyx_t_2 = (__pyx_v_i < __pyx_v_n);
    if (!__pyx_t_2) break;

    /* "pyteomics/cparser.pyx":393
 * 
 *     while i < n:
 *         c = csequence[i]             # <<<<<<<<<<<<<<
 *         # We've reached an amino acid. Process the current modX block
 *         if isupper(c):
*/
    __pyx_v_c = (__pyx_v_csequence[__pyx_v_i]);

    /* "pyteomics/cparser.pyx":395
 *         c = csequence[i]
 *         # We've reached an amino acid. Process the current modX block
 *         if isupper(c):             # <<<<<<<<<<<<<<
 *             end = i + 1
 *             if not split:
*/
    __pyx_t_2 = (isupper(__pyx_v_c) != 0);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":396
 *         # We've reached an amino acid. Process the current modX block
 *         if isupper(c):
 *             end = i + 1             # <<<<<<<<<<<<<<
 *             if not split:
 *                 tokens.append(PyStr_FromStringAndSize(&csequence[start], end - start))
*/
      __pyx_v_end = (__pyx_v_i + 1);

      /* "pyteomics/cparser.pyx":397
 *         if isupper(c):
 *             end = i + 1
 *             if not split:             # <<<<<<<<<<<<<<
 *                 tokens.append(PyStr_FromStringAndSize(&csequence[start], end - start))
 *             else:
*/
      __pyx_t_2 = (!__pyx_v_split);
      if (__pyx_t_2) {

        /* "pyteomics/cparser.pyx":398
 *             end = i + 1
 *             if not split:
 *                 tokens.append(PyStr_FromStringAndSize(&csequence[start], end - start))             # <<<<<<<<<<<<<<
 *             else:
 *                 if end - start > 1:
*/
        __pyx_t_1 = PyStr_FromStringAndSize((&(__pyx_v_csequence[__pyx_v_start])), (__pyx_v_end - __pyx_v_start)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 398, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_tokens, __pyx_t_1); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 398, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyteomics/cparser.pyx":397
 *         if isupper(c):
 *             end = i + 1
 *             if not split:             # <<<<<<<<<<<<<<
 *                 tokens.append(PyStr_FromStringAndSize(&csequence[start], end - start))
 *             else:
*/
        goto __pyx_L6;
      }

      /* "pyteomics/cparser.pyx":400
 *                 tokens.append(PyStr_FromStringAndSize(&csequence[start], end - start))
 *             else:
 *                 if end - start > 1:             # <<<<<<<<<<<<<<
 *                     mod = PyStr_FromStringAndSize(&csequence[start], (end - start) - 1)
 *                     x = PyStr_FromStringAndSize(&csequence[end - 1], 1)
*/
      /*else*/ {
        __pyx_t_2 = ((__pyx_v_end - __pyx_v_start) > 1);
        if (__pyx_t_2) {

          /* "pyteomics/cparser.pyx":401
 *             else:
 *                 if end - start > 1:
 *                     mod = PyStr_FromStringAndSize(&csequence[start], (end - start) - 1)             # <<<<<<<<<<<<<<
 *                     x = PyStr_FromStringAndSize(&csequence[end - 1], 1)
 *                     tokens.append((mod, x))
*/
          __pyx_t_1 = PyStr_FromStringAndSize((&(__pyx_v_csequence[__pyx_v_start])), ((__pyx_v_end - __pyx_v_start) - 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 401, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_XDECREF_SET(__pyx_v_mod, ((PyObject*)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "pyteomics/cparser.pyx":402
 *                 if end - start > 1:
 *                     mod = PyStr_FromStringAndSize(&csequence[start], (end - start) - 1)
 *                     x = PyStr_FromStringAndSize(&csequence[end - 1], 1)             # <<<<<<<<<<<<<<
 *                     tokens.append((mod, x))
 *                 else:
*/
          __pyx_t_1 = PyStr_FromStringAndSize((&(__pyx_v_csequence[(__pyx_v_end - 1)])), 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 402, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_XDECREF_SET(__pyx_v_x, ((PyObject*)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "pyteomics/cparser.pyx":403
 *                     mod = PyStr_FromStringAndSize(&csequence[start], (end - start) - 1)
 *                     x = PyStr_FromStringAndSize(&csequence[end - 1], 1)
 *                     tokens.append((mod, x))             # <<<<<<<<<<<<<<
 *                 else:
 *                     x = PyStr_FromStringAndSize(&csequence[end - 1], 1)
*/
          __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 403, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_v_mod);
          __Pyx_GIVEREF(__pyx_v_mod);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_mod) != (0)) __PYX_ERR(0, 403, __pyx_L1_error);
          __Pyx_INCREF(__pyx_v_x);
          __Pyx_GIVEREF(__pyx_v_x);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_x) != (0)) __PYX_ERR(0, 403, __pyx_L1_error);
          __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_tokens, __pyx_t_1); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 403, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "pyteomics/cparser.pyx":400
 *                 tokens.append(PyStr_FromStringAndSize(&csequence[start], end - start))
 *             else:
 *                 if end - start > 1:             # <<<<<<<<<<<<<<
 *                     mod = PyStr_FromStringAndSize(&csequence[start], (end - start) - 1)
 *                     x = PyStr_FromStringAndSize(&csequence[end - 1], 1)
*/
          goto __pyx_L7;
        }

        /* "pyteomics/cparser.pyx":405
 *                     tokens.append((mod, x))
 *                 else:
 *                     x = PyStr_FromStringAndSize(&csequence[end - 1], 1)             # <<<<<<<<<<<<<<
 *                     tokens.append((x, ))
 *             parsed_n += 1
*/
        /*else*/ {
          __pyx_t_1 = PyStr_FromStringAndSize((&(__pyx_v_csequence[(__pyx_v_end - 1)])), 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 405, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_XDECREF_SET(__pyx_v_x, ((PyObject*)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "pyteomics/cparser.pyx":406
 *                 else:
 *                     x = PyStr_FromStringAndSize(&csequence[end - 1], 1)
 *                     tokens.append((x, ))             # <<<<<<<<<<<<<<
 *             parsed_n += 1
 *             start = i + 1
*/
          __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 406, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_v_x);
          __Pyx_GIVEREF(__pyx_v_x);
          if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_x) != (0)) __PYX_ERR(0, 406, __pyx_L1_error);
          __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_tokens, __pyx_t_1); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 406, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        }
        __pyx_L7:;
      }
      __pyx_L6:;

      /* "pyteomics/cparser.pyx":407
 *                     x = PyStr_FromStringAndSize(&csequence[end - 1], 1)
 *                     tokens.append((x, ))
 *             parsed_n += 1             # <<<<<<<<<<<<<<
 *             start = i + 1
 *         # We've reached a terminal modification boundary
*/
      __pyx_v_parsed_n = (__pyx_v_parsed_n + 1);

      /* "pyteomics/cparser.pyx":408
 *                     tokens.append((x, ))
 *             parsed_n += 1
 *             start = i + 1             # <<<<<<<<<<<<<<
 *         # We've reached a terminal modification boundary
 *         elif c == '-':
*/
      __pyx_v_start = (__pyx_v_i + 1);

      /* "pyteomics/cparser.pyx":395
 *         c = csequence[i]
 *         # We've reached an amino acid. Process the current modX block
 *         if isupper(c):             # <<<<<<<<<<<<<<
 *             end = i + 1
 *             if not split:
*/
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":410
 *             start = i + 1
 *         # We've reached a terminal modification boundary
 *         elif c == '-':             # <<<<<<<<<<<<<<
 *             # If we haven't parsed any other tokens
 *             if parsed_n == 0:
*/
    __pyx_t_2 = (__pyx_v_c == '-');
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":412
 *         elif c == '-':
 *             # If we haven't parsed any other tokens
 *             if parsed_n == 0:             # <<<<<<<<<<<<<<
 *                 end = i + 1
 *                 if split:
*/
      __pyx_t_2 = (__pyx_v_parsed_n == 0);
      if (__pyx_t_2) {

        /* "pyteomics/cparser.pyx":413
 *             # If we haven't parsed any other tokens
 *             if parsed_n == 0:
 *                 end = i + 1             # <<<<<<<<<<<<<<
 *                 if split:
 *                     nterm = PyStr_FromStringAndSize(&csequence[start], end - start)
*/
        __pyx_v_end = (__pyx_v_i + 1);

        /* "pyteomics/cparser.pyx":414
 *             if parsed_n == 0:
 *                 end = i + 1
 *                 if split:             # <<<<<<<<<<<<<<
 *                     nterm = PyStr_FromStringAndSize(&csequence[start], end - start)
 *                 else:
*/
        if (__pyx_v_split) {

          /* "pyteomics/cparser.pyx":415
 *                 end = i + 1
 *                 if split:
 *                     nterm = PyStr_FromStringAndSize(&csequence[start], end - start)             # <<<<<<<<<<<<<<
 *                 else:
 *                     tokens.append(nterm)
*/
          __pyx_t_1 = PyStr_FromStringAndSize((&(__pyx_v_csequence[__pyx_v_start])), (__pyx_v_end - __pyx_v_start)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 415, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF_SET(__pyx_v_nterm, ((PyObject*)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "pyteomics/cparser.pyx":414
 *             if parsed_n == 0:
 *                 end = i + 1
 *                 if split:             # <<<<<<<<<<<<<<
 *                     nterm = PyStr_FromStringAndSize(&csequence[start], end - start)
 *                 else:
*/
          goto __pyx_L9;
        }

        /* "pyteomics/cparser.pyx":417
 *                     nterm = PyStr_FromStringAndSize(&csequence[start], end - start)
 *                 else:
 *                     tokens.append(nterm)             # <<<<<<<<<<<<<<
 *                 start = i + 1
 * 
*/
        /*else*/ {
          __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_tokens, __pyx_v_nterm); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 417, __pyx_L1_error)
        }
        __pyx_L9:;

        /* "pyteomics/cparser.pyx":418
 *                 else:
 *                     tokens.append(nterm)
 *                 start = i + 1             # <<<<<<<<<<<<<<
 * 
 *         i += 1
*/
        __pyx_v_start = (__pyx_v_i + 1);

        /* "pyteomics/cparser.pyx":412
 *         elif c == '-':
 *             # If we haven't parsed any other tokens
 *             if parsed_n == 0:             # <<<<<<<<<<<<<<
 *                 end = i + 1
 *                 if split:
*/
      }

      /* "pyteomics/cparser.pyx":410
 *             start = i + 1
 *         # We've reached a terminal modification boundary
 *         elif c == '-':             # <<<<<<<<<<<<<<
 *             # If we haven't parsed any other tokens
 *             if parsed_n == 0:
*/
    }
    __pyx_L5:;

    /* "pyteomics/cparser.pyx":420
 *                 start = i + 1
 * 
 *         i += 1             # <<<<<<<<<<<<<<
 *     end = n
 *     # The end of the last position was different from the end of the string so we must have
*/
    __pyx_v_i = (__pyx_v_i + 1);
  }

  /* "pyteomics/cparser.pyx":421
 * 
 *         i += 1
 *     end = n             # <<<<<<<<<<<<<<
 *     # The end of the last position was different from the end of the string so we must have
 *     # a C-terminal
*/
  __pyx_v_end = __pyx_v_n;

  /* "pyteomics/cparser.pyx":424
 *     # The end of the last position was different from the end of the string so we must have
 *     # a C-terminal
 *     if start != end:             # <<<<<<<<<<<<<<
 *         cterm = PyStr_FromStringAndSize(&csequence[start], end - start)
 *         if not split:
*/
  __pyx_t_2 = (__pyx_v_start != __pyx_v_end);
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":425
 *     # a C-terminal
 *     if start != end:
 *         cterm = PyStr_FromStringAndSize(&csequence[start], end - start)             # <<<<<<<<<<<<<<
 *         if not split:
 *             tokens.append(cterm)
*/
    __pyx_t_1 = PyStr_FromStringAndSize((&(__pyx_v_csequence[__pyx_v_start])), (__pyx_v_end - __pyx_v_start)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_cterm, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":426
 *     if start != end:
 *         cterm = PyStr_FromStringAndSize(&csequence[start], end - start)
 *         if not split:             # <<<<<<<<<<<<<<
 *             tokens.append(cterm)
 *         else:
*/
    __pyx_t_2 = (!__pyx_v_split);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":427
 *         cterm = PyStr_FromStringAndSize(&csequence[start], end - start)
 *         if not split:
 *             tokens.append(cterm)             # <<<<<<<<<<<<<<
 *         else:
 *             tokens[-1] = tokens[-1] + (cterm, )
*/
      __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_tokens, __pyx_v_cterm); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 427, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":426
 *     if start != end:
 *         cterm = PyStr_FromStringAndSize(&csequence[start], end - start)
 *         if not split:             # <<<<<<<<<<<<<<
 *             tokens.append(cterm)
 *         else:
*/
      goto __pyx_L11;
    }

    /* "pyteomics/cparser.pyx":429
 *             tokens.append(cterm)
 *         else:
 *             tokens[-1] = tokens[-1] + (cterm, )             # <<<<<<<<<<<<<<
 *     if split and nterm:
 *         tokens[0] = (nterm, ) + tokens[0]
*/
    /*else*/ {
      __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_tokens, -1L, long, 1, __Pyx_PyLong_From_long, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_v_cterm);
      __Pyx_GIVEREF(__pyx_v_cterm);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_cterm) != (0)) __PYX_ERR(0, 429, __pyx_L1_error);
      __pyx_t_5 = PyNumber_Add(__pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely((__Pyx_SetItemInt(__pyx_v_tokens, -1L, __pyx_t_5, long, 1, __Pyx_PyLong_From_long, 1, 1, 1) < 0))) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_L11:;

    /* "pyteomics/cparser.pyx":424
 *     # The end of the last position was different from the end of the string so we must have
 *     # a C-terminal
 *     if start != end:             # <<<<<<<<<<<<<<
 *         cterm = PyStr_FromStringAndSize(&csequence[start], end - start)
 *         if not split:
*/
  }

  /* "pyteomics/cparser.pyx":430
 *         else:
 *             tokens[-1] = tokens[-1] + (cterm, )
 *     if split and nterm:             # <<<<<<<<<<<<<<
 *         tokens[0] = (nterm, ) + tokens[0]
 *     return tokens
*/
  if (__pyx_v_split) {
  } else {
    __pyx_t_2 = __pyx_v_split;
    goto __pyx_L13_bool_binop_done;
  }
  __pyx_t_6 = (__pyx_v_nterm != Py_None)&&(__Pyx_PyUnicode_IS_TRUE(__pyx_v_nterm) != 0);
  if (unlikely(((!CYTHON_ASSUME_SAFE_MACROS) && __pyx_t_6 < 0))) __PYX_ERR(0, 430, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_6;
  __pyx_L13_bool_binop_done:;
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":431
 *             tokens[-1] = tokens[-1] + (cterm, )
 *     if split and nterm:
 *         tokens[0] = (nterm, ) + tokens[0]             # <<<<<<<<<<<<<<
 *     return tokens
 * 
*/
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_nterm);
    __Pyx_GIVEREF(__pyx_v_nterm);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_nterm) != (0)) __PYX_ERR(0, 431, __pyx_L1_error);
    __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_tokens, 0, long, 1, __Pyx_PyLong_From_long, 1, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyNumber_Add(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely((__Pyx_SetItemInt(__pyx_v_tokens, 0, __pyx_t_1, long, 1, __Pyx_PyLong_From_long, 1, 0, 1) < 0))) __PYX_ERR(0, 431, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":430
 *         else:
 *             tokens[-1] = tokens[-1] + (cterm, )
 *     if split and nterm:             # <<<<<<<<<<<<<<
 *         tokens[0] = (nterm, ) + tokens[0]
 *     return tokens
*/
  }

  /* "pyteomics/cparser.pyx":432
 *     if split and nterm:
 *         tokens[0] = (nterm, ) + tokens[0]
 *     return tokens             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_tokens);
  __pyx_r = __pyx_v_tokens;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":375
 * 
 * 
 * cdef list _tokenize_modx_sequence(str sequence, bint split=True):             # <<<<<<<<<<<<<<
 *     cdef:
 *         char* csequence
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyteomics.cparser._tokenize_modx_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tokens);
  __Pyx_XDECREF(__pyx_v_nterm);
  __Pyx_XDECREF(__pyx_v_cterm);
  __Pyx_XDECREF(__pyx_v_mod);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":435
 * 
 * 
 * cpdef str tostring(object parsed_sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     """Create a string from a parsed sequence.
 * 
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_7tostring(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_tostring(PyObject *__pyx_v_parsed_sequence, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_tostring *__pyx_optional_args) {
  int __pyx_v_show_unmodified_termini = ((int)1);
  PyObject *__pyx_v_group = 0;
  PyObject *__pyx_v_group_j = 0;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_n;
  Py_ssize_t __pyx_v_acc;
  Py_ssize_t __pyx_v_sz;
  Py_ssize_t __pyx_v_j;
  Py_ssize_t __pyx_v_m;
  Py_ssize_t __pyx_v_k;
  PyObject *__pyx_v_sequence_fast = 0;
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_pos = 0;
  PyObject *__pyx_v_result = 0;
  char *__pyx_v_cstr;
  char __pyx_v_prealloc_buffer[0x96];
  char *__pyx_v_string_buffer;
  int __pyx_v_needs_free;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("tostring", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_show_unmodified_termini = __pyx_optional_args->show_unmodified_termini;
    }
  }

  /* "pyteomics/cparser.pyx":467
 *         char* string_buffer
 *         bint needs_free
 *     needs_free = False             # <<<<<<<<<<<<<<
 *     sz = 0
 *     acc = 0
*/
  __pyx_v_needs_free = 0;

  /* "pyteomics/cparser.pyx":468
 *         bint needs_free
 *     needs_free = False
 *     sz = 0             # <<<<<<<<<<<<<<
 *     acc = 0
 *     sequence_fast = PySequence_Fast(parsed_sequence, "tostring requires an iterable sequence!")
*/
  __pyx_v_sz = 0;

  /* "pyteomics/cparser.pyx":469
 *     needs_free = False
 *     sz = 0
 *     acc = 0             # <<<<<<<<<<<<<<
 *     sequence_fast = PySequence_Fast(parsed_sequence, "tostring requires an iterable sequence!")
 *     n = PySequence_Fast_GET_SIZE(sequence_fast)
*/
  __pyx_v_acc = 0;

  /* "pyteomics/cparser.pyx":470
 *     sz = 0
 *     acc = 0
 *     sequence_fast = PySequence_Fast(parsed_sequence, "tostring requires an iterable sequence!")             # <<<<<<<<<<<<<<
 *     n = PySequence_Fast_GET_SIZE(sequence_fast)
 *     for i in range(n):
*/
  __pyx_t_1 = PySequence_Fast(__pyx_v_parsed_sequence, ((char *)"tostring requires an iterable sequence!")); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_sequence_fast = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":471
 *     acc = 0
 *     sequence_fast = PySequence_Fast(parsed_sequence, "tostring requires an iterable sequence!")
 *     n = PySequence_Fast_GET_SIZE(sequence_fast)             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)
*/
  __pyx_v_n = PySequence_Fast_GET_SIZE(__pyx_v_sequence_fast);

  /* "pyteomics/cparser.pyx":472
 *     sequence_fast = PySequence_Fast(parsed_sequence, "tostring requires an iterable sequence!")
 *     n = PySequence_Fast_GET_SIZE(sequence_fast)
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)
 *         if isinstance(group, tuple):
*/
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pyteomics/cparser.pyx":473
 *     n = PySequence_Fast_GET_SIZE(sequence_fast)
 *     for i in range(n):
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)             # <<<<<<<<<<<<<<
 *         if isinstance(group, tuple):
 *             m = PyTuple_Size(group)
*/
    __pyx_t_5 = PySequence_Fast_GET_ITEM(__pyx_v_sequence_fast, __pyx_v_i);
    __pyx_t_1 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_group, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":474
 *     for i in range(n):
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)
 *         if isinstance(group, tuple):             # <<<<<<<<<<<<<<
 *             m = PyTuple_Size(group)
 *             for j in range(m):
*/
    __pyx_t_6 = PyTuple_Check(__pyx_v_group); 
    if (__pyx_t_6) {

      /* "pyteomics/cparser.pyx":475
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)
 *         if isinstance(group, tuple):
 *             m = PyTuple_Size(group)             # <<<<<<<<<<<<<<
 *             for j in range(m):
 *                 group_j = <str>PyTuple_GetItem(group, j)
*/
      __pyx_t_7 = PyTuple_Size(__pyx_v_group); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 475, __pyx_L1_error)
      __pyx_v_m = __pyx_t_7;

      /* "pyteomics/cparser.pyx":476
 *         if isinstance(group, tuple):
 *             m = PyTuple_Size(group)
 *             for j in range(m):             # <<<<<<<<<<<<<<
 *                 group_j = <str>PyTuple_GetItem(group, j)
 *                 PyStr_AsUTF8AndSize(group_j, &sz)
*/
      __pyx_t_7 = __pyx_v_m;
      __pyx_t_8 = __pyx_t_7;
      for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_j = __pyx_t_9;

        /* "pyteomics/cparser.pyx":477
 *             m = PyTuple_Size(group)
 *             for j in range(m):
 *                 group_j = <str>PyTuple_GetItem(group, j)             # <<<<<<<<<<<<<<
 *                 PyStr_AsUTF8AndSize(group_j, &sz)
 *                 acc += sz
*/
        __pyx_t_5 = PyTuple_GetItem(__pyx_v_group, __pyx_v_j); if (unlikely(__pyx_t_5 == ((PyObject *)0))) __PYX_ERR(0, 477, __pyx_L1_error)
        __pyx_t_1 = ((PyObject *)__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_XDECREF_SET(__pyx_v_group_j, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "pyteomics/cparser.pyx":478
 *             for j in range(m):
 *                 group_j = <str>PyTuple_GetItem(group, j)
 *                 PyStr_AsUTF8AndSize(group_j, &sz)             # <<<<<<<<<<<<<<
 *                 acc += sz
 *         elif isinstance(group, str):
*/
        (void)(PyStr_AsUTF8AndSize(__pyx_v_group_j, (&__pyx_v_sz)));

        /* "pyteomics/cparser.pyx":479
 *                 group_j = <str>PyTuple_GetItem(group, j)
 *                 PyStr_AsUTF8AndSize(group_j, &sz)
 *                 acc += sz             # <<<<<<<<<<<<<<
 *         elif isinstance(group, str):
 *             PyStr_AsUTF8AndSize(group, &sz)
*/
        __pyx_v_acc = (__pyx_v_acc + __pyx_v_sz);
      }

      /* "pyteomics/cparser.pyx":474
 *     for i in range(n):
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)
 *         if isinstance(group, tuple):             # <<<<<<<<<<<<<<
 *             m = PyTuple_Size(group)
 *             for j in range(m):
*/
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":480
 *                 PyStr_AsUTF8AndSize(group_j, &sz)
 *                 acc += sz
 *         elif isinstance(group, str):             # <<<<<<<<<<<<<<
 *             PyStr_AsUTF8AndSize(group, &sz)
 *             acc += sz
*/
    __pyx_t_6 = PyUnicode_Check(__pyx_v_group); 
    if (__pyx_t_6) {

      /* "pyteomics/cparser.pyx":481
 *                 acc += sz
 *         elif isinstance(group, str):
 *             PyStr_AsUTF8AndSize(group, &sz)             # <<<<<<<<<<<<<<
 *             acc += sz
 *         elif isinstance(group, SequencePosition):
*/
      if (!(likely(PyUnicode_CheckExact(__pyx_v_group))||((__pyx_v_group) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_group))) __PYX_ERR(0, 481, __pyx_L1_error)
      (void)(PyStr_AsUTF8AndSize(((PyObject*)__pyx_v_group), (&__pyx_v_sz)));

      /* "pyteomics/cparser.pyx":482
 *         elif isinstance(group, str):
 *             PyStr_AsUTF8AndSize(group, &sz)
 *             acc += sz             # <<<<<<<<<<<<<<
 *         elif isinstance(group, SequencePosition):
 *             pos = <SequencePosition>group
*/
      __pyx_v_acc = (__pyx_v_acc + __pyx_v_sz);

      /* "pyteomics/cparser.pyx":480
 *                 PyStr_AsUTF8AndSize(group_j, &sz)
 *                 acc += sz
 *         elif isinstance(group, str):             # <<<<<<<<<<<<<<
 *             PyStr_AsUTF8AndSize(group, &sz)
 *             acc += sz
*/
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":483
 *             PyStr_AsUTF8AndSize(group, &sz)
 *             acc += sz
 *         elif isinstance(group, SequencePosition):             # <<<<<<<<<<<<<<
 *             pos = <SequencePosition>group
 *             if pos.terminal is not None:
*/
    __pyx_t_6 = __Pyx_TypeCheck(__pyx_v_group, __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition); 
    if (likely(__pyx_t_6)) {

      /* "pyteomics/cparser.pyx":484
 *             acc += sz
 *         elif isinstance(group, SequencePosition):
 *             pos = <SequencePosition>group             # <<<<<<<<<<<<<<
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
*/
      __pyx_t_1 = __pyx_v_group;
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_pos, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":485
 *         elif isinstance(group, SequencePosition):
 *             pos = <SequencePosition>group
 *             if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 acc += sz
*/
      __pyx_t_6 = (__pyx_v_pos->terminal != ((PyObject*)Py_None));
      if (__pyx_t_6) {

        /* "pyteomics/cparser.pyx":486
 *             pos = <SequencePosition>group
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)             # <<<<<<<<<<<<<<
 *                 acc += sz
 *             if pos.modification is not None:
*/
        __pyx_t_1 = __pyx_v_pos->terminal;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_1), (&__pyx_v_sz));
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyteomics/cparser.pyx":487
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 acc += sz             # <<<<<<<<<<<<<<
 *             if pos.modification is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
*/
        __pyx_v_acc = (__pyx_v_acc + __pyx_v_sz);

        /* "pyteomics/cparser.pyx":485
 *         elif isinstance(group, SequencePosition):
 *             pos = <SequencePosition>group
 *             if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 acc += sz
*/
      }

      /* "pyteomics/cparser.pyx":488
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 acc += sz
 *             if pos.modification is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *                 acc += sz
*/
      __pyx_t_6 = (__pyx_v_pos->modification != ((PyObject*)Py_None));
      if (__pyx_t_6) {

        /* "pyteomics/cparser.pyx":489
 *                 acc += sz
 *             if pos.modification is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)             # <<<<<<<<<<<<<<
 *                 acc += sz
 *             if pos.amino_acid is not None:
*/
        __pyx_t_1 = __pyx_v_pos->modification;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_1), (&__pyx_v_sz));
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyteomics/cparser.pyx":490
 *             if pos.modification is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *                 acc += sz             # <<<<<<<<<<<<<<
 *             if pos.amino_acid is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
*/
        __pyx_v_acc = (__pyx_v_acc + __pyx_v_sz);

        /* "pyteomics/cparser.pyx":488
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 acc += sz
 *             if pos.modification is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *                 acc += sz
*/
      }

      /* "pyteomics/cparser.pyx":491
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *                 acc += sz
 *             if pos.amino_acid is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *                 acc += sz
*/
      __pyx_t_6 = (__pyx_v_pos->amino_acid != ((PyObject*)Py_None));
      if (__pyx_t_6) {

        /* "pyteomics/cparser.pyx":492
 *                 acc += sz
 *             if pos.amino_acid is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)             # <<<<<<<<<<<<<<
 *                 acc += sz
 *         else:
*/
        __pyx_t_1 = __pyx_v_pos->amino_acid;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_1), (&__pyx_v_sz));
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyteomics/cparser.pyx":493
 *             if pos.amino_acid is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *                 acc += sz             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError(type(group))
*/
        __pyx_v_acc = (__pyx_v_acc + __pyx_v_sz);

        /* "pyteomics/cparser.pyx":491
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *                 acc += sz
 *             if pos.amino_acid is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *                 acc += sz
*/
      }

      /* "pyteomics/cparser.pyx":483
 *             PyStr_AsUTF8AndSize(group, &sz)
 *             acc += sz
 *         elif isinstance(group, SequencePosition):             # <<<<<<<<<<<<<<
 *             pos = <SequencePosition>group
 *             if pos.terminal is not None:
*/
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":495
 *                 acc += sz
 *         else:
 *             raise TypeError(type(group))             # <<<<<<<<<<<<<<
 * 
 *     if acc < TOSTRING_STACK_BUFFER_SIZE:
*/
    /*else*/ {
      __pyx_t_10 = NULL;
      __Pyx_INCREF(__pyx_builtin_TypeError);
      __pyx_t_11 = __pyx_builtin_TypeError; 
      __pyx_t_12 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, ((PyObject *)Py_TYPE(__pyx_v_group))};
        __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 495, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 495, __pyx_L1_error)
    }
    __pyx_L5:;
  }

  /* "pyteomics/cparser.pyx":497
 *             raise TypeError(type(group))
 * 
 *     if acc < TOSTRING_STACK_BUFFER_SIZE:             # <<<<<<<<<<<<<<
 *         string_buffer = prealloc_buffer
 *     else:
*/
  __pyx_t_6 = (__pyx_v_acc < 0x96);
  if (__pyx_t_6) {

    /* "pyteomics/cparser.pyx":498
 * 
 *     if acc < TOSTRING_STACK_BUFFER_SIZE:
 *         string_buffer = prealloc_buffer             # <<<<<<<<<<<<<<
 *     else:
 *         string_buffer = <char*>malloc(sizeof(char*) * acc)
*/
    __pyx_v_string_buffer = __pyx_v_prealloc_buffer;

    /* "pyteomics/cparser.pyx":497
 *             raise TypeError(type(group))
 * 
 *     if acc < TOSTRING_STACK_BUFFER_SIZE:             # <<<<<<<<<<<<<<
 *         string_buffer = prealloc_buffer
 *     else:
*/
    goto __pyx_L11;
  }

  /* "pyteomics/cparser.pyx":500
 *         string_buffer = prealloc_buffer
 *     else:
 *         string_buffer = <char*>malloc(sizeof(char*) * acc)             # <<<<<<<<<<<<<<
 *         needs_free = True
 *     acc = 0
*/
  /*else*/ {
    __pyx_v_string_buffer = ((char *)malloc(((sizeof(char *)) * __pyx_v_acc)));

    /* "pyteomics/cparser.pyx":501
 *     else:
 *         string_buffer = <char*>malloc(sizeof(char*) * acc)
 *         needs_free = True             # <<<<<<<<<<<<<<
 *     acc = 0
 *     for i in range(n):
*/
    __pyx_v_needs_free = 1;
  }
  __pyx_L11:;

  /* "pyteomics/cparser.pyx":502
 *         string_buffer = <char*>malloc(sizeof(char*) * acc)
 *         needs_free = True
 *     acc = 0             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)
*/
  __pyx_v_acc = 0;

  /* "pyteomics/cparser.pyx":503
 *         needs_free = True
 *     acc = 0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)
 *         if isinstance(group, tuple):
*/
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pyteomics/cparser.pyx":504
 *     acc = 0
 *     for i in range(n):
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)             # <<<<<<<<<<<<<<
 *         if isinstance(group, tuple):
 *             m = PyTuple_Size(group)
*/
    __pyx_t_5 = PySequence_Fast_GET_ITEM(__pyx_v_sequence_fast, __pyx_v_i);
    __pyx_t_1 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_group, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":505
 *     for i in range(n):
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)
 *         if isinstance(group, tuple):             # <<<<<<<<<<<<<<
 *             m = PyTuple_Size(group)
 *             for j in range(m):
*/
    __pyx_t_6 = PyTuple_Check(__pyx_v_group); 
    if (__pyx_t_6) {

      /* "pyteomics/cparser.pyx":506
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)
 *         if isinstance(group, tuple):
 *             m = PyTuple_Size(group)             # <<<<<<<<<<<<<<
 *             for j in range(m):
 *                 group_j = <str>PyTuple_GetItem(group, j)
*/
      __pyx_t_7 = PyTuple_Size(__pyx_v_group); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 506, __pyx_L1_error)
      __pyx_v_m = __pyx_t_7;

      /* "pyteomics/cparser.pyx":507
 *         if isinstance(group, tuple):
 *             m = PyTuple_Size(group)
 *             for j in range(m):             # <<<<<<<<<<<<<<
 *                 group_j = <str>PyTuple_GetItem(group, j)
 *                 if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:
*/
      __pyx_t_7 = __pyx_v_m;
      __pyx_t_8 = __pyx_t_7;
      for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_j = __pyx_t_9;

        /* "pyteomics/cparser.pyx":508
 *             m = PyTuple_Size(group)
 *             for j in range(m):
 *                 group_j = <str>PyTuple_GetItem(group, j)             # <<<<<<<<<<<<<<
 *                 if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:
 *                     continue
*/
        __pyx_t_5 = PyTuple_GetItem(__pyx_v_group, __pyx_v_j); if (unlikely(__pyx_t_5 == ((PyObject *)0))) __PYX_ERR(0, 508, __pyx_L1_error)
        __pyx_t_1 = ((PyObject *)__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_XDECREF_SET(__pyx_v_group_j, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "pyteomics/cparser.pyx":509
 *             for j in range(m):
 *                 group_j = <str>PyTuple_GetItem(group, j)
 *                 if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:             # <<<<<<<<<<<<<<
 *                     continue
 * 
*/
        __pyx_t_13 = (__Pyx_PyUnicode_Equals(__pyx_v_group_j, __pyx_v_9pyteomics_7cparser_std_cterm, Py_EQ)); if (unlikely((__pyx_t_13 < 0))) __PYX_ERR(0, 509, __pyx_L1_error)
        if (!__pyx_t_13) {
        } else {
          goto __pyx_L19_next_and;
        }
        __pyx_t_13 = (__Pyx_PyUnicode_Equals(__pyx_v_group_j, __pyx_v_9pyteomics_7cparser_std_nterm, Py_EQ)); if (unlikely((__pyx_t_13 < 0))) __PYX_ERR(0, 509, __pyx_L1_error)
        if (__pyx_t_13) {
        } else {
          __pyx_t_6 = __pyx_t_13;
          goto __pyx_L18_bool_binop_done;
        }
        __pyx_L19_next_and:;
        __pyx_t_13 = (!__pyx_v_show_unmodified_termini);
        __pyx_t_6 = __pyx_t_13;
        __pyx_L18_bool_binop_done:;
        if (__pyx_t_6) {

          /* "pyteomics/cparser.pyx":510
 *                 group_j = <str>PyTuple_GetItem(group, j)
 *                 if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:
 *                     continue             # <<<<<<<<<<<<<<
 * 
 *                 cstr = PyStr_AsUTF8AndSize(group_j, &sz)
*/
          goto __pyx_L15_continue;

          /* "pyteomics/cparser.pyx":509
 *             for j in range(m):
 *                 group_j = <str>PyTuple_GetItem(group, j)
 *                 if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:             # <<<<<<<<<<<<<<
 *                     continue
 * 
*/
        }

        /* "pyteomics/cparser.pyx":512
 *                     continue
 * 
 *                 cstr = PyStr_AsUTF8AndSize(group_j, &sz)             # <<<<<<<<<<<<<<
 *                 for k in range(sz):
 *                     string_buffer[acc] = cstr[k]
*/
        __pyx_v_cstr = PyStr_AsUTF8AndSize(__pyx_v_group_j, (&__pyx_v_sz));

        /* "pyteomics/cparser.pyx":513
 * 
 *                 cstr = PyStr_AsUTF8AndSize(group_j, &sz)
 *                 for k in range(sz):             # <<<<<<<<<<<<<<
 *                     string_buffer[acc] = cstr[k]
 *                     acc += 1
*/
        __pyx_t_14 = __pyx_v_sz;
        __pyx_t_15 = __pyx_t_14;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_k = __pyx_t_16;

          /* "pyteomics/cparser.pyx":514
 *                 cstr = PyStr_AsUTF8AndSize(group_j, &sz)
 *                 for k in range(sz):
 *                     string_buffer[acc] = cstr[k]             # <<<<<<<<<<<<<<
 *                     acc += 1
 * 
*/
          (__pyx_v_string_buffer[__pyx_v_acc]) = (__pyx_v_cstr[__pyx_v_k]);

          /* "pyteomics/cparser.pyx":515
 *                 for k in range(sz):
 *                     string_buffer[acc] = cstr[k]
 *                     acc += 1             # <<<<<<<<<<<<<<
 * 
 *         elif isinstance(group, str):
*/
          __pyx_v_acc = (__pyx_v_acc + 1);
        }
        __pyx_L15_continue:;
      }

      /* "pyteomics/cparser.pyx":505
 *     for i in range(n):
 *         group = <object>PySequence_Fast_GET_ITEM(sequence_fast, i)
 *         if isinstance(group, tuple):             # <<<<<<<<<<<<<<
 *             m = PyTuple_Size(group)
 *             for j in range(m):
*/
      goto __pyx_L14;
    }

    /* "pyteomics/cparser.pyx":517
 *                     acc += 1
 * 
 *         elif isinstance(group, str):             # <<<<<<<<<<<<<<
 *             group_j = <str>group
 *             if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:
*/
    __pyx_t_6 = PyUnicode_Check(__pyx_v_group); 
    if (__pyx_t_6) {

      /* "pyteomics/cparser.pyx":518
 * 
 *         elif isinstance(group, str):
 *             group_j = <str>group             # <<<<<<<<<<<<<<
 *             if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:
 *                 continue
*/
      __pyx_t_1 = __pyx_v_group;
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_group_j, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":519
 *         elif isinstance(group, str):
 *             group_j = <str>group
 *             if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:             # <<<<<<<<<<<<<<
 *                 continue
 * 
*/
      __pyx_t_13 = (__Pyx_PyUnicode_Equals(__pyx_v_group_j, __pyx_v_9pyteomics_7cparser_std_cterm, Py_EQ)); if (unlikely((__pyx_t_13 < 0))) __PYX_ERR(0, 519, __pyx_L1_error)
      if (!__pyx_t_13) {
      } else {
        goto __pyx_L25_next_and;
      }
      __pyx_t_13 = (__Pyx_PyUnicode_Equals(__pyx_v_group_j, __pyx_v_9pyteomics_7cparser_std_nterm, Py_EQ)); if (unlikely((__pyx_t_13 < 0))) __PYX_ERR(0, 519, __pyx_L1_error)
      if (__pyx_t_13) {
      } else {
        __pyx_t_6 = __pyx_t_13;
        goto __pyx_L24_bool_binop_done;
      }
      __pyx_L25_next_and:;
      __pyx_t_13 = (!__pyx_v_show_unmodified_termini);
      __pyx_t_6 = __pyx_t_13;
      __pyx_L24_bool_binop_done:;
      if (__pyx_t_6) {

        /* "pyteomics/cparser.pyx":520
 *             group_j = <str>group
 *             if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:
 *                 continue             # <<<<<<<<<<<<<<
 * 
 *             PyStr_AsUTF8AndSize(group, &sz)
*/
        goto __pyx_L12_continue;

        /* "pyteomics/cparser.pyx":519
 *         elif isinstance(group, str):
 *             group_j = <str>group
 *             if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:             # <<<<<<<<<<<<<<
 *                 continue
 * 
*/
      }

      /* "pyteomics/cparser.pyx":522
 *                 continue
 * 
 *             PyStr_AsUTF8AndSize(group, &sz)             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(group, &sz)
 *             for k in range(sz):
*/
      if (!(likely(PyUnicode_CheckExact(__pyx_v_group))||((__pyx_v_group) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_group))) __PYX_ERR(0, 522, __pyx_L1_error)
      (void)(PyStr_AsUTF8AndSize(((PyObject*)__pyx_v_group), (&__pyx_v_sz)));

      /* "pyteomics/cparser.pyx":523
 * 
 *             PyStr_AsUTF8AndSize(group, &sz)
 *             cstr = PyStr_AsUTF8AndSize(group, &sz)             # <<<<<<<<<<<<<<
 *             for k in range(sz):
 *                 string_buffer[acc] = cstr[k]
*/
      if (!(likely(PyUnicode_CheckExact(__pyx_v_group))||((__pyx_v_group) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_v_group))) __PYX_ERR(0, 523, __pyx_L1_error)
      __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_v_group), (&__pyx_v_sz));

      /* "pyteomics/cparser.pyx":524
 *             PyStr_AsUTF8AndSize(group, &sz)
 *             cstr = PyStr_AsUTF8AndSize(group, &sz)
 *             for k in range(sz):             # <<<<<<<<<<<<<<
 *                 string_buffer[acc] = cstr[k]
 *                 acc += 1
*/
      __pyx_t_7 = __pyx_v_sz;
      __pyx_t_8 = __pyx_t_7;
      for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_k = __pyx_t_9;

        /* "pyteomics/cparser.pyx":525
 *             cstr = PyStr_AsUTF8AndSize(group, &sz)
 *             for k in range(sz):
 *                 string_buffer[acc] = cstr[k]             # <<<<<<<<<<<<<<
 *                 acc += 1
 * 
*/
        (__pyx_v_string_buffer[__pyx_v_acc]) = (__pyx_v_cstr[__pyx_v_k]);

        /* "pyteomics/cparser.pyx":526
 *             for k in range(sz):
 *                 string_buffer[acc] = cstr[k]
 *                 acc += 1             # <<<<<<<<<<<<<<
 * 
 *         elif isinstance(group, SequencePosition):
*/
        __pyx_v_acc = (__pyx_v_acc + 1);
      }

      /* "pyteomics/cparser.pyx":517
 *                     acc += 1
 * 
 *         elif isinstance(group, str):             # <<<<<<<<<<<<<<
 *             group_j = <str>group
 *             if (group_j == std_cterm or group_j == std_nterm) and not show_unmodified_termini:
*/
      goto __pyx_L14;
    }

    /* "pyteomics/cparser.pyx":528
 *                 acc += 1
 * 
 *         elif isinstance(group, SequencePosition):             # <<<<<<<<<<<<<<
 *             pos = <SequencePosition>group
 *             if pos.position_type == PositionType.n_term:
*/
    __pyx_t_6 = __Pyx_TypeCheck(__pyx_v_group, __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition); 
    if (__pyx_t_6) {

      /* "pyteomics/cparser.pyx":529
 * 
 *         elif isinstance(group, SequencePosition):
 *             pos = <SequencePosition>group             # <<<<<<<<<<<<<<
 *             if pos.position_type == PositionType.n_term:
 *                 if pos.terminal is not None:
*/
      __pyx_t_1 = __pyx_v_group;
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_pos, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":530
 *         elif isinstance(group, SequencePosition):
 *             pos = <SequencePosition>group
 *             if pos.position_type == PositionType.n_term:             # <<<<<<<<<<<<<<
 *                 if pos.terminal is not None:
 *                     if show_unmodified_termini or pos.terminal != std_nterm:
*/
      __pyx_t_6 = (__pyx_v_pos->position_type == __pyx_e_9pyteomics_7cparser_n_term);
      if (__pyx_t_6) {

        /* "pyteomics/cparser.pyx":531
 *             pos = <SequencePosition>group
 *             if pos.position_type == PositionType.n_term:
 *                 if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *                     if show_unmodified_termini or pos.terminal != std_nterm:
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
*/
        __pyx_t_6 = (__pyx_v_pos->terminal != ((PyObject*)Py_None));
        if (__pyx_t_6) {

          /* "pyteomics/cparser.pyx":532
 *             if pos.position_type == PositionType.n_term:
 *                 if pos.terminal is not None:
 *                     if show_unmodified_termini or pos.terminal != std_nterm:             # <<<<<<<<<<<<<<
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                         for j in range(sz):
*/
          if (!__pyx_v_show_unmodified_termini) {
          } else {
            __pyx_t_6 = __pyx_v_show_unmodified_termini;
            goto __pyx_L32_bool_binop_done;
          }
          __pyx_t_13 = (__Pyx_PyUnicode_Equals(__pyx_v_pos->terminal, __pyx_v_9pyteomics_7cparser_std_nterm, Py_NE)); if (unlikely((__pyx_t_13 < 0))) __PYX_ERR(0, 532, __pyx_L1_error)
          __pyx_t_6 = __pyx_t_13;
          __pyx_L32_bool_binop_done:;
          if (__pyx_t_6) {

            /* "pyteomics/cparser.pyx":533
 *                 if pos.terminal is not None:
 *                     if show_unmodified_termini or pos.terminal != std_nterm:
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)             # <<<<<<<<<<<<<<
 *                         for j in range(sz):
 *                             string_buffer[acc] = cstr[j]
*/
            __pyx_t_1 = __pyx_v_pos->terminal;
            __Pyx_INCREF(__pyx_t_1);
            __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_1), (&__pyx_v_sz));
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "pyteomics/cparser.pyx":534
 *                     if show_unmodified_termini or pos.terminal != std_nterm:
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                         for j in range(sz):             # <<<<<<<<<<<<<<
 *                             string_buffer[acc] = cstr[j]
 *                             acc += 1
*/
            __pyx_t_7 = __pyx_v_sz;
            __pyx_t_8 = __pyx_t_7;
            for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
              __pyx_v_j = __pyx_t_9;

              /* "pyteomics/cparser.pyx":535
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                         for j in range(sz):
 *                             string_buffer[acc] = cstr[j]             # <<<<<<<<<<<<<<
 *                             acc += 1
 * 
*/
              (__pyx_v_string_buffer[__pyx_v_acc]) = (__pyx_v_cstr[__pyx_v_j]);

              /* "pyteomics/cparser.pyx":536
 *                         for j in range(sz):
 *                             string_buffer[acc] = cstr[j]
 *                             acc += 1             # <<<<<<<<<<<<<<
 * 
 *             if pos.modification is not None:
*/
              __pyx_v_acc = (__pyx_v_acc + 1);
            }

            /* "pyteomics/cparser.pyx":532
 *             if pos.position_type == PositionType.n_term:
 *                 if pos.terminal is not None:
 *                     if show_unmodified_termini or pos.terminal != std_nterm:             # <<<<<<<<<<<<<<
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                         for j in range(sz):
*/
          }

          /* "pyteomics/cparser.pyx":531
 *             pos = <SequencePosition>group
 *             if pos.position_type == PositionType.n_term:
 *                 if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *                     if show_unmodified_termini or pos.terminal != std_nterm:
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
*/
        }

        /* "pyteomics/cparser.pyx":530
 *         elif isinstance(group, SequencePosition):
 *             pos = <SequencePosition>group
 *             if pos.position_type == PositionType.n_term:             # <<<<<<<<<<<<<<
 *                 if pos.terminal is not None:
 *                     if show_unmodified_termini or pos.terminal != std_nterm:
*/
      }

      /* "pyteomics/cparser.pyx":538
 *                             acc += 1
 * 
 *             if pos.modification is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *                 for j in range(sz):
*/
      __pyx_t_6 = (__pyx_v_pos->modification != ((PyObject*)Py_None));
      if (__pyx_t_6) {

        /* "pyteomics/cparser.pyx":539
 * 
 *             if pos.modification is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)             # <<<<<<<<<<<<<<
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]
*/
        __pyx_t_1 = __pyx_v_pos->modification;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_1), (&__pyx_v_sz));
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyteomics/cparser.pyx":540
 *             if pos.modification is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *                 for j in range(sz):             # <<<<<<<<<<<<<<
 *                     string_buffer[acc] = cstr[j]
 *                     acc += 1
*/
        __pyx_t_7 = __pyx_v_sz;
        __pyx_t_8 = __pyx_t_7;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_j = __pyx_t_9;

          /* "pyteomics/cparser.pyx":541
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]             # <<<<<<<<<<<<<<
 *                     acc += 1
 * 
*/
          (__pyx_v_string_buffer[__pyx_v_acc]) = (__pyx_v_cstr[__pyx_v_j]);

          /* "pyteomics/cparser.pyx":542
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]
 *                     acc += 1             # <<<<<<<<<<<<<<
 * 
 *             if pos.amino_acid is not None:
*/
          __pyx_v_acc = (__pyx_v_acc + 1);
        }

        /* "pyteomics/cparser.pyx":538
 *                             acc += 1
 * 
 *             if pos.modification is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *                 for j in range(sz):
*/
      }

      /* "pyteomics/cparser.pyx":544
 *                     acc += 1
 * 
 *             if pos.amino_acid is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *                 for j in range(sz):
*/
      __pyx_t_6 = (__pyx_v_pos->amino_acid != ((PyObject*)Py_None));
      if (__pyx_t_6) {

        /* "pyteomics/cparser.pyx":545
 * 
 *             if pos.amino_acid is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)             # <<<<<<<<<<<<<<
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]
*/
        __pyx_t_1 = __pyx_v_pos->amino_acid;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_1), (&__pyx_v_sz));
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyteomics/cparser.pyx":546
 *             if pos.amino_acid is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *                 for j in range(sz):             # <<<<<<<<<<<<<<
 *                     string_buffer[acc] = cstr[j]
 *                     acc += 1
*/
        __pyx_t_7 = __pyx_v_sz;
        __pyx_t_8 = __pyx_t_7;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_j = __pyx_t_9;

          /* "pyteomics/cparser.pyx":547
 *                 cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]             # <<<<<<<<<<<<<<
 *                     acc += 1
 * 
*/
          (__pyx_v_string_buffer[__pyx_v_acc]) = (__pyx_v_cstr[__pyx_v_j]);

          /* "pyteomics/cparser.pyx":548
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]
 *                     acc += 1             # <<<<<<<<<<<<<<
 * 
 *             if pos.position_type == PositionType.c_term:
*/
          __pyx_v_acc = (__pyx_v_acc + 1);
        }

        /* "pyteomics/cparser.pyx":544
 *                     acc += 1
 * 
 *             if pos.amino_acid is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *                 for j in range(sz):
*/
      }

      /* "pyteomics/cparser.pyx":550
 *                     acc += 1
 * 
 *             if pos.position_type == PositionType.c_term:             # <<<<<<<<<<<<<<
 *                 if pos.terminal is not None:
 *                     if show_unmodified_termini or pos.terminal != std_cterm:
*/
      __pyx_t_6 = (__pyx_v_pos->position_type == __pyx_e_9pyteomics_7cparser_c_term);
      if (__pyx_t_6) {

        /* "pyteomics/cparser.pyx":551
 * 
 *             if pos.position_type == PositionType.c_term:
 *                 if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *                     if show_unmodified_termini or pos.terminal != std_cterm:
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
*/
        __pyx_t_6 = (__pyx_v_pos->terminal != ((PyObject*)Py_None));
        if (__pyx_t_6) {

          /* "pyteomics/cparser.pyx":552
 *             if pos.position_type == PositionType.c_term:
 *                 if pos.terminal is not None:
 *                     if show_unmodified_termini or pos.terminal != std_cterm:             # <<<<<<<<<<<<<<
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                         for j in range(sz):
*/
          if (!__pyx_v_show_unmodified_termini) {
          } else {
            __pyx_t_6 = __pyx_v_show_unmodified_termini;
            goto __pyx_L45_bool_binop_done;
          }
          __pyx_t_13 = (__Pyx_PyUnicode_Equals(__pyx_v_pos->terminal, __pyx_v_9pyteomics_7cparser_std_cterm, Py_NE)); if (unlikely((__pyx_t_13 < 0))) __PYX_ERR(0, 552, __pyx_L1_error)
          __pyx_t_6 = __pyx_t_13;
          __pyx_L45_bool_binop_done:;
          if (__pyx_t_6) {

            /* "pyteomics/cparser.pyx":553
 *                 if pos.terminal is not None:
 *                     if show_unmodified_termini or pos.terminal != std_cterm:
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)             # <<<<<<<<<<<<<<
 *                         for j in range(sz):
 *                             string_buffer[acc] = cstr[j]
*/
            __pyx_t_1 = __pyx_v_pos->terminal;
            __Pyx_INCREF(__pyx_t_1);
            __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_1), (&__pyx_v_sz));
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "pyteomics/cparser.pyx":554
 *                     if show_unmodified_termini or pos.terminal != std_cterm:
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                         for j in range(sz):             # <<<<<<<<<<<<<<
 *                             string_buffer[acc] = cstr[j]
 *                             acc += 1
*/
            __pyx_t_7 = __pyx_v_sz;
            __pyx_t_8 = __pyx_t_7;
            for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
              __pyx_v_j = __pyx_t_9;

              /* "pyteomics/cparser.pyx":555
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                         for j in range(sz):
 *                             string_buffer[acc] = cstr[j]             # <<<<<<<<<<<<<<
 *                             acc += 1
 * 
*/
              (__pyx_v_string_buffer[__pyx_v_acc]) = (__pyx_v_cstr[__pyx_v_j]);

              /* "pyteomics/cparser.pyx":556
 *                         for j in range(sz):
 *                             string_buffer[acc] = cstr[j]
 *                             acc += 1             # <<<<<<<<<<<<<<
 * 
 *     result = PyStr_FromStringAndSize(string_buffer, acc)
*/
              __pyx_v_acc = (__pyx_v_acc + 1);
            }

            /* "pyteomics/cparser.pyx":552
 *             if pos.position_type == PositionType.c_term:
 *                 if pos.terminal is not None:
 *                     if show_unmodified_termini or pos.terminal != std_cterm:             # <<<<<<<<<<<<<<
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                         for j in range(sz):
*/
          }

          /* "pyteomics/cparser.pyx":551
 * 
 *             if pos.position_type == PositionType.c_term:
 *                 if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *                     if show_unmodified_termini or pos.terminal != std_cterm:
 *                         cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
*/
        }

        /* "pyteomics/cparser.pyx":550
 *                     acc += 1
 * 
 *             if pos.position_type == PositionType.c_term:             # <<<<<<<<<<<<<<
 *                 if pos.terminal is not None:
 *                     if show_unmodified_termini or pos.terminal != std_cterm:
*/
      }

      /* "pyteomics/cparser.pyx":528
 *                 acc += 1
 * 
 *         elif isinstance(group, SequencePosition):             # <<<<<<<<<<<<<<
 *             pos = <SequencePosition>group
 *             if pos.position_type == PositionType.n_term:
*/
    }
    __pyx_L14:;
    __pyx_L12_continue:;
  }

  /* "pyteomics/cparser.pyx":558
 *                             acc += 1
 * 
 *     result = PyStr_FromStringAndSize(string_buffer, acc)             # <<<<<<<<<<<<<<
 *     if needs_free:
 *         free(string_buffer)
*/
  __pyx_t_1 = PyStr_FromStringAndSize(__pyx_v_string_buffer, __pyx_v_acc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":559
 * 
 *     result = PyStr_FromStringAndSize(string_buffer, acc)
 *     if needs_free:             # <<<<<<<<<<<<<<
 *         free(string_buffer)
 *     return result
*/
  if (__pyx_v_needs_free) {

    /* "pyteomics/cparser.pyx":560
 *     result = PyStr_FromStringAndSize(string_buffer, acc)
 *     if needs_free:
 *         free(string_buffer)             # <<<<<<<<<<<<<<
 *     return result
 * 
*/
    free(__pyx_v_string_buffer);

    /* "pyteomics/cparser.pyx":559
 * 
 *     result = PyStr_FromStringAndSize(string_buffer, acc)
 *     if needs_free:             # <<<<<<<<<<<<<<
 *         free(string_buffer)
 *     return result
*/
  }

  /* "pyteomics/cparser.pyx":561
 *     if needs_free:
 *         free(string_buffer)
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":435
 * 
 * 
 * cpdef str tostring(object parsed_sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     """Create a string from a parsed sequence.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pyteomics.cparser.tostring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_group);
  __Pyx_XDECREF(__pyx_v_group_j);
  __Pyx_XDECREF(__pyx_v_sequence_fast);
  __Pyx_XDECREF((PyObject *)__pyx_v_pos);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_7tostring(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_6tostring, "tostring(parsed_sequence, bool show_unmodified_termini=True) -> str\nCreate a string from a parsed sequence.\n\nParameters\n----------\nparsed_sequence : iterable\n    Expected to be in one of the formats returned by\n    :py:func:`parse`, i.e. list of labels or list of tuples.\nshow_unmodified_termini : bool, optional\n    Defines the behavior towards standard terminal groups in the input.\n    :py:const:`True` means that they will be preserved if present (default).\n    :py:const:`False` means that they will be removed. Standard terminal\n    groups will not be added if not shown in `parsed_sequence`,\n    regardless of this setting.\n\nReturns\n-------\nsequence : str");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_7tostring = {"tostring", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_7tostring, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_6tostring};
static PyObject *__pyx_pw_9pyteomics_7cparser_7tostring(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_parsed_sequence = 0;
  int __pyx_v_show_unmodified_termini;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("tostring (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_parsed_sequence,&__pyx_mstate_global->__pyx_n_u_show_unmodified_termini,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 435, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 435, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 435, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "tostring", 0) < 0) __PYX_ERR(0, 435, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("tostring", 0, 1, 2, i); __PYX_ERR(0, 435, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 435, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 435, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_parsed_sequence = values[0];
    if (values[1]) {
      __pyx_v_show_unmodified_termini = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_show_unmodified_termini == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 435, __pyx_L3_error)
    } else {
      __pyx_v_show_unmodified_termini = ((int)1);
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("tostring", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 435, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.tostring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_6tostring(__pyx_self, __pyx_v_parsed_sequence, __pyx_v_show_unmodified_termini);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_6tostring(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_parsed_sequence, int __pyx_v_show_unmodified_termini) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_9pyteomics_7cparser_tostring __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("tostring", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.show_unmodified_termini = __pyx_v_show_unmodified_termini;
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_tostring(__pyx_v_parsed_sequence, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.tostring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":565
 * 
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=0, bint term_aa=0,             # <<<<<<<<<<<<<<
 *                                   bint allow_unknown_modifications=0, object labels=None):
 *     """Calculate amino acid composition of a polypeptide.
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_9amino_acid_composition(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_amino_acid_composition(PyObject *__pyx_v_sequence, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition *__pyx_optional_args) {
  int __pyx_v_show_unmodified_termini = ((int)0);
  int __pyx_v_term_aa = ((int)0);
  int __pyx_v_allow_unknown_modifications = ((int)0);

  /* "pyteomics/cparser.pyx":566
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=0, bint term_aa=0,
 *                                   bint allow_unknown_modifications=0, object labels=None):             # <<<<<<<<<<<<<<
 *     """Calculate amino acid composition of a polypeptide.
 * 
*/
  PyObject *__pyx_v_labels = ((PyObject *)Py_None);
  PyObject *__pyx_v_parsed_sequence = 0;
  int __pyx_v_nterm_aa_position;
  int __pyx_v_cterm_aa_position;
  int __pyx_v_aa_count;
  PyObject *__pyx_v_aa_dict = 0;
  PyObject *__pyx_v_aa = 0;
  PyObject *__pyx_v__labels = 0;
  PyObject *__pyx_v_temp = 0;
  PyObject *__pyx_v_pvalue;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  struct __pyx_opt_args_9pyteomics_7cparser_parse __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5;
  struct __pyx_opt_args_9pyteomics_7cparser_tostring __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("amino_acid_composition", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_show_unmodified_termini = __pyx_optional_args->show_unmodified_termini;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_term_aa = __pyx_optional_args->term_aa;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_allow_unknown_modifications = __pyx_optional_args->allow_unknown_modifications;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_labels = __pyx_optional_args->labels;
          }
        }
      }
    }
  }

  /* "pyteomics/cparser.pyx":614
 *         Py_ssize_t i
 * 
 *     if labels is None:             # <<<<<<<<<<<<<<
 *         allow_unknown_modifications = True
 * 
*/
  __pyx_t_1 = (__pyx_v_labels == Py_None);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":615
 * 
 *     if labels is None:
 *         allow_unknown_modifications = True             # <<<<<<<<<<<<<<
 * 
 *     _labels = interpolate_labels(labels)
*/
    __pyx_v_allow_unknown_modifications = 1;

    /* "pyteomics/cparser.pyx":614
 *         Py_ssize_t i
 * 
 *     if labels is None:             # <<<<<<<<<<<<<<
 *         allow_unknown_modifications = True
 * 
*/
  }

  /* "pyteomics/cparser.pyx":617
 *         allow_unknown_modifications = True
 * 
 *     _labels = interpolate_labels(labels)             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(sequence, str):
*/
  __pyx_t_2 = __pyx_f_9pyteomics_7cparser_interpolate_labels(__pyx_v_labels); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__labels = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":619
 *     _labels = interpolate_labels(labels)
 * 
 *     if isinstance(sequence, str):             # <<<<<<<<<<<<<<
 *         parsed_sequence = parse(<str>sequence, show_unmodified_termini, split=False,
 *                                 allow_unknown_modifications=allow_unknown_modifications,
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_v_sequence); 
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":620
 * 
 *     if isinstance(sequence, str):
 *         parsed_sequence = parse(<str>sequence, show_unmodified_termini, split=False,             # <<<<<<<<<<<<<<
 *                                 allow_unknown_modifications=allow_unknown_modifications,
 *                                 labels=_labels)
*/
    __pyx_t_3.__pyx_n = 4;
    __pyx_t_3.show_unmodified_termini = __pyx_v_show_unmodified_termini;
    __pyx_t_3.split = 0;
    __pyx_t_3.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
    __pyx_t_3.labels = __pyx_v__labels;
    __pyx_t_2 = __pyx_f_9pyteomics_7cparser_parse(((PyObject*)__pyx_v_sequence), 0, &__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 620, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":619
 *     _labels = interpolate_labels(labels)
 * 
 *     if isinstance(sequence, str):             # <<<<<<<<<<<<<<
 *         parsed_sequence = parse(<str>sequence, show_unmodified_termini, split=False,
 *                                 allow_unknown_modifications=allow_unknown_modifications,
*/
    goto __pyx_L4;
  }

  /* "pyteomics/cparser.pyx":623
 *                                 allow_unknown_modifications=allow_unknown_modifications,
 *                                 labels=_labels)
 *     elif isinstance(sequence, list):             # <<<<<<<<<<<<<<
 *         if sequence and isinstance(<object>PyList_GetItem(sequence, 0), tuple):
 *             parsed_sequence = parse(tostring(sequence, True),
*/
  __pyx_t_1 = PyList_Check(__pyx_v_sequence); 
  if (likely(__pyx_t_1)) {

    /* "pyteomics/cparser.pyx":624
 *                                 labels=_labels)
 *     elif isinstance(sequence, list):
 *         if sequence and isinstance(<object>PyList_GetItem(sequence, 0), tuple):             # <<<<<<<<<<<<<<
 *             parsed_sequence = parse(tostring(sequence, True),
 *                 show_unmodified_termini,
*/
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_sequence); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 624, __pyx_L1_error)
    if (__pyx_t_4) {
    } else {
      __pyx_t_1 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = PyList_GetItem(__pyx_v_sequence, 0); if (unlikely(__pyx_t_5 == ((PyObject *)0))) __PYX_ERR(0, 624, __pyx_L1_error)
    __pyx_t_4 = PyTuple_Check(((PyObject *)__pyx_t_5)); 
    __pyx_t_1 = __pyx_t_4;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_1) {

      /* "pyteomics/cparser.pyx":625
 *     elif isinstance(sequence, list):
 *         if sequence and isinstance(<object>PyList_GetItem(sequence, 0), tuple):
 *             parsed_sequence = parse(tostring(sequence, True),             # <<<<<<<<<<<<<<
 *                 show_unmodified_termini,
 *                 split=False,
*/
      __pyx_t_6.__pyx_n = 1;
      __pyx_t_6.show_unmodified_termini = 1;
      __pyx_t_2 = __pyx_f_9pyteomics_7cparser_tostring(__pyx_v_sequence, 0, &__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 625, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "pyteomics/cparser.pyx":629
 *                 split=False,
 *                 allow_unknown_modifications=allow_unknown_modifications,
 *                 labels=_labels)             # <<<<<<<<<<<<<<
 *         else:
 *             parsed_sequence = sequence
*/
      __pyx_t_3.__pyx_n = 4;
      __pyx_t_3.show_unmodified_termini = __pyx_v_show_unmodified_termini;
      __pyx_t_3.split = 0;
      __pyx_t_3.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
      __pyx_t_3.labels = __pyx_v__labels;
      __pyx_t_7 = __pyx_f_9pyteomics_7cparser_parse(((PyObject*)__pyx_t_2), 0, &__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 625, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_7);
      __pyx_t_7 = 0;

      /* "pyteomics/cparser.pyx":624
 *                                 labels=_labels)
 *     elif isinstance(sequence, list):
 *         if sequence and isinstance(<object>PyList_GetItem(sequence, 0), tuple):             # <<<<<<<<<<<<<<
 *             parsed_sequence = parse(tostring(sequence, True),
 *                 show_unmodified_termini,
*/
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":631
 *                 labels=_labels)
 *         else:
 *             parsed_sequence = sequence             # <<<<<<<<<<<<<<
 *     else:
 *         raise PyteomicsError('Unsupported type of a sequence.'
*/
    /*else*/ {
      __pyx_t_7 = __pyx_v_sequence;
      __Pyx_INCREF(__pyx_t_7);
      if (!(likely(PyList_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_7))) __PYX_ERR(0, 631, __pyx_L1_error)
      __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_7);
      __pyx_t_7 = 0;
    }
    __pyx_L5:;

    /* "pyteomics/cparser.pyx":623
 *                                 allow_unknown_modifications=allow_unknown_modifications,
 *                                 labels=_labels)
 *     elif isinstance(sequence, list):             # <<<<<<<<<<<<<<
 *         if sequence and isinstance(<object>PyList_GetItem(sequence, 0), tuple):
 *             parsed_sequence = parse(tostring(sequence, True),
*/
    goto __pyx_L4;
  }

  /* "pyteomics/cparser.pyx":633
 *             parsed_sequence = sequence
 *     else:
 *         raise PyteomicsError('Unsupported type of a sequence.'             # <<<<<<<<<<<<<<
 *                 'Must be str or list, not %s' % type(sequence))
 * 
*/
  /*else*/ {
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_PyteomicsError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 633, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);

    /* "pyteomics/cparser.pyx":634
 *     else:
 *         raise PyteomicsError('Unsupported type of a sequence.'
 *                 'Must be str or list, not %s' % type(sequence))             # <<<<<<<<<<<<<<
 * 
 *     aa_dict = {}
*/
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_mstate_global->__pyx_kp_u_Unsupported_type_of_a_sequence_M, ((PyObject *)Py_TYPE(__pyx_v_sequence))); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 634, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_8);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
      __pyx_t_10 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_9};
      __pyx_t_7 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 633, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 633, __pyx_L1_error)
  }
  __pyx_L4:;

  /* "pyteomics/cparser.pyx":636
 *                 'Must be str or list, not %s' % type(sequence))
 * 
 *     aa_dict = {}             # <<<<<<<<<<<<<<
 *     # Process terminal amino acids.
 *     if term_aa:
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 636, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_v_aa_dict = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;

  /* "pyteomics/cparser.pyx":638
 *     aa_dict = {}
 *     # Process terminal amino acids.
 *     if term_aa:             # <<<<<<<<<<<<<<
 *         if is_term_mod(<object>PyList_GetItem(parsed_sequence, 0)):
 *             nterm_aa_position = 1
*/
  if (__pyx_v_term_aa) {

    /* "pyteomics/cparser.pyx":639
 *     # Process terminal amino acids.
 *     if term_aa:
 *         if is_term_mod(<object>PyList_GetItem(parsed_sequence, 0)):             # <<<<<<<<<<<<<<
 *             nterm_aa_position = 1
 *         else:
*/
    __pyx_t_5 = PyList_GetItem(__pyx_v_parsed_sequence, 0); if (unlikely(__pyx_t_5 == ((PyObject *)0))) __PYX_ERR(0, 639, __pyx_L1_error)
    if (!(likely(PyUnicode_CheckExact(((PyObject *)__pyx_t_5)))||((((PyObject *)__pyx_t_5)) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", ((PyObject *)__pyx_t_5)))) __PYX_ERR(0, 639, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser_is_term_mod(((PyObject*)((PyObject *)__pyx_t_5))); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 639, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "pyteomics/cparser.pyx":640
 *     if term_aa:
 *         if is_term_mod(<object>PyList_GetItem(parsed_sequence, 0)):
 *             nterm_aa_position = 1             # <<<<<<<<<<<<<<
 *         else:
 *             nterm_aa_position = 0
*/
      __pyx_v_nterm_aa_position = 1;

      /* "pyteomics/cparser.pyx":639
 *     # Process terminal amino acids.
 *     if term_aa:
 *         if is_term_mod(<object>PyList_GetItem(parsed_sequence, 0)):             # <<<<<<<<<<<<<<
 *             nterm_aa_position = 1
 *         else:
*/
      goto __pyx_L9;
    }

    /* "pyteomics/cparser.pyx":642
 *             nterm_aa_position = 1
 *         else:
 *             nterm_aa_position = 0             # <<<<<<<<<<<<<<
 *         if is_term_mod(parsed_sequence[-1]):
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 2
*/
    /*else*/ {
      __pyx_v_nterm_aa_position = 0;
    }
    __pyx_L9:;

    /* "pyteomics/cparser.pyx":643
 *         else:
 *             nterm_aa_position = 0
 *         if is_term_mod(parsed_sequence[-1]):             # <<<<<<<<<<<<<<
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 2
 *         else:
*/
    if (unlikely(__pyx_v_parsed_sequence == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 643, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_List(__pyx_v_parsed_sequence, -1L, long, 1, __Pyx_PyLong_From_long, 1, 1, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_7))) __PYX_ERR(0, 643, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser_is_term_mod(((PyObject*)__pyx_t_7)); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (__pyx_t_1) {

      /* "pyteomics/cparser.pyx":644
 *             nterm_aa_position = 0
 *         if is_term_mod(parsed_sequence[-1]):
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 2             # <<<<<<<<<<<<<<
 *         else:
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 1
*/
      __pyx_t_11 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 644, __pyx_L1_error)
      __pyx_v_cterm_aa_position = (__pyx_t_11 - 2);

      /* "pyteomics/cparser.pyx":643
 *         else:
 *             nterm_aa_position = 0
 *         if is_term_mod(parsed_sequence[-1]):             # <<<<<<<<<<<<<<
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 2
 *         else:
*/
      goto __pyx_L10;
    }

    /* "pyteomics/cparser.pyx":646
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 2
 *         else:
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 1             # <<<<<<<<<<<<<<
 *         if PyList_Size(parsed_sequence) > 1:
 *             # temp = PyInt_FromLong(cterm_aa_position)
*/
    /*else*/ {
      __pyx_t_11 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 646, __pyx_L1_error)
      __pyx_v_cterm_aa_position = (__pyx_t_11 - 1);
    }
    __pyx_L10:;

    /* "pyteomics/cparser.pyx":647
 *         else:
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 1
 *         if PyList_Size(parsed_sequence) > 1:             # <<<<<<<<<<<<<<
 *             # temp = PyInt_FromLong(cterm_aa_position)
 *             # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
*/
    __pyx_t_11 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 647, __pyx_L1_error)
    __pyx_t_1 = (__pyx_t_11 > 1);
    if (__pyx_t_1) {

      /* "pyteomics/cparser.pyx":650
 *             # temp = PyInt_FromLong(cterm_aa_position)
 *             # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 *             temp = parsed_sequence.pop(cterm_aa_position)             # <<<<<<<<<<<<<<
 *             PyDict_SetItem(aa_dict, 'cterm' + temp, 1)
 *         # temp = PyInt_FromLong(nterm_aa_position)
*/
      if (unlikely(__pyx_v_parsed_sequence == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
        __PYX_ERR(0, 650, __pyx_L1_error)
      }
      __pyx_t_7 = __Pyx_PyList_PopIndex(__pyx_v_parsed_sequence, Py_None, __pyx_v_cterm_aa_position, 1, int, __Pyx_PyLong_From_int); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 650, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_v_temp = __pyx_t_7;
      __pyx_t_7 = 0;

      /* "pyteomics/cparser.pyx":651
 *             # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 *             temp = parsed_sequence.pop(cterm_aa_position)
 *             PyDict_SetItem(aa_dict, 'cterm' + temp, 1)             # <<<<<<<<<<<<<<
 *         # temp = PyInt_FromLong(nterm_aa_position)
 *         # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
*/
      __pyx_t_7 = PyNumber_Add(__pyx_mstate_global->__pyx_n_u_cterm, __pyx_v_temp); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_12 = PyDict_SetItem(__pyx_v_aa_dict, __pyx_t_7, __pyx_mstate_global->__pyx_int_1); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "pyteomics/cparser.pyx":647
 *         else:
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 1
 *         if PyList_Size(parsed_sequence) > 1:             # <<<<<<<<<<<<<<
 *             # temp = PyInt_FromLong(cterm_aa_position)
 *             # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
*/
    }

    /* "pyteomics/cparser.pyx":654
 *         # temp = PyInt_FromLong(nterm_aa_position)
 *         # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 *         temp = parsed_sequence.pop(nterm_aa_position)             # <<<<<<<<<<<<<<
 *         PyDict_SetItem(aa_dict, 'nterm' + temp, 1)
 *     # Process core amino acids.
*/
    if (unlikely(__pyx_v_parsed_sequence == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
      __PYX_ERR(0, 654, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_PyList_PopIndex(__pyx_v_parsed_sequence, Py_None, __pyx_v_nterm_aa_position, 1, int, __Pyx_PyLong_From_int); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 654, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_XDECREF_SET(__pyx_v_temp, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "pyteomics/cparser.pyx":655
 *         # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 *         temp = parsed_sequence.pop(nterm_aa_position)
 *         PyDict_SetItem(aa_dict, 'nterm' + temp, 1)             # <<<<<<<<<<<<<<
 *     # Process core amino acids.
 *     i = 0
*/
    __pyx_t_7 = PyNumber_Add(__pyx_mstate_global->__pyx_n_u_nterm, __pyx_v_temp); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_12 = PyDict_SetItem(__pyx_v_aa_dict, __pyx_t_7, __pyx_mstate_global->__pyx_int_1); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 655, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "pyteomics/cparser.pyx":638
 *     aa_dict = {}
 *     # Process terminal amino acids.
 *     if term_aa:             # <<<<<<<<<<<<<<
 *         if is_term_mod(<object>PyList_GetItem(parsed_sequence, 0)):
 *             nterm_aa_position = 1
*/
  }

  /* "pyteomics/cparser.pyx":657
 *         PyDict_SetItem(aa_dict, 'nterm' + temp, 1)
 *     # Process core amino acids.
 *     i = 0             # <<<<<<<<<<<<<<
 *     for i in range(PyList_Size(parsed_sequence)):
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)
*/
  __pyx_v_i = 0;

  /* "pyteomics/cparser.pyx":658
 *     # Process core amino acids.
 *     i = 0
 *     for i in range(PyList_Size(parsed_sequence)):             # <<<<<<<<<<<<<<
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *         pvalue = PyDict_GetItem(aa_dict, aa)
*/
  __pyx_t_11 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 658, __pyx_L1_error)
  __pyx_t_13 = __pyx_t_11;
  for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
    __pyx_v_i = __pyx_t_14;

    /* "pyteomics/cparser.pyx":659
 *     i = 0
 *     for i in range(PyList_Size(parsed_sequence)):
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)             # <<<<<<<<<<<<<<
 *         pvalue = PyDict_GetItem(aa_dict, aa)
 *         if pvalue == NULL:
*/
    __pyx_t_5 = PyList_GET_ITEM(__pyx_v_parsed_sequence, __pyx_v_i);
    __pyx_t_7 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_XDECREF_SET(__pyx_v_aa, ((PyObject*)__pyx_t_7));
    __pyx_t_7 = 0;

    /* "pyteomics/cparser.pyx":660
 *     for i in range(PyList_Size(parsed_sequence)):
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *         pvalue = PyDict_GetItem(aa_dict, aa)             # <<<<<<<<<<<<<<
 *         if pvalue == NULL:
 *             aa_count = 0
*/
    __pyx_v_pvalue = PyDict_GetItem(__pyx_v_aa_dict, __pyx_v_aa);

    /* "pyteomics/cparser.pyx":661
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *         pvalue = PyDict_GetItem(aa_dict, aa)
 *         if pvalue == NULL:             # <<<<<<<<<<<<<<
 *             aa_count = 0
 *         else:
*/
    __pyx_t_1 = (__pyx_v_pvalue == NULL);
    if (__pyx_t_1) {

      /* "pyteomics/cparser.pyx":662
 *         pvalue = PyDict_GetItem(aa_dict, aa)
 *         if pvalue == NULL:
 *             aa_count = 0             # <<<<<<<<<<<<<<
 *         else:
 *             aa_count = PyInt_AsLong(<object>pvalue)
*/
      __pyx_v_aa_count = 0;

      /* "pyteomics/cparser.pyx":661
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *         pvalue = PyDict_GetItem(aa_dict, aa)
 *         if pvalue == NULL:             # <<<<<<<<<<<<<<
 *             aa_count = 0
 *         else:
*/
      goto __pyx_L14;
    }

    /* "pyteomics/cparser.pyx":664
 *             aa_count = 0
 *         else:
 *             aa_count = PyInt_AsLong(<object>pvalue)             # <<<<<<<<<<<<<<
 *         aa_count += 1
 *         PyDict_SetItem(aa_dict, aa, aa_count)
*/
    /*else*/ {
      __pyx_v_aa_count = PyInt_AsLong(((PyObject *)__pyx_v_pvalue));
    }
    __pyx_L14:;

    /* "pyteomics/cparser.pyx":665
 *         else:
 *             aa_count = PyInt_AsLong(<object>pvalue)
 *         aa_count += 1             # <<<<<<<<<<<<<<
 *         PyDict_SetItem(aa_dict, aa, aa_count)
 * 
*/
    __pyx_v_aa_count = (__pyx_v_aa_count + 1);

    /* "pyteomics/cparser.pyx":666
 *             aa_count = PyInt_AsLong(<object>pvalue)
 *         aa_count += 1
 *         PyDict_SetItem(aa_dict, aa, aa_count)             # <<<<<<<<<<<<<<
 * 
 *     return aa_dict
*/
    __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_aa_count); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_12 = PyDict_SetItem(__pyx_v_aa_dict, __pyx_v_aa, __pyx_t_7); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }

  /* "pyteomics/cparser.pyx":668
 *         PyDict_SetItem(aa_dict, aa, aa_count)
 * 
 *     return aa_dict             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_aa_dict);
  __pyx_r = __pyx_v_aa_dict;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":565
 * 
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=0, bint term_aa=0,             # <<<<<<<<<<<<<<
 *                                   bint allow_unknown_modifications=0, object labels=None):
 *     """Calculate amino acid composition of a polypeptide.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("pyteomics.cparser.amino_acid_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_parsed_sequence);
  __Pyx_XDECREF(__pyx_v_aa_dict);
  __Pyx_XDECREF(__pyx_v_aa);
  __Pyx_XDECREF(__pyx_v__labels);
  __Pyx_XDECREF(__pyx_v_temp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_9amino_acid_composition(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_8amino_acid_composition, "amino_acid_composition(sequence, bool show_unmodified_termini=0, bool term_aa=0, bool allow_unknown_modifications=0, labels=None) -> dict\nCalculate amino acid composition of a polypeptide.\n\nParameters\n----------\nsequence : str or list\n    The sequence of a polypeptide or a list with a parsed sequence.\nshow_unmodified_termini : bool, optional\n    If :py:const:`True` then the unmodified N- and C-terminus are explicitly\n    shown in the returned dict. Default value is :py:const:`False`.\nterm_aa : bool, optional\n    If :py:const:`True` then the terminal amino acid residues are\n    artificially modified with `nterm` or `cterm` modification.\n    Default value is :py:const:`False`.\nallow_unknown_modifications : bool, optional\n    If :py:const:`True` then do not raise an exception when an unknown\n    modification of a known amino acid residue is found in the sequence.\n    Default value is :py:const:`False`.\nlabels : list, optional\n    A list of allowed labels for amino acids and terminal modifications.\n\nReturns\n-------\nout : dict\n    A dictionary of amino acid composition.\n\nExamples\n--------\n>>> amino_acid_composition('PEPTIDE') ==     {'I': 1, 'P': 2, 'E': 2, 'T': 1, 'D': 1}\nTrue\n>>> amino_acid_composition('PEPTDE', term_aa=True) ==     {'ctermE': 1, 'E': 1, 'D': 1, 'P': 1, 'T': 1, 'ntermP': 1}\nTrue\n>>> amino_acid_composition('PEPpTIDE', labels=std_labels+['pT']) ==     {'I': 1, 'P': 2, 'E': 2, 'D': 1, 'pT': 1}\nTrue");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_9amino_acid_composition = {"amino_acid_composition", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_9amino_acid_composition, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_8amino_acid_composition};
static PyObject *__pyx_pw_9pyteomics_7cparser_9amino_acid_composition(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  int __pyx_v_show_unmodified_termini;
  int __pyx_v_term_aa;
  int __pyx_v_allow_unknown_modifications;
  PyObject *__pyx_v_labels = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("amino_acid_composition (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,&__pyx_mstate_global->__pyx_n_u_show_unmodified_termini,&__pyx_mstate_global->__pyx_n_u_term_aa,&__pyx_mstate_global->__pyx_n_u_allow_unknown_modifications,&__pyx_mstate_global->__pyx_n_u_labels,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 565, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 565, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 565, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 565, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 565, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 565, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "amino_acid_composition", 0) < 0) __PYX_ERR(0, 565, __pyx_L3_error)

      /* "pyteomics/cparser.pyx":566
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=0, bint term_aa=0,
 *                                   bint allow_unknown_modifications=0, object labels=None):             # <<<<<<<<<<<<<<
 *     """Calculate amino acid composition of a polypeptide.
 * 
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("amino_acid_composition", 0, 1, 5, i); __PYX_ERR(0, 565, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 565, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 565, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 565, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 565, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 565, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_sequence = values[0];
    if (values[1]) {
      __pyx_v_show_unmodified_termini = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_show_unmodified_termini == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 565, __pyx_L3_error)
    } else {
      __pyx_v_show_unmodified_termini = ((int)0);
    }
    if (values[2]) {
      __pyx_v_term_aa = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_term_aa == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 565, __pyx_L3_error)
    } else {
      __pyx_v_term_aa = ((int)0);
    }
    if (values[3]) {
      __pyx_v_allow_unknown_modifications = __Pyx_PyObject_IsTrue(values[3]); if (unlikely((__pyx_v_allow_unknown_modifications == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 566, __pyx_L3_error)
    } else {
      __pyx_v_allow_unknown_modifications = ((int)0);
    }
    __pyx_v_labels = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("amino_acid_composition", 0, 1, 5, __pyx_nargs); __PYX_ERR(0, 565, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.amino_acid_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_8amino_acid_composition(__pyx_self, __pyx_v_sequence, __pyx_v_show_unmodified_termini, __pyx_v_term_aa, __pyx_v_allow_unknown_modifications, __pyx_v_labels);

  /* "pyteomics/cparser.pyx":565
 * 
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=0, bint term_aa=0,             # <<<<<<<<<<<<<<
 *                                   bint allow_unknown_modifications=0, object labels=None):
 *     """Calculate amino acid composition of a polypeptide.
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_8amino_acid_composition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_term_aa, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("amino_acid_composition", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 4;
  __pyx_t_2.show_unmodified_termini = __pyx_v_show_unmodified_termini;
  __pyx_t_2.term_aa = __pyx_v_term_aa;
  __pyx_t_2.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
  __pyx_t_2.labels = __pyx_v_labels;
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_amino_acid_composition(__pyx_v_sequence, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.amino_acid_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":683
 * 
 * 
 * cdef inline list copy_list(list source):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
*/

static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser_copy_list(PyObject *__pyx_v_source) {
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_v_val = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_list", 0);

  /* "pyteomics/cparser.pyx":688
 *         list result
 *         object val
 *     n = PyList_Size(source)             # <<<<<<<<<<<<<<
 *     result = PyList_New(n)
 *     for i in range(n):
*/
  __pyx_t_1 = PyList_Size(__pyx_v_source); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 688, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "pyteomics/cparser.pyx":689
 *         object val
 *     n = PyList_Size(source)
 *     result = PyList_New(n)             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         val = <object>PyList_GET_ITEM(source, i)
*/
  __pyx_t_2 = PyList_New(__pyx_v_n); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_result = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":690
 *     n = PyList_Size(source)
 *     result = PyList_New(n)
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         val = <object>PyList_GET_ITEM(source, i)
 *         Py_INCREF(val)
*/
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_i = __pyx_t_5;

    /* "pyteomics/cparser.pyx":691
 *     result = PyList_New(n)
 *     for i in range(n):
 *         val = <object>PyList_GET_ITEM(source, i)             # <<<<<<<<<<<<<<
 *         Py_INCREF(val)
 *         PyList_SET_ITEM(result, i, val)
*/
    __pyx_t_6 = PyList_GET_ITEM(__pyx_v_source, __pyx_v_i);
    __pyx_t_2 = ((PyObject *)__pyx_t_6);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":692
 *     for i in range(n):
 *         val = <object>PyList_GET_ITEM(source, i)
 *         Py_INCREF(val)             # <<<<<<<<<<<<<<
 *         PyList_SET_ITEM(result, i, val)
 *     return result
*/
    Py_INCREF(__pyx_v_val);

    /* "pyteomics/cparser.pyx":693
 *         val = <object>PyList_GET_ITEM(source, i)
 *         Py_INCREF(val)
 *         PyList_SET_ITEM(result, i, val)             # <<<<<<<<<<<<<<
 *     return result
 * 
*/
    PyList_SET_ITEM(__pyx_v_result, __pyx_v_i, __pyx_v_val);
  }

  /* "pyteomics/cparser.pyx":694
 *         Py_INCREF(val)
 *         PyList_SET_ITEM(result, i, val)
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":683
 * 
 * 
 * cdef inline list copy_list(list source):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.copy_list", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":706
 *         public str terminal
 * 
 *     def __init__(self, amino_acid, modification=None, terminal=None, position_type=None):             # <<<<<<<<<<<<<<
 *         if position_type is None:
 *             position_type = PositionType.internal
*/

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_amino_acid = 0;
  PyObject *__pyx_v_modification = 0;
  PyObject *__pyx_v_terminal = 0;
  PyObject *__pyx_v_position_type = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_amino_acid,&__pyx_mstate_global->__pyx_n_u_modification,&__pyx_mstate_global->__pyx_n_u_terminal,&__pyx_mstate_global->__pyx_n_u_position_type,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 706, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 706, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 706, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 706, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 706, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < 0) __PYX_ERR(0, 706, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 4, i); __PYX_ERR(0, 706, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 706, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 706, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 706, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 706, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_amino_acid = values[0];
    __pyx_v_modification = values[1];
    __pyx_v_terminal = values[2];
    __pyx_v_position_type = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 4, __pyx_nargs); __PYX_ERR(0, 706, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition___init__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), __pyx_v_amino_acid, __pyx_v_modification, __pyx_v_terminal, __pyx_v_position_type);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16SequencePosition___init__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_amino_acid, PyObject *__pyx_v_modification, PyObject *__pyx_v_terminal, PyObject *__pyx_v_position_type) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_position_type);

  /* "pyteomics/cparser.pyx":707
 * 
 *     def __init__(self, amino_acid, modification=None, terminal=None, position_type=None):
 *         if position_type is None:             # <<<<<<<<<<<<<<
 *             position_type = PositionType.internal
 *         self.amino_acid = amino_acid
*/
  __pyx_t_1 = (__pyx_v_position_type == Py_None);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":708
 *     def __init__(self, amino_acid, modification=None, terminal=None, position_type=None):
 *         if position_type is None:
 *             position_type = PositionType.internal             # <<<<<<<<<<<<<<
 *         self.amino_acid = amino_acid
 *         self.modification = modification
*/
    __pyx_t_2 = __Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(__pyx_e_9pyteomics_7cparser_internal); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_position_type, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":707
 * 
 *     def __init__(self, amino_acid, modification=None, terminal=None, position_type=None):
 *         if position_type is None:             # <<<<<<<<<<<<<<
 *             position_type = PositionType.internal
 *         self.amino_acid = amino_acid
*/
  }

  /* "pyteomics/cparser.pyx":709
 *         if position_type is None:
 *             position_type = PositionType.internal
 *         self.amino_acid = amino_acid             # <<<<<<<<<<<<<<
 *         self.modification = modification
 *         self.terminal = terminal
*/
  __pyx_t_2 = __pyx_v_amino_acid;
  __Pyx_INCREF(__pyx_t_2);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 709, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->amino_acid);
  __Pyx_DECREF(__pyx_v_self->amino_acid);
  __pyx_v_self->amino_acid = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":710
 *             position_type = PositionType.internal
 *         self.amino_acid = amino_acid
 *         self.modification = modification             # <<<<<<<<<<<<<<
 *         self.terminal = terminal
 *         self.position_type = position_type
*/
  __pyx_t_2 = __pyx_v_modification;
  __Pyx_INCREF(__pyx_t_2);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->modification);
  __Pyx_DECREF(__pyx_v_self->modification);
  __pyx_v_self->modification = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":711
 *         self.amino_acid = amino_acid
 *         self.modification = modification
 *         self.terminal = terminal             # <<<<<<<<<<<<<<
 *         self.position_type = position_type
 * 
*/
  __pyx_t_2 = __pyx_v_terminal;
  __Pyx_INCREF(__pyx_t_2);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->terminal);
  __Pyx_DECREF(__pyx_v_self->terminal);
  __pyx_v_self->terminal = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":712
 *         self.modification = modification
 *         self.terminal = terminal
 *         self.position_type = position_type             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
*/
  __pyx_t_3 = ((enum __pyx_t_9pyteomics_7cparser_PositionType)__Pyx_PyLong_As_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_v_position_type)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 712, __pyx_L1_error)
  __pyx_v_self->position_type = __pyx_t_3;

  /* "pyteomics/cparser.pyx":706
 *         public str terminal
 * 
 *     def __init__(self, amino_acid, modification=None, terminal=None, position_type=None):             # <<<<<<<<<<<<<<
 *         if position_type is None:
 *             position_type = PositionType.internal
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_position_type);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":714
 *         self.position_type = position_type
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef SequencePosition _create(str amino_acid, str modification, str terminal, PositionType position_type):
 *         cdef SequencePosition self = SequencePosition.__new__(SequencePosition)
*/

static struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_f_9pyteomics_7cparser_16SequencePosition__create(PyObject *__pyx_v_amino_acid, PyObject *__pyx_v_modification, PyObject *__pyx_v_terminal, enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_v_position_type) {
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self = 0;
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create", 0);

  /* "pyteomics/cparser.pyx":716
 *     @staticmethod
 *     cdef SequencePosition _create(str amino_acid, str modification, str terminal, PositionType position_type):
 *         cdef SequencePosition self = SequencePosition.__new__(SequencePosition)             # <<<<<<<<<<<<<<
 *         self.amino_acid = amino_acid
 *         self.modification = modification
*/
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_9pyteomics_7cparser_SequencePosition(((PyTypeObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition), __pyx_mstate_global->__pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF((PyObject *)__pyx_t_1);
  __pyx_v_self = ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":717
 *     cdef SequencePosition _create(str amino_acid, str modification, str terminal, PositionType position_type):
 *         cdef SequencePosition self = SequencePosition.__new__(SequencePosition)
 *         self.amino_acid = amino_acid             # <<<<<<<<<<<<<<
 *         self.modification = modification
 *         self.terminal = terminal
*/
  __Pyx_INCREF(__pyx_v_amino_acid);
  __Pyx_GIVEREF(__pyx_v_amino_acid);
  __Pyx_GOTREF(__pyx_v_self->amino_acid);
  __Pyx_DECREF(__pyx_v_self->amino_acid);
  __pyx_v_self->amino_acid = __pyx_v_amino_acid;

  /* "pyteomics/cparser.pyx":718
 *         cdef SequencePosition self = SequencePosition.__new__(SequencePosition)
 *         self.amino_acid = amino_acid
 *         self.modification = modification             # <<<<<<<<<<<<<<
 *         self.terminal = terminal
 *         self.position_type = position_type
*/
  __Pyx_INCREF(__pyx_v_modification);
  __Pyx_GIVEREF(__pyx_v_modification);
  __Pyx_GOTREF(__pyx_v_self->modification);
  __Pyx_DECREF(__pyx_v_self->modification);
  __pyx_v_self->modification = __pyx_v_modification;

  /* "pyteomics/cparser.pyx":719
 *         self.amino_acid = amino_acid
 *         self.modification = modification
 *         self.terminal = terminal             # <<<<<<<<<<<<<<
 *         self.position_type = position_type
 *         return self
*/
  __Pyx_INCREF(__pyx_v_terminal);
  __Pyx_GIVEREF(__pyx_v_terminal);
  __Pyx_GOTREF(__pyx_v_self->terminal);
  __Pyx_DECREF(__pyx_v_self->terminal);
  __pyx_v_self->terminal = __pyx_v_terminal;

  /* "pyteomics/cparser.pyx":720
 *         self.modification = modification
 *         self.terminal = terminal
 *         self.position_type = position_type             # <<<<<<<<<<<<<<
 *         return self
 * 
*/
  __pyx_v_self->position_type = __pyx_v_position_type;

  /* "pyteomics/cparser.pyx":721
 *         self.terminal = terminal
 *         self.position_type = position_type
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, i):
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = __pyx_v_self;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":714
 *         self.position_type = position_type
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef SequencePosition _create(str amino_acid, str modification, str terminal, PositionType position_type):
 *         cdef SequencePosition self = SequencePosition.__new__(SequencePosition)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition._create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_self);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":723
 *         return self
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         if isinstance(i, slice):
 *             return self.as_tuple()[i]
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_2__getitem__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_i));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_2__getitem__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_i) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  long __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "pyteomics/cparser.pyx":724
 * 
 *     def __getitem__(self, i):
 *         if isinstance(i, slice):             # <<<<<<<<<<<<<<
 *             return self.as_tuple()[i]
 *         return self.get_index(i)
*/
  __pyx_t_1 = PySlice_Check(__pyx_v_i); 
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":725
 *     def __getitem__(self, i):
 *         if isinstance(i, slice):
 *             return self.as_tuple()[i]             # <<<<<<<<<<<<<<
 *         return self.get_index(i)
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_9pyteomics_7cparser_16SequencePosition_as_tuple(__pyx_v_self, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 725, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(__pyx_t_2 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 725, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 725, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":724
 * 
 *     def __getitem__(self, i):
 *         if isinstance(i, slice):             # <<<<<<<<<<<<<<
 *             return self.as_tuple()[i]
 *         return self.get_index(i)
*/
  }

  /* "pyteomics/cparser.pyx":726
 *         if isinstance(i, slice):
 *             return self.as_tuple()[i]
 *         return self.get_index(i)             # <<<<<<<<<<<<<<
 * 
 *     cpdef str get_index(self, long i):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyLong_As_long(__pyx_v_i); if (unlikely((__pyx_t_4 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 726, __pyx_L1_error)
  __pyx_t_3 = __pyx_f_9pyteomics_7cparser_16SequencePosition_get_index(__pyx_v_self, __pyx_t_4, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 726, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":723
 *         return self
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         if isinstance(i, slice):
 *             return self.as_tuple()[i]
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":728
 *         return self.get_index(i)
 * 
 *     cpdef str get_index(self, long i):             # <<<<<<<<<<<<<<
 *         cdef:
 *             long size, n_mods
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_5get_index(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16SequencePosition_get_index(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, long __pyx_v_i, CYTHON_UNUSED int __pyx_skip_dispatch) {
  long __pyx_v_size;
  long __pyx_v_n_mods;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_index", 0);

  /* "pyteomics/cparser.pyx":731
 *         cdef:
 *             long size, n_mods
 *         size = self.get_size()             # <<<<<<<<<<<<<<
 *         n_mods = self.modification is not None
 *         if i >= size:
*/
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition_get_size(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 731, __pyx_L1_error)
  __pyx_v_size = __pyx_t_1;

  /* "pyteomics/cparser.pyx":732
 *             long size, n_mods
 *         size = self.get_size()
 *         n_mods = self.modification is not None             # <<<<<<<<<<<<<<
 *         if i >= size:
 *             raise IndexError(i)
*/
  __pyx_t_2 = (__pyx_v_self->modification != ((PyObject*)Py_None));
  __pyx_v_n_mods = __pyx_t_2;

  /* "pyteomics/cparser.pyx":733
 *         size = self.get_size()
 *         n_mods = self.modification is not None
 *         if i >= size:             # <<<<<<<<<<<<<<
 *             raise IndexError(i)
 *         elif i < 0:
*/
  __pyx_t_2 = (__pyx_v_i >= __pyx_v_size);
  if (unlikely(__pyx_t_2)) {

    /* "pyteomics/cparser.pyx":734
 *         n_mods = self.modification is not None
 *         if i >= size:
 *             raise IndexError(i)             # <<<<<<<<<<<<<<
 *         elif i < 0:
 *             if size + i < 0:
*/
    __pyx_t_4 = NULL;
    __Pyx_INCREF(__pyx_builtin_IndexError);
    __pyx_t_5 = __pyx_builtin_IndexError; 
    __pyx_t_6 = __Pyx_PyLong_From_long(__pyx_v_i); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 734, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_6};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_1, (2-__pyx_t_1) | (__pyx_t_1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 734, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 734, __pyx_L1_error)

    /* "pyteomics/cparser.pyx":733
 *         size = self.get_size()
 *         n_mods = self.modification is not None
 *         if i >= size:             # <<<<<<<<<<<<<<
 *             raise IndexError(i)
 *         elif i < 0:
*/
  }

  /* "pyteomics/cparser.pyx":735
 *         if i >= size:
 *             raise IndexError(i)
 *         elif i < 0:             # <<<<<<<<<<<<<<
 *             if size + i < 0:
 *                 raise IndexError(i)
*/
  __pyx_t_2 = (__pyx_v_i < 0);
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":736
 *             raise IndexError(i)
 *         elif i < 0:
 *             if size + i < 0:             # <<<<<<<<<<<<<<
 *                 raise IndexError(i)
 *             i = size + i
*/
    __pyx_t_2 = ((__pyx_v_size + __pyx_v_i) < 0);
    if (unlikely(__pyx_t_2)) {

      /* "pyteomics/cparser.pyx":737
 *         elif i < 0:
 *             if size + i < 0:
 *                 raise IndexError(i)             # <<<<<<<<<<<<<<
 *             i = size + i
 *         if self.position_type == PositionType.n_term:
*/
      __pyx_t_5 = NULL;
      __Pyx_INCREF(__pyx_builtin_IndexError);
      __pyx_t_6 = __pyx_builtin_IndexError; 
      __pyx_t_4 = __Pyx_PyLong_From_long(__pyx_v_i); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 737, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+__pyx_t_1, (2-__pyx_t_1) | (__pyx_t_1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 737, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 737, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":736
 *             raise IndexError(i)
 *         elif i < 0:
 *             if size + i < 0:             # <<<<<<<<<<<<<<
 *                 raise IndexError(i)
 *             i = size + i
*/
    }

    /* "pyteomics/cparser.pyx":738
 *             if size + i < 0:
 *                 raise IndexError(i)
 *             i = size + i             # <<<<<<<<<<<<<<
 *         if self.position_type == PositionType.n_term:
 *             if i == 0:
*/
    __pyx_v_i = (__pyx_v_size + __pyx_v_i);

    /* "pyteomics/cparser.pyx":735
 *         if i >= size:
 *             raise IndexError(i)
 *         elif i < 0:             # <<<<<<<<<<<<<<
 *             if size + i < 0:
 *                 raise IndexError(i)
*/
  }

  /* "pyteomics/cparser.pyx":739
 *                 raise IndexError(i)
 *             i = size + i
 *         if self.position_type == PositionType.n_term:             # <<<<<<<<<<<<<<
 *             if i == 0:
 *                 return self.terminal
*/
  switch (__pyx_v_self->position_type) {
    case __pyx_e_9pyteomics_7cparser_n_term:

    /* "pyteomics/cparser.pyx":740
 *             i = size + i
 *         if self.position_type == PositionType.n_term:
 *             if i == 0:             # <<<<<<<<<<<<<<
 *                 return self.terminal
 *             elif (i - 1) == n_mods:
*/
    __pyx_t_2 = (__pyx_v_i == 0);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":741
 *         if self.position_type == PositionType.n_term:
 *             if i == 0:
 *                 return self.terminal             # <<<<<<<<<<<<<<
 *             elif (i - 1) == n_mods:
 *                 return self.amino_acid
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_self->terminal);
      __pyx_r = __pyx_v_self->terminal;
      goto __pyx_L0;

      /* "pyteomics/cparser.pyx":740
 *             i = size + i
 *         if self.position_type == PositionType.n_term:
 *             if i == 0:             # <<<<<<<<<<<<<<
 *                 return self.terminal
 *             elif (i - 1) == n_mods:
*/
    }

    /* "pyteomics/cparser.pyx":742
 *             if i == 0:
 *                 return self.terminal
 *             elif (i - 1) == n_mods:             # <<<<<<<<<<<<<<
 *                 return self.amino_acid
 *             else:
*/
    __pyx_t_2 = ((__pyx_v_i - 1) == __pyx_v_n_mods);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":743
 *                 return self.terminal
 *             elif (i - 1) == n_mods:
 *                 return self.amino_acid             # <<<<<<<<<<<<<<
 *             else:
 *                 return self.modification
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_self->amino_acid);
      __pyx_r = __pyx_v_self->amino_acid;
      goto __pyx_L0;

      /* "pyteomics/cparser.pyx":742
 *             if i == 0:
 *                 return self.terminal
 *             elif (i - 1) == n_mods:             # <<<<<<<<<<<<<<
 *                 return self.amino_acid
 *             else:
*/
    }

    /* "pyteomics/cparser.pyx":745
 *                 return self.amino_acid
 *             else:
 *                 return self.modification             # <<<<<<<<<<<<<<
 *         elif self.position_type == PositionType.internal:
 *             if i == n_mods:
*/
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_self->modification);
      __pyx_r = __pyx_v_self->modification;
      goto __pyx_L0;
    }

    /* "pyteomics/cparser.pyx":739
 *                 raise IndexError(i)
 *             i = size + i
 *         if self.position_type == PositionType.n_term:             # <<<<<<<<<<<<<<
 *             if i == 0:
 *                 return self.terminal
*/
    break;
    case __pyx_e_9pyteomics_7cparser_internal:

    /* "pyteomics/cparser.pyx":747
 *                 return self.modification
 *         elif self.position_type == PositionType.internal:
 *             if i == n_mods:             # <<<<<<<<<<<<<<
 *                 return self.amino_acid
 *             else:
*/
    __pyx_t_2 = (__pyx_v_i == __pyx_v_n_mods);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":748
 *         elif self.position_type == PositionType.internal:
 *             if i == n_mods:
 *                 return self.amino_acid             # <<<<<<<<<<<<<<
 *             else:
 *                 return self.modification
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_self->amino_acid);
      __pyx_r = __pyx_v_self->amino_acid;
      goto __pyx_L0;

      /* "pyteomics/cparser.pyx":747
 *                 return self.modification
 *         elif self.position_type == PositionType.internal:
 *             if i == n_mods:             # <<<<<<<<<<<<<<
 *                 return self.amino_acid
 *             else:
*/
    }

    /* "pyteomics/cparser.pyx":750
 *                 return self.amino_acid
 *             else:
 *                 return self.modification             # <<<<<<<<<<<<<<
 *         else:
 *             if i - n_mods == 0:
*/
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_self->modification);
      __pyx_r = __pyx_v_self->modification;
      goto __pyx_L0;
    }

    /* "pyteomics/cparser.pyx":746
 *             else:
 *                 return self.modification
 *         elif self.position_type == PositionType.internal:             # <<<<<<<<<<<<<<
 *             if i == n_mods:
 *                 return self.amino_acid
*/
    break;
    default:

    /* "pyteomics/cparser.pyx":752
 *                 return self.modification
 *         else:
 *             if i - n_mods == 0:             # <<<<<<<<<<<<<<
 *                 return self.amino_acid
 *             elif i - n_mods == 1:
*/
    __pyx_t_2 = ((__pyx_v_i - __pyx_v_n_mods) == 0);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":753
 *         else:
 *             if i - n_mods == 0:
 *                 return self.amino_acid             # <<<<<<<<<<<<<<
 *             elif i - n_mods == 1:
 *                 return self.terminal
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_self->amino_acid);
      __pyx_r = __pyx_v_self->amino_acid;
      goto __pyx_L0;

      /* "pyteomics/cparser.pyx":752
 *                 return self.modification
 *         else:
 *             if i - n_mods == 0:             # <<<<<<<<<<<<<<
 *                 return self.amino_acid
 *             elif i - n_mods == 1:
*/
    }

    /* "pyteomics/cparser.pyx":754
 *             if i - n_mods == 0:
 *                 return self.amino_acid
 *             elif i - n_mods == 1:             # <<<<<<<<<<<<<<
 *                 return self.terminal
 *             else:
*/
    __pyx_t_2 = ((__pyx_v_i - __pyx_v_n_mods) == 1);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":755
 *                 return self.amino_acid
 *             elif i - n_mods == 1:
 *                 return self.terminal             # <<<<<<<<<<<<<<
 *             else:
 *                 return self.modification
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_self->terminal);
      __pyx_r = __pyx_v_self->terminal;
      goto __pyx_L0;

      /* "pyteomics/cparser.pyx":754
 *             if i - n_mods == 0:
 *                 return self.amino_acid
 *             elif i - n_mods == 1:             # <<<<<<<<<<<<<<
 *                 return self.terminal
 *             else:
*/
    }

    /* "pyteomics/cparser.pyx":757
 *                 return self.terminal
 *             else:
 *                 return self.modification             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t get_size(self):
*/
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_self->modification);
      __pyx_r = __pyx_v_self->modification;
      goto __pyx_L0;
    }
    break;
  }

  /* "pyteomics/cparser.pyx":728
 *         return self.get_index(i)
 * 
 *     cpdef str get_index(self, long i):             # <<<<<<<<<<<<<<
 *         cdef:
 *             long size, n_mods
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.get_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_5get_index(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16SequencePosition_4get_index, "SequencePosition.get_index(self, long i) -> str");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16SequencePosition_5get_index = {"get_index", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_5get_index, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_4get_index};
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_5get_index(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  long __pyx_v_i;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_index (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_i,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 728, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 728, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_index", 0) < 0) __PYX_ERR(0, 728, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_index", 1, 1, 1, i); __PYX_ERR(0, 728, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 728, __pyx_L3_error)
    }
    __pyx_v_i = __Pyx_PyLong_As_long(values[0]); if (unlikely((__pyx_v_i == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 728, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_index", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 728, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.get_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_4get_index(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), __pyx_v_i);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_4get_index(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, long __pyx_v_i) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_index", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition_get_index(__pyx_v_self, __pyx_v_i, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.get_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":759
 *                 return self.modification
 * 
 *     cdef size_t get_size(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t n_mods
*/

static size_t __pyx_f_9pyteomics_7cparser_16SequencePosition_get_size(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  size_t __pyx_v_n_mods;
  size_t __pyx_r;
  int __pyx_t_1;

  /* "pyteomics/cparser.pyx":763
 *             size_t n_mods
 * 
 *         n_mods = self.modification is not None             # <<<<<<<<<<<<<<
 *         if self.position_type == PositionType.internal:
 *             return n_mods + 1
*/
  __pyx_t_1 = (__pyx_v_self->modification != ((PyObject*)Py_None));
  __pyx_v_n_mods = __pyx_t_1;

  /* "pyteomics/cparser.pyx":764
 * 
 *         n_mods = self.modification is not None
 *         if self.position_type == PositionType.internal:             # <<<<<<<<<<<<<<
 *             return n_mods + 1
 *         else:
*/
  __pyx_t_1 = (__pyx_v_self->position_type == __pyx_e_9pyteomics_7cparser_internal);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":765
 *         n_mods = self.modification is not None
 *         if self.position_type == PositionType.internal:
 *             return n_mods + 1             # <<<<<<<<<<<<<<
 *         else:
 *             return n_mods + 2
*/
    __pyx_r = (__pyx_v_n_mods + 1);
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":764
 * 
 *         n_mods = self.modification is not None
 *         if self.position_type == PositionType.internal:             # <<<<<<<<<<<<<<
 *             return n_mods + 1
 *         else:
*/
  }

  /* "pyteomics/cparser.pyx":767
 *             return n_mods + 1
 *         else:
 *             return n_mods + 2             # <<<<<<<<<<<<<<
 * 
 *     cdef bint is_modified(self):
*/
  /*else*/ {
    __pyx_r = (__pyx_v_n_mods + 2);
    goto __pyx_L0;
  }

  /* "pyteomics/cparser.pyx":759
 *                 return self.modification
 * 
 *     cdef size_t get_size(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t n_mods
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":769
 *             return n_mods + 2
 * 
 *     cdef bint is_modified(self):             # <<<<<<<<<<<<<<
 *         return self.modification is not None
 * 
*/

static int __pyx_f_9pyteomics_7cparser_16SequencePosition_is_modified(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  int __pyx_r;
  int __pyx_t_1;

  /* "pyteomics/cparser.pyx":770
 * 
 *     cdef bint is_modified(self):
 *         return self.modification is not None             # <<<<<<<<<<<<<<
 * 
 *     cpdef add_modification(self, str mod):
*/
  __pyx_t_1 = (__pyx_v_self->modification != ((PyObject*)Py_None));
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":769
 *             return n_mods + 2
 * 
 *     cdef bint is_modified(self):             # <<<<<<<<<<<<<<
 *         return self.modification is not None
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":772
 *         return self.modification is not None
 * 
 *     cpdef add_modification(self, str mod):             # <<<<<<<<<<<<<<
 *         self.modification = mod
 * 
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_7add_modification(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16SequencePosition_add_modification(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_mod, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_modification", 0);

  /* "pyteomics/cparser.pyx":773
 * 
 *     cpdef add_modification(self, str mod):
 *         self.modification = mod             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __Pyx_INCREF(__pyx_v_mod);
  __Pyx_GIVEREF(__pyx_v_mod);
  __Pyx_GOTREF(__pyx_v_self->modification);
  __Pyx_DECREF(__pyx_v_self->modification);
  __pyx_v_self->modification = __pyx_v_mod;

  /* "pyteomics/cparser.pyx":772
 *         return self.modification is not None
 * 
 *     cpdef add_modification(self, str mod):             # <<<<<<<<<<<<<<
 *         self.modification = mod
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_7add_modification(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16SequencePosition_6add_modification, "SequencePosition.add_modification(self, str mod)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16SequencePosition_7add_modification = {"add_modification", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_7add_modification, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_6add_modification};
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_7add_modification(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_mod = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_modification (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_mod,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 772, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 772, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "add_modification", 0) < 0) __PYX_ERR(0, 772, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("add_modification", 1, 1, 1, i); __PYX_ERR(0, 772, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 772, __pyx_L3_error)
    }
    __pyx_v_mod = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_modification", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 772, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.add_modification", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_mod), (&PyUnicode_Type), 1, "mod", 1))) __PYX_ERR(0, 772, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_6add_modification(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), __pyx_v_mod);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_6add_modification(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_mod) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_modification", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition_add_modification(__pyx_v_self, __pyx_v_mod, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 772, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.add_modification", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":775
 *         self.modification = mod
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return str(self.as_tuple())
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_9__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_9__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_8__repr__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_8__repr__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "pyteomics/cparser.pyx":776
 * 
 *     def __repr__(self):
 *         return str(self.as_tuple())             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition_as_tuple(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Unicode(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":775
 *         self.modification = mod
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return str(self.as_tuple())
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":778
 *         return str(self.as_tuple())
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self.get_size()
 * 
*/

/* Python wrapper */
static Py_ssize_t __pyx_pw_9pyteomics_7cparser_16SequencePosition_11__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_9pyteomics_7cparser_16SequencePosition_11__len__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_10__len__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_9pyteomics_7cparser_16SequencePosition_10__len__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  size_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "pyteomics/cparser.pyx":779
 * 
 *     def __len__(self):
 *         return self.get_size()             # <<<<<<<<<<<<<<
 * 
 *     cpdef tuple as_tuple(self):
*/
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition_get_size(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 779, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":778
 *         return str(self.as_tuple())
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self.get_size()
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":781
 *         return self.get_size()
 * 
 *     cpdef tuple as_tuple(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t size, i, n_mods, j
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_13as_tuple(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16SequencePosition_as_tuple(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_v_size;
  size_t __pyx_v_i;
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_v_mod = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("as_tuple", 0);

  /* "pyteomics/cparser.pyx":786
 *             tuple result
 *             str mod
 *         size = self.get_size()             # <<<<<<<<<<<<<<
 *         result = PyTuple_New(size)
 *         i = 0
*/
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition_get_size(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 786, __pyx_L1_error)
  __pyx_v_size = __pyx_t_1;

  /* "pyteomics/cparser.pyx":787
 *             str mod
 *         size = self.get_size()
 *         result = PyTuple_New(size)             # <<<<<<<<<<<<<<
 *         i = 0
 *         if self.position_type == PositionType.n_term:
*/
  __pyx_t_2 = PyTuple_New(__pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_result = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":788
 *         size = self.get_size()
 *         result = PyTuple_New(size)
 *         i = 0             # <<<<<<<<<<<<<<
 *         if self.position_type == PositionType.n_term:
 *             Py_INCREF(self.terminal)
*/
  __pyx_v_i = 0;

  /* "pyteomics/cparser.pyx":789
 *         result = PyTuple_New(size)
 *         i = 0
 *         if self.position_type == PositionType.n_term:             # <<<<<<<<<<<<<<
 *             Py_INCREF(self.terminal)
 *             PyTuple_SET_ITEM(result, i, self.terminal)
*/
  __pyx_t_3 = (__pyx_v_self->position_type == __pyx_e_9pyteomics_7cparser_n_term);
  if (__pyx_t_3) {

    /* "pyteomics/cparser.pyx":790
 *         i = 0
 *         if self.position_type == PositionType.n_term:
 *             Py_INCREF(self.terminal)             # <<<<<<<<<<<<<<
 *             PyTuple_SET_ITEM(result, i, self.terminal)
 *             i += 1
*/
    __pyx_t_2 = __pyx_v_self->terminal;
    __Pyx_INCREF(__pyx_t_2);
    Py_INCREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":791
 *         if self.position_type == PositionType.n_term:
 *             Py_INCREF(self.terminal)
 *             PyTuple_SET_ITEM(result, i, self.terminal)             # <<<<<<<<<<<<<<
 *             i += 1
 *         if self.is_modified():
*/
    __pyx_t_2 = __pyx_v_self->terminal;
    __Pyx_INCREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_v_result, __pyx_v_i, __pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":792
 *             Py_INCREF(self.terminal)
 *             PyTuple_SET_ITEM(result, i, self.terminal)
 *             i += 1             # <<<<<<<<<<<<<<
 *         if self.is_modified():
 *             mod = self.modification
*/
    __pyx_v_i = (__pyx_v_i + 1);

    /* "pyteomics/cparser.pyx":789
 *         result = PyTuple_New(size)
 *         i = 0
 *         if self.position_type == PositionType.n_term:             # <<<<<<<<<<<<<<
 *             Py_INCREF(self.terminal)
 *             PyTuple_SET_ITEM(result, i, self.terminal)
*/
  }

  /* "pyteomics/cparser.pyx":793
 *             PyTuple_SET_ITEM(result, i, self.terminal)
 *             i += 1
 *         if self.is_modified():             # <<<<<<<<<<<<<<
 *             mod = self.modification
 *             Py_INCREF(mod)
*/
  __pyx_t_3 = __pyx_f_9pyteomics_7cparser_16SequencePosition_is_modified(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 793, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "pyteomics/cparser.pyx":794
 *             i += 1
 *         if self.is_modified():
 *             mod = self.modification             # <<<<<<<<<<<<<<
 *             Py_INCREF(mod)
 *             PyTuple_SET_ITEM(result, i, mod)
*/
    __pyx_t_2 = __pyx_v_self->modification;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_mod = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":795
 *         if self.is_modified():
 *             mod = self.modification
 *             Py_INCREF(mod)             # <<<<<<<<<<<<<<
 *             PyTuple_SET_ITEM(result, i, mod)
 *             i += 1
*/
    Py_INCREF(__pyx_v_mod);

    /* "pyteomics/cparser.pyx":796
 *             mod = self.modification
 *             Py_INCREF(mod)
 *             PyTuple_SET_ITEM(result, i, mod)             # <<<<<<<<<<<<<<
 *             i += 1
 *         Py_INCREF(self.amino_acid)
*/
    PyTuple_SET_ITEM(__pyx_v_result, __pyx_v_i, __pyx_v_mod);

    /* "pyteomics/cparser.pyx":797
 *             Py_INCREF(mod)
 *             PyTuple_SET_ITEM(result, i, mod)
 *             i += 1             # <<<<<<<<<<<<<<
 *         Py_INCREF(self.amino_acid)
 *         PyTuple_SET_ITEM(result, i, self.amino_acid)
*/
    __pyx_v_i = (__pyx_v_i + 1);

    /* "pyteomics/cparser.pyx":793
 *             PyTuple_SET_ITEM(result, i, self.terminal)
 *             i += 1
 *         if self.is_modified():             # <<<<<<<<<<<<<<
 *             mod = self.modification
 *             Py_INCREF(mod)
*/
  }

  /* "pyteomics/cparser.pyx":798
 *             PyTuple_SET_ITEM(result, i, mod)
 *             i += 1
 *         Py_INCREF(self.amino_acid)             # <<<<<<<<<<<<<<
 *         PyTuple_SET_ITEM(result, i, self.amino_acid)
 *         i += 1
*/
  __pyx_t_2 = __pyx_v_self->amino_acid;
  __Pyx_INCREF(__pyx_t_2);
  Py_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":799
 *             i += 1
 *         Py_INCREF(self.amino_acid)
 *         PyTuple_SET_ITEM(result, i, self.amino_acid)             # <<<<<<<<<<<<<<
 *         i += 1
 *         if self.position_type == PositionType.c_term:
*/
  __pyx_t_2 = __pyx_v_self->amino_acid;
  __Pyx_INCREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_v_result, __pyx_v_i, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":800
 *         Py_INCREF(self.amino_acid)
 *         PyTuple_SET_ITEM(result, i, self.amino_acid)
 *         i += 1             # <<<<<<<<<<<<<<
 *         if self.position_type == PositionType.c_term:
 *             Py_INCREF(self.terminal)
*/
  __pyx_v_i = (__pyx_v_i + 1);

  /* "pyteomics/cparser.pyx":801
 *         PyTuple_SET_ITEM(result, i, self.amino_acid)
 *         i += 1
 *         if self.position_type == PositionType.c_term:             # <<<<<<<<<<<<<<
 *             Py_INCREF(self.terminal)
 *             PyTuple_SET_ITEM(result, i, self.terminal)
*/
  __pyx_t_3 = (__pyx_v_self->position_type == __pyx_e_9pyteomics_7cparser_c_term);
  if (__pyx_t_3) {

    /* "pyteomics/cparser.pyx":802
 *         i += 1
 *         if self.position_type == PositionType.c_term:
 *             Py_INCREF(self.terminal)             # <<<<<<<<<<<<<<
 *             PyTuple_SET_ITEM(result, i, self.terminal)
 *             i += 1
*/
    __pyx_t_2 = __pyx_v_self->terminal;
    __Pyx_INCREF(__pyx_t_2);
    Py_INCREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":803
 *         if self.position_type == PositionType.c_term:
 *             Py_INCREF(self.terminal)
 *             PyTuple_SET_ITEM(result, i, self.terminal)             # <<<<<<<<<<<<<<
 *             i += 1
 *         return result
*/
    __pyx_t_2 = __pyx_v_self->terminal;
    __Pyx_INCREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_v_result, __pyx_v_i, __pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":804
 *             Py_INCREF(self.terminal)
 *             PyTuple_SET_ITEM(result, i, self.terminal)
 *             i += 1             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
    __pyx_v_i = (__pyx_v_i + 1);

    /* "pyteomics/cparser.pyx":801
 *         PyTuple_SET_ITEM(result, i, self.amino_acid)
 *         i += 1
 *         if self.position_type == PositionType.c_term:             # <<<<<<<<<<<<<<
 *             Py_INCREF(self.terminal)
 *             PyTuple_SET_ITEM(result, i, self.terminal)
*/
  }

  /* "pyteomics/cparser.pyx":805
 *             PyTuple_SET_ITEM(result, i, self.terminal)
 *             i += 1
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cdef SequencePosition copy(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":781
 *         return self.get_size()
 * 
 *     cpdef tuple as_tuple(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t size, i, n_mods, j
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.as_tuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_mod);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_13as_tuple(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16SequencePosition_12as_tuple, "SequencePosition.as_tuple(self) -> tuple");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16SequencePosition_13as_tuple = {"as_tuple", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_13as_tuple, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_12as_tuple};
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_13as_tuple(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("as_tuple (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("as_tuple", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("as_tuple", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_12as_tuple(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_12as_tuple(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("as_tuple", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition_as_tuple(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 781, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.as_tuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":807
 *         return result
 * 
 *     cdef SequencePosition copy(self):             # <<<<<<<<<<<<<<
 *         return SequencePosition._create(
 *             self.amino_acid, (self.modification), self.terminal, self.position_type)
*/

static struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_f_9pyteomics_7cparser_16SequencePosition_copy(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);

  /* "pyteomics/cparser.pyx":808
 * 
 *     cdef SequencePosition copy(self):
 *         return SequencePosition._create(             # <<<<<<<<<<<<<<
 *             self.amino_acid, (self.modification), self.terminal, self.position_type)
 * 
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);

  /* "pyteomics/cparser.pyx":809
 *     cdef SequencePosition copy(self):
 *         return SequencePosition._create(
 *             self.amino_acid, (self.modification), self.terminal, self.position_type)             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
*/
  __pyx_t_1 = __pyx_v_self->amino_acid;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_v_self->modification;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = __pyx_v_self->terminal;
  __Pyx_INCREF(__pyx_t_3);

  /* "pyteomics/cparser.pyx":808
 * 
 *     cdef SequencePosition copy(self):
 *         return SequencePosition._create(             # <<<<<<<<<<<<<<
 *             self.amino_acid, (self.modification), self.terminal, self.position_type)
 * 
*/
  __pyx_t_4 = ((PyObject *)__pyx_f_9pyteomics_7cparser_16SequencePosition__create(((PyObject*)__pyx_t_1), ((PyObject*)__pyx_t_2), ((PyObject*)__pyx_t_3), __pyx_v_self->position_type)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_4);
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":807
 *         return result
 * 
 *     cdef SequencePosition copy(self):             # <<<<<<<<<<<<<<
 *         return SequencePosition._create(
 *             self.amino_acid, (self.modification), self.terminal, self.position_type)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":811
 *             self.amino_acid, (self.modification), self.terminal, self.position_type)
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_tuple(cls, tuple source):
 *         cdef:
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_15from_tuple(PyObject *__pyx_v_cls, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16SequencePosition_14from_tuple, "SequencePosition.from_tuple(cls, tuple source)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16SequencePosition_15from_tuple = {"from_tuple", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_15from_tuple, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_14from_tuple};
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_15from_tuple(PyObject *__pyx_v_cls, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_source = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("from_tuple (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_source,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 811, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 811, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "from_tuple", 0) < 0) __PYX_ERR(0, 811, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("from_tuple", 1, 1, 1, i); __PYX_ERR(0, 811, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 811, __pyx_L3_error)
    }
    __pyx_v_source = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("from_tuple", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 811, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.from_tuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_source), (&PyTuple_Type), 1, "source", 1))) __PYX_ERR(0, 812, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_14from_tuple(((PyTypeObject*)__pyx_v_cls), __pyx_v_source);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_14from_tuple(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_source) {
  size_t __pyx_v_size;
  size_t __pyx_v_i;
  PyObject *__pyx_v_entry = 0;
  PyObject *__pyx_v_last_entry = 0;
  PyObject *__pyx_v_terminal = 0;
  PyObject *__pyx_v_amino_acid = 0;
  PyObject *__pyx_v_modification = 0;
  enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_v_position_type;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12[4];
  PyObject *__pyx_t_13 = NULL;
  size_t __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_tuple", 0);

  /* "pyteomics/cparser.pyx":822
 *             PositionType position_type
 * 
 *         position_type = PositionType.internal             # <<<<<<<<<<<<<<
 *         modification = None
 *         size = PyTuple_Size(source)
*/
  __pyx_v_position_type = __pyx_e_9pyteomics_7cparser_internal;

  /* "pyteomics/cparser.pyx":823
 * 
 *         position_type = PositionType.internal
 *         modification = None             # <<<<<<<<<<<<<<
 *         size = PyTuple_Size(source)
 *         last_entry = None
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_modification = ((PyObject*)Py_None);

  /* "pyteomics/cparser.pyx":824
 *         position_type = PositionType.internal
 *         modification = None
 *         size = PyTuple_Size(source)             # <<<<<<<<<<<<<<
 *         last_entry = None
 *         terminal = None
*/
  __pyx_t_1 = PyTuple_Size(__pyx_v_source); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 824, __pyx_L1_error)
  __pyx_v_size = __pyx_t_1;

  /* "pyteomics/cparser.pyx":825
 *         modification = None
 *         size = PyTuple_Size(source)
 *         last_entry = None             # <<<<<<<<<<<<<<
 *         terminal = None
 *         for i in range(size):
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_last_entry = ((PyObject*)Py_None);

  /* "pyteomics/cparser.pyx":826
 *         size = PyTuple_Size(source)
 *         last_entry = None
 *         terminal = None             # <<<<<<<<<<<<<<
 *         for i in range(size):
 *             entry = <str>PyTuple_GET_ITEM(source, i)
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_terminal = ((PyObject*)Py_None);

  /* "pyteomics/cparser.pyx":827
 *         last_entry = None
 *         terminal = None
 *         for i in range(size):             # <<<<<<<<<<<<<<
 *             entry = <str>PyTuple_GET_ITEM(source, i)
 *             if entry.endswith("-"):
*/
  __pyx_t_2 = __pyx_v_size;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pyteomics/cparser.pyx":828
 *         terminal = None
 *         for i in range(size):
 *             entry = <str>PyTuple_GET_ITEM(source, i)             # <<<<<<<<<<<<<<
 *             if entry.endswith("-"):
 *                 if position_type != PositionType.internal:
*/
    __pyx_t_5 = PyTuple_GET_ITEM(__pyx_v_source, __pyx_v_i);
    __pyx_t_6 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_XDECREF_SET(__pyx_v_entry, ((PyObject*)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "pyteomics/cparser.pyx":829
 *         for i in range(size):
 *             entry = <str>PyTuple_GET_ITEM(source, i)
 *             if entry.endswith("-"):             # <<<<<<<<<<<<<<
 *                 if position_type != PositionType.internal:
 *                     raise ValueError(
*/
    if (unlikely(__pyx_v_entry == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "endswith");
      __PYX_ERR(0, 829, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_PyUnicode_Tailmatch(__pyx_v_entry, __pyx_mstate_global->__pyx_kp_u_, 0, PY_SSIZE_T_MAX, 1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 829, __pyx_L1_error)
    if (__pyx_t_7) {

      /* "pyteomics/cparser.pyx":830
 *             entry = <str>PyTuple_GET_ITEM(source, i)
 *             if entry.endswith("-"):
 *                 if position_type != PositionType.internal:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         "Position already has position type %s, second type provided %s" % (
*/
      __pyx_t_7 = (__pyx_v_position_type != __pyx_e_9pyteomics_7cparser_internal);
      if (unlikely(__pyx_t_7)) {

        /* "pyteomics/cparser.pyx":831
 *             if entry.endswith("-"):
 *                 if position_type != PositionType.internal:
 *                     raise ValueError(             # <<<<<<<<<<<<<<
 *                         "Position already has position type %s, second type provided %s" % (
 *                             position_type, PositionType.n_term))
*/
        __pyx_t_8 = NULL;
        __Pyx_INCREF(__pyx_builtin_ValueError);
        __pyx_t_9 = __pyx_builtin_ValueError; 

        /* "pyteomics/cparser.pyx":833
 *                     raise ValueError(
 *                         "Position already has position type %s, second type provided %s" % (
 *                             position_type, PositionType.n_term))             # <<<<<<<<<<<<<<
 *                 position_type = PositionType.n_term
 *                 terminal = entry
*/
        __pyx_t_10 = __Pyx_PyUnicode_From_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_v_position_type, 0, ' ', 'd'); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 833, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = __Pyx_PyUnicode_From_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_e_9pyteomics_7cparser_n_term, 0, ' ', 'd'); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 833, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12[0] = __pyx_mstate_global->__pyx_kp_u_Position_already_has_position_ty;
        __pyx_t_12[1] = __pyx_t_10;
        __pyx_t_12[2] = __pyx_mstate_global->__pyx_kp_u_second_type_provided;
        __pyx_t_12[3] = __pyx_t_11;

        /* "pyteomics/cparser.pyx":832
 *                 if position_type != PositionType.internal:
 *                     raise ValueError(
 *                         "Position already has position type %s, second type provided %s" % (             # <<<<<<<<<<<<<<
 *                             position_type, PositionType.n_term))
 *                 position_type = PositionType.n_term
*/
        __pyx_t_13 = __Pyx_PyUnicode_Join(__pyx_t_12, 4, 35 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10) + 23 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11));
        if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 832, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_14 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_13};
          __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 831, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_Raise(__pyx_t_6, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __PYX_ERR(0, 831, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":830
 *             entry = <str>PyTuple_GET_ITEM(source, i)
 *             if entry.endswith("-"):
 *                 if position_type != PositionType.internal:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         "Position already has position type %s, second type provided %s" % (
*/
      }

      /* "pyteomics/cparser.pyx":834
 *                         "Position already has position type %s, second type provided %s" % (
 *                             position_type, PositionType.n_term))
 *                 position_type = PositionType.n_term             # <<<<<<<<<<<<<<
 *                 terminal = entry
 *             elif entry.startswith("-"):
*/
      __pyx_v_position_type = __pyx_e_9pyteomics_7cparser_n_term;

      /* "pyteomics/cparser.pyx":835
 *                             position_type, PositionType.n_term))
 *                 position_type = PositionType.n_term
 *                 terminal = entry             # <<<<<<<<<<<<<<
 *             elif entry.startswith("-"):
 *                 if position_type != PositionType.internal:
*/
      __Pyx_INCREF(__pyx_v_entry);
      __Pyx_DECREF_SET(__pyx_v_terminal, __pyx_v_entry);

      /* "pyteomics/cparser.pyx":829
 *         for i in range(size):
 *             entry = <str>PyTuple_GET_ITEM(source, i)
 *             if entry.endswith("-"):             # <<<<<<<<<<<<<<
 *                 if position_type != PositionType.internal:
 *                     raise ValueError(
*/
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":836
 *                 position_type = PositionType.n_term
 *                 terminal = entry
 *             elif entry.startswith("-"):             # <<<<<<<<<<<<<<
 *                 if position_type != PositionType.internal:
 *                     raise ValueError(
*/
    if (unlikely(__pyx_v_entry == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "startswith");
      __PYX_ERR(0, 836, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_PyUnicode_Tailmatch(__pyx_v_entry, __pyx_mstate_global->__pyx_kp_u_, 0, PY_SSIZE_T_MAX, -1); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 836, __pyx_L1_error)
    if (__pyx_t_7) {

      /* "pyteomics/cparser.pyx":837
 *                 terminal = entry
 *             elif entry.startswith("-"):
 *                 if position_type != PositionType.internal:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         "Position already has position type %s, second type provided %s" % (
*/
      __pyx_t_7 = (__pyx_v_position_type != __pyx_e_9pyteomics_7cparser_internal);
      if (unlikely(__pyx_t_7)) {

        /* "pyteomics/cparser.pyx":838
 *             elif entry.startswith("-"):
 *                 if position_type != PositionType.internal:
 *                     raise ValueError(             # <<<<<<<<<<<<<<
 *                         "Position already has position type %s, second type provided %s" % (
 *                             position_type, PositionType.c_term))
*/
        __pyx_t_9 = NULL;
        __Pyx_INCREF(__pyx_builtin_ValueError);
        __pyx_t_13 = __pyx_builtin_ValueError; 

        /* "pyteomics/cparser.pyx":840
 *                     raise ValueError(
 *                         "Position already has position type %s, second type provided %s" % (
 *                             position_type, PositionType.c_term))             # <<<<<<<<<<<<<<
 *                 position_type = PositionType.c_term
 *                 terminal = entry
*/
        __pyx_t_8 = __Pyx_PyUnicode_From_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_v_position_type, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 840, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyUnicode_From_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_e_9pyteomics_7cparser_c_term, 0, ' ', 'd'); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 840, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12[0] = __pyx_mstate_global->__pyx_kp_u_Position_already_has_position_ty;
        __pyx_t_12[1] = __pyx_t_8;
        __pyx_t_12[2] = __pyx_mstate_global->__pyx_kp_u_second_type_provided;
        __pyx_t_12[3] = __pyx_t_11;

        /* "pyteomics/cparser.pyx":839
 *                 if position_type != PositionType.internal:
 *                     raise ValueError(
 *                         "Position already has position type %s, second type provided %s" % (             # <<<<<<<<<<<<<<
 *                             position_type, PositionType.c_term))
 *                 position_type = PositionType.c_term
*/
        __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_12, 4, 35 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 23 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11));
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 839, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_14 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_10};
          __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_13, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 838, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_Raise(__pyx_t_6, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __PYX_ERR(0, 838, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":837
 *                 terminal = entry
 *             elif entry.startswith("-"):
 *                 if position_type != PositionType.internal:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         "Position already has position type %s, second type provided %s" % (
*/
      }

      /* "pyteomics/cparser.pyx":841
 *                         "Position already has position type %s, second type provided %s" % (
 *                             position_type, PositionType.c_term))
 *                 position_type = PositionType.c_term             # <<<<<<<<<<<<<<
 *                 terminal = entry
 *             else:
*/
      __pyx_v_position_type = __pyx_e_9pyteomics_7cparser_c_term;

      /* "pyteomics/cparser.pyx":842
 *                             position_type, PositionType.c_term))
 *                 position_type = PositionType.c_term
 *                 terminal = entry             # <<<<<<<<<<<<<<
 *             else:
 *                 if last_entry is not None:
*/
      __Pyx_INCREF(__pyx_v_entry);
      __Pyx_DECREF_SET(__pyx_v_terminal, __pyx_v_entry);

      /* "pyteomics/cparser.pyx":836
 *                 position_type = PositionType.n_term
 *                 terminal = entry
 *             elif entry.startswith("-"):             # <<<<<<<<<<<<<<
 *                 if position_type != PositionType.internal:
 *                     raise ValueError(
*/
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":844
 *                 terminal = entry
 *             else:
 *                 if last_entry is not None:             # <<<<<<<<<<<<<<
 *                     modification = last_entry
 *                 last_entry = entry
*/
    /*else*/ {
      __pyx_t_7 = (__pyx_v_last_entry != ((PyObject*)Py_None));
      if (__pyx_t_7) {

        /* "pyteomics/cparser.pyx":845
 *             else:
 *                 if last_entry is not None:
 *                     modification = last_entry             # <<<<<<<<<<<<<<
 *                 last_entry = entry
 *         amino_acid = last_entry
*/
        __Pyx_INCREF(__pyx_v_last_entry);
        __Pyx_DECREF_SET(__pyx_v_modification, __pyx_v_last_entry);

        /* "pyteomics/cparser.pyx":844
 *                 terminal = entry
 *             else:
 *                 if last_entry is not None:             # <<<<<<<<<<<<<<
 *                     modification = last_entry
 *                 last_entry = entry
*/
      }

      /* "pyteomics/cparser.pyx":846
 *                 if last_entry is not None:
 *                     modification = last_entry
 *                 last_entry = entry             # <<<<<<<<<<<<<<
 *         amino_acid = last_entry
 *         return cls(amino_acid, modification, terminal, position_type)
*/
      __Pyx_INCREF(__pyx_v_entry);
      __Pyx_DECREF_SET(__pyx_v_last_entry, __pyx_v_entry);
    }
    __pyx_L5:;
  }

  /* "pyteomics/cparser.pyx":847
 *                     modification = last_entry
 *                 last_entry = entry
 *         amino_acid = last_entry             # <<<<<<<<<<<<<<
 *         return cls(amino_acid, modification, terminal, position_type)
 * 
*/
  __Pyx_INCREF(__pyx_v_last_entry);
  __pyx_v_amino_acid = __pyx_v_last_entry;

  /* "pyteomics/cparser.pyx":848
 *                 last_entry = entry
 *         amino_acid = last_entry
 *         return cls(amino_acid, modification, terminal, position_type)             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = NULL;
  __Pyx_INCREF((PyObject *)__pyx_v_cls);
  __pyx_t_10 = ((PyObject *)__pyx_v_cls); 
  __pyx_t_9 = __Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(__pyx_v_position_type); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 848, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = 1;
  {
    PyObject *__pyx_callargs[5] = {__pyx_t_13, __pyx_v_amino_acid, __pyx_v_modification, __pyx_v_terminal, __pyx_t_9};
    __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+__pyx_t_2, (5-__pyx_t_2) | (__pyx_t_2*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 848, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
  }
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":811
 *             self.amino_acid, (self.modification), self.terminal, self.position_type)
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_tuple(cls, tuple source):
 *         cdef:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.from_tuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_entry);
  __Pyx_XDECREF(__pyx_v_last_entry);
  __Pyx_XDECREF(__pyx_v_terminal);
  __Pyx_XDECREF(__pyx_v_amino_acid);
  __Pyx_XDECREF(__pyx_v_modification);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":850
 *         return cls(amino_acid, modification, terminal, position_type)
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def parse(cls, str sequence, **kwargs):
 *         cdef:
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_17parse(PyObject *__pyx_v_cls, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16SequencePosition_16parse, "SequencePosition.parse(cls, str sequence, **kwargs)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16SequencePosition_17parse = {"parse", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_17parse, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_16parse};
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_17parse(PyObject *__pyx_v_cls, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  PyObject *__pyx_v_kwargs = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("parse (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 850, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 850, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, __pyx_v_kwargs, values, kwd_pos_args, __pyx_kwds_len, "parse", 1) < 0) __PYX_ERR(0, 850, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("parse", 1, 1, 1, i); __PYX_ERR(0, 850, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 850, __pyx_L3_error)
    }
    __pyx_v_sequence = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("parse", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 850, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.parse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sequence), (&PyUnicode_Type), 1, "sequence", 1))) __PYX_ERR(0, 851, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_16parse(((PyTypeObject*)__pyx_v_cls), __pyx_v_sequence, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_DECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_16parse(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_sequence, PyObject *__pyx_v_kwargs) {
  size_t __pyx_v_n;
  size_t __pyx_v_i;
  PyObject *__pyx_v_tokens = 0;
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_v_converter = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10;
  size_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("parse", 0);

  /* "pyteomics/cparser.pyx":856
 *             list tokens, result
 *             object converter
 *         converter = cls.from_tuple             # <<<<<<<<<<<<<<
 *         tokens = parse(sequence, show_unmodified_termini=True, split=True, **kwargs)
 *         n = PyList_Size(tokens)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_cls), __pyx_mstate_global->__pyx_n_u_from_tuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_converter = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":857
 *             object converter
 *         converter = cls.from_tuple
 *         tokens = parse(sequence, show_unmodified_termini=True, split=True, **kwargs)             # <<<<<<<<<<<<<<
 *         n = PyList_Size(tokens)
 *         result = []
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_parse); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 857, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 857, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_show_unmodified_termini, Py_True) < 0) __PYX_ERR(0, 857, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_split, Py_True) < 0) __PYX_ERR(0, 857, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_5;
  __pyx_t_5 = 0;
  if (__Pyx_MergeKeywords(__pyx_t_4, __pyx_v_kwargs) < 0) __PYX_ERR(0, 857, __pyx_L1_error)
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_sequence};
    __pyx_t_1 = __Pyx_PyObject_FastCallDict(__pyx_t_3, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 857, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(0, 857, __pyx_L1_error)
  __pyx_v_tokens = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":858
 *         converter = cls.from_tuple
 *         tokens = parse(sequence, show_unmodified_termini=True, split=True, **kwargs)
 *         n = PyList_Size(tokens)             # <<<<<<<<<<<<<<
 *         result = []
 *         for i in range(n):
*/
  __pyx_t_7 = PyList_Size(__pyx_v_tokens); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 858, __pyx_L1_error)
  __pyx_v_n = __pyx_t_7;

  /* "pyteomics/cparser.pyx":859
 *         tokens = parse(sequence, show_unmodified_termini=True, split=True, **kwargs)
 *         n = PyList_Size(tokens)
 *         result = []             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             result.append(converter(<tuple>PyList_GET_ITEM(tokens, i)))
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":860
 *         n = PyList_Size(tokens)
 *         result = []
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             result.append(converter(<tuple>PyList_GET_ITEM(tokens, i)))
 *         return result
*/
  __pyx_t_6 = __pyx_v_n;
  __pyx_t_8 = __pyx_t_6;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "pyteomics/cparser.pyx":861
 *         result = []
 *         for i in range(n):
 *             result.append(converter(<tuple>PyList_GET_ITEM(tokens, i)))             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
    __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_converter);
    __pyx_t_4 = __pyx_v_converter; 
    __pyx_t_10 = PyList_GET_ITEM(__pyx_v_tokens, __pyx_v_i);
    __pyx_t_11 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_11 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, ((PyObject *)__pyx_t_10)};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_11, (2-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 861, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_12 = __Pyx_PyList_Append(__pyx_v_result, __pyx_t_1); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 861, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pyteomics/cparser.pyx":862
 *         for i in range(n):
 *             result.append(converter(<tuple>PyList_GET_ITEM(tokens, i)))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":850
 *         return cls(amino_acid, modification, terminal, position_type)
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def parse(cls, str sequence, **kwargs):
 *         cdef:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.parse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tokens);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_converter);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":864
 *         return result
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_iterable(cls, object iterable):
 *         cdef:
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_19from_iterable(PyObject *__pyx_v_cls, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16SequencePosition_18from_iterable, "SequencePosition.from_iterable(cls, iterable)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16SequencePosition_19from_iterable = {"from_iterable", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_19from_iterable, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_18from_iterable};
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_19from_iterable(PyObject *__pyx_v_cls, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_iterable = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("from_iterable (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_iterable,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 864, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 864, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "from_iterable", 0) < 0) __PYX_ERR(0, 864, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("from_iterable", 1, 1, 1, i); __PYX_ERR(0, 864, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 864, __pyx_L3_error)
    }
    __pyx_v_iterable = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("from_iterable", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 864, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.from_iterable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_18from_iterable(((PyTypeObject*)__pyx_v_cls), __pyx_v_iterable);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_18from_iterable(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_iterable) {
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_v_converter = 0;
  PyObject *__pyx_v_obj = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_iterable", 0);

  /* "pyteomics/cparser.pyx":871
 *             object converter
 * 
 *         converter = cls.from_tuple             # <<<<<<<<<<<<<<
 *         result = []
 *         for obj in iterable:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_cls), __pyx_mstate_global->__pyx_n_u_from_tuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 871, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_converter = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":872
 * 
 *         converter = cls.from_tuple
 *         result = []             # <<<<<<<<<<<<<<
 *         for obj in iterable:
 *             if isinstance(obj, SequencePosition):
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 872, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":873
 *         converter = cls.from_tuple
 *         result = []
 *         for obj in iterable:             # <<<<<<<<<<<<<<
 *             if isinstance(obj, SequencePosition):
 *                 result.append((<SequencePosition>obj).copy())
*/
  if (likely(PyList_CheckExact(__pyx_v_iterable)) || PyTuple_CheckExact(__pyx_v_iterable)) {
    __pyx_t_1 = __pyx_v_iterable; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_iterable); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 873, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 873, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 873, __pyx_L1_error)
          #endif
          if (__pyx_t_2 >= __pyx_temp) break;
        }
        __pyx_t_4 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_2);
        ++__pyx_t_2;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 873, __pyx_L1_error)
          #endif
          if (__pyx_t_2 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2));
        #else
        __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2);
        #endif
        ++__pyx_t_2;
      }
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 873, __pyx_L1_error)
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 873, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_XDECREF_SET(__pyx_v_obj, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "pyteomics/cparser.pyx":874
 *         result = []
 *         for obj in iterable:
 *             if isinstance(obj, SequencePosition):             # <<<<<<<<<<<<<<
 *                 result.append((<SequencePosition>obj).copy())
 *             else:
*/
    __pyx_t_5 = __Pyx_TypeCheck(__pyx_v_obj, __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition); 
    if (__pyx_t_5) {

      /* "pyteomics/cparser.pyx":875
 *         for obj in iterable:
 *             if isinstance(obj, SequencePosition):
 *                 result.append((<SequencePosition>obj).copy())             # <<<<<<<<<<<<<<
 *             else:
 *                 result.append(converter(<tuple?>obj))
*/
      __pyx_t_4 = ((PyObject *)__pyx_f_9pyteomics_7cparser_16SequencePosition_copy(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_obj))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 875, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_result, __pyx_t_4); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 875, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyteomics/cparser.pyx":874
 *         result = []
 *         for obj in iterable:
 *             if isinstance(obj, SequencePosition):             # <<<<<<<<<<<<<<
 *                 result.append((<SequencePosition>obj).copy())
 *             else:
*/
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":877
 *                 result.append((<SequencePosition>obj).copy())
 *             else:
 *                 result.append(converter(<tuple?>obj))             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
    /*else*/ {
      __pyx_t_7 = NULL;
      __Pyx_INCREF(__pyx_v_converter);
      __pyx_t_8 = __pyx_v_converter; 
      if (!(likely(PyTuple_CheckExact(__pyx_v_obj)) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v_obj))) __PYX_ERR(0, 877, __pyx_L1_error)
      __pyx_t_9 = 1;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
        __pyx_t_9 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_v_obj};
        __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 877, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
      }
      __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_result, __pyx_t_4); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 877, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __pyx_L5:;

    /* "pyteomics/cparser.pyx":873
 *         converter = cls.from_tuple
 *         result = []
 *         for obj in iterable:             # <<<<<<<<<<<<<<
 *             if isinstance(obj, SequencePosition):
 *                 result.append((<SequencePosition>obj).copy())
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":878
 *             else:
 *                 result.append(converter(<tuple?>obj))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cdef bint _eq(self, SequencePosition other):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":864
 *         return result
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_iterable(cls, object iterable):
 *         cdef:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.from_iterable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_converter);
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":880
 *         return result
 * 
 *     cdef bint _eq(self, SequencePosition other):             # <<<<<<<<<<<<<<
 *         if self.amino_acid != other.amino_acid:
 *             return False
*/

static int __pyx_f_9pyteomics_7cparser_16SequencePosition__eq(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_other) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "pyteomics/cparser.pyx":881
 * 
 *     cdef bint _eq(self, SequencePosition other):
 *         if self.amino_acid != other.amino_acid:             # <<<<<<<<<<<<<<
 *             return False
 *         elif self.modification != other.modification:
*/
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_self->amino_acid, __pyx_v_other->amino_acid, Py_NE)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 881, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":882
 *     cdef bint _eq(self, SequencePosition other):
 *         if self.amino_acid != other.amino_acid:
 *             return False             # <<<<<<<<<<<<<<
 *         elif self.modification != other.modification:
 *             return False
*/
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":881
 * 
 *     cdef bint _eq(self, SequencePosition other):
 *         if self.amino_acid != other.amino_acid:             # <<<<<<<<<<<<<<
 *             return False
 *         elif self.modification != other.modification:
*/
  }

  /* "pyteomics/cparser.pyx":883
 *         if self.amino_acid != other.amino_acid:
 *             return False
 *         elif self.modification != other.modification:             # <<<<<<<<<<<<<<
 *             return False
 *         elif self.terminal != other.terminal:
*/
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_self->modification, __pyx_v_other->modification, Py_NE)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 883, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":884
 *             return False
 *         elif self.modification != other.modification:
 *             return False             # <<<<<<<<<<<<<<
 *         elif self.terminal != other.terminal:
 *             return False
*/
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":883
 *         if self.amino_acid != other.amino_acid:
 *             return False
 *         elif self.modification != other.modification:             # <<<<<<<<<<<<<<
 *             return False
 *         elif self.terminal != other.terminal:
*/
  }

  /* "pyteomics/cparser.pyx":885
 *         elif self.modification != other.modification:
 *             return False
 *         elif self.terminal != other.terminal:             # <<<<<<<<<<<<<<
 *             return False
 *         return True
*/
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_self->terminal, __pyx_v_other->terminal, Py_NE)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 885, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":886
 *             return False
 *         elif self.terminal != other.terminal:
 *             return False             # <<<<<<<<<<<<<<
 *         return True
 * 
*/
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":885
 *         elif self.modification != other.modification:
 *             return False
 *         elif self.terminal != other.terminal:             # <<<<<<<<<<<<<<
 *             return False
 *         return True
*/
  }

  /* "pyteomics/cparser.pyx":887
 *         elif self.terminal != other.terminal:
 *             return False
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     cdef bint _eq_tuple(self, tuple other):
*/
  __pyx_r = 1;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":880
 *         return result
 * 
 *     cdef bint _eq(self, SequencePosition other):             # <<<<<<<<<<<<<<
 *         if self.amino_acid != other.amino_acid:
 *             return False
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition._eq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":889
 *         return True
 * 
 *     cdef bint _eq_tuple(self, tuple other):             # <<<<<<<<<<<<<<
 *         return self.as_tuple() == other
 * 
*/

static int __pyx_f_9pyteomics_7cparser_16SequencePosition__eq_tuple(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_other) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_eq_tuple", 0);

  /* "pyteomics/cparser.pyx":890
 * 
 *     cdef bint _eq_tuple(self, tuple other):
 *         return self.as_tuple() == other             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, other):
*/
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition_as_tuple(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 890, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_v_other, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 890, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 890, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":889
 *         return True
 * 
 *     cdef bint _eq_tuple(self, tuple other):             # <<<<<<<<<<<<<<
 *         return self.as_tuple() == other
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition._eq_tuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":892
 *         return self.as_tuple() == other
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         if other is None:
 *             return False
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_21__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_21__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_20__eq__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_20__eq__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "pyteomics/cparser.pyx":893
 * 
 *     def __eq__(self, other):
 *         if other is None:             # <<<<<<<<<<<<<<
 *             return False
 *         if isinstance(other, SequencePosition):
*/
  __pyx_t_1 = (__pyx_v_other == Py_None);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":894
 *     def __eq__(self, other):
 *         if other is None:
 *             return False             # <<<<<<<<<<<<<<
 *         if isinstance(other, SequencePosition):
 *             return self._eq(<SequencePosition>other)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":893
 * 
 *     def __eq__(self, other):
 *         if other is None:             # <<<<<<<<<<<<<<
 *             return False
 *         if isinstance(other, SequencePosition):
*/
  }

  /* "pyteomics/cparser.pyx":895
 *         if other is None:
 *             return False
 *         if isinstance(other, SequencePosition):             # <<<<<<<<<<<<<<
 *             return self._eq(<SequencePosition>other)
 *         elif isinstance(other, tuple):
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_other, __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition); 
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":896
 *             return False
 *         if isinstance(other, SequencePosition):
 *             return self._eq(<SequencePosition>other)             # <<<<<<<<<<<<<<
 *         elif isinstance(other, tuple):
 *             return self._eq_tuple(<tuple>other)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition__eq(__pyx_v_self, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_other)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 896, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 896, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":895
 *         if other is None:
 *             return False
 *         if isinstance(other, SequencePosition):             # <<<<<<<<<<<<<<
 *             return self._eq(<SequencePosition>other)
 *         elif isinstance(other, tuple):
*/
  }

  /* "pyteomics/cparser.pyx":897
 *         if isinstance(other, SequencePosition):
 *             return self._eq(<SequencePosition>other)
 *         elif isinstance(other, tuple):             # <<<<<<<<<<<<<<
 *             return self._eq_tuple(<tuple>other)
 *         else:
*/
  __pyx_t_1 = PyTuple_Check(__pyx_v_other); 
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":898
 *             return self._eq(<SequencePosition>other)
 *         elif isinstance(other, tuple):
 *             return self._eq_tuple(<tuple>other)             # <<<<<<<<<<<<<<
 *         else:
 *             return self._eq_tuple(tuple(other))
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition__eq_tuple(__pyx_v_self, ((PyObject*)__pyx_v_other)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 898, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 898, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":897
 *         if isinstance(other, SequencePosition):
 *             return self._eq(<SequencePosition>other)
 *         elif isinstance(other, tuple):             # <<<<<<<<<<<<<<
 *             return self._eq_tuple(<tuple>other)
 *         else:
*/
  }

  /* "pyteomics/cparser.pyx":900
 *             return self._eq_tuple(<tuple>other)
 *         else:
 *             return self._eq_tuple(tuple(other))             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, other):
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PySequence_Tuple(__pyx_v_other); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 900, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16SequencePosition__eq_tuple(__pyx_v_self, ((PyObject*)__pyx_t_2)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 900, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 900, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pyteomics/cparser.pyx":892
 *         return self.as_tuple() == other
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         if other is None:
 *             return False
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":902
 *             return self._eq_tuple(tuple(other))
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         return not (self == other)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_23__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_23__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_22__ne__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_22__ne__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "pyteomics/cparser.pyx":903
 * 
 *     def __ne__(self, other):
 *         return not (self == other)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)__pyx_v_self), __pyx_v_other, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 903, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 903, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyBool_FromLong((!__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 903, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":902
 *             return self._eq_tuple(tuple(other))
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         return not (self == other)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":701
 * cdef class SequencePosition(object):
 *     cdef:
 *         public str modification             # <<<<<<<<<<<<<<
 *         public PositionType position_type
 *         public str amino_acid
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_12modification_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_12modification_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_12modification___get__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_12modification___get__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->modification);
  __pyx_r = __pyx_v_self->modification;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_12modification_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_12modification_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_12modification_2__set__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_12modification_2__set__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->modification);
  __Pyx_DECREF(__pyx_v_self->modification);
  __pyx_v_self->modification = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.modification.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_12modification_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_12modification_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_12modification_4__del__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_12modification_4__del__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->modification);
  __Pyx_DECREF(__pyx_v_self->modification);
  __pyx_v_self->modification = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":702
 *     cdef:
 *         public str modification
 *         public PositionType position_type             # <<<<<<<<<<<<<<
 *         public str amino_acid
 *         public str terminal
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_13position_type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_13position_type_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_13position_type___get__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_13position_type___get__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(__pyx_v_self->position_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 702, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.position_type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_13position_type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_13position_type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_13position_type_2__set__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_13position_type_2__set__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = ((enum __pyx_t_9pyteomics_7cparser_PositionType)__Pyx_PyLong_As_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_v_value)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 702, __pyx_L1_error)
  __pyx_v_self->position_type = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.position_type.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":703
 *         public str modification
 *         public PositionType position_type
 *         public str amino_acid             # <<<<<<<<<<<<<<
 *         public str terminal
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_10amino_acid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_10amino_acid_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_10amino_acid___get__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_10amino_acid___get__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->amino_acid);
  __pyx_r = __pyx_v_self->amino_acid;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_10amino_acid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_10amino_acid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_10amino_acid_2__set__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_10amino_acid_2__set__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->amino_acid);
  __Pyx_DECREF(__pyx_v_self->amino_acid);
  __pyx_v_self->amino_acid = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.amino_acid.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_10amino_acid_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_10amino_acid_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_10amino_acid_4__del__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_10amino_acid_4__del__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->amino_acid);
  __Pyx_DECREF(__pyx_v_self->amino_acid);
  __pyx_v_self->amino_acid = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":704
 *         public PositionType position_type
 *         public str amino_acid
 *         public str terminal             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, amino_acid, modification=None, terminal=None, position_type=None):
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_8terminal_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_8terminal_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_8terminal___get__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_8terminal___get__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->terminal);
  __pyx_r = __pyx_v_self->terminal;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_8terminal_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_8terminal_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_8terminal_2__set__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_8terminal_2__set__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 704, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->terminal);
  __Pyx_DECREF(__pyx_v_self->terminal);
  __pyx_v_self->terminal = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.terminal.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_8terminal_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16SequencePosition_8terminal_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_8terminal_4__del__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16SequencePosition_8terminal_4__del__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->terminal);
  __Pyx_DECREF(__pyx_v_self->terminal);
  __pyx_v_self->terminal = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_25__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16SequencePosition_24__reduce_cython__, "SequencePosition.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16SequencePosition_25__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_25__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_24__reduce_cython__};
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_25__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_24__reduce_cython__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_24__reduce_cython__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.amino_acid, self.modification, self.position_type, self.terminal)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
*/
  __pyx_t_1 = __Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(__pyx_v_self->position_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->amino_acid);
  __Pyx_GIVEREF(__pyx_v_self->amino_acid);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->amino_acid) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->modification);
  __Pyx_GIVEREF(__pyx_v_self->modification);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->modification) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_1) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->terminal);
  __Pyx_GIVEREF(__pyx_v_self->terminal);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_v_self->terminal) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.amino_acid, self.modification, self.position_type, self.terminal)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
*/
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self.amino_acid, self.modification, self.position_type, self.terminal)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict) != (0)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.amino_acid is not None or self.modification is not None or self.terminal is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.amino_acid, self.modification, self.position_type, self.terminal)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.amino_acid is not None or self.modification is not None or self.terminal is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, None), state
*/
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->amino_acid != ((PyObject*)Py_None));
    if (!__pyx_t_4) {
    } else {
      __pyx_t_3 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->modification != ((PyObject*)Py_None));
    if (!__pyx_t_4) {
    } else {
      __pyx_t_3 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->terminal != ((PyObject*)Py_None));
    __pyx_t_3 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.amino_acid is not None or self.modification is not None or self.terminal is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.amino_acid is not None or self.modification is not None or self.terminal is not None
 *     if use_setstate:
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_SequencePosition); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_237805793);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_237805793);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_mstate_global->__pyx_int_237805793) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.amino_acid is not None or self.modification is not None or self.terminal is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, None), state
 *     else:
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SequencePosition__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_SequencePosition); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_237805793);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_237805793);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_mstate_global->__pyx_int_237805793) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SequencePosition__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_27__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16SequencePosition_26__setstate_cython__, "SequencePosition.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16SequencePosition_27__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_27__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_26__setstate_cython__};
static PyObject *__pyx_pw_9pyteomics_7cparser_16SequencePosition_27__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16SequencePosition_26__setstate_cython__(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16SequencePosition_26__setstate_cython__(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SequencePosition__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser___pyx_unpickle_SequencePosition__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SequencePosition__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.SequencePosition.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":906
 * 
 * 
 * cpdef str _sequence_tostring(list sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n, acc, j
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_11_sequence_tostring(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser__sequence_tostring(PyObject *__pyx_v_sequence, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser__sequence_tostring *__pyx_optional_args) {
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  size_t __pyx_v_acc;
  size_t __pyx_v_j;
  Py_ssize_t __pyx_v_sz;
  char *__pyx_v_cstr;
  char __pyx_v_prealloc_buffer[0x96];
  char *__pyx_v_string_buffer;
  int __pyx_v_needs_free;
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_pos = 0;
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  size_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_sequence_tostring", 0);
  if (__pyx_optional_args) {
  }

  /* "pyteomics/cparser.pyx":917
 *         str result
 * 
 *     needs_free = False             # <<<<<<<<<<<<<<
 *     sz = 0
 *     n = PyList_Size(sequence)
*/
  __pyx_v_needs_free = 0;

  /* "pyteomics/cparser.pyx":918
 * 
 *     needs_free = False
 *     sz = 0             # <<<<<<<<<<<<<<
 *     n = PyList_Size(sequence)
 *     acc = 0
*/
  __pyx_v_sz = 0;

  /* "pyteomics/cparser.pyx":919
 *     needs_free = False
 *     sz = 0
 *     n = PyList_Size(sequence)             # <<<<<<<<<<<<<<
 *     acc = 0
 *     for i in range(n):
*/
  __pyx_t_1 = PyList_Size(__pyx_v_sequence); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 919, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "pyteomics/cparser.pyx":920
 *     sz = 0
 *     n = PyList_Size(sequence)
 *     acc = 0             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
*/
  __pyx_v_acc = 0;

  /* "pyteomics/cparser.pyx":921
 *     n = PyList_Size(sequence)
 *     acc = 0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
 *         if pos.terminal is not None:
*/
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pyteomics/cparser.pyx":922
 *     acc = 0
 *     for i in range(n):
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)             # <<<<<<<<<<<<<<
 *         if pos.terminal is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
*/
    __pyx_t_5 = PyList_GET_ITEM(__pyx_v_sequence, __pyx_v_i);
    __pyx_t_6 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_XDECREF_SET(__pyx_v_pos, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "pyteomics/cparser.pyx":923
 *     for i in range(n):
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
 *         if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *             acc += sz
*/
    __pyx_t_7 = (__pyx_v_pos->terminal != ((PyObject*)Py_None));
    if (__pyx_t_7) {

      /* "pyteomics/cparser.pyx":924
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
 *         if pos.terminal is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)             # <<<<<<<<<<<<<<
 *             acc += sz
 *         if pos.modification is not None:
*/
      __pyx_t_6 = __pyx_v_pos->terminal;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_6), (&__pyx_v_sz));
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyteomics/cparser.pyx":925
 *         if pos.terminal is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *             acc += sz             # <<<<<<<<<<<<<<
 *         if pos.modification is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
*/
      __pyx_v_acc = (__pyx_v_acc + __pyx_v_sz);

      /* "pyteomics/cparser.pyx":923
 *     for i in range(n):
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
 *         if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *             acc += sz
*/
    }

    /* "pyteomics/cparser.pyx":926
 *             cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *             acc += sz
 *         if pos.modification is not None:             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *             acc += sz
*/
    __pyx_t_7 = (__pyx_v_pos->modification != ((PyObject*)Py_None));
    if (__pyx_t_7) {

      /* "pyteomics/cparser.pyx":927
 *             acc += sz
 *         if pos.modification is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)             # <<<<<<<<<<<<<<
 *             acc += sz
 *         if pos.amino_acid is not None:
*/
      __pyx_t_6 = __pyx_v_pos->modification;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_6), (&__pyx_v_sz));
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyteomics/cparser.pyx":928
 *         if pos.modification is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *             acc += sz             # <<<<<<<<<<<<<<
 *         if pos.amino_acid is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
*/
      __pyx_v_acc = (__pyx_v_acc + __pyx_v_sz);

      /* "pyteomics/cparser.pyx":926
 *             cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *             acc += sz
 *         if pos.modification is not None:             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *             acc += sz
*/
    }

    /* "pyteomics/cparser.pyx":929
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *             acc += sz
 *         if pos.amino_acid is not None:             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *             acc += sz
*/
    __pyx_t_7 = (__pyx_v_pos->amino_acid != ((PyObject*)Py_None));
    if (__pyx_t_7) {

      /* "pyteomics/cparser.pyx":930
 *             acc += sz
 *         if pos.amino_acid is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)             # <<<<<<<<<<<<<<
 *             acc += sz
 *     if acc < TOSTRING_STACK_BUFFER_SIZE:
*/
      __pyx_t_6 = __pyx_v_pos->amino_acid;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_6), (&__pyx_v_sz));
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyteomics/cparser.pyx":931
 *         if pos.amino_acid is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *             acc += sz             # <<<<<<<<<<<<<<
 *     if acc < TOSTRING_STACK_BUFFER_SIZE:
 *         string_buffer = prealloc_buffer
*/
      __pyx_v_acc = (__pyx_v_acc + __pyx_v_sz);

      /* "pyteomics/cparser.pyx":929
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *             acc += sz
 *         if pos.amino_acid is not None:             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *             acc += sz
*/
    }
  }

  /* "pyteomics/cparser.pyx":932
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *             acc += sz
 *     if acc < TOSTRING_STACK_BUFFER_SIZE:             # <<<<<<<<<<<<<<
 *         string_buffer = prealloc_buffer
 *     else:
*/
  __pyx_t_7 = (__pyx_v_acc < 0x96);
  if (__pyx_t_7) {

    /* "pyteomics/cparser.pyx":933
 *             acc += sz
 *     if acc < TOSTRING_STACK_BUFFER_SIZE:
 *         string_buffer = prealloc_buffer             # <<<<<<<<<<<<<<
 *     else:
 *         string_buffer = <char*>malloc(sizeof(char*) * acc)
*/
    __pyx_v_string_buffer = __pyx_v_prealloc_buffer;

    /* "pyteomics/cparser.pyx":932
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *             acc += sz
 *     if acc < TOSTRING_STACK_BUFFER_SIZE:             # <<<<<<<<<<<<<<
 *         string_buffer = prealloc_buffer
 *     else:
*/
    goto __pyx_L8;
  }

  /* "pyteomics/cparser.pyx":935
 *         string_buffer = prealloc_buffer
 *     else:
 *         string_buffer = <char*>malloc(sizeof(char*) * acc)             # <<<<<<<<<<<<<<
 *         needs_free = True
 *     acc = 0
*/
  /*else*/ {
    __pyx_v_string_buffer = ((char *)malloc(((sizeof(char *)) * __pyx_v_acc)));

    /* "pyteomics/cparser.pyx":936
 *     else:
 *         string_buffer = <char*>malloc(sizeof(char*) * acc)
 *         needs_free = True             # <<<<<<<<<<<<<<
 *     acc = 0
 *     for i in range(n):
*/
    __pyx_v_needs_free = 1;
  }
  __pyx_L8:;

  /* "pyteomics/cparser.pyx":937
 *         string_buffer = <char*>malloc(sizeof(char*) * acc)
 *         needs_free = True
 *     acc = 0             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
*/
  __pyx_v_acc = 0;

  /* "pyteomics/cparser.pyx":938
 *         needs_free = True
 *     acc = 0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
 *         if pos.position_type == PositionType.n_term:
*/
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pyteomics/cparser.pyx":939
 *     acc = 0
 *     for i in range(n):
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)             # <<<<<<<<<<<<<<
 *         if pos.position_type == PositionType.n_term:
 *             if pos.terminal is not None:
*/
    __pyx_t_5 = PyList_GET_ITEM(__pyx_v_sequence, __pyx_v_i);
    __pyx_t_6 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_XDECREF_SET(__pyx_v_pos, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "pyteomics/cparser.pyx":940
 *     for i in range(n):
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
 *         if pos.position_type == PositionType.n_term:             # <<<<<<<<<<<<<<
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
*/
    __pyx_t_7 = (__pyx_v_pos->position_type == __pyx_e_9pyteomics_7cparser_n_term);
    if (__pyx_t_7) {

      /* "pyteomics/cparser.pyx":941
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
 *         if pos.position_type == PositionType.n_term:
 *             if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 for j in range(sz):
*/
      __pyx_t_7 = (__pyx_v_pos->terminal != ((PyObject*)Py_None));
      if (__pyx_t_7) {

        /* "pyteomics/cparser.pyx":942
 *         if pos.position_type == PositionType.n_term:
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)             # <<<<<<<<<<<<<<
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]
*/
        __pyx_t_6 = __pyx_v_pos->terminal;
        __Pyx_INCREF(__pyx_t_6);
        __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_6), (&__pyx_v_sz));
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pyteomics/cparser.pyx":943
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 for j in range(sz):             # <<<<<<<<<<<<<<
 *                     string_buffer[acc] = cstr[j]
 *                     acc += 1
*/
        __pyx_t_1 = __pyx_v_sz;
        __pyx_t_8 = __pyx_t_1;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_j = __pyx_t_9;

          /* "pyteomics/cparser.pyx":944
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]             # <<<<<<<<<<<<<<
 *                     acc += 1
 * 
*/
          (__pyx_v_string_buffer[__pyx_v_acc]) = (__pyx_v_cstr[__pyx_v_j]);

          /* "pyteomics/cparser.pyx":945
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]
 *                     acc += 1             # <<<<<<<<<<<<<<
 * 
 *         if pos.modification is not None:
*/
          __pyx_v_acc = (__pyx_v_acc + 1);
        }

        /* "pyteomics/cparser.pyx":941
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
 *         if pos.position_type == PositionType.n_term:
 *             if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 for j in range(sz):
*/
      }

      /* "pyteomics/cparser.pyx":940
 *     for i in range(n):
 *         pos = <SequencePosition?>PyList_GET_ITEM(sequence, i)
 *         if pos.position_type == PositionType.n_term:             # <<<<<<<<<<<<<<
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
*/
    }

    /* "pyteomics/cparser.pyx":947
 *                     acc += 1
 * 
 *         if pos.modification is not None:             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *             for j in range(sz):
*/
    __pyx_t_7 = (__pyx_v_pos->modification != ((PyObject*)Py_None));
    if (__pyx_t_7) {

      /* "pyteomics/cparser.pyx":948
 * 
 *         if pos.modification is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)             # <<<<<<<<<<<<<<
 *             for j in range(sz):
 *                 string_buffer[acc] = cstr[j]
*/
      __pyx_t_6 = __pyx_v_pos->modification;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_6), (&__pyx_v_sz));
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyteomics/cparser.pyx":949
 *         if pos.modification is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *             for j in range(sz):             # <<<<<<<<<<<<<<
 *                 string_buffer[acc] = cstr[j]
 *                 acc += 1
*/
      __pyx_t_1 = __pyx_v_sz;
      __pyx_t_8 = __pyx_t_1;
      for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_j = __pyx_t_9;

        /* "pyteomics/cparser.pyx":950
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *             for j in range(sz):
 *                 string_buffer[acc] = cstr[j]             # <<<<<<<<<<<<<<
 *                 acc += 1
 * 
*/
        (__pyx_v_string_buffer[__pyx_v_acc]) = (__pyx_v_cstr[__pyx_v_j]);

        /* "pyteomics/cparser.pyx":951
 *             for j in range(sz):
 *                 string_buffer[acc] = cstr[j]
 *                 acc += 1             # <<<<<<<<<<<<<<
 * 
 *         if pos.amino_acid is not None:
*/
        __pyx_v_acc = (__pyx_v_acc + 1);
      }

      /* "pyteomics/cparser.pyx":947
 *                     acc += 1
 * 
 *         if pos.modification is not None:             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(pos.modification, &sz)
 *             for j in range(sz):
*/
    }

    /* "pyteomics/cparser.pyx":953
 *                 acc += 1
 * 
 *         if pos.amino_acid is not None:             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *             for j in range(sz):
*/
    __pyx_t_7 = (__pyx_v_pos->amino_acid != ((PyObject*)Py_None));
    if (__pyx_t_7) {

      /* "pyteomics/cparser.pyx":954
 * 
 *         if pos.amino_acid is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)             # <<<<<<<<<<<<<<
 *             for j in range(sz):
 *                 string_buffer[acc] = cstr[j]
*/
      __pyx_t_6 = __pyx_v_pos->amino_acid;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_6), (&__pyx_v_sz));
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyteomics/cparser.pyx":955
 *         if pos.amino_acid is not None:
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *             for j in range(sz):             # <<<<<<<<<<<<<<
 *                 string_buffer[acc] = cstr[j]
 *                 acc += 1
*/
      __pyx_t_1 = __pyx_v_sz;
      __pyx_t_8 = __pyx_t_1;
      for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_j = __pyx_t_9;

        /* "pyteomics/cparser.pyx":956
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *             for j in range(sz):
 *                 string_buffer[acc] = cstr[j]             # <<<<<<<<<<<<<<
 *                 acc += 1
 * 
*/
        (__pyx_v_string_buffer[__pyx_v_acc]) = (__pyx_v_cstr[__pyx_v_j]);

        /* "pyteomics/cparser.pyx":957
 *             for j in range(sz):
 *                 string_buffer[acc] = cstr[j]
 *                 acc += 1             # <<<<<<<<<<<<<<
 * 
 *         if pos.position_type == PositionType.c_term:
*/
        __pyx_v_acc = (__pyx_v_acc + 1);
      }

      /* "pyteomics/cparser.pyx":953
 *                 acc += 1
 * 
 *         if pos.amino_acid is not None:             # <<<<<<<<<<<<<<
 *             cstr = PyStr_AsUTF8AndSize(pos.amino_acid, &sz)
 *             for j in range(sz):
*/
    }

    /* "pyteomics/cparser.pyx":959
 *                 acc += 1
 * 
 *         if pos.position_type == PositionType.c_term:             # <<<<<<<<<<<<<<
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
*/
    __pyx_t_7 = (__pyx_v_pos->position_type == __pyx_e_9pyteomics_7cparser_c_term);
    if (__pyx_t_7) {

      /* "pyteomics/cparser.pyx":960
 * 
 *         if pos.position_type == PositionType.c_term:
 *             if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 for j in range(sz):
*/
      __pyx_t_7 = (__pyx_v_pos->terminal != ((PyObject*)Py_None));
      if (__pyx_t_7) {

        /* "pyteomics/cparser.pyx":961
 *         if pos.position_type == PositionType.c_term:
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)             # <<<<<<<<<<<<<<
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]
*/
        __pyx_t_6 = __pyx_v_pos->terminal;
        __Pyx_INCREF(__pyx_t_6);
        __pyx_v_cstr = PyStr_AsUTF8AndSize(((PyObject*)__pyx_t_6), (&__pyx_v_sz));
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pyteomics/cparser.pyx":962
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 for j in range(sz):             # <<<<<<<<<<<<<<
 *                     string_buffer[acc] = cstr[j]
 *                     acc += 1
*/
        __pyx_t_1 = __pyx_v_sz;
        __pyx_t_8 = __pyx_t_1;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_j = __pyx_t_9;

          /* "pyteomics/cparser.pyx":963
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]             # <<<<<<<<<<<<<<
 *                     acc += 1
 * 
*/
          (__pyx_v_string_buffer[__pyx_v_acc]) = (__pyx_v_cstr[__pyx_v_j]);

          /* "pyteomics/cparser.pyx":964
 *                 for j in range(sz):
 *                     string_buffer[acc] = cstr[j]
 *                     acc += 1             # <<<<<<<<<<<<<<
 * 
 *     result = PyStr_FromStringAndSize(string_buffer, acc)
*/
          __pyx_v_acc = (__pyx_v_acc + 1);
        }

        /* "pyteomics/cparser.pyx":960
 * 
 *         if pos.position_type == PositionType.c_term:
 *             if pos.terminal is not None:             # <<<<<<<<<<<<<<
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
 *                 for j in range(sz):
*/
      }

      /* "pyteomics/cparser.pyx":959
 *                 acc += 1
 * 
 *         if pos.position_type == PositionType.c_term:             # <<<<<<<<<<<<<<
 *             if pos.terminal is not None:
 *                 cstr = PyStr_AsUTF8AndSize(pos.terminal, &sz)
*/
    }
  }

  /* "pyteomics/cparser.pyx":966
 *                     acc += 1
 * 
 *     result = PyStr_FromStringAndSize(string_buffer, acc)             # <<<<<<<<<<<<<<
 *     if needs_free:
 *         free(string_buffer)
*/
  __pyx_t_6 = PyStr_FromStringAndSize(__pyx_v_string_buffer, __pyx_v_acc); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 966, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v_result = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "pyteomics/cparser.pyx":967
 * 
 *     result = PyStr_FromStringAndSize(string_buffer, acc)
 *     if needs_free:             # <<<<<<<<<<<<<<
 *         free(string_buffer)
 *     return result
*/
  if (__pyx_v_needs_free) {

    /* "pyteomics/cparser.pyx":968
 *     result = PyStr_FromStringAndSize(string_buffer, acc)
 *     if needs_free:
 *         free(string_buffer)             # <<<<<<<<<<<<<<
 *     return result
 * 
*/
    free(__pyx_v_string_buffer);

    /* "pyteomics/cparser.pyx":967
 * 
 *     result = PyStr_FromStringAndSize(string_buffer, acc)
 *     if needs_free:             # <<<<<<<<<<<<<<
 *         free(string_buffer)
 *     return result
*/
  }

  /* "pyteomics/cparser.pyx":969
 *     if needs_free:
 *         free(string_buffer)
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":906
 * 
 * 
 * cpdef str _sequence_tostring(list sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n, acc, j
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyteomics.cparser._sequence_tostring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_pos);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_11_sequence_tostring(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_10_sequence_tostring, "_sequence_tostring(list sequence, bool show_unmodified_termini=True) -> str");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_11_sequence_tostring = {"_sequence_tostring", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_11_sequence_tostring, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_10_sequence_tostring};
static PyObject *__pyx_pw_9pyteomics_7cparser_11_sequence_tostring(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  int __pyx_v_show_unmodified_termini;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_sequence_tostring (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,&__pyx_mstate_global->__pyx_n_u_show_unmodified_termini,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 906, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 906, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 906, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_sequence_tostring", 0) < 0) __PYX_ERR(0, 906, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_sequence_tostring", 0, 1, 2, i); __PYX_ERR(0, 906, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 906, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 906, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sequence = ((PyObject*)values[0]);
    if (values[1]) {
      __pyx_v_show_unmodified_termini = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_show_unmodified_termini == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 906, __pyx_L3_error)
    } else {
      __pyx_v_show_unmodified_termini = ((int)1);
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_sequence_tostring", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 906, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser._sequence_tostring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sequence), (&PyList_Type), 1, "sequence", 1))) __PYX_ERR(0, 906, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_10_sequence_tostring(__pyx_self, __pyx_v_sequence, __pyx_v_show_unmodified_termini);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_10_sequence_tostring(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_9pyteomics_7cparser__sequence_tostring __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_sequence_tostring", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.show_unmodified_termini = __pyx_v_show_unmodified_termini;
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser__sequence_tostring(__pyx_v_sequence, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 906, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser._sequence_tostring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":972
 * 
 * 
 * cpdef list copy_sequence(list sequence):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list result
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_13copy_sequence(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_copy_sequence(PyObject *__pyx_v_sequence, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_result = 0;
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_sequence", 0);

  /* "pyteomics/cparser.pyx":977
 *         size_t i, n
 *         SequencePosition position
 *     n = PyList_Size(sequence)             # <<<<<<<<<<<<<<
 *     result = PyList_New(n)
 *     for i in range(n):
*/
  __pyx_t_1 = PyList_Size(__pyx_v_sequence); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 977, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "pyteomics/cparser.pyx":978
 *         SequencePosition position
 *     n = PyList_Size(sequence)
 *     result = PyList_New(n)             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         position = <SequencePosition>PyList_GET_ITEM(sequence, i)
*/
  __pyx_t_2 = PyList_New(__pyx_v_n); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 978, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_result = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":979
 *     n = PyList_Size(sequence)
 *     result = PyList_New(n)
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         position = <SequencePosition>PyList_GET_ITEM(sequence, i)
 *         position = position.copy()
*/
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_i = __pyx_t_5;

    /* "pyteomics/cparser.pyx":980
 *     result = PyList_New(n)
 *     for i in range(n):
 *         position = <SequencePosition>PyList_GET_ITEM(sequence, i)             # <<<<<<<<<<<<<<
 *         position = position.copy()
 *         Py_INCREF(position)
*/
    __pyx_t_6 = PyList_GET_ITEM(__pyx_v_sequence, __pyx_v_i);
    __pyx_t_2 = ((PyObject *)__pyx_t_6);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":981
 *     for i in range(n):
 *         position = <SequencePosition>PyList_GET_ITEM(sequence, i)
 *         position = position.copy()             # <<<<<<<<<<<<<<
 *         Py_INCREF(position)
 *         PyList_SET_ITEM(result, i, position)
*/
    __pyx_t_2 = ((PyObject *)__pyx_f_9pyteomics_7cparser_16SequencePosition_copy(__pyx_v_position)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_position, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":982
 *         position = <SequencePosition>PyList_GET_ITEM(sequence, i)
 *         position = position.copy()
 *         Py_INCREF(position)             # <<<<<<<<<<<<<<
 *         PyList_SET_ITEM(result, i, position)
 *     return result
*/
    Py_INCREF(((PyObject *)__pyx_v_position));

    /* "pyteomics/cparser.pyx":983
 *         position = position.copy()
 *         Py_INCREF(position)
 *         PyList_SET_ITEM(result, i, position)             # <<<<<<<<<<<<<<
 *     return result
 * 
*/
    PyList_SET_ITEM(__pyx_v_result, __pyx_v_i, ((PyObject *)__pyx_v_position));
  }

  /* "pyteomics/cparser.pyx":984
 *         Py_INCREF(position)
 *         PyList_SET_ITEM(result, i, position)
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":972
 * 
 * 
 * cpdef list copy_sequence(list sequence):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list result
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.copy_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_13copy_sequence(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_12copy_sequence, "copy_sequence(list sequence) -> list");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_13copy_sequence = {"copy_sequence", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_13copy_sequence, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_12copy_sequence};
static PyObject *__pyx_pw_9pyteomics_7cparser_13copy_sequence(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_sequence (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 972, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 972, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "copy_sequence", 0) < 0) __PYX_ERR(0, 972, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("copy_sequence", 1, 1, 1, i); __PYX_ERR(0, 972, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 972, __pyx_L3_error)
    }
    __pyx_v_sequence = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("copy_sequence", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 972, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.copy_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sequence), (&PyList_Type), 1, "sequence", 1))) __PYX_ERR(0, 972, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_12copy_sequence(__pyx_self, __pyx_v_sequence);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_12copy_sequence(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_sequence", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_copy_sequence(__pyx_v_sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.copy_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1004
 *         public list last_indices
 * 
 *     def __init__(self, variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False,             # <<<<<<<<<<<<<<
 *                  show_unmodified_termini=False, persist=True):
 *         if variable_mods is None:
*/

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_variable_mods = 0;
  PyObject *__pyx_v_fixed_mods = 0;
  PyObject *__pyx_v_labels = 0;
  PyObject *__pyx_v_max_mods = 0;
  PyObject *__pyx_v_override = 0;
  PyObject *__pyx_v_show_unmodified_termini = 0;
  PyObject *__pyx_v_persist = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[7] = {0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_variable_mods,&__pyx_mstate_global->__pyx_n_u_fixed_mods,&__pyx_mstate_global->__pyx_n_u_labels,&__pyx_mstate_global->__pyx_n_u_max_mods,&__pyx_mstate_global->__pyx_n_u_override,&__pyx_mstate_global->__pyx_n_u_show_unmodified_termini,&__pyx_mstate_global->__pyx_n_u_persist,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1004, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  7:
        values[6] = __Pyx_ArgRef_VARARGS(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_VARARGS(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < 0) __PYX_ERR(0, 1004, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_False));

      /* "pyteomics/cparser.pyx":1005
 * 
 *     def __init__(self, variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False,
 *                  show_unmodified_termini=False, persist=True):             # <<<<<<<<<<<<<<
 *         if variable_mods is None:
 *             variable_mods = {}
*/
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_True));
    } else {
      switch (__pyx_nargs) {
        case  7:
        values[6] = __Pyx_ArgRef_VARARGS(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_VARARGS(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "pyteomics/cparser.pyx":1004
 *         public list last_indices
 * 
 *     def __init__(self, variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False,             # <<<<<<<<<<<<<<
 *                  show_unmodified_termini=False, persist=True):
 *         if variable_mods is None:
*/
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_False));

      /* "pyteomics/cparser.pyx":1005
 * 
 *     def __init__(self, variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False,
 *                  show_unmodified_termini=False, persist=True):             # <<<<<<<<<<<<<<
 *         if variable_mods is None:
 *             variable_mods = {}
*/
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_True));
    }
    __pyx_v_variable_mods = values[0];
    __pyx_v_fixed_mods = values[1];
    __pyx_v_labels = values[2];
    __pyx_v_max_mods = values[3];
    __pyx_v_override = values[4];
    __pyx_v_show_unmodified_termini = values[5];
    __pyx_v_persist = values[6];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 7, __pyx_nargs); __PYX_ERR(0, 1004, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator___init__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), __pyx_v_variable_mods, __pyx_v_fixed_mods, __pyx_v_labels, __pyx_v_max_mods, __pyx_v_override, __pyx_v_show_unmodified_termini, __pyx_v_persist);

  /* "pyteomics/cparser.pyx":1004
 *         public list last_indices
 * 
 *     def __init__(self, variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False,             # <<<<<<<<<<<<<<
 *                  show_unmodified_termini=False, persist=True):
 *         if variable_mods is None:
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator___init__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_variable_mods, PyObject *__pyx_v_fixed_mods, PyObject *__pyx_v_labels, PyObject *__pyx_v_max_mods, PyObject *__pyx_v_override, PyObject *__pyx_v_show_unmodified_termini, PyObject *__pyx_v_persist) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_variable_mods);
  __Pyx_INCREF(__pyx_v_fixed_mods);
  __Pyx_INCREF(__pyx_v_labels);
  __Pyx_INCREF(__pyx_v_max_mods);

  /* "pyteomics/cparser.pyx":1006
 *     def __init__(self, variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False,
 *                  show_unmodified_termini=False, persist=True):
 *         if variable_mods is None:             # <<<<<<<<<<<<<<
 *             variable_mods = {}
 *         if fixed_mods is None:
*/
  __pyx_t_1 = (__pyx_v_variable_mods == Py_None);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":1007
 *                  show_unmodified_termini=False, persist=True):
 *         if variable_mods is None:
 *             variable_mods = {}             # <<<<<<<<<<<<<<
 *         if fixed_mods is None:
 *             fixed_mods = {}
*/
    __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1007, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_variable_mods, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":1006
 *     def __init__(self, variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False,
 *                  show_unmodified_termini=False, persist=True):
 *         if variable_mods is None:             # <<<<<<<<<<<<<<
 *             variable_mods = {}
 *         if fixed_mods is None:
*/
  }

  /* "pyteomics/cparser.pyx":1008
 *         if variable_mods is None:
 *             variable_mods = {}
 *         if fixed_mods is None:             # <<<<<<<<<<<<<<
 *             fixed_mods = {}
 *         if labels is None:
*/
  __pyx_t_1 = (__pyx_v_fixed_mods == Py_None);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":1009
 *             variable_mods = {}
 *         if fixed_mods is None:
 *             fixed_mods = {}             # <<<<<<<<<<<<<<
 *         if labels is None:
 *             labels = list(std_labels)
*/
    __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1009, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_fixed_mods, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":1008
 *         if variable_mods is None:
 *             variable_mods = {}
 *         if fixed_mods is None:             # <<<<<<<<<<<<<<
 *             fixed_mods = {}
 *         if labels is None:
*/
  }

  /* "pyteomics/cparser.pyx":1010
 *         if fixed_mods is None:
 *             fixed_mods = {}
 *         if labels is None:             # <<<<<<<<<<<<<<
 *             labels = list(std_labels)
 *         if max_mods is None:
*/
  __pyx_t_1 = (__pyx_v_labels == Py_None);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":1011
 *             fixed_mods = {}
 *         if labels is None:
 *             labels = list(std_labels)             # <<<<<<<<<<<<<<
 *         if max_mods is None:
 *             max_mods = -1
*/
    __pyx_t_2 = PySequence_List(__pyx_v_9pyteomics_7cparser_std_labels); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1011, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_labels, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":1010
 *         if fixed_mods is None:
 *             fixed_mods = {}
 *         if labels is None:             # <<<<<<<<<<<<<<
 *             labels = list(std_labels)
 *         if max_mods is None:
*/
  }

  /* "pyteomics/cparser.pyx":1012
 *         if labels is None:
 *             labels = list(std_labels)
 *         if max_mods is None:             # <<<<<<<<<<<<<<
 *             max_mods = -1
 *         self.variable_mods = self.coerce_modification_dict(variable_mods)
*/
  __pyx_t_1 = (__pyx_v_max_mods == Py_None);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":1013
 *             labels = list(std_labels)
 *         if max_mods is None:
 *             max_mods = -1             # <<<<<<<<<<<<<<
 *         self.variable_mods = self.coerce_modification_dict(variable_mods)
 *         self.fixed_mods = self.coerce_modification_dict(fixed_mods)
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_neg_1);
    __Pyx_DECREF_SET(__pyx_v_max_mods, __pyx_mstate_global->__pyx_int_neg_1);

    /* "pyteomics/cparser.pyx":1012
 *         if labels is None:
 *             labels = list(std_labels)
 *         if max_mods is None:             # <<<<<<<<<<<<<<
 *             max_mods = -1
 *         self.variable_mods = self.coerce_modification_dict(variable_mods)
*/
  }

  /* "pyteomics/cparser.pyx":1014
 *         if max_mods is None:
 *             max_mods = -1
 *         self.variable_mods = self.coerce_modification_dict(variable_mods)             # <<<<<<<<<<<<<<
 *         self.fixed_mods = self.coerce_modification_dict(fixed_mods)
 *         self.labels = labels
*/
  if (!(likely(PyDict_CheckExact(__pyx_v_variable_mods))||((__pyx_v_variable_mods) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_v_variable_mods))) __PYX_ERR(0, 1014, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self->__pyx_vtab)->coerce_modification_dict(__pyx_v_self, ((PyObject*)__pyx_v_variable_mods), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1014, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->variable_mods);
  __Pyx_DECREF(__pyx_v_self->variable_mods);
  __pyx_v_self->variable_mods = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1015
 *             max_mods = -1
 *         self.variable_mods = self.coerce_modification_dict(variable_mods)
 *         self.fixed_mods = self.coerce_modification_dict(fixed_mods)             # <<<<<<<<<<<<<<
 *         self.labels = labels
 *         self.max_mods = PyInt_AsLong(max_mods)
*/
  if (!(likely(PyDict_CheckExact(__pyx_v_fixed_mods))||((__pyx_v_fixed_mods) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_v_fixed_mods))) __PYX_ERR(0, 1015, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self->__pyx_vtab)->coerce_modification_dict(__pyx_v_self, ((PyObject*)__pyx_v_fixed_mods), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1015, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->fixed_mods);
  __Pyx_DECREF(__pyx_v_self->fixed_mods);
  __pyx_v_self->fixed_mods = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1016
 *         self.variable_mods = self.coerce_modification_dict(variable_mods)
 *         self.fixed_mods = self.coerce_modification_dict(fixed_mods)
 *         self.labels = labels             # <<<<<<<<<<<<<<
 *         self.max_mods = PyInt_AsLong(max_mods)
 *         self.override = override
*/
  __pyx_t_2 = __pyx_v_labels;
  __Pyx_INCREF(__pyx_t_2);
  if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_2))) __PYX_ERR(0, 1016, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->labels);
  __Pyx_DECREF(__pyx_v_self->labels);
  __pyx_v_self->labels = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1017
 *         self.fixed_mods = self.coerce_modification_dict(fixed_mods)
 *         self.labels = labels
 *         self.max_mods = PyInt_AsLong(max_mods)             # <<<<<<<<<<<<<<
 *         self.override = override
 *         self.show_unmodified_termini = show_unmodified_termini
*/
  __pyx_v_self->max_mods = PyInt_AsLong(__pyx_v_max_mods);

  /* "pyteomics/cparser.pyx":1018
 *         self.labels = labels
 *         self.max_mods = PyInt_AsLong(max_mods)
 *         self.override = override             # <<<<<<<<<<<<<<
 *         self.show_unmodified_termini = show_unmodified_termini
 *         self.persist = persist
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_override); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1018, __pyx_L1_error)
  __pyx_v_self->override = __pyx_t_1;

  /* "pyteomics/cparser.pyx":1019
 *         self.max_mods = PyInt_AsLong(max_mods)
 *         self.override = override
 *         self.show_unmodified_termini = show_unmodified_termini             # <<<<<<<<<<<<<<
 *         self.persist = persist
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_show_unmodified_termini); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1019, __pyx_L1_error)
  __pyx_v_self->show_unmodified_termini = __pyx_t_1;

  /* "pyteomics/cparser.pyx":1020
 *         self.override = override
 *         self.show_unmodified_termini = show_unmodified_termini
 *         self.persist = persist             # <<<<<<<<<<<<<<
 * 
 *         self.sequence = None
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_persist); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1020, __pyx_L1_error)
  __pyx_v_self->persist = __pyx_t_1;

  /* "pyteomics/cparser.pyx":1022
 *         self.persist = persist
 * 
 *         self.sequence = None             # <<<<<<<<<<<<<<
 *         self.original_sequence = None
 *         self.last_indices = []
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->sequence);
  __Pyx_DECREF(__pyx_v_self->sequence);
  __pyx_v_self->sequence = ((PyObject*)Py_None);

  /* "pyteomics/cparser.pyx":1023
 * 
 *         self.sequence = None
 *         self.original_sequence = None             # <<<<<<<<<<<<<<
 *         self.last_indices = []
 * 
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->original_sequence);
  __Pyx_DECREF(__pyx_v_self->original_sequence);
  __pyx_v_self->original_sequence = ((PyObject*)Py_None);

  /* "pyteomics/cparser.pyx":1024
 *         self.sequence = None
 *         self.original_sequence = None
 *         self.last_indices = []             # <<<<<<<<<<<<<<
 * 
 *     cpdef dict coerce_modification_dict(self, dict source):
*/
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1024, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->last_indices);
  __Pyx_DECREF(__pyx_v_self->last_indices);
  __pyx_v_self->last_indices = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1004
 *         public list last_indices
 * 
 *     def __init__(self, variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False,             # <<<<<<<<<<<<<<
 *                  show_unmodified_termini=False, persist=True):
 *         if variable_mods is None:
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_variable_mods);
  __Pyx_XDECREF(__pyx_v_fixed_mods);
  __Pyx_XDECREF(__pyx_v_labels);
  __Pyx_XDECREF(__pyx_v_max_mods);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1026
 *         self.last_indices = []
 * 
 *     cpdef dict coerce_modification_dict(self, dict source):             # <<<<<<<<<<<<<<
 *         cdef:
 *             str label
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_3coerce_modification_dict(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16IsoformGenerator_coerce_modification_dict(CYTHON_UNUSED struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_source, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_label = 0;
  struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_rule = 0;
  PyObject *__pyx_v_targets = 0;
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("coerce_modification_dict", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_coerce_modification_dict); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1026, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_3coerce_modification_dict)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_source};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1026, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        if (!(likely(PyDict_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_2))) __PYX_ERR(0, 1026, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "pyteomics/cparser.pyx":1032
 *             object targets
 *             dict result
 *         result = {}             # <<<<<<<<<<<<<<
 *         for label, targets in source.items():
 *             rule = ModificationRule(label, list(targets))
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1032, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1033
 *             dict result
 *         result = {}
 *         for label, targets in source.items():             # <<<<<<<<<<<<<<
 *             rule = ModificationRule(label, list(targets))
 *             PyDict_SetItem(result, label, rule)
*/
  __pyx_t_6 = 0;
  if (unlikely(__pyx_v_source == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
    __PYX_ERR(0, 1033, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_dict_iterator(__pyx_v_source, 1, __pyx_mstate_global->__pyx_n_u_items, (&__pyx_t_7), (&__pyx_t_8)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1033, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_1);
  __pyx_t_1 = __pyx_t_2;
  __pyx_t_2 = 0;
  while (1) {
    __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_7, &__pyx_t_6, &__pyx_t_2, &__pyx_t_4, NULL, __pyx_t_8);
    if (unlikely(__pyx_t_9 == 0)) break;
    if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 1033, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GOTREF(__pyx_t_4);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 1033, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_label, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;
    __Pyx_XDECREF_SET(__pyx_v_targets, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "pyteomics/cparser.pyx":1034
 *         result = {}
 *         for label, targets in source.items():
 *             rule = ModificationRule(label, list(targets))             # <<<<<<<<<<<<<<
 *             PyDict_SetItem(result, label, rule)
 *         return result
*/
    __pyx_t_2 = NULL;
    __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationRule);
    __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationRule); 
    __pyx_t_10 = PySequence_List(__pyx_v_targets); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1034, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_v_label, __pyx_t_10};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1034, __pyx_L1_error)
      __Pyx_GOTREF((PyObject *)__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_rule, ((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "pyteomics/cparser.pyx":1035
 *         for label, targets in source.items():
 *             rule = ModificationRule(label, list(targets))
 *             PyDict_SetItem(result, label, rule)             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
    __pyx_t_9 = PyDict_SetItem(__pyx_v_result, __pyx_v_label, ((PyObject *)__pyx_v_rule)); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1035, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1036
 *             rule = ModificationRule(label, list(targets))
 *             PyDict_SetItem(result, label, rule)
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cpdef list initialize_from_str(self, str sequence):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1026
 *         self.last_indices = []
 * 
 *     cpdef dict coerce_modification_dict(self, dict source):             # <<<<<<<<<<<<<<
 *         cdef:
 *             str label
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.coerce_modification_dict", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_label);
  __Pyx_XDECREF((PyObject *)__pyx_v_rule);
  __Pyx_XDECREF(__pyx_v_targets);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_3coerce_modification_dict(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16IsoformGenerator_2coerce_modification_dict, "IsoformGenerator.coerce_modification_dict(self, dict source) -> dict");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_3coerce_modification_dict = {"coerce_modification_dict", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_3coerce_modification_dict, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_2coerce_modification_dict};
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_3coerce_modification_dict(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_source = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("coerce_modification_dict (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_source,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1026, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1026, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "coerce_modification_dict", 0) < 0) __PYX_ERR(0, 1026, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("coerce_modification_dict", 1, 1, 1, i); __PYX_ERR(0, 1026, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1026, __pyx_L3_error)
    }
    __pyx_v_source = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("coerce_modification_dict", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1026, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.coerce_modification_dict", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_source), (&PyDict_Type), 1, "source", 1))) __PYX_ERR(0, 1026, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_2coerce_modification_dict(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), __pyx_v_source);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_2coerce_modification_dict(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_source) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("coerce_modification_dict", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16IsoformGenerator_coerce_modification_dict(__pyx_v_self, __pyx_v_source, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.coerce_modification_dict", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1038
 *         return result
 * 
 *     cpdef list initialize_from_str(self, str sequence):             # <<<<<<<<<<<<<<
 *         labels = self.labels + list(self.fixed_mods)
 *         self.sequence = SequencePosition.parse(sequence, labels=labels)
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_5initialize_from_str(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16IsoformGenerator_initialize_from_str(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_labels = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("initialize_from_str", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_initialize_from_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1038, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_5initialize_from_str)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_sequence};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1038, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_2))) __PYX_ERR(0, 1038, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "pyteomics/cparser.pyx":1039
 * 
 *     cpdef list initialize_from_str(self, str sequence):
 *         labels = self.labels + list(self.fixed_mods)             # <<<<<<<<<<<<<<
 *         self.sequence = SequencePosition.parse(sequence, labels=labels)
 *         return self.sequence
*/
  __pyx_t_1 = PySequence_List(__pyx_v_self->fixed_mods); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1039, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_v_self->labels, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1039, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_labels = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1040
 *     cpdef list initialize_from_str(self, str sequence):
 *         labels = self.labels + list(self.fixed_mods)
 *         self.sequence = SequencePosition.parse(sequence, labels=labels)             # <<<<<<<<<<<<<<
 *         return self.sequence
 * 
*/
  __pyx_t_1 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_v_sequence};
    __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1040, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_labels, __pyx_v_labels, __pyx_t_4, __pyx_callargs+2, 0) < 0) __PYX_ERR(0, 1040, __pyx_L1_error)
    __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_parse, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1040, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_2))) __PYX_ERR(0, 1040, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->sequence);
  __Pyx_DECREF(__pyx_v_self->sequence);
  __pyx_v_self->sequence = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1041
 *         labels = self.labels + list(self.fixed_mods)
 *         self.sequence = SequencePosition.parse(sequence, labels=labels)
 *         return self.sequence             # <<<<<<<<<<<<<<
 * 
 *     cpdef list initialize_from_iterable(self, object sequence):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->sequence);
  __pyx_r = __pyx_v_self->sequence;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1038
 *         return result
 * 
 *     cpdef list initialize_from_str(self, str sequence):             # <<<<<<<<<<<<<<
 *         labels = self.labels + list(self.fixed_mods)
 *         self.sequence = SequencePosition.parse(sequence, labels=labels)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.initialize_from_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_labels);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_5initialize_from_str(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16IsoformGenerator_4initialize_from_str, "IsoformGenerator.initialize_from_str(self, str sequence) -> list");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_5initialize_from_str = {"initialize_from_str", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_5initialize_from_str, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_4initialize_from_str};
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_5initialize_from_str(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("initialize_from_str (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1038, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1038, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "initialize_from_str", 0) < 0) __PYX_ERR(0, 1038, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("initialize_from_str", 1, 1, 1, i); __PYX_ERR(0, 1038, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1038, __pyx_L3_error)
    }
    __pyx_v_sequence = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("initialize_from_str", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1038, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.initialize_from_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sequence), (&PyUnicode_Type), 1, "sequence", 1))) __PYX_ERR(0, 1038, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_4initialize_from_str(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), __pyx_v_sequence);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_4initialize_from_str(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("initialize_from_str", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16IsoformGenerator_initialize_from_str(__pyx_v_self, __pyx_v_sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1038, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.initialize_from_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1043
 *         return self.sequence
 * 
 *     cpdef list initialize_from_iterable(self, object sequence):             # <<<<<<<<<<<<<<
 *         self.sequence = SequencePosition.from_iterable(sequence)
 *         return self.sequence
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7initialize_from_iterable(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16IsoformGenerator_initialize_from_iterable(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("initialize_from_iterable", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_initialize_from_iterable); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1043, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7initialize_from_iterable)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_sequence};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1043, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_2))) __PYX_ERR(0, 1043, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "pyteomics/cparser.pyx":1044
 * 
 *     cpdef list initialize_from_iterable(self, object sequence):
 *         self.sequence = SequencePosition.from_iterable(sequence)             # <<<<<<<<<<<<<<
 *         return self.sequence
 * 
*/
  __pyx_t_2 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_sequence};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_from_iterable, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1044, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(0, 1044, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->sequence);
  __Pyx_DECREF(__pyx_v_self->sequence);
  __pyx_v_self->sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1045
 *     cpdef list initialize_from_iterable(self, object sequence):
 *         self.sequence = SequencePosition.from_iterable(sequence)
 *         return self.sequence             # <<<<<<<<<<<<<<
 * 
 *     cpdef list apply_fixed_mods(self, list sequence):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->sequence);
  __pyx_r = __pyx_v_self->sequence;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1043
 *         return self.sequence
 * 
 *     cpdef list initialize_from_iterable(self, object sequence):             # <<<<<<<<<<<<<<
 *         self.sequence = SequencePosition.from_iterable(sequence)
 *         return self.sequence
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.initialize_from_iterable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7initialize_from_iterable(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16IsoformGenerator_6initialize_from_iterable, "IsoformGenerator.initialize_from_iterable(self, sequence) -> list");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_7initialize_from_iterable = {"initialize_from_iterable", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7initialize_from_iterable, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_6initialize_from_iterable};
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7initialize_from_iterable(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("initialize_from_iterable (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1043, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1043, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "initialize_from_iterable", 0) < 0) __PYX_ERR(0, 1043, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("initialize_from_iterable", 1, 1, 1, i); __PYX_ERR(0, 1043, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1043, __pyx_L3_error)
    }
    __pyx_v_sequence = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("initialize_from_iterable", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1043, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.initialize_from_iterable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6initialize_from_iterable(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), __pyx_v_sequence);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6initialize_from_iterable(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("initialize_from_iterable", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16IsoformGenerator_initialize_from_iterable(__pyx_v_self, __pyx_v_sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1043, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.initialize_from_iterable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1047
 *         return self.sequence
 * 
 *     cpdef list apply_fixed_mods(self, list sequence):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_9apply_fixed_mods(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16IsoformGenerator_apply_fixed_mods(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, int __pyx_skip_dispatch) {
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  Py_ssize_t __pyx_v_j;
  PyObject *__pyx_v_pkey;
  PyObject *__pyx_v_pvalue;
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position = 0;
  struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_rule = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  PyObject *__pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("apply_fixed_mods", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_apply_fixed_mods); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_9apply_fixed_mods)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_sequence};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1047, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_2))) __PYX_ERR(0, 1047, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "pyteomics/cparser.pyx":1056
 *             ModificationRule rule
 * 
 *         n = PyList_Size(sequence)             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             position = <SequencePosition>PyList_GET_ITEM(sequence, i)
*/
  __pyx_t_6 = PyList_Size(__pyx_v_sequence); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1056, __pyx_L1_error)
  __pyx_v_n = __pyx_t_6;

  /* "pyteomics/cparser.pyx":1057
 * 
 *         n = PyList_Size(sequence)
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             position = <SequencePosition>PyList_GET_ITEM(sequence, i)
 *             j = 0
*/
  __pyx_t_5 = __pyx_v_n;
  __pyx_t_7 = __pyx_t_5;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "pyteomics/cparser.pyx":1058
 *         n = PyList_Size(sequence)
 *         for i in range(n):
 *             position = <SequencePosition>PyList_GET_ITEM(sequence, i)             # <<<<<<<<<<<<<<
 *             j = 0
 *             while PyDict_Next(self.fixed_mods, &j, &pkey, &pvalue):
*/
    __pyx_t_9 = PyList_GET_ITEM(__pyx_v_sequence, __pyx_v_i);
    __pyx_t_1 = ((PyObject *)__pyx_t_9);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":1059
 *         for i in range(n):
 *             position = <SequencePosition>PyList_GET_ITEM(sequence, i)
 *             j = 0             # <<<<<<<<<<<<<<
 *             while PyDict_Next(self.fixed_mods, &j, &pkey, &pvalue):
 *                 rule = <ModificationRule>pvalue
*/
    __pyx_v_j = 0;

    /* "pyteomics/cparser.pyx":1060
 *             position = <SequencePosition>PyList_GET_ITEM(sequence, i)
 *             j = 0
 *             while PyDict_Next(self.fixed_mods, &j, &pkey, &pvalue):             # <<<<<<<<<<<<<<
 *                 rule = <ModificationRule>pvalue
 *                 if rule.test(position):
*/
    while (1) {
      __pyx_t_1 = __pyx_v_self->fixed_mods;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_10 = (PyDict_Next(__pyx_t_1, (&__pyx_v_j), (&__pyx_v_pkey), (&__pyx_v_pvalue)) != 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) break;

      /* "pyteomics/cparser.pyx":1061
 *             j = 0
 *             while PyDict_Next(self.fixed_mods, &j, &pkey, &pvalue):
 *                 rule = <ModificationRule>pvalue             # <<<<<<<<<<<<<<
 *                 if rule.test(position):
 *                     if not position.is_modified():
*/
      __pyx_t_1 = ((PyObject *)__pyx_v_pvalue);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_rule, ((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":1062
 *             while PyDict_Next(self.fixed_mods, &j, &pkey, &pvalue):
 *                 rule = <ModificationRule>pvalue
 *                 if rule.test(position):             # <<<<<<<<<<<<<<
 *                     if not position.is_modified():
 *                         position.add_modification(rule.modification)
*/
      __pyx_t_10 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationRule *)__pyx_v_rule->__pyx_vtab)->test(__pyx_v_rule, __pyx_v_position, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1062, __pyx_L1_error)
      if (__pyx_t_10) {

        /* "pyteomics/cparser.pyx":1063
 *                 rule = <ModificationRule>pvalue
 *                 if rule.test(position):
 *                     if not position.is_modified():             # <<<<<<<<<<<<<<
 *                         position.add_modification(rule.modification)
 *         return sequence
*/
        __pyx_t_10 = __pyx_f_9pyteomics_7cparser_16SequencePosition_is_modified(__pyx_v_position); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1063, __pyx_L1_error)
        __pyx_t_11 = (!__pyx_t_10);
        if (__pyx_t_11) {

          /* "pyteomics/cparser.pyx":1064
 *                 if rule.test(position):
 *                     if not position.is_modified():
 *                         position.add_modification(rule.modification)             # <<<<<<<<<<<<<<
 *         return sequence
 * 
*/
          __pyx_t_1 = __pyx_v_rule->modification;
          __Pyx_INCREF(__pyx_t_1);
          __pyx_t_2 = __pyx_f_9pyteomics_7cparser_16SequencePosition_add_modification(__pyx_v_position, ((PyObject*)__pyx_t_1), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1064, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyteomics/cparser.pyx":1063
 *                 rule = <ModificationRule>pvalue
 *                 if rule.test(position):
 *                     if not position.is_modified():             # <<<<<<<<<<<<<<
 *                         position.add_modification(rule.modification)
 *         return sequence
*/
        }

        /* "pyteomics/cparser.pyx":1062
 *             while PyDict_Next(self.fixed_mods, &j, &pkey, &pvalue):
 *                 rule = <ModificationRule>pvalue
 *                 if rule.test(position):             # <<<<<<<<<<<<<<
 *                     if not position.is_modified():
 *                         position.add_modification(rule.modification)
*/
      }
    }
  }

  /* "pyteomics/cparser.pyx":1065
 *                     if not position.is_modified():
 *                         position.add_modification(rule.modification)
 *         return sequence             # <<<<<<<<<<<<<<
 * 
 *     cpdef dict build_site_map(self, list sequence, bint include_unmodified=True):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_sequence);
  __pyx_r = __pyx_v_sequence;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1047
 *         return self.sequence
 * 
 *     cpdef list apply_fixed_mods(self, list sequence):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.apply_fixed_mods", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_XDECREF((PyObject *)__pyx_v_rule);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_9apply_fixed_mods(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16IsoformGenerator_8apply_fixed_mods, "IsoformGenerator.apply_fixed_mods(self, list sequence) -> list");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_9apply_fixed_mods = {"apply_fixed_mods", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_9apply_fixed_mods, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_8apply_fixed_mods};
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_9apply_fixed_mods(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("apply_fixed_mods (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1047, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1047, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "apply_fixed_mods", 0) < 0) __PYX_ERR(0, 1047, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("apply_fixed_mods", 1, 1, 1, i); __PYX_ERR(0, 1047, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1047, __pyx_L3_error)
    }
    __pyx_v_sequence = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("apply_fixed_mods", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1047, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.apply_fixed_mods", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sequence), (&PyList_Type), 1, "sequence", 1))) __PYX_ERR(0, 1047, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8apply_fixed_mods(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), __pyx_v_sequence);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8apply_fixed_mods(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("apply_fixed_mods", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16IsoformGenerator_apply_fixed_mods(__pyx_v_self, __pyx_v_sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1047, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.apply_fixed_mods", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1067
 *         return sequence
 * 
 *     cpdef dict build_site_map(self, list sequence, bint include_unmodified=True):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_11build_site_map(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_16IsoformGenerator_build_site_map(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_16IsoformGenerator_build_site_map *__pyx_optional_args) {
  int __pyx_v_include_unmodified = ((int)1);
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  Py_ssize_t __pyx_v_j;
  PyObject *__pyx_v_pkey;
  PyObject *__pyx_v_pvalue;
  PyObject *__pyx_v_psites;
  PyObject *__pyx_v_sites = 0;
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position = 0;
  struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_rule = 0;
  PyObject *__pyx_v_site_map = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("build_site_map", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_include_unmodified = __pyx_optional_args->include_unmodified;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_build_site_map); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1067, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_11build_site_map)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_include_unmodified); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1067, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_sequence, __pyx_t_5};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1067, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        if (!(likely(PyDict_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_2))) __PYX_ERR(0, 1067, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "pyteomics/cparser.pyx":1079
 *             dict site_map
 * 
 *         site_map = dict()             # <<<<<<<<<<<<<<
 *         n = PyList_Size(sequence)
 *         for i in range(n):
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1079, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_site_map = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1080
 * 
 *         site_map = dict()
 *         n = PyList_Size(sequence)             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             position = <SequencePosition>PyList_GET_ITEM(sequence, i)
*/
  __pyx_t_7 = PyList_Size(__pyx_v_sequence); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1080, __pyx_L1_error)
  __pyx_v_n = __pyx_t_7;

  /* "pyteomics/cparser.pyx":1081
 *         site_map = dict()
 *         n = PyList_Size(sequence)
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             position = <SequencePosition>PyList_GET_ITEM(sequence, i)
 *             j = 0
*/
  __pyx_t_6 = __pyx_v_n;
  __pyx_t_8 = __pyx_t_6;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "pyteomics/cparser.pyx":1082
 *         n = PyList_Size(sequence)
 *         for i in range(n):
 *             position = <SequencePosition>PyList_GET_ITEM(sequence, i)             # <<<<<<<<<<<<<<
 *             j = 0
 *             while PyDict_Next(self.variable_mods, &j, &pkey, &pvalue):
*/
    __pyx_t_10 = PyList_GET_ITEM(__pyx_v_sequence, __pyx_v_i);
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":1083
 *         for i in range(n):
 *             position = <SequencePosition>PyList_GET_ITEM(sequence, i)
 *             j = 0             # <<<<<<<<<<<<<<
 *             while PyDict_Next(self.variable_mods, &j, &pkey, &pvalue):
 *                 rule = <ModificationRule>pvalue
*/
    __pyx_v_j = 0;

    /* "pyteomics/cparser.pyx":1084
 *             position = <SequencePosition>PyList_GET_ITEM(sequence, i)
 *             j = 0
 *             while PyDict_Next(self.variable_mods, &j, &pkey, &pvalue):             # <<<<<<<<<<<<<<
 *                 rule = <ModificationRule>pvalue
 *                 if rule.test(position):
*/
    while (1) {
      __pyx_t_1 = __pyx_v_self->variable_mods;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_11 = (PyDict_Next(__pyx_t_1, (&__pyx_v_j), (&__pyx_v_pkey), (&__pyx_v_pvalue)) != 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_11) break;

      /* "pyteomics/cparser.pyx":1085
 *             j = 0
 *             while PyDict_Next(self.variable_mods, &j, &pkey, &pvalue):
 *                 rule = <ModificationRule>pvalue             # <<<<<<<<<<<<<<
 *                 if rule.test(position):
 *                     if not position.is_modified():
*/
      __pyx_t_1 = ((PyObject *)__pyx_v_pvalue);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_rule, ((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":1086
 *             while PyDict_Next(self.variable_mods, &j, &pkey, &pvalue):
 *                 rule = <ModificationRule>pvalue
 *                 if rule.test(position):             # <<<<<<<<<<<<<<
 *                     if not position.is_modified():
 *                         psites = PyDict_GetItem(site_map, i)
*/
      __pyx_t_11 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationRule *)__pyx_v_rule->__pyx_vtab)->test(__pyx_v_rule, __pyx_v_position, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1086, __pyx_L1_error)
      if (__pyx_t_11) {

        /* "pyteomics/cparser.pyx":1087
 *                 rule = <ModificationRule>pvalue
 *                 if rule.test(position):
 *                     if not position.is_modified():             # <<<<<<<<<<<<<<
 *                         psites = PyDict_GetItem(site_map, i)
 *                         if psites == NULL:
*/
        __pyx_t_11 = __pyx_f_9pyteomics_7cparser_16SequencePosition_is_modified(__pyx_v_position); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1087, __pyx_L1_error)
        __pyx_t_12 = (!__pyx_t_11);
        if (__pyx_t_12) {

          /* "pyteomics/cparser.pyx":1088
 *                 if rule.test(position):
 *                     if not position.is_modified():
 *                         psites = PyDict_GetItem(site_map, i)             # <<<<<<<<<<<<<<
 *                         if psites == NULL:
 *                             sites = []
*/
          __pyx_t_1 = __Pyx_PyLong_FromSize_t(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1088, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_v_psites = PyDict_GetItem(__pyx_v_site_map, __pyx_t_1);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "pyteomics/cparser.pyx":1089
 *                     if not position.is_modified():
 *                         psites = PyDict_GetItem(site_map, i)
 *                         if psites == NULL:             # <<<<<<<<<<<<<<
 *                             sites = []
 *                             PyDict_SetItem(site_map, i, sites)
*/
          __pyx_t_12 = (__pyx_v_psites == NULL);
          if (__pyx_t_12) {

            /* "pyteomics/cparser.pyx":1090
 *                         psites = PyDict_GetItem(site_map, i)
 *                         if psites == NULL:
 *                             sites = []             # <<<<<<<<<<<<<<
 *                             PyDict_SetItem(site_map, i, sites)
 *                         else:
*/
            __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1090, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_XDECREF_SET(__pyx_v_sites, ((PyObject*)__pyx_t_1));
            __pyx_t_1 = 0;

            /* "pyteomics/cparser.pyx":1091
 *                         if psites == NULL:
 *                             sites = []
 *                             PyDict_SetItem(site_map, i, sites)             # <<<<<<<<<<<<<<
 *                         else:
 *                             sites = <list>psites
*/
            __pyx_t_1 = __Pyx_PyLong_FromSize_t(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1091, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_13 = PyDict_SetItem(__pyx_v_site_map, __pyx_t_1, __pyx_v_sites); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 1091, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "pyteomics/cparser.pyx":1089
 *                     if not position.is_modified():
 *                         psites = PyDict_GetItem(site_map, i)
 *                         if psites == NULL:             # <<<<<<<<<<<<<<
 *                             sites = []
 *                             PyDict_SetItem(site_map, i, sites)
*/
            goto __pyx_L9;
          }

          /* "pyteomics/cparser.pyx":1093
 *                             PyDict_SetItem(site_map, i, sites)
 *                         else:
 *                             sites = <list>psites             # <<<<<<<<<<<<<<
 *                         position = position.copy()
 *                         position.modification = rule.modification
*/
          /*else*/ {
            __pyx_t_1 = ((PyObject *)__pyx_v_psites);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_XDECREF_SET(__pyx_v_sites, ((PyObject*)__pyx_t_1));
            __pyx_t_1 = 0;
          }
          __pyx_L9:;

          /* "pyteomics/cparser.pyx":1094
 *                         else:
 *                             sites = <list>psites
 *                         position = position.copy()             # <<<<<<<<<<<<<<
 *                         position.modification = rule.modification
 *                         sites.append(position)
*/
          __pyx_t_1 = ((PyObject *)__pyx_f_9pyteomics_7cparser_16SequencePosition_copy(__pyx_v_position)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1094, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF_SET(__pyx_v_position, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "pyteomics/cparser.pyx":1095
 *                             sites = <list>psites
 *                         position = position.copy()
 *                         position.modification = rule.modification             # <<<<<<<<<<<<<<
 *                         sites.append(position)
 *         if include_unmodified:
*/
          __pyx_t_1 = __pyx_v_rule->modification;
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_v_position->modification);
          __Pyx_DECREF(__pyx_v_position->modification);
          __pyx_v_position->modification = ((PyObject*)__pyx_t_1);
          __pyx_t_1 = 0;

          /* "pyteomics/cparser.pyx":1096
 *                         position = position.copy()
 *                         position.modification = rule.modification
 *                         sites.append(position)             # <<<<<<<<<<<<<<
 *         if include_unmodified:
 *             j = 0
*/
          if (unlikely(__pyx_v_sites == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
            __PYX_ERR(0, 1096, __pyx_L1_error)
          }
          __pyx_t_14 = __Pyx_PyList_Append(__pyx_v_sites, ((PyObject *)__pyx_v_position)); if (unlikely(__pyx_t_14 == ((int)-1))) __PYX_ERR(0, 1096, __pyx_L1_error)

          /* "pyteomics/cparser.pyx":1087
 *                 rule = <ModificationRule>pvalue
 *                 if rule.test(position):
 *                     if not position.is_modified():             # <<<<<<<<<<<<<<
 *                         psites = PyDict_GetItem(site_map, i)
 *                         if psites == NULL:
*/
        }

        /* "pyteomics/cparser.pyx":1086
 *             while PyDict_Next(self.variable_mods, &j, &pkey, &pvalue):
 *                 rule = <ModificationRule>pvalue
 *                 if rule.test(position):             # <<<<<<<<<<<<<<
 *                     if not position.is_modified():
 *                         psites = PyDict_GetItem(site_map, i)
*/
      }
    }
  }

  /* "pyteomics/cparser.pyx":1097
 *                         position.modification = rule.modification
 *                         sites.append(position)
 *         if include_unmodified:             # <<<<<<<<<<<<<<
 *             j = 0
 *             while PyDict_Next(site_map, &j, &pkey, &pvalue):
*/
  if (__pyx_v_include_unmodified) {

    /* "pyteomics/cparser.pyx":1098
 *                         sites.append(position)
 *         if include_unmodified:
 *             j = 0             # <<<<<<<<<<<<<<
 *             while PyDict_Next(site_map, &j, &pkey, &pvalue):
 *                 position = <SequencePosition>PyList_GET_ITEM(sequence, PyInt_AsLong(<object>pkey))
*/
    __pyx_v_j = 0;

    /* "pyteomics/cparser.pyx":1099
 *         if include_unmodified:
 *             j = 0
 *             while PyDict_Next(site_map, &j, &pkey, &pvalue):             # <<<<<<<<<<<<<<
 *                 position = <SequencePosition>PyList_GET_ITEM(sequence, PyInt_AsLong(<object>pkey))
 *                 position = position.copy()
*/
    while (1) {
      __pyx_t_12 = (PyDict_Next(__pyx_v_site_map, (&__pyx_v_j), (&__pyx_v_pkey), (&__pyx_v_pvalue)) != 0);
      if (!__pyx_t_12) break;

      /* "pyteomics/cparser.pyx":1100
 *             j = 0
 *             while PyDict_Next(site_map, &j, &pkey, &pvalue):
 *                 position = <SequencePosition>PyList_GET_ITEM(sequence, PyInt_AsLong(<object>pkey))             # <<<<<<<<<<<<<<
 *                 position = position.copy()
 *                 (<list>pvalue).append(position)
*/
      __pyx_t_10 = PyList_GET_ITEM(__pyx_v_sequence, PyInt_AsLong(((PyObject *)__pyx_v_pkey)));
      __pyx_t_1 = ((PyObject *)__pyx_t_10);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":1101
 *             while PyDict_Next(site_map, &j, &pkey, &pvalue):
 *                 position = <SequencePosition>PyList_GET_ITEM(sequence, PyInt_AsLong(<object>pkey))
 *                 position = position.copy()             # <<<<<<<<<<<<<<
 *                 (<list>pvalue).append(position)
 *         return site_map
*/
      __pyx_t_1 = ((PyObject *)__pyx_f_9pyteomics_7cparser_16SequencePosition_copy(__pyx_v_position)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1101, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_position, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":1102
 *                 position = <SequencePosition>PyList_GET_ITEM(sequence, PyInt_AsLong(<object>pkey))
 *                 position = position.copy()
 *                 (<list>pvalue).append(position)             # <<<<<<<<<<<<<<
 *         return site_map
 * 
*/
      if (unlikely(((PyObject *)__pyx_v_pvalue) == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
        __PYX_ERR(0, 1102, __pyx_L1_error)
      }
      __pyx_t_14 = __Pyx_PyList_Append(((PyObject*)__pyx_v_pvalue), ((PyObject *)__pyx_v_position)); if (unlikely(__pyx_t_14 == ((int)-1))) __PYX_ERR(0, 1102, __pyx_L1_error)
    }

    /* "pyteomics/cparser.pyx":1097
 *                         position.modification = rule.modification
 *                         sites.append(position)
 *         if include_unmodified:             # <<<<<<<<<<<<<<
 *             j = 0
 *             while PyDict_Next(site_map, &j, &pkey, &pvalue):
*/
  }

  /* "pyteomics/cparser.pyx":1103
 *                 position = position.copy()
 *                 (<list>pvalue).append(position)
 *         return site_map             # <<<<<<<<<<<<<<
 * 
 *     @cython.boundscheck(False)
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_site_map);
  __pyx_r = __pyx_v_site_map;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1067
 *         return sequence
 * 
 *     cpdef dict build_site_map(self, list sequence, bint include_unmodified=True):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.build_site_map", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_sites);
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_XDECREF((PyObject *)__pyx_v_rule);
  __Pyx_XDECREF(__pyx_v_site_map);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_11build_site_map(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16IsoformGenerator_10build_site_map, "IsoformGenerator.build_site_map(self, list sequence, bool include_unmodified=True) -> dict");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_11build_site_map = {"build_site_map", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_11build_site_map, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_10build_site_map};
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_11build_site_map(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  int __pyx_v_include_unmodified;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("build_site_map (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,&__pyx_mstate_global->__pyx_n_u_include_unmodified,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1067, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1067, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1067, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "build_site_map", 0) < 0) __PYX_ERR(0, 1067, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("build_site_map", 0, 1, 2, i); __PYX_ERR(0, 1067, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1067, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1067, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sequence = ((PyObject*)values[0]);
    if (values[1]) {
      __pyx_v_include_unmodified = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_include_unmodified == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1067, __pyx_L3_error)
    } else {
      __pyx_v_include_unmodified = ((int)1);
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("build_site_map", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 1067, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.build_site_map", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sequence), (&PyList_Type), 1, "sequence", 1))) __PYX_ERR(0, 1067, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10build_site_map(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), __pyx_v_sequence, __pyx_v_include_unmodified);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10build_site_map(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, int __pyx_v_include_unmodified) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_9pyteomics_7cparser_16IsoformGenerator_build_site_map __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("build_site_map", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.include_unmodified = __pyx_v_include_unmodified;
  __pyx_t_1 = __pyx_vtabptr_9pyteomics_7cparser_IsoformGenerator->build_site_map(__pyx_v_self, __pyx_v_sequence, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1067, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.build_site_map", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_9pyteomics_7cparser_16IsoformGenerator_14generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyteomics/cparser.pyx":1105
 *         return site_map
 * 
 *     @cython.boundscheck(False)             # <<<<<<<<<<<<<<
 *     def generate(self, sequence_spec):
 *         cdef:
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13generate(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16IsoformGenerator_12generate, "IsoformGenerator.generate(self, sequence_spec)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_13generate = {"generate", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13generate, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_12generate};
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13generate(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence_spec = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("generate (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence_spec,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1105, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1105, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "generate", 0) < 0) __PYX_ERR(0, 1105, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("generate", 1, 1, 1, i); __PYX_ERR(0, 1105, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1105, __pyx_L3_error)
    }
    __pyx_v_sequence_spec = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("generate", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1105, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.generate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12generate(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), __pyx_v_sequence_spec);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12generate(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence_spec) {
  struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("generate", 0);
  __pyx_cur_scope = (struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *)__pyx_tp_new_9pyteomics_7cparser___pyx_scope_struct__generate(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1105, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_sequence_spec = __pyx_v_sequence_spec;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_sequence_spec);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_sequence_spec);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_9pyteomics_7cparser_16IsoformGenerator_14generator, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[0]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_generate, __pyx_mstate_global->__pyx_n_u_IsoformGenerator_generate, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser); if (unlikely(!gen)) __PYX_ERR(0, 1105, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.generate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_9pyteomics_7cparser_16IsoformGenerator_14generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *__pyx_cur_scope = ((struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  long __pyx_t_12;
  long __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  Py_ssize_t __pyx_t_15;
  size_t __pyx_t_16;
  PyObject *(*__pyx_t_17)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("generate", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L13_resume_from_yield;
    case 2: goto __pyx_L23_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(0, 1105, __pyx_L1_error)
  }

  /* "pyteomics/cparser.pyx":1118
 *             long* pindices
 * 
 *         if isinstance(sequence_spec, str):             # <<<<<<<<<<<<<<
 *             self.initialize_from_str(sequence_spec)
 *         else:
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_cur_scope->__pyx_v_sequence_spec); 
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":1119
 * 
 *         if isinstance(sequence_spec, str):
 *             self.initialize_from_str(sequence_spec)             # <<<<<<<<<<<<<<
 *         else:
 *             self.initialize_from_iterable(sequence_spec)
*/
    if (!(likely(PyUnicode_CheckExact(__pyx_cur_scope->__pyx_v_sequence_spec))||((__pyx_cur_scope->__pyx_v_sequence_spec) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_cur_scope->__pyx_v_sequence_spec))) __PYX_ERR(0, 1119, __pyx_L1_error)
    __pyx_t_2 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->initialize_from_str(__pyx_cur_scope->__pyx_v_self, ((PyObject*)__pyx_cur_scope->__pyx_v_sequence_spec), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":1118
 *             long* pindices
 * 
 *         if isinstance(sequence_spec, str):             # <<<<<<<<<<<<<<
 *             self.initialize_from_str(sequence_spec)
 *         else:
*/
    goto __pyx_L4;
  }

  /* "pyteomics/cparser.pyx":1121
 *             self.initialize_from_str(sequence_spec)
 *         else:
 *             self.initialize_from_iterable(sequence_spec)             # <<<<<<<<<<<<<<
 * 
 *         self.apply_fixed_mods(self.sequence)
*/
  /*else*/ {
    __pyx_t_2 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->initialize_from_iterable(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_sequence_spec, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_L4:;

  /* "pyteomics/cparser.pyx":1123
 *             self.initialize_from_iterable(sequence_spec)
 * 
 *         self.apply_fixed_mods(self.sequence)             # <<<<<<<<<<<<<<
 *         variable_mod_sites = self.build_site_map(self.sequence)
 *         indices = PyDict_Keys(variable_mod_sites)
*/
  __pyx_t_2 = __pyx_cur_scope->__pyx_v_self->sequence;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->apply_fixed_mods(__pyx_cur_scope->__pyx_v_self, ((PyObject*)__pyx_t_2), 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyteomics/cparser.pyx":1124
 * 
 *         self.apply_fixed_mods(self.sequence)
 *         variable_mod_sites = self.build_site_map(self.sequence)             # <<<<<<<<<<<<<<
 *         indices = PyDict_Keys(variable_mod_sites)
 *         assignments = PyDict_Values(variable_mod_sites)
*/
  __pyx_t_3 = __pyx_cur_scope->__pyx_v_self->sequence;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_2 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->build_site_map(__pyx_cur_scope->__pyx_v_self, ((PyObject*)__pyx_t_3), 0, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_variable_mod_sites = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1125
 *         self.apply_fixed_mods(self.sequence)
 *         variable_mod_sites = self.build_site_map(self.sequence)
 *         indices = PyDict_Keys(variable_mod_sites)             # <<<<<<<<<<<<<<
 *         assignments = PyDict_Values(variable_mod_sites)
 *         k = PyList_Size(indices)
*/
  __pyx_t_2 = PyDict_Keys(__pyx_cur_scope->__pyx_v_variable_mod_sites); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_indices = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1126
 *         variable_mod_sites = self.build_site_map(self.sequence)
 *         indices = PyDict_Keys(variable_mod_sites)
 *         assignments = PyDict_Values(variable_mod_sites)             # <<<<<<<<<<<<<<
 *         k = PyList_Size(indices)
 * 
*/
  __pyx_t_2 = PyDict_Values(__pyx_cur_scope->__pyx_v_variable_mod_sites); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_assignments = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1127
 *         indices = PyDict_Keys(variable_mod_sites)
 *         assignments = PyDict_Values(variable_mod_sites)
 *         k = PyList_Size(indices)             # <<<<<<<<<<<<<<
 * 
 *         if self.persist:
*/
  __pyx_t_4 = PyList_Size(__pyx_cur_scope->__pyx_v_indices); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1127, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_k = __pyx_t_4;

  /* "pyteomics/cparser.pyx":1129
 *         k = PyList_Size(indices)
 * 
 *         if self.persist:             # <<<<<<<<<<<<<<
 *             self.original_sequence = list(self.sequence)
 * 
*/
  if (__pyx_cur_scope->__pyx_v_self->persist) {

    /* "pyteomics/cparser.pyx":1130
 * 
 *         if self.persist:
 *             self.original_sequence = list(self.sequence)             # <<<<<<<<<<<<<<
 * 
 *         if self.max_mods < 0 or self.max_mods > k:
*/
    __pyx_t_2 = PySequence_List(__pyx_cur_scope->__pyx_v_self->sequence); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1130, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->original_sequence);
    __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->original_sequence);
    __pyx_cur_scope->__pyx_v_self->original_sequence = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":1129
 *         k = PyList_Size(indices)
 * 
 *         if self.persist:             # <<<<<<<<<<<<<<
 *             self.original_sequence = list(self.sequence)
 * 
*/
  }

  /* "pyteomics/cparser.pyx":1132
 *             self.original_sequence = list(self.sequence)
 * 
 *         if self.max_mods < 0 or self.max_mods > k:             # <<<<<<<<<<<<<<
 *             self.last_indices = indices
 *             assignments_product = it.product(*assignments)
*/
  __pyx_t_5 = (__pyx_cur_scope->__pyx_v_self->max_mods < 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_1 = __pyx_t_5;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_5 = (__pyx_cur_scope->__pyx_v_self->max_mods > __pyx_cur_scope->__pyx_v_k);
  __pyx_t_1 = __pyx_t_5;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":1133
 * 
 *         if self.max_mods < 0 or self.max_mods > k:
 *             self.last_indices = indices             # <<<<<<<<<<<<<<
 *             assignments_product = it.product(*assignments)
 *             pindices = <long*>malloc(sizeof(long) * k)
*/
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_indices);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_indices);
    __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->last_indices);
    __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->last_indices);
    __pyx_cur_scope->__pyx_v_self->last_indices = __pyx_cur_scope->__pyx_v_indices;

    /* "pyteomics/cparser.pyx":1134
 *         if self.max_mods < 0 or self.max_mods > k:
 *             self.last_indices = indices
 *             assignments_product = it.product(*assignments)             # <<<<<<<<<<<<<<
 *             pindices = <long*>malloc(sizeof(long) * k)
 *             for i in range(k):
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_it); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1134, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_product); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1134, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PySequence_Tuple(__pyx_cur_scope->__pyx_v_assignments); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1134, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1134, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_6);
    __pyx_cur_scope->__pyx_v_assignments_product = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "pyteomics/cparser.pyx":1135
 *             self.last_indices = indices
 *             assignments_product = it.product(*assignments)
 *             pindices = <long*>malloc(sizeof(long) * k)             # <<<<<<<<<<<<<<
 *             for i in range(k):
 *                 pindices[i] = PyInt_AsLong(<object>PyList_GET_ITEM(indices, i))
*/
    __pyx_cur_scope->__pyx_v_pindices = ((long *)malloc(((sizeof(long)) * __pyx_cur_scope->__pyx_v_k)));

    /* "pyteomics/cparser.pyx":1136
 *             assignments_product = it.product(*assignments)
 *             pindices = <long*>malloc(sizeof(long) * k)
 *             for i in range(k):             # <<<<<<<<<<<<<<
 *                 pindices[i] = PyInt_AsLong(<object>PyList_GET_ITEM(indices, i))
 *             for a in assignments_product:
*/
    __pyx_t_7 = __pyx_cur_scope->__pyx_v_k;
    __pyx_t_8 = __pyx_t_7;
    for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
      __pyx_cur_scope->__pyx_v_i = __pyx_t_9;

      /* "pyteomics/cparser.pyx":1137
 *             pindices = <long*>malloc(sizeof(long) * k)
 *             for i in range(k):
 *                 pindices[i] = PyInt_AsLong(<object>PyList_GET_ITEM(indices, i))             # <<<<<<<<<<<<<<
 *             for a in assignments_product:
 *                 assignment = <tuple>a
*/
      __pyx_t_10 = PyList_GET_ITEM(__pyx_cur_scope->__pyx_v_indices, __pyx_cur_scope->__pyx_v_i);
      (__pyx_cur_scope->__pyx_v_pindices[__pyx_cur_scope->__pyx_v_i]) = PyInt_AsLong(((PyObject *)__pyx_t_10));
    }

    /* "pyteomics/cparser.pyx":1138
 *             for i in range(k):
 *                 pindices[i] = PyInt_AsLong(<object>PyList_GET_ITEM(indices, i))
 *             for a in assignments_product:             # <<<<<<<<<<<<<<
 *                 assignment = <tuple>a
 *                 yield self._apply_variable_assignment(self.sequence, pindices, assignment, k)
*/
    if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_assignments_product)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_assignments_product)) {
      __pyx_t_6 = __pyx_cur_scope->__pyx_v_assignments_product; __Pyx_INCREF(__pyx_t_6);
      __pyx_t_4 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_4 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_assignments_product); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1138, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_11 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1138, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_6))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_6);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1138, __pyx_L1_error)
            #endif
            if (__pyx_t_4 >= __pyx_temp) break;
          }
          __pyx_t_2 = __Pyx_PyList_GetItemRef(__pyx_t_6, __pyx_t_4);
          ++__pyx_t_4;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_6);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1138, __pyx_L1_error)
            #endif
            if (__pyx_t_4 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_4));
          #else
          __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_6, __pyx_t_4);
          #endif
          ++__pyx_t_4;
        }
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1138, __pyx_L1_error)
      } else {
        __pyx_t_2 = __pyx_t_11(__pyx_t_6);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1138, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_a);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_a, __pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":1139
 *                 pindices[i] = PyInt_AsLong(<object>PyList_GET_ITEM(indices, i))
 *             for a in assignments_product:
 *                 assignment = <tuple>a             # <<<<<<<<<<<<<<
 *                 yield self._apply_variable_assignment(self.sequence, pindices, assignment, k)
 *             free(pindices)
*/
      __pyx_t_2 = __pyx_cur_scope->__pyx_v_a;
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_assignment);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_assignment, ((PyObject*)__pyx_t_2));
      __Pyx_GIVEREF(__pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":1140
 *             for a in assignments_product:
 *                 assignment = <tuple>a
 *                 yield self._apply_variable_assignment(self.sequence, pindices, assignment, k)             # <<<<<<<<<<<<<<
 *             free(pindices)
 *         else:
*/
      __pyx_t_2 = __pyx_cur_scope->__pyx_v_self->sequence;
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_3 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->__pyx_fuse_0_apply_variable_assignment(__pyx_cur_scope->__pyx_v_self, ((PyObject*)__pyx_t_2), __pyx_cur_scope->__pyx_v_pindices, __pyx_cur_scope->__pyx_v_assignment, __pyx_cur_scope->__pyx_v_k); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1140, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_3;
      __pyx_t_3 = 0;
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_4;
      __Pyx_XGIVEREF(__pyx_t_6);
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
      __pyx_cur_scope->__pyx_t_2 = __pyx_t_11;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L13_resume_from_yield:;
      __pyx_t_4 = __pyx_cur_scope->__pyx_t_0;
      __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
      __pyx_cur_scope->__pyx_t_1 = 0;
      __Pyx_XGOTREF(__pyx_t_6);
      __pyx_t_11 = __pyx_cur_scope->__pyx_t_2;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1140, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":1138
 *             for i in range(k):
 *                 pindices[i] = PyInt_AsLong(<object>PyList_GET_ITEM(indices, i))
 *             for a in assignments_product:             # <<<<<<<<<<<<<<
 *                 assignment = <tuple>a
 *                 yield self._apply_variable_assignment(self.sequence, pindices, assignment, k)
*/
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "pyteomics/cparser.pyx":1141
 *                 assignment = <tuple>a
 *                 yield self._apply_variable_assignment(self.sequence, pindices, assignment, k)
 *             free(pindices)             # <<<<<<<<<<<<<<
 *         else:
 *             self._reset_sequence()
*/
    free(__pyx_cur_scope->__pyx_v_pindices);

    /* "pyteomics/cparser.pyx":1132
 *             self.original_sequence = list(self.sequence)
 * 
 *         if self.max_mods < 0 or self.max_mods > k:             # <<<<<<<<<<<<<<
 *             self.last_indices = indices
 *             assignments_product = it.product(*assignments)
*/
    goto __pyx_L6;
  }

  /* "pyteomics/cparser.pyx":1143
 *             free(pindices)
 *         else:
 *             self._reset_sequence()             # <<<<<<<<<<<<<<
 *             for n_mods in range(0, self.max_mods):
 *                 for subindices in it.combinations(indices, n_mods):
*/
  /*else*/ {
    ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_reset_sequence(__pyx_cur_scope->__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1143, __pyx_L1_error)

    /* "pyteomics/cparser.pyx":1144
 *         else:
 *             self._reset_sequence()
 *             for n_mods in range(0, self.max_mods):             # <<<<<<<<<<<<<<
 *                 for subindices in it.combinations(indices, n_mods):
 *                     self._reset_sequence()
*/
    __pyx_t_12 = __pyx_cur_scope->__pyx_v_self->max_mods;
    __pyx_t_13 = __pyx_t_12;
    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_13; __pyx_t_7+=1) {
      __pyx_cur_scope->__pyx_v_n_mods = __pyx_t_7;

      /* "pyteomics/cparser.pyx":1145
 *             self._reset_sequence()
 *             for n_mods in range(0, self.max_mods):
 *                 for subindices in it.combinations(indices, n_mods):             # <<<<<<<<<<<<<<
 *                     self._reset_sequence()
 *                     self.last_indices = list(subindices)
*/
      __pyx_t_3 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_it); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_combinations); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_cur_scope->__pyx_v_n_mods); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_14))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_14);
        assert(__pyx_t_3);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
        __pyx_t_8 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_cur_scope->__pyx_v_indices, __pyx_t_2};
        __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+__pyx_t_8, (3-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1145, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
        __pyx_t_14 = __pyx_t_6; __Pyx_INCREF(__pyx_t_14);
        __pyx_t_4 = 0;
        __pyx_t_11 = NULL;
      } else {
        __pyx_t_4 = -1; __pyx_t_14 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1145, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_11 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_14); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1145, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      for (;;) {
        if (likely(!__pyx_t_11)) {
          if (likely(PyList_CheckExact(__pyx_t_14))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_14);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1145, __pyx_L1_error)
              #endif
              if (__pyx_t_4 >= __pyx_temp) break;
            }
            __pyx_t_6 = __Pyx_PyList_GetItemRef(__pyx_t_14, __pyx_t_4);
            ++__pyx_t_4;
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_14);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1145, __pyx_L1_error)
              #endif
              if (__pyx_t_4 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_6 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_14, __pyx_t_4));
            #else
            __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_14, __pyx_t_4);
            #endif
            ++__pyx_t_4;
          }
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1145, __pyx_L1_error)
        } else {
          __pyx_t_6 = __pyx_t_11(__pyx_t_14);
          if (unlikely(!__pyx_t_6)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1145, __pyx_L1_error)
              PyErr_Clear();
            }
            break;
          }
        }
        __Pyx_GOTREF(__pyx_t_6);
        if (!(likely(PyTuple_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_6))) __PYX_ERR(0, 1145, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_subindices);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_subindices, ((PyObject*)__pyx_t_6));
        __Pyx_GIVEREF(__pyx_t_6);
        __pyx_t_6 = 0;

        /* "pyteomics/cparser.pyx":1146
 *             for n_mods in range(0, self.max_mods):
 *                 for subindices in it.combinations(indices, n_mods):
 *                     self._reset_sequence()             # <<<<<<<<<<<<<<
 *                     self.last_indices = list(subindices)
 *                     k = PyTuple_Size(subindices)
*/
        ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_reset_sequence(__pyx_cur_scope->__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1146, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":1147
 *                 for subindices in it.combinations(indices, n_mods):
 *                     self._reset_sequence()
 *                     self.last_indices = list(subindices)             # <<<<<<<<<<<<<<
 *                     k = PyTuple_Size(subindices)
 *                     pindices = <long*>malloc(sizeof(long) * k)
*/
        __pyx_t_6 = PySequence_List(__pyx_cur_scope->__pyx_v_subindices); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1147, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_6);
        __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->last_indices);
        __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->last_indices);
        __pyx_cur_scope->__pyx_v_self->last_indices = ((PyObject*)__pyx_t_6);
        __pyx_t_6 = 0;

        /* "pyteomics/cparser.pyx":1148
 *                     self._reset_sequence()
 *                     self.last_indices = list(subindices)
 *                     k = PyTuple_Size(subindices)             # <<<<<<<<<<<<<<
 *                     pindices = <long*>malloc(sizeof(long) * k)
 *                     subassignments = PyTuple_New(k)
*/
        __pyx_t_15 = PyTuple_Size(__pyx_cur_scope->__pyx_v_subindices); if (unlikely(__pyx_t_15 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1148, __pyx_L1_error)
        __pyx_cur_scope->__pyx_v_k = __pyx_t_15;

        /* "pyteomics/cparser.pyx":1149
 *                     self.last_indices = list(subindices)
 *                     k = PyTuple_Size(subindices)
 *                     pindices = <long*>malloc(sizeof(long) * k)             # <<<<<<<<<<<<<<
 *                     subassignments = PyTuple_New(k)
 *                     for i in range(k):
*/
        __pyx_cur_scope->__pyx_v_pindices = ((long *)malloc(((sizeof(long)) * __pyx_cur_scope->__pyx_v_k)));

        /* "pyteomics/cparser.pyx":1150
 *                     k = PyTuple_Size(subindices)
 *                     pindices = <long*>malloc(sizeof(long) * k)
 *                     subassignments = PyTuple_New(k)             # <<<<<<<<<<<<<<
 *                     for i in range(k):
 *                         temp1 = <object>PyTuple_GET_ITEM(subindices, i)
*/
        __pyx_t_6 = PyTuple_New(__pyx_cur_scope->__pyx_v_k); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1150, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_subassignments);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_subassignments, ((PyObject*)__pyx_t_6));
        __Pyx_GIVEREF(__pyx_t_6);
        __pyx_t_6 = 0;

        /* "pyteomics/cparser.pyx":1151
 *                     pindices = <long*>malloc(sizeof(long) * k)
 *                     subassignments = PyTuple_New(k)
 *                     for i in range(k):             # <<<<<<<<<<<<<<
 *                         temp1 = <object>PyTuple_GET_ITEM(subindices, i)
 *                         pindices[i] = PyInt_AsLong(temp1)
*/
        __pyx_t_8 = __pyx_cur_scope->__pyx_v_k;
        __pyx_t_9 = __pyx_t_8;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_9; __pyx_t_16+=1) {
          __pyx_cur_scope->__pyx_v_i = __pyx_t_16;

          /* "pyteomics/cparser.pyx":1152
 *                     subassignments = PyTuple_New(k)
 *                     for i in range(k):
 *                         temp1 = <object>PyTuple_GET_ITEM(subindices, i)             # <<<<<<<<<<<<<<
 *                         pindices[i] = PyInt_AsLong(temp1)
 *                         temp2 = <object>PyDict_GetItem(variable_mod_sites, temp1)
*/
          __pyx_t_10 = PyTuple_GET_ITEM(__pyx_cur_scope->__pyx_v_subindices, __pyx_cur_scope->__pyx_v_i);
          __pyx_t_6 = ((PyObject *)__pyx_t_10);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_temp1);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_temp1, __pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_6);
          __pyx_t_6 = 0;

          /* "pyteomics/cparser.pyx":1153
 *                     for i in range(k):
 *                         temp1 = <object>PyTuple_GET_ITEM(subindices, i)
 *                         pindices[i] = PyInt_AsLong(temp1)             # <<<<<<<<<<<<<<
 *                         temp2 = <object>PyDict_GetItem(variable_mod_sites, temp1)
 *                         Py_INCREF(temp2)
*/
          (__pyx_cur_scope->__pyx_v_pindices[__pyx_cur_scope->__pyx_v_i]) = PyInt_AsLong(__pyx_cur_scope->__pyx_v_temp1);

          /* "pyteomics/cparser.pyx":1154
 *                         temp1 = <object>PyTuple_GET_ITEM(subindices, i)
 *                         pindices[i] = PyInt_AsLong(temp1)
 *                         temp2 = <object>PyDict_GetItem(variable_mod_sites, temp1)             # <<<<<<<<<<<<<<
 *                         Py_INCREF(temp2)
 *                         PyTuple_SET_ITEM(subassignments, i, temp2)
*/
          __pyx_t_10 = PyDict_GetItem(__pyx_cur_scope->__pyx_v_variable_mod_sites, __pyx_cur_scope->__pyx_v_temp1);
          __pyx_t_6 = ((PyObject *)__pyx_t_10);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_temp2);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_temp2, __pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_6);
          __pyx_t_6 = 0;

          /* "pyteomics/cparser.pyx":1155
 *                         pindices[i] = PyInt_AsLong(temp1)
 *                         temp2 = <object>PyDict_GetItem(variable_mod_sites, temp1)
 *                         Py_INCREF(temp2)             # <<<<<<<<<<<<<<
 *                         PyTuple_SET_ITEM(subassignments, i, temp2)
 *                     assignments_product = it.product(*subassignments)
*/
          Py_INCREF(__pyx_cur_scope->__pyx_v_temp2);

          /* "pyteomics/cparser.pyx":1156
 *                         temp2 = <object>PyDict_GetItem(variable_mod_sites, temp1)
 *                         Py_INCREF(temp2)
 *                         PyTuple_SET_ITEM(subassignments, i, temp2)             # <<<<<<<<<<<<<<
 *                     assignments_product = it.product(*subassignments)
 *                     for a in assignments_product:
*/
          PyTuple_SET_ITEM(__pyx_cur_scope->__pyx_v_subassignments, __pyx_cur_scope->__pyx_v_i, __pyx_cur_scope->__pyx_v_temp2);
        }

        /* "pyteomics/cparser.pyx":1157
 *                         Py_INCREF(temp2)
 *                         PyTuple_SET_ITEM(subassignments, i, temp2)
 *                     assignments_product = it.product(*subassignments)             # <<<<<<<<<<<<<<
 *                     for a in assignments_product:
 *                         yield self._apply_variable_assignment(self.sequence, pindices, <tuple>a, k)
*/
        __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_it); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1157, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_product); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1157, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(__pyx_cur_scope->__pyx_v_subassignments == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(0, 1157, __pyx_L1_error)
        }
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_cur_scope->__pyx_v_subassignments, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1157, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_assignments_product);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_assignments_product, __pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_6);
        __pyx_t_6 = 0;

        /* "pyteomics/cparser.pyx":1158
 *                         PyTuple_SET_ITEM(subassignments, i, temp2)
 *                     assignments_product = it.product(*subassignments)
 *                     for a in assignments_product:             # <<<<<<<<<<<<<<
 *                         yield self._apply_variable_assignment(self.sequence, pindices, <tuple>a, k)
 *                     free(pindices)
*/
        if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_assignments_product)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_assignments_product)) {
          __pyx_t_6 = __pyx_cur_scope->__pyx_v_assignments_product; __Pyx_INCREF(__pyx_t_6);
          __pyx_t_15 = 0;
          __pyx_t_17 = NULL;
        } else {
          __pyx_t_15 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_assignments_product); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1158, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_17 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1158, __pyx_L1_error)
        }
        for (;;) {
          if (likely(!__pyx_t_17)) {
            if (likely(PyList_CheckExact(__pyx_t_6))) {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_6);
                #if !CYTHON_ASSUME_SAFE_SIZE
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1158, __pyx_L1_error)
                #endif
                if (__pyx_t_15 >= __pyx_temp) break;
              }
              __pyx_t_2 = __Pyx_PyList_GetItemRef(__pyx_t_6, __pyx_t_15);
              ++__pyx_t_15;
            } else {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_6);
                #if !CYTHON_ASSUME_SAFE_SIZE
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1158, __pyx_L1_error)
                #endif
                if (__pyx_t_15 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_2 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_15));
              #else
              __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_6, __pyx_t_15);
              #endif
              ++__pyx_t_15;
            }
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1158, __pyx_L1_error)
          } else {
            __pyx_t_2 = __pyx_t_17(__pyx_t_6);
            if (unlikely(!__pyx_t_2)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1158, __pyx_L1_error)
                PyErr_Clear();
              }
              break;
            }
          }
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_a);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_a, __pyx_t_2);
          __Pyx_GIVEREF(__pyx_t_2);
          __pyx_t_2 = 0;

          /* "pyteomics/cparser.pyx":1159
 *                     assignments_product = it.product(*subassignments)
 *                     for a in assignments_product:
 *                         yield self._apply_variable_assignment(self.sequence, pindices, <tuple>a, k)             # <<<<<<<<<<<<<<
 *                     free(pindices)
 *         self.sequence = None
*/
          __pyx_t_2 = __pyx_cur_scope->__pyx_v_self->sequence;
          __Pyx_INCREF(__pyx_t_2);
          __pyx_t_3 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->__pyx_fuse_0_apply_variable_assignment(__pyx_cur_scope->__pyx_v_self, ((PyObject*)__pyx_t_2), __pyx_cur_scope->__pyx_v_pindices, ((PyObject*)__pyx_cur_scope->__pyx_v_a), __pyx_cur_scope->__pyx_v_k); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1159, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_r = __pyx_t_3;
          __pyx_t_3 = 0;
          __pyx_cur_scope->__pyx_t_0 = __pyx_t_4;
          __Pyx_XGIVEREF(__pyx_t_6);
          __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
          __pyx_cur_scope->__pyx_t_3 = __pyx_t_7;
          __pyx_cur_scope->__pyx_t_2 = __pyx_t_11;
          __pyx_cur_scope->__pyx_t_4 = __pyx_t_12;
          __pyx_cur_scope->__pyx_t_5 = __pyx_t_13;
          __Pyx_XGIVEREF(__pyx_t_14);
          __pyx_cur_scope->__pyx_t_6 = __pyx_t_14;
          __pyx_cur_scope->__pyx_t_7 = __pyx_t_15;
          __pyx_cur_scope->__pyx_t_8 = __pyx_t_17;
          __Pyx_XGIVEREF(__pyx_r);
          __Pyx_RefNannyFinishContext();
          __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
          /* return from generator, yielding value */
          __pyx_generator->resume_label = 2;
          return __pyx_r;
          __pyx_L23_resume_from_yield:;
          __pyx_t_4 = __pyx_cur_scope->__pyx_t_0;
          __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
          __pyx_cur_scope->__pyx_t_1 = 0;
          __Pyx_XGOTREF(__pyx_t_6);
          __pyx_t_7 = __pyx_cur_scope->__pyx_t_3;
          __pyx_t_11 = __pyx_cur_scope->__pyx_t_2;
          __pyx_t_12 = __pyx_cur_scope->__pyx_t_4;
          __pyx_t_13 = __pyx_cur_scope->__pyx_t_5;
          __pyx_t_14 = __pyx_cur_scope->__pyx_t_6;
          __pyx_cur_scope->__pyx_t_6 = 0;
          __Pyx_XGOTREF(__pyx_t_14);
          __pyx_t_15 = __pyx_cur_scope->__pyx_t_7;
          __pyx_t_17 = __pyx_cur_scope->__pyx_t_8;
          if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1159, __pyx_L1_error)

          /* "pyteomics/cparser.pyx":1158
 *                         PyTuple_SET_ITEM(subassignments, i, temp2)
 *                     assignments_product = it.product(*subassignments)
 *                     for a in assignments_product:             # <<<<<<<<<<<<<<
 *                         yield self._apply_variable_assignment(self.sequence, pindices, <tuple>a, k)
 *                     free(pindices)
*/
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pyteomics/cparser.pyx":1160
 *                     for a in assignments_product:
 *                         yield self._apply_variable_assignment(self.sequence, pindices, <tuple>a, k)
 *                     free(pindices)             # <<<<<<<<<<<<<<
 *         self.sequence = None
 * 
*/
        free(__pyx_cur_scope->__pyx_v_pindices);

        /* "pyteomics/cparser.pyx":1145
 *             self._reset_sequence()
 *             for n_mods in range(0, self.max_mods):
 *                 for subindices in it.combinations(indices, n_mods):             # <<<<<<<<<<<<<<
 *                     self._reset_sequence()
 *                     self.last_indices = list(subindices)
*/
      }
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    }
  }
  __pyx_L6:;

  /* "pyteomics/cparser.pyx":1161
 *                         yield self._apply_variable_assignment(self.sequence, pindices, <tuple>a, k)
 *                     free(pindices)
 *         self.sequence = None             # <<<<<<<<<<<<<<
 * 
 *     @cython.boundscheck(False)
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->sequence);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->sequence);
  __pyx_cur_scope->__pyx_v_self->sequence = ((PyObject*)Py_None);
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "pyteomics/cparser.pyx":1105
 *         return site_map
 * 
 *     @cython.boundscheck(False)             # <<<<<<<<<<<<<<
 *     def generate(self, sequence_spec):
 *         cdef:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_14);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("generate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1163
 *         self.sequence = None
 * 
 *     @cython.boundscheck(False)             # <<<<<<<<<<<<<<
 *     cdef list _apply_variable_assignment(self, list sequence, long* indices, tuple_or_list assignment, size_t n):
 *         cdef:
*/

static PyObject *__pyx_fuse_0__pyx_f_9pyteomics_7cparser_16IsoformGenerator__apply_variable_assignment(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, long *__pyx_v_indices, PyObject *__pyx_v_assignment, size_t __pyx_v_n) {
  size_t __pyx_v_i;
  long __pyx_v_ix;
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0_apply_variable_assignment", 0);
  __Pyx_INCREF(__pyx_v_sequence);

  /* "pyteomics/cparser.pyx":1170
 *             SequencePosition position
 *             str mod
 *         if self.persist:             # <<<<<<<<<<<<<<
 *             sequence = list(sequence)
 *         else:
*/
  if (__pyx_v_self->persist) {

    /* "pyteomics/cparser.pyx":1171
 *             str mod
 *         if self.persist:
 *             sequence = list(sequence)             # <<<<<<<<<<<<<<
 *         else:
 *             self._reset_sequence()
*/
    __pyx_t_1 = PySequence_List(__pyx_v_sequence); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_sequence, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":1170
 *             SequencePosition position
 *             str mod
 *         if self.persist:             # <<<<<<<<<<<<<<
 *             sequence = list(sequence)
 *         else:
*/
    goto __pyx_L3;
  }

  /* "pyteomics/cparser.pyx":1173
 *             sequence = list(sequence)
 *         else:
 *             self._reset_sequence()             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             if tuple_or_list is tuple:
*/
  /*else*/ {
    ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self->__pyx_vtab)->_reset_sequence(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1173, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "pyteomics/cparser.pyx":1174
 *         else:
 *             self._reset_sequence()
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             if tuple_or_list is tuple:
 *                 position = <SequencePosition>PyTuple_GET_ITEM(assignment, i)
*/
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pyteomics/cparser.pyx":1176
 *         for i in range(n):
 *             if tuple_or_list is tuple:
 *                 position = <SequencePosition>PyTuple_GET_ITEM(assignment, i)             # <<<<<<<<<<<<<<
 *             else:
 *                 position = <SequencePosition>PyList_GET_ITEM(assignment, i)
*/
    __pyx_t_5 = PyTuple_GET_ITEM(__pyx_v_assignment, __pyx_v_i);
    __pyx_t_1 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":1179
 *             else:
 *                 position = <SequencePosition>PyList_GET_ITEM(assignment, i)
 *             ix = indices[i]             # <<<<<<<<<<<<<<
 *             Py_INCREF(position)
 *             PyList_SET_ITEM(sequence, ix, position)
*/
    __pyx_v_ix = (__pyx_v_indices[__pyx_v_i]);

    /* "pyteomics/cparser.pyx":1180
 *                 position = <SequencePosition>PyList_GET_ITEM(assignment, i)
 *             ix = indices[i]
 *             Py_INCREF(position)             # <<<<<<<<<<<<<<
 *             PyList_SET_ITEM(sequence, ix, position)
 *         return sequence
*/
    Py_INCREF(((PyObject *)__pyx_v_position));

    /* "pyteomics/cparser.pyx":1181
 *             ix = indices[i]
 *             Py_INCREF(position)
 *             PyList_SET_ITEM(sequence, ix, position)             # <<<<<<<<<<<<<<
 *         return sequence
 * 
*/
    PyList_SET_ITEM(__pyx_v_sequence, __pyx_v_ix, ((PyObject *)__pyx_v_position));
  }

  /* "pyteomics/cparser.pyx":1182
 *             Py_INCREF(position)
 *             PyList_SET_ITEM(sequence, ix, position)
 *         return sequence             # <<<<<<<<<<<<<<
 * 
 *     cdef void _reset_sequence(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_sequence);
  __pyx_r = __pyx_v_sequence;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1163
 *         self.sequence = None
 * 
 *     @cython.boundscheck(False)             # <<<<<<<<<<<<<<
 *     cdef list _apply_variable_assignment(self, list sequence, long* indices, tuple_or_list assignment, size_t n):
 *         cdef:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator._apply_variable_assignment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_XDECREF(__pyx_v_sequence);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_fuse_1__pyx_f_9pyteomics_7cparser_16IsoformGenerator__apply_variable_assignment(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence, long *__pyx_v_indices, PyObject *__pyx_v_assignment, size_t __pyx_v_n) {
  size_t __pyx_v_i;
  long __pyx_v_ix;
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1_apply_variable_assignment", 0);
  __Pyx_INCREF(__pyx_v_sequence);

  /* "pyteomics/cparser.pyx":1170
 *             SequencePosition position
 *             str mod
 *         if self.persist:             # <<<<<<<<<<<<<<
 *             sequence = list(sequence)
 *         else:
*/
  if (__pyx_v_self->persist) {

    /* "pyteomics/cparser.pyx":1171
 *             str mod
 *         if self.persist:
 *             sequence = list(sequence)             # <<<<<<<<<<<<<<
 *         else:
 *             self._reset_sequence()
*/
    __pyx_t_1 = PySequence_List(__pyx_v_sequence); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_sequence, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":1170
 *             SequencePosition position
 *             str mod
 *         if self.persist:             # <<<<<<<<<<<<<<
 *             sequence = list(sequence)
 *         else:
*/
    goto __pyx_L3;
  }

  /* "pyteomics/cparser.pyx":1173
 *             sequence = list(sequence)
 *         else:
 *             self._reset_sequence()             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             if tuple_or_list is tuple:
*/
  /*else*/ {
    ((struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self->__pyx_vtab)->_reset_sequence(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1173, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "pyteomics/cparser.pyx":1174
 *         else:
 *             self._reset_sequence()
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             if tuple_or_list is tuple:
 *                 position = <SequencePosition>PyTuple_GET_ITEM(assignment, i)
*/
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pyteomics/cparser.pyx":1178
 *                 position = <SequencePosition>PyTuple_GET_ITEM(assignment, i)
 *             else:
 *                 position = <SequencePosition>PyList_GET_ITEM(assignment, i)             # <<<<<<<<<<<<<<
 *             ix = indices[i]
 *             Py_INCREF(position)
*/
    __pyx_t_5 = PyList_GET_ITEM(__pyx_v_assignment, __pyx_v_i);
    __pyx_t_1 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":1179
 *             else:
 *                 position = <SequencePosition>PyList_GET_ITEM(assignment, i)
 *             ix = indices[i]             # <<<<<<<<<<<<<<
 *             Py_INCREF(position)
 *             PyList_SET_ITEM(sequence, ix, position)
*/
    __pyx_v_ix = (__pyx_v_indices[__pyx_v_i]);

    /* "pyteomics/cparser.pyx":1180
 *                 position = <SequencePosition>PyList_GET_ITEM(assignment, i)
 *             ix = indices[i]
 *             Py_INCREF(position)             # <<<<<<<<<<<<<<
 *             PyList_SET_ITEM(sequence, ix, position)
 *         return sequence
*/
    Py_INCREF(((PyObject *)__pyx_v_position));

    /* "pyteomics/cparser.pyx":1181
 *             ix = indices[i]
 *             Py_INCREF(position)
 *             PyList_SET_ITEM(sequence, ix, position)             # <<<<<<<<<<<<<<
 *         return sequence
 * 
*/
    PyList_SET_ITEM(__pyx_v_sequence, __pyx_v_ix, ((PyObject *)__pyx_v_position));
  }

  /* "pyteomics/cparser.pyx":1182
 *             Py_INCREF(position)
 *             PyList_SET_ITEM(sequence, ix, position)
 *         return sequence             # <<<<<<<<<<<<<<
 * 
 *     cdef void _reset_sequence(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_sequence);
  __pyx_r = __pyx_v_sequence;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1163
 *         self.sequence = None
 * 
 *     @cython.boundscheck(False)             # <<<<<<<<<<<<<<
 *     cdef list _apply_variable_assignment(self, list sequence, long* indices, tuple_or_list assignment, size_t n):
 *         cdef:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator._apply_variable_assignment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_XDECREF(__pyx_v_sequence);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1184
 *         return sequence
 * 
 *     cdef void _reset_sequence(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/

static void __pyx_f_9pyteomics_7cparser_16IsoformGenerator__reset_sequence(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  long __pyx_v_ix;
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_reset_sequence", 0);

  /* "pyteomics/cparser.pyx":1190
 *             SequencePosition position
 * 
 *         n = PyList_Size(self.last_indices)             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             ix = PyInt_AsLong(<object>PyList_GET_ITEM(self.last_indices, i))
*/
  __pyx_t_1 = __pyx_v_self->last_indices;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyList_Size(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1190, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_n = __pyx_t_2;

  /* "pyteomics/cparser.pyx":1191
 * 
 *         n = PyList_Size(self.last_indices)
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             ix = PyInt_AsLong(<object>PyList_GET_ITEM(self.last_indices, i))
 *             position = <SequencePosition>PyList_GET_ITEM(self.original_sequence, ix)
*/
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_i = __pyx_t_5;

    /* "pyteomics/cparser.pyx":1192
 *         n = PyList_Size(self.last_indices)
 *         for i in range(n):
 *             ix = PyInt_AsLong(<object>PyList_GET_ITEM(self.last_indices, i))             # <<<<<<<<<<<<<<
 *             position = <SequencePosition>PyList_GET_ITEM(self.original_sequence, ix)
 *             Py_INCREF(position)
*/
    __pyx_t_1 = __pyx_v_self->last_indices;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_6 = PyList_GET_ITEM(__pyx_t_1, __pyx_v_i);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_ix = PyInt_AsLong(((PyObject *)__pyx_t_6));

    /* "pyteomics/cparser.pyx":1193
 *         for i in range(n):
 *             ix = PyInt_AsLong(<object>PyList_GET_ITEM(self.last_indices, i))
 *             position = <SequencePosition>PyList_GET_ITEM(self.original_sequence, ix)             # <<<<<<<<<<<<<<
 *             Py_INCREF(position)
 *             PyList_SET_ITEM(self.sequence, ix, position)
*/
    __pyx_t_1 = __pyx_v_self->original_sequence;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_6 = PyList_GET_ITEM(__pyx_t_1, __pyx_v_ix);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)__pyx_t_6);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":1194
 *             ix = PyInt_AsLong(<object>PyList_GET_ITEM(self.last_indices, i))
 *             position = <SequencePosition>PyList_GET_ITEM(self.original_sequence, ix)
 *             Py_INCREF(position)             # <<<<<<<<<<<<<<
 *             PyList_SET_ITEM(self.sequence, ix, position)
 * 
*/
    Py_INCREF(((PyObject *)__pyx_v_position));

    /* "pyteomics/cparser.pyx":1195
 *             position = <SequencePosition>PyList_GET_ITEM(self.original_sequence, ix)
 *             Py_INCREF(position)
 *             PyList_SET_ITEM(self.sequence, ix, position)             # <<<<<<<<<<<<<<
 * 
 *     def __call__(self, sequence_spec):
*/
    __pyx_t_1 = __pyx_v_self->sequence;
    __Pyx_INCREF(__pyx_t_1);
    PyList_SET_ITEM(__pyx_t_1, __pyx_v_ix, ((PyObject *)__pyx_v_position));
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pyteomics/cparser.pyx":1184
 *         return sequence
 * 
 *     cdef void _reset_sequence(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator._reset_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_RefNannyFinishContext();
}

/* "pyteomics/cparser.pyx":1197
 *             PyList_SET_ITEM(self.sequence, ix, position)
 * 
 *     def __call__(self, sequence_spec):             # <<<<<<<<<<<<<<
 *         return self.generate(sequence_spec)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_16__call__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_16__call__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_sequence_spec = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__call__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence_spec,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1197, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1197, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__call__", 0) < 0) __PYX_ERR(0, 1197, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__call__", 1, 1, 1, i); __PYX_ERR(0, 1197, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1197, __pyx_L3_error)
    }
    __pyx_v_sequence_spec = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__call__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1197, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.__call__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_15__call__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), __pyx_v_sequence_spec);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_15__call__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_sequence_spec) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__call__", 0);

  /* "pyteomics/cparser.pyx":1198
 * 
 *     def __call__(self, sequence_spec):
 *         return self.generate(sequence_spec)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_sequence_spec};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_generate, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1197
 *             PyList_SET_ITEM(self.sequence, ix, position)
 * 
 *     def __call__(self, sequence_spec):             # <<<<<<<<<<<<<<
 *         return self.generate(sequence_spec)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.__call__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":990
 * cdef class IsoformGenerator(object):
 *     cdef:
 *         public dict variable_mods             # <<<<<<<<<<<<<<
 *         public dict fixed_mods
 *         public list labels
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_13variable_mods___get__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_13variable_mods___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->variable_mods);
  __pyx_r = __pyx_v_self->variable_mods;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_2__set__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_1))) __PYX_ERR(0, 990, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->variable_mods);
  __Pyx_DECREF(__pyx_v_self->variable_mods);
  __pyx_v_self->variable_mods = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.variable_mods.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_4__del__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->variable_mods);
  __Pyx_DECREF(__pyx_v_self->variable_mods);
  __pyx_v_self->variable_mods = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":991
 *     cdef:
 *         public dict variable_mods
 *         public dict fixed_mods             # <<<<<<<<<<<<<<
 *         public list labels
 *         public long max_mods
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods___get__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->fixed_mods);
  __pyx_r = __pyx_v_self->fixed_mods;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_2__set__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_1))) __PYX_ERR(0, 991, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->fixed_mods);
  __Pyx_DECREF(__pyx_v_self->fixed_mods);
  __pyx_v_self->fixed_mods = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.fixed_mods.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_4__del__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->fixed_mods);
  __Pyx_DECREF(__pyx_v_self->fixed_mods);
  __pyx_v_self->fixed_mods = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":992
 *         public dict variable_mods
 *         public dict fixed_mods
 *         public list labels             # <<<<<<<<<<<<<<
 *         public long max_mods
 *         public bint override
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_6labels_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_6labels_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6labels___get__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6labels___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->labels);
  __pyx_r = __pyx_v_self->labels;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_6labels_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_6labels_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6labels_2__set__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6labels_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(0, 992, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->labels);
  __Pyx_DECREF(__pyx_v_self->labels);
  __pyx_v_self->labels = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.labels.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_6labels_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_6labels_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6labels_4__del__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_6labels_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->labels);
  __Pyx_DECREF(__pyx_v_self->labels);
  __pyx_v_self->labels = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":993
 *         public dict fixed_mods
 *         public list labels
 *         public long max_mods             # <<<<<<<<<<<<<<
 *         public bint override
 *         public bint show_unmodified_termini
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8max_mods_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8max_mods_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8max_mods___get__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8max_mods___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_long(__pyx_v_self->max_mods); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 993, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.max_mods.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8max_mods_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8max_mods_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8max_mods_2__set__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8max_mods_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  long __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyLong_As_long(__pyx_v_value); if (unlikely((__pyx_t_1 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 993, __pyx_L1_error)
  __pyx_v_self->max_mods = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.max_mods.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":994
 *         public list labels
 *         public long max_mods
 *         public bint override             # <<<<<<<<<<<<<<
 *         public bint show_unmodified_termini
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8override_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8override_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8override___get__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8override___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->override); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 994, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.override.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8override_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8override_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8override_2__set__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8override_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 994, __pyx_L1_error)
  __pyx_v_self->override = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.override.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":995
 *         public long max_mods
 *         public bint override
 *         public bint show_unmodified_termini             # <<<<<<<<<<<<<<
 * 
 *         public bint persist
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini___get__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->show_unmodified_termini); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 995, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.show_unmodified_termini.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini_2__set__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 995, __pyx_L1_error)
  __pyx_v_self->show_unmodified_termini = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.show_unmodified_termini.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":997
 *         public bint show_unmodified_termini
 * 
 *         public bint persist             # <<<<<<<<<<<<<<
 * 
 *         # Temporary State
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7persist_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7persist_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_7persist___get__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_7persist___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->persist); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 997, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.persist.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7persist_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7persist_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_7persist_2__set__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_7persist_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 997, __pyx_L1_error)
  __pyx_v_self->persist = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.persist.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1000
 * 
 *         # Temporary State
 *         public list sequence             # <<<<<<<<<<<<<<
 *         public list original_sequence
 *         public list last_indices
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8sequence_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8sequence_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8sequence___get__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8sequence___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->sequence);
  __pyx_r = __pyx_v_self->sequence;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8sequence_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8sequence_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8sequence_2__set__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8sequence_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(0, 1000, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->sequence);
  __Pyx_DECREF(__pyx_v_self->sequence);
  __pyx_v_self->sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.sequence.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8sequence_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8sequence_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8sequence_4__del__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_8sequence_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->sequence);
  __Pyx_DECREF(__pyx_v_self->sequence);
  __pyx_v_self->sequence = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1001
 *         # Temporary State
 *         public list sequence
 *         public list original_sequence             # <<<<<<<<<<<<<<
 *         public list last_indices
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17original_sequence___get__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17original_sequence___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->original_sequence);
  __pyx_r = __pyx_v_self->original_sequence;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_2__set__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(0, 1001, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->original_sequence);
  __Pyx_DECREF(__pyx_v_self->original_sequence);
  __pyx_v_self->original_sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.original_sequence.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_4__del__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->original_sequence);
  __Pyx_DECREF(__pyx_v_self->original_sequence);
  __pyx_v_self->original_sequence = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1002
 *         public list sequence
 *         public list original_sequence
 *         public list last_indices             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False,
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_12last_indices_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_12last_indices_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12last_indices___get__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12last_indices___get__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->last_indices);
  __pyx_r = __pyx_v_self->last_indices;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_12last_indices_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_12last_indices_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12last_indices_2__set__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12last_indices_2__set__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(0, 1002, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->last_indices);
  __Pyx_DECREF(__pyx_v_self->last_indices);
  __pyx_v_self->last_indices = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.last_indices.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_12last_indices_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_12last_indices_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12last_indices_4__del__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_12last_indices_4__del__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->last_indices);
  __Pyx_DECREF(__pyx_v_self->last_indices);
  __pyx_v_self->last_indices = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_18__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16IsoformGenerator_17__reduce_cython__, "IsoformGenerator.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_18__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_18__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_17__reduce_cython__};
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_18__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17__reduce_cython__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_17__reduce_cython__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.fixed_mods, self.labels, self.last_indices, self.max_mods, self.original_sequence, self.override, self.persist, self.sequence, self.show_unmodified_termini, self.variable_mods)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
*/
  __pyx_t_1 = __Pyx_PyLong_From_long(__pyx_v_self->max_mods); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->override); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->persist); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->show_unmodified_termini); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(10); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->fixed_mods);
  __Pyx_GIVEREF(__pyx_v_self->fixed_mods);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_self->fixed_mods) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->labels);
  __Pyx_GIVEREF(__pyx_v_self->labels);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_self->labels) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->last_indices);
  __Pyx_GIVEREF(__pyx_v_self->last_indices);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_self->last_indices) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_1) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->original_sequence);
  __Pyx_GIVEREF(__pyx_v_self->original_sequence);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_v_self->original_sequence) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_2) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 6, __pyx_t_3) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->sequence);
  __Pyx_GIVEREF(__pyx_v_self->sequence);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 7, __pyx_v_self->sequence) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_4);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 8, __pyx_t_4) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->variable_mods);
  __Pyx_GIVEREF(__pyx_v_self->variable_mods);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 9, __pyx_v_self->variable_mods) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.fixed_mods, self.labels, self.last_indices, self.max_mods, self.original_sequence, self.override, self.persist, self.sequence, self.show_unmodified_termini, self.variable_mods)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
*/
  __pyx_t_5 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v__dict = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "(tree fragment)":7
 *     state = (self.fixed_mods, self.labels, self.last_indices, self.max_mods, self.original_sequence, self.override, self.persist, self.sequence, self.show_unmodified_termini, self.variable_mods)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v__dict) != (0)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.fixed_mods is not None or self.labels is not None or self.last_indices is not None or self.original_sequence is not None or self.sequence is not None or self.variable_mods is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.fixed_mods, self.labels, self.last_indices, self.max_mods, self.original_sequence, self.override, self.persist, self.sequence, self.show_unmodified_termini, self.variable_mods)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.fixed_mods is not None or self.labels is not None or self.last_indices is not None or self.original_sequence is not None or self.sequence is not None or self.variable_mods is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, None), state
*/
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->fixed_mods != ((PyObject*)Py_None));
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->labels != ((PyObject*)Py_None));
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->last_indices != ((PyObject*)Py_None));
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->original_sequence != ((PyObject*)Py_None));
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->sequence != ((PyObject*)Py_None));
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->variable_mods != ((PyObject*)Py_None));
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.fixed_mods is not None or self.labels is not None or self.last_indices is not None or self.original_sequence is not None or self.sequence is not None or self.variable_mods is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.fixed_mods is not None or self.labels is not None or self.last_indices is not None or self.original_sequence is not None or self.sequence is not None or self.variable_mods is not None
 *     if use_setstate:
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_IsoformGenerator); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_10332774);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_10332774);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_mstate_global->__pyx_int_10332774) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, Py_None) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.fixed_mods is not None or self.labels is not None or self.last_indices is not None or self.original_sequence is not None or self.sequence is not None or self.variable_mods is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, None), state
 *     else:
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_IsoformGenerator__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_IsoformGenerator); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_10332774);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_10332774);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_mstate_global->__pyx_int_10332774) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IsoformGenerator__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_20__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16IsoformGenerator_19__setstate_cython__, "IsoformGenerator.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_20__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_20__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_19__setstate_cython__};
static PyObject *__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_20__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16IsoformGenerator_19__setstate_cython__(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16IsoformGenerator_19__setstate_cython__(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_IsoformGenerator__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser___pyx_unpickle_IsoformGenerator__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IsoformGenerator__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.IsoformGenerator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1201
 * 
 * 
 * cdef bint _is_nterm_mod(str target):             # <<<<<<<<<<<<<<
 *     return target.startswith("nterm")
 * 
*/

static int __pyx_f_9pyteomics_7cparser__is_nterm_mod(PyObject *__pyx_v_target) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "pyteomics/cparser.pyx":1202
 * 
 * cdef bint _is_nterm_mod(str target):
 *     return target.startswith("nterm")             # <<<<<<<<<<<<<<
 * 
 * 
*/
  if (unlikely(__pyx_v_target == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "startswith");
    __PYX_ERR(0, 1202, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyUnicode_Tailmatch(__pyx_v_target, __pyx_mstate_global->__pyx_n_u_nterm, 0, PY_SSIZE_T_MAX, -1); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 1202, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1201
 * 
 * 
 * cdef bint _is_nterm_mod(str target):             # <<<<<<<<<<<<<<
 *     return target.startswith("nterm")
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser._is_nterm_mod", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1205
 * 
 * 
 * cdef bint _is_cterm_mod(str target):             # <<<<<<<<<<<<<<
 *     return target.startswith("cterm")
 * 
*/

static int __pyx_f_9pyteomics_7cparser__is_cterm_mod(PyObject *__pyx_v_target) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "pyteomics/cparser.pyx":1206
 * 
 * cdef bint _is_cterm_mod(str target):
 *     return target.startswith("cterm")             # <<<<<<<<<<<<<<
 * 
 * 
*/
  if (unlikely(__pyx_v_target == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "startswith");
    __PYX_ERR(0, 1206, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyUnicode_Tailmatch(__pyx_v_target, __pyx_mstate_global->__pyx_n_u_cterm, 0, PY_SSIZE_T_MAX, -1); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 1206, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1205
 * 
 * 
 * cdef bint _is_cterm_mod(str target):             # <<<<<<<<<<<<<<
 *     return target.startswith("cterm")
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser._is_cterm_mod", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1215
 *         public list targets
 * 
 *     def __init__(self, str label, targets):             # <<<<<<<<<<<<<<
 *         self.modification = label
 *         self.targets = [ModificationTarget._from_string(x) if isinstance(x, str)
*/

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16ModificationRule_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16ModificationRule_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_label = 0;
  PyObject *__pyx_v_targets = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_label,&__pyx_mstate_global->__pyx_n_u_targets,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1215, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1215, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1215, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < 0) __PYX_ERR(0, 1215, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, i); __PYX_ERR(0, 1215, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1215, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1215, __pyx_L3_error)
    }
    __pyx_v_label = ((PyObject*)values[0]);
    __pyx_v_targets = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1215, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_label), (&PyUnicode_Type), 1, "label", 1))) __PYX_ERR(0, 1215, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule___init__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self), __pyx_v_label, __pyx_v_targets);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16ModificationRule___init__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, PyObject *__pyx_v_label, PyObject *__pyx_v_targets) {
  PyObject *__pyx_8genexpr1__pyx_v_x = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyteomics/cparser.pyx":1216
 * 
 *     def __init__(self, str label, targets):
 *         self.modification = label             # <<<<<<<<<<<<<<
 *         self.targets = [ModificationTarget._from_string(x) if isinstance(x, str)
 *                         else ModificationTarget._create(None, PositionType.universal)
*/
  __Pyx_INCREF(__pyx_v_label);
  __Pyx_GIVEREF(__pyx_v_label);
  __Pyx_GOTREF(__pyx_v_self->modification);
  __Pyx_DECREF(__pyx_v_self->modification);
  __pyx_v_self->modification = __pyx_v_label;

  /* "pyteomics/cparser.pyx":1217
 *     def __init__(self, str label, targets):
 *         self.modification = label
 *         self.targets = [ModificationTarget._from_string(x) if isinstance(x, str)             # <<<<<<<<<<<<<<
 *                         else ModificationTarget._create(None, PositionType.universal)
 *                         for x in targets]
*/
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1217, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "pyteomics/cparser.pyx":1219
 *         self.targets = [ModificationTarget._from_string(x) if isinstance(x, str)
 *                         else ModificationTarget._create(None, PositionType.universal)
 *                         for x in targets]             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint test(self, SequencePosition position):
*/
    if (likely(PyList_CheckExact(__pyx_v_targets)) || PyTuple_CheckExact(__pyx_v_targets)) {
      __pyx_t_2 = __pyx_v_targets; __Pyx_INCREF(__pyx_t_2);
      __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_targets); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1219, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1219, __pyx_L5_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1219, __pyx_L5_error)
            #endif
            if (__pyx_t_3 >= __pyx_temp) break;
          }
          __pyx_t_5 = __Pyx_PyList_GetItemRef(__pyx_t_2, __pyx_t_3);
          ++__pyx_t_3;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1219, __pyx_L5_error)
            #endif
            if (__pyx_t_3 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3));
          #else
          __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_3);
          #endif
          ++__pyx_t_3;
        }
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1219, __pyx_L5_error)
      } else {
        __pyx_t_5 = __pyx_t_4(__pyx_t_2);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1219, __pyx_L5_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_x, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "pyteomics/cparser.pyx":1217
 *     def __init__(self, str label, targets):
 *         self.modification = label
 *         self.targets = [ModificationTarget._from_string(x) if isinstance(x, str)             # <<<<<<<<<<<<<<
 *                         else ModificationTarget._create(None, PositionType.universal)
 *                         for x in targets]
*/
      __pyx_t_6 = PyUnicode_Check(__pyx_8genexpr1__pyx_v_x); 
      if (__pyx_t_6) {
        if (!(likely(PyUnicode_CheckExact(__pyx_8genexpr1__pyx_v_x))||((__pyx_8genexpr1__pyx_v_x) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_8genexpr1__pyx_v_x))) __PYX_ERR(0, 1217, __pyx_L5_error)
        __pyx_t_7 = ((PyObject *)__pyx_f_9pyteomics_7cparser_18ModificationTarget__from_string(((PyObject*)__pyx_8genexpr1__pyx_v_x))); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1217, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_5 = __pyx_t_7;
        __pyx_t_7 = 0;
      } else {

        /* "pyteomics/cparser.pyx":1218
 *         self.modification = label
 *         self.targets = [ModificationTarget._from_string(x) if isinstance(x, str)
 *                         else ModificationTarget._create(None, PositionType.universal)             # <<<<<<<<<<<<<<
 *                         for x in targets]
 * 
*/
        __pyx_t_7 = ((PyObject *)__pyx_f_9pyteomics_7cparser_18ModificationTarget__create(((PyObject*)Py_None), __pyx_e_9pyteomics_7cparser_universal)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1218, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_5 = __pyx_t_7;
        __pyx_t_7 = 0;
      }
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 1217, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "pyteomics/cparser.pyx":1219
 *         self.targets = [ModificationTarget._from_string(x) if isinstance(x, str)
 *                         else ModificationTarget._create(None, PositionType.universal)
 *                         for x in targets]             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint test(self, SequencePosition position):
*/
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_x); __pyx_8genexpr1__pyx_v_x = 0;
    goto __pyx_L9_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_x); __pyx_8genexpr1__pyx_v_x = 0;
    goto __pyx_L1_error;
    __pyx_L9_exit_scope:;
  } /* exit inner scope */

  /* "pyteomics/cparser.pyx":1217
 *     def __init__(self, str label, targets):
 *         self.modification = label
 *         self.targets = [ModificationTarget._from_string(x) if isinstance(x, str)             # <<<<<<<<<<<<<<
 *                         else ModificationTarget._create(None, PositionType.universal)
 *                         for x in targets]
*/
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->targets);
  __Pyx_DECREF(__pyx_v_self->targets);
  __pyx_v_self->targets = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1215
 *         public list targets
 * 
 *     def __init__(self, str label, targets):             # <<<<<<<<<<<<<<
 *         self.modification = label
 *         self.targets = [ModificationTarget._from_string(x) if isinstance(x, str)
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_x);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1221
 *                         for x in targets]
 * 
 *     cpdef bint test(self, SequencePosition position):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_3test(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static int __pyx_f_9pyteomics_7cparser_16ModificationRule_test(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position, int __pyx_skip_dispatch) {
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_t = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("test", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationRule &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_test); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1221, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_9pyteomics_7cparser_16ModificationRule_3test)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, ((PyObject *)__pyx_v_position)};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1221, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1221, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_6;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "pyteomics/cparser.pyx":1225
 *             size_t i, n
 *             ModificationTarget t
 *         n = PyList_Size(self.targets)             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             t = <ModificationTarget>PyList_GET_ITEM(self.targets, i)
*/
  __pyx_t_1 = __pyx_v_self->targets;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_7 = PyList_Size(__pyx_t_1); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1225, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_n = __pyx_t_7;

  /* "pyteomics/cparser.pyx":1226
 *             ModificationTarget t
 *         n = PyList_Size(self.targets)
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             t = <ModificationTarget>PyList_GET_ITEM(self.targets, i)
 *             if t.test(position):
*/
  __pyx_t_5 = __pyx_v_n;
  __pyx_t_8 = __pyx_t_5;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "pyteomics/cparser.pyx":1227
 *         n = PyList_Size(self.targets)
 *         for i in range(n):
 *             t = <ModificationTarget>PyList_GET_ITEM(self.targets, i)             # <<<<<<<<<<<<<<
 *             if t.test(position):
 *                 return True
*/
    __pyx_t_1 = __pyx_v_self->targets;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_10 = PyList_GET_ITEM(__pyx_t_1, __pyx_v_i);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_t, ((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":1228
 *         for i in range(n):
 *             t = <ModificationTarget>PyList_GET_ITEM(self.targets, i)
 *             if t.test(position):             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
*/
    __pyx_t_6 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationTarget *)__pyx_v_t->__pyx_vtab)->test(__pyx_v_t, __pyx_v_position, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1228, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "pyteomics/cparser.pyx":1229
 *             t = <ModificationTarget>PyList_GET_ITEM(self.targets, i)
 *             if t.test(position):
 *                 return True             # <<<<<<<<<<<<<<
 *         return False
 * 
*/
      __pyx_r = 1;
      goto __pyx_L0;

      /* "pyteomics/cparser.pyx":1228
 *         for i in range(n):
 *             t = <ModificationTarget>PyList_GET_ITEM(self.targets, i)
 *             if t.test(position):             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
*/
    }
  }

  /* "pyteomics/cparser.pyx":1230
 *             if t.test(position):
 *                 return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     def __call__(self, position):
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1221
 *                         for x in targets]
 * 
 *     cpdef bint test(self, SequencePosition position):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.test", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_3test(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16ModificationRule_2test, "ModificationRule.test(self, SequencePosition position) -> bool");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16ModificationRule_3test = {"test", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16ModificationRule_3test, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16ModificationRule_2test};
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_3test(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("test (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_position,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1221, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1221, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "test", 0) < 0) __PYX_ERR(0, 1221, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("test", 1, 1, 1, i); __PYX_ERR(0, 1221, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1221, __pyx_L3_error)
    }
    __pyx_v_position = ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("test", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1221, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.test", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_position), __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, 1, "position", 0))) __PYX_ERR(0, 1221, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_2test(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self), __pyx_v_position);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_2test(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("test", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_16ModificationRule_test(__pyx_v_self, __pyx_v_position, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1221, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.test", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1232
 *         return False
 * 
 *     def __call__(self, position):             # <<<<<<<<<<<<<<
 *         return self.test(position)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_5__call__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_5__call__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_position = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__call__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_position,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1232, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1232, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__call__", 0) < 0) __PYX_ERR(0, 1232, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__call__", 1, 1, 1, i); __PYX_ERR(0, 1232, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1232, __pyx_L3_error)
    }
    __pyx_v_position = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__call__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1232, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.__call__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_4__call__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self), __pyx_v_position);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_4__call__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, PyObject *__pyx_v_position) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__call__", 0);

  /* "pyteomics/cparser.pyx":1233
 * 
 *     def __call__(self, position):
 *         return self.test(position)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  if (!(likely(((__pyx_v_position) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_position, __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition))))) __PYX_ERR(0, 1233, __pyx_L1_error)
  __pyx_t_1 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationRule *)__pyx_v_self->__pyx_vtab)->test(__pyx_v_self, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_position), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1233, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1232
 *         return False
 * 
 *     def __call__(self, position):             # <<<<<<<<<<<<<<
 *         return self.test(position)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.__call__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1235
 *         return self.test(position)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         t = "{self.__class__.__name__}({self.modification!r}, {self.targets})"
 *         return t.format(self=self)
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_7__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_7__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_6__repr__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_6__repr__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self) {
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "pyteomics/cparser.pyx":1236
 * 
 *     def __repr__(self):
 *         t = "{self.__class__.__name__}({self.modification!r}, {self.targets})"             # <<<<<<<<<<<<<<
 *         return t.format(self=self)
 * 
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_self___class_____name___self_mo);
  __pyx_v_t = __pyx_mstate_global->__pyx_kp_u_self___class_____name___self_mo;

  /* "pyteomics/cparser.pyx":1237
 *     def __repr__(self):
 *         t = "{self.__class__.__name__}({self.modification!r}, {self.targets})"
 *         return t.format(self=self)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_v_t;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_self, ((PyObject *)__pyx_v_self), __pyx_t_4, __pyx_callargs+1, 0) < 0) __PYX_ERR(0, 1237, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1235
 *         return self.test(position)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         t = "{self.__class__.__name__}({self.modification!r}, {self.targets})"
 *         return t.format(self=self)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1212
 * cdef class ModificationRule(object):
 *     cdef:
 *         public str modification             # <<<<<<<<<<<<<<
 *         public list targets
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_12modification_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_12modification_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_12modification___get__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_12modification___get__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->modification);
  __pyx_r = __pyx_v_self->modification;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16ModificationRule_12modification_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16ModificationRule_12modification_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_12modification_2__set__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16ModificationRule_12modification_2__set__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 1212, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->modification);
  __Pyx_DECREF(__pyx_v_self->modification);
  __pyx_v_self->modification = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.modification.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16ModificationRule_12modification_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16ModificationRule_12modification_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_12modification_4__del__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16ModificationRule_12modification_4__del__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->modification);
  __Pyx_DECREF(__pyx_v_self->modification);
  __pyx_v_self->modification = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1213
 *     cdef:
 *         public str modification
 *         public list targets             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, str label, targets):
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_7targets_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_7targets_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_7targets___get__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_7targets___get__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->targets);
  __pyx_r = __pyx_v_self->targets;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16ModificationRule_7targets_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16ModificationRule_7targets_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_7targets_2__set__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16ModificationRule_7targets_2__set__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(0, 1213, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->targets);
  __Pyx_DECREF(__pyx_v_self->targets);
  __pyx_v_self->targets = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.targets.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_16ModificationRule_7targets_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_16ModificationRule_7targets_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_7targets_4__del__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_16ModificationRule_7targets_4__del__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->targets);
  __Pyx_DECREF(__pyx_v_self->targets);
  __pyx_v_self->targets = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_9__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16ModificationRule_8__reduce_cython__, "ModificationRule.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16ModificationRule_9__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16ModificationRule_9__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16ModificationRule_8__reduce_cython__};
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_9__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_8__reduce_cython__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_8__reduce_cython__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.modification, self.targets)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
*/
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->modification);
  __Pyx_GIVEREF(__pyx_v_self->modification);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->modification) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->targets);
  __Pyx_GIVEREF(__pyx_v_self->targets);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->targets) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.modification, self.targets)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
*/
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.modification, self.targets)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  if (__pyx_t_2) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict) != (0)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.modification is not None or self.targets is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.modification, self.targets)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.modification is not None or self.targets is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, None), state
*/
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->modification != ((PyObject*)Py_None));
    if (!__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->targets != ((PyObject*)Py_None));
    __pyx_t_2 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_2;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.modification is not None or self.targets is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.modification is not None or self.targets is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ModificationRule); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_214727337);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_214727337);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_214727337) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.modification is not None or self.targets is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, None), state
 *     else:
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ModificationRule__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ModificationRule); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_214727337);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_214727337);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_214727337) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ModificationRule__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_11__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_16ModificationRule_10__setstate_cython__, "ModificationRule.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_16ModificationRule_11__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16ModificationRule_11__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16ModificationRule_10__setstate_cython__};
static PyObject *__pyx_pw_9pyteomics_7cparser_16ModificationRule_11__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_16ModificationRule_10__setstate_cython__(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_16ModificationRule_10__setstate_cython__(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ModificationRule__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser___pyx_unpickle_ModificationRule__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ModificationRule__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationRule.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1246
 *         public PositionType position_type
 * 
 *     def __init__(self, target, position_type=PositionType.internal):             # <<<<<<<<<<<<<<
 *         self.target = target
 *         self.position_type = position_type
*/

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_18ModificationTarget_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_18ModificationTarget_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_position_type = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_target,&__pyx_mstate_global->__pyx_n_u_position_type,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1246, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1246, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1246, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < 0) __PYX_ERR(0, 1246, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__4);
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, i); __PYX_ERR(0, 1246, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1246, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1246, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__4);
    }
    __pyx_v_target = values[0];
    __pyx_v_position_type = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 1246, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget___init__(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self), __pyx_v_target, __pyx_v_position_type);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_18ModificationTarget___init__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, PyObject *__pyx_v_target, PyObject *__pyx_v_position_type) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyteomics/cparser.pyx":1247
 * 
 *     def __init__(self, target, position_type=PositionType.internal):
 *         self.target = target             # <<<<<<<<<<<<<<
 *         self.position_type = position_type
 * 
*/
  __pyx_t_1 = __pyx_v_target;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 1247, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->target);
  __Pyx_DECREF(__pyx_v_self->target);
  __pyx_v_self->target = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1248
 *     def __init__(self, target, position_type=PositionType.internal):
 *         self.target = target
 *         self.position_type = position_type             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint test(self, SequencePosition position):
*/
  __pyx_t_2 = ((enum __pyx_t_9pyteomics_7cparser_PositionType)__Pyx_PyLong_As_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_v_position_type)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1248, __pyx_L1_error)
  __pyx_v_self->position_type = __pyx_t_2;

  /* "pyteomics/cparser.pyx":1246
 *         public PositionType position_type
 * 
 *     def __init__(self, target, position_type=PositionType.internal):             # <<<<<<<<<<<<<<
 *         self.target = target
 *         self.position_type = position_type
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1250
 *         self.position_type = position_type
 * 
 *     cpdef bint test(self, SequencePosition position):             # <<<<<<<<<<<<<<
 *         if self.position_type == PositionType.universal:
 *             return True
*/

static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_3test(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static int __pyx_f_9pyteomics_7cparser_18ModificationTarget_test(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("test", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationTarget &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_test); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1250, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_9pyteomics_7cparser_18ModificationTarget_3test)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, ((PyObject *)__pyx_v_position)};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1250, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1250, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_6;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "pyteomics/cparser.pyx":1251
 * 
 *     cpdef bint test(self, SequencePosition position):
 *         if self.position_type == PositionType.universal:             # <<<<<<<<<<<<<<
 *             return True
 *         if self.position_type != PositionType.internal:
*/
  __pyx_t_6 = (__pyx_v_self->position_type == __pyx_e_9pyteomics_7cparser_universal);
  if (__pyx_t_6) {

    /* "pyteomics/cparser.pyx":1252
 *     cpdef bint test(self, SequencePosition position):
 *         if self.position_type == PositionType.universal:
 *             return True             # <<<<<<<<<<<<<<
 *         if self.position_type != PositionType.internal:
 *             if position.position_type != self.position_type:
*/
    __pyx_r = 1;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":1251
 * 
 *     cpdef bint test(self, SequencePosition position):
 *         if self.position_type == PositionType.universal:             # <<<<<<<<<<<<<<
 *             return True
 *         if self.position_type != PositionType.internal:
*/
  }

  /* "pyteomics/cparser.pyx":1253
 *         if self.position_type == PositionType.universal:
 *             return True
 *         if self.position_type != PositionType.internal:             # <<<<<<<<<<<<<<
 *             if position.position_type != self.position_type:
 *                 return False
*/
  __pyx_t_6 = (__pyx_v_self->position_type != __pyx_e_9pyteomics_7cparser_internal);
  if (__pyx_t_6) {

    /* "pyteomics/cparser.pyx":1254
 *             return True
 *         if self.position_type != PositionType.internal:
 *             if position.position_type != self.position_type:             # <<<<<<<<<<<<<<
 *                 return False
 *         if self.target is not None:
*/
    __pyx_t_6 = (__pyx_v_position->position_type != __pyx_v_self->position_type);
    if (__pyx_t_6) {

      /* "pyteomics/cparser.pyx":1255
 *         if self.position_type != PositionType.internal:
 *             if position.position_type != self.position_type:
 *                 return False             # <<<<<<<<<<<<<<
 *         if self.target is not None:
 *             if self.target != position.amino_acid:
*/
      __pyx_r = 0;
      goto __pyx_L0;

      /* "pyteomics/cparser.pyx":1254
 *             return True
 *         if self.position_type != PositionType.internal:
 *             if position.position_type != self.position_type:             # <<<<<<<<<<<<<<
 *                 return False
 *         if self.target is not None:
*/
    }

    /* "pyteomics/cparser.pyx":1253
 *         if self.position_type == PositionType.universal:
 *             return True
 *         if self.position_type != PositionType.internal:             # <<<<<<<<<<<<<<
 *             if position.position_type != self.position_type:
 *                 return False
*/
  }

  /* "pyteomics/cparser.pyx":1256
 *             if position.position_type != self.position_type:
 *                 return False
 *         if self.target is not None:             # <<<<<<<<<<<<<<
 *             if self.target != position.amino_acid:
 *                 return False
*/
  __pyx_t_6 = (__pyx_v_self->target != ((PyObject*)Py_None));
  if (__pyx_t_6) {

    /* "pyteomics/cparser.pyx":1257
 *                 return False
 *         if self.target is not None:
 *             if self.target != position.amino_acid:             # <<<<<<<<<<<<<<
 *                 return False
 *         return True
*/
    __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_v_self->target, __pyx_v_position->amino_acid, Py_NE)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1257, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "pyteomics/cparser.pyx":1258
 *         if self.target is not None:
 *             if self.target != position.amino_acid:
 *                 return False             # <<<<<<<<<<<<<<
 *         return True
 * 
*/
      __pyx_r = 0;
      goto __pyx_L0;

      /* "pyteomics/cparser.pyx":1257
 *                 return False
 *         if self.target is not None:
 *             if self.target != position.amino_acid:             # <<<<<<<<<<<<<<
 *                 return False
 *         return True
*/
    }

    /* "pyteomics/cparser.pyx":1256
 *             if position.position_type != self.position_type:
 *                 return False
 *         if self.target is not None:             # <<<<<<<<<<<<<<
 *             if self.target != position.amino_acid:
 *                 return False
*/
  }

  /* "pyteomics/cparser.pyx":1259
 *             if self.target != position.amino_acid:
 *                 return False
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     def __call__(self, position):
*/
  __pyx_r = 1;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1250
 *         self.position_type = position_type
 * 
 *     cpdef bint test(self, SequencePosition position):             # <<<<<<<<<<<<<<
 *         if self.position_type == PositionType.universal:
 *             return True
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.test", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_3test(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_18ModificationTarget_2test, "ModificationTarget.test(self, SequencePosition position) -> bool");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_18ModificationTarget_3test = {"test", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_18ModificationTarget_3test, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_18ModificationTarget_2test};
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_3test(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("test (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_position,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1250, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1250, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "test", 0) < 0) __PYX_ERR(0, 1250, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("test", 1, 1, 1, i); __PYX_ERR(0, 1250, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1250, __pyx_L3_error)
    }
    __pyx_v_position = ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("test", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1250, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.test", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_position), __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, 1, "position", 0))) __PYX_ERR(0, 1250, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_2test(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self), __pyx_v_position);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_2test(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v_position) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("test", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_18ModificationTarget_test(__pyx_v_self, __pyx_v_position, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1250, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.test", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1261
 *         return True
 * 
 *     def __call__(self, position):             # <<<<<<<<<<<<<<
 *         return self.test(position)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_5__call__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_5__call__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_position = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__call__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_position,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1261, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1261, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__call__", 0) < 0) __PYX_ERR(0, 1261, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__call__", 1, 1, 1, i); __PYX_ERR(0, 1261, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1261, __pyx_L3_error)
    }
    __pyx_v_position = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__call__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1261, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.__call__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_4__call__(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self), __pyx_v_position);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_4__call__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, PyObject *__pyx_v_position) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__call__", 0);

  /* "pyteomics/cparser.pyx":1262
 * 
 *     def __call__(self, position):
 *         return self.test(position)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  if (!(likely(((__pyx_v_position) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_position, __pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition))))) __PYX_ERR(0, 1262, __pyx_L1_error)
  __pyx_t_1 = ((struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self->__pyx_vtab)->test(__pyx_v_self, ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v_position), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1262, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1261
 *         return True
 * 
 *     def __call__(self, position):             # <<<<<<<<<<<<<<
 *         return self.test(position)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.__call__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1264
 *         return self.test(position)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "{self.__class__.__name__}({self.target!r}, {self.position_type!r})".format(self=self)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_7__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_7__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_6__repr__(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_6__repr__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "pyteomics/cparser.pyx":1265
 * 
 *     def __repr__(self):
 *         return "{self.__class__.__name__}({self.target!r}, {self.position_type!r})".format(self=self)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_mstate_global->__pyx_kp_u_self___class_____name___self_ta;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1265, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_self, ((PyObject *)__pyx_v_self), __pyx_t_4, __pyx_callargs+1, 0) < 0) __PYX_ERR(0, 1265, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder(__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1265, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1264
 *         return self.test(position)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "{self.__class__.__name__}({self.target!r}, {self.position_type!r})".format(self=self)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1267
 *         return "{self.__class__.__name__}({self.target!r}, {self.position_type!r})".format(self=self)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef ModificationTarget _create(str target, PositionType position_type):
 *         cdef ModificationTarget inst = ModificationTarget.__new__(ModificationTarget)
*/

static struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_f_9pyteomics_7cparser_18ModificationTarget__create(PyObject *__pyx_v_target, enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_v_position_type) {
  struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_inst = 0;
  struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create", 0);

  /* "pyteomics/cparser.pyx":1269
 *     @staticmethod
 *     cdef ModificationTarget _create(str target, PositionType position_type):
 *         cdef ModificationTarget inst = ModificationTarget.__new__(ModificationTarget)             # <<<<<<<<<<<<<<
 *         inst.target = target
 *         inst.position_type = position_type
*/
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_9pyteomics_7cparser_ModificationTarget(((PyTypeObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationTarget), __pyx_mstate_global->__pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1269, __pyx_L1_error)
  __Pyx_GOTREF((PyObject *)__pyx_t_1);
  __pyx_v_inst = ((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1270
 *     cdef ModificationTarget _create(str target, PositionType position_type):
 *         cdef ModificationTarget inst = ModificationTarget.__new__(ModificationTarget)
 *         inst.target = target             # <<<<<<<<<<<<<<
 *         inst.position_type = position_type
 *         return inst
*/
  __Pyx_INCREF(__pyx_v_target);
  __Pyx_GIVEREF(__pyx_v_target);
  __Pyx_GOTREF(__pyx_v_inst->target);
  __Pyx_DECREF(__pyx_v_inst->target);
  __pyx_v_inst->target = __pyx_v_target;

  /* "pyteomics/cparser.pyx":1271
 *         cdef ModificationTarget inst = ModificationTarget.__new__(ModificationTarget)
 *         inst.target = target
 *         inst.position_type = position_type             # <<<<<<<<<<<<<<
 *         return inst
 * 
*/
  __pyx_v_inst->position_type = __pyx_v_position_type;

  /* "pyteomics/cparser.pyx":1272
 *         inst.target = target
 *         inst.position_type = position_type
 *         return inst             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_inst);
  __pyx_r = __pyx_v_inst;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1267
 *         return "{self.__class__.__name__}({self.target!r}, {self.position_type!r})".format(self=self)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef ModificationTarget _create(str target, PositionType position_type):
 *         cdef ModificationTarget inst = ModificationTarget.__new__(ModificationTarget)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget._create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1274
 *         return inst
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def from_string(str string):
 *         return ModificationTarget._from_string(string)
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_9from_string(CYTHON_UNUSED PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_18ModificationTarget_8from_string, "ModificationTarget.from_string(str string)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_18ModificationTarget_9from_string = {"from_string", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_18ModificationTarget_9from_string, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_18ModificationTarget_8from_string};
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_9from_string(CYTHON_UNUSED PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_string = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("from_string (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_string,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1274, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1274, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "from_string", 0) < 0) __PYX_ERR(0, 1274, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("from_string", 1, 1, 1, i); __PYX_ERR(0, 1274, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1274, __pyx_L3_error)
    }
    __pyx_v_string = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("from_string", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1274, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.from_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_string), (&PyUnicode_Type), 1, "string", 1))) __PYX_ERR(0, 1275, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_8from_string(__pyx_v_string);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_8from_string(PyObject *__pyx_v_string) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_string", 0);

  /* "pyteomics/cparser.pyx":1276
 *     @staticmethod
 *     def from_string(str string):
 *         return ModificationTarget._from_string(string)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9pyteomics_7cparser_18ModificationTarget__from_string(__pyx_v_string)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1274
 *         return inst
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def from_string(str string):
 *         return ModificationTarget._from_string(string)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.from_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1278
 *         return ModificationTarget._from_string(string)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef ModificationTarget _from_string(str string):
 *         cdef:
*/

static struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_f_9pyteomics_7cparser_18ModificationTarget__from_string(PyObject *__pyx_v_string) {
  enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_v_position_type;
  size_t __pyx_v_n;
  PyObject *__pyx_v_target = 0;
  struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_inst = 0;
  struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  Py_UCS4 __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_from_string", 0);

  /* "pyteomics/cparser.pyx":1285
 *             str target
 *             ModificationTarget inst
 *         n = len(string)             # <<<<<<<<<<<<<<
 *         target = None
 *         position_type = PositionType.internal
*/
  if (unlikely(__pyx_v_string == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 1285, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_string); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1285, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "pyteomics/cparser.pyx":1286
 *             ModificationTarget inst
 *         n = len(string)
 *         target = None             # <<<<<<<<<<<<<<
 *         position_type = PositionType.internal
 *         if _is_cterm_mod(string):
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_target = ((PyObject*)Py_None);

  /* "pyteomics/cparser.pyx":1287
 *         n = len(string)
 *         target = None
 *         position_type = PositionType.internal             # <<<<<<<<<<<<<<
 *         if _is_cterm_mod(string):
 *             position_type = PositionType.c_term
*/
  __pyx_v_position_type = __pyx_e_9pyteomics_7cparser_internal;

  /* "pyteomics/cparser.pyx":1288
 *         target = None
 *         position_type = PositionType.internal
 *         if _is_cterm_mod(string):             # <<<<<<<<<<<<<<
 *             position_type = PositionType.c_term
 *             if n > 5:
*/
  __pyx_t_2 = __pyx_f_9pyteomics_7cparser__is_cterm_mod(__pyx_v_string); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1288, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":1289
 *         position_type = PositionType.internal
 *         if _is_cterm_mod(string):
 *             position_type = PositionType.c_term             # <<<<<<<<<<<<<<
 *             if n > 5:
 *                 if n > 6:
*/
    __pyx_v_position_type = __pyx_e_9pyteomics_7cparser_c_term;

    /* "pyteomics/cparser.pyx":1290
 *         if _is_cterm_mod(string):
 *             position_type = PositionType.c_term
 *             if n > 5:             # <<<<<<<<<<<<<<
 *                 if n > 6:
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
*/
    __pyx_t_2 = (__pyx_v_n > 5);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":1291
 *             position_type = PositionType.c_term
 *             if n > 5:
 *                 if n > 6:             # <<<<<<<<<<<<<<
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
 *                 target = string[-1]
*/
      __pyx_t_2 = (__pyx_v_n > 6);
      if (unlikely(__pyx_t_2)) {

        /* "pyteomics/cparser.pyx":1292
 *             if n > 5:
 *                 if n > 6:
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))             # <<<<<<<<<<<<<<
 *                 target = string[-1]
 *         elif _is_nterm_mod(string):
*/
        __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_builtin_ValueError);
        __pyx_t_5 = __pyx_builtin_ValueError; 
        __pyx_t_6 = __Pyx_PyUnicode_Unicode(__pyx_v_string); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1292, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Terminal_Modifications_with_Targ, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1292, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_8 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_7};
          __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1292, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 1292, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":1291
 *             position_type = PositionType.c_term
 *             if n > 5:
 *                 if n > 6:             # <<<<<<<<<<<<<<
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
 *                 target = string[-1]
*/
      }

      /* "pyteomics/cparser.pyx":1293
 *                 if n > 6:
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
 *                 target = string[-1]             # <<<<<<<<<<<<<<
 *         elif _is_nterm_mod(string):
 *             position_type = PositionType.n_term
*/
      __pyx_t_9 = __Pyx_GetItemInt_Unicode(__pyx_v_string, -1L, long, 1, __Pyx_PyLong_From_long, 0, 1, 1); if (unlikely(__pyx_t_9 == (Py_UCS4)-1)) __PYX_ERR(0, 1293, __pyx_L1_error)
      __pyx_t_3 = __Pyx_PyUnicode_FromOrdinal(__pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1293, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_3)) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_3))) __PYX_ERR(0, 1293, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_target, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":1290
 *         if _is_cterm_mod(string):
 *             position_type = PositionType.c_term
 *             if n > 5:             # <<<<<<<<<<<<<<
 *                 if n > 6:
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
*/
    }

    /* "pyteomics/cparser.pyx":1288
 *         target = None
 *         position_type = PositionType.internal
 *         if _is_cterm_mod(string):             # <<<<<<<<<<<<<<
 *             position_type = PositionType.c_term
 *             if n > 5:
*/
    goto __pyx_L3;
  }

  /* "pyteomics/cparser.pyx":1294
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
 *                 target = string[-1]
 *         elif _is_nterm_mod(string):             # <<<<<<<<<<<<<<
 *             position_type = PositionType.n_term
 *             if n > 5:
*/
  __pyx_t_2 = __pyx_f_9pyteomics_7cparser__is_nterm_mod(__pyx_v_string); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1294, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":1295
 *                 target = string[-1]
 *         elif _is_nterm_mod(string):
 *             position_type = PositionType.n_term             # <<<<<<<<<<<<<<
 *             if n > 5:
 *                 if n > 6:
*/
    __pyx_v_position_type = __pyx_e_9pyteomics_7cparser_n_term;

    /* "pyteomics/cparser.pyx":1296
 *         elif _is_nterm_mod(string):
 *             position_type = PositionType.n_term
 *             if n > 5:             # <<<<<<<<<<<<<<
 *                 if n > 6:
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
*/
    __pyx_t_2 = (__pyx_v_n > 5);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":1297
 *             position_type = PositionType.n_term
 *             if n > 5:
 *                 if n > 6:             # <<<<<<<<<<<<<<
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
 *                 target = string[-1]
*/
      __pyx_t_2 = (__pyx_v_n > 6);
      if (unlikely(__pyx_t_2)) {

        /* "pyteomics/cparser.pyx":1298
 *             if n > 5:
 *                 if n > 6:
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))             # <<<<<<<<<<<<<<
 *                 target = string[-1]
 *         else:
*/
        __pyx_t_5 = NULL;
        __Pyx_INCREF(__pyx_builtin_ValueError);
        __pyx_t_7 = __pyx_builtin_ValueError; 
        __pyx_t_4 = __Pyx_PyUnicode_Unicode(__pyx_v_string); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1298, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_6 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Terminal_Modifications_with_Targ, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1298, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_8 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_6};
          __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1298, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 1298, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":1297
 *             position_type = PositionType.n_term
 *             if n > 5:
 *                 if n > 6:             # <<<<<<<<<<<<<<
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
 *                 target = string[-1]
*/
      }

      /* "pyteomics/cparser.pyx":1299
 *                 if n > 6:
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
 *                 target = string[-1]             # <<<<<<<<<<<<<<
 *         else:
 *             if n > 1:
*/
      __pyx_t_9 = __Pyx_GetItemInt_Unicode(__pyx_v_string, -1L, long, 1, __Pyx_PyLong_From_long, 0, 1, 1); if (unlikely(__pyx_t_9 == (Py_UCS4)-1)) __PYX_ERR(0, 1299, __pyx_L1_error)
      __pyx_t_3 = __Pyx_PyUnicode_FromOrdinal(__pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1299, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_3)) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_3))) __PYX_ERR(0, 1299, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_target, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":1296
 *         elif _is_nterm_mod(string):
 *             position_type = PositionType.n_term
 *             if n > 5:             # <<<<<<<<<<<<<<
 *                 if n > 6:
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
*/
    }

    /* "pyteomics/cparser.pyx":1294
 *                     raise ValueError("Terminal Modifications with Target must be a single Amino Acid, got %s" % (string, ))
 *                 target = string[-1]
 *         elif _is_nterm_mod(string):             # <<<<<<<<<<<<<<
 *             position_type = PositionType.n_term
 *             if n > 5:
*/
    goto __pyx_L3;
  }

  /* "pyteomics/cparser.pyx":1301
 *                 target = string[-1]
 *         else:
 *             if n > 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Target must be a single Amino Acid, got %s" % (string, ))
 *             target = string
*/
  /*else*/ {
    __pyx_t_2 = (__pyx_v_n > 1);
    if (unlikely(__pyx_t_2)) {

      /* "pyteomics/cparser.pyx":1302
 *         else:
 *             if n > 1:
 *                 raise ValueError("Target must be a single Amino Acid, got %s" % (string, ))             # <<<<<<<<<<<<<<
 *             target = string
 *         inst = ModificationTarget._create(target, position_type)
*/
      __pyx_t_7 = NULL;
      __Pyx_INCREF(__pyx_builtin_ValueError);
      __pyx_t_6 = __pyx_builtin_ValueError; 
      __pyx_t_5 = __Pyx_PyUnicode_Unicode(__pyx_v_string); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1302, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Target_must_be_a_single_Amino_Ac, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1302, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_8 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 1302, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":1301
 *                 target = string[-1]
 *         else:
 *             if n > 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Target must be a single Amino Acid, got %s" % (string, ))
 *             target = string
*/
    }

    /* "pyteomics/cparser.pyx":1303
 *             if n > 1:
 *                 raise ValueError("Target must be a single Amino Acid, got %s" % (string, ))
 *             target = string             # <<<<<<<<<<<<<<
 *         inst = ModificationTarget._create(target, position_type)
 *         return inst
*/
    __Pyx_INCREF(__pyx_v_string);
    __Pyx_DECREF_SET(__pyx_v_target, __pyx_v_string);
  }
  __pyx_L3:;

  /* "pyteomics/cparser.pyx":1304
 *                 raise ValueError("Target must be a single Amino Acid, got %s" % (string, ))
 *             target = string
 *         inst = ModificationTarget._create(target, position_type)             # <<<<<<<<<<<<<<
 *         return inst
 * 
*/
  __pyx_t_3 = ((PyObject *)__pyx_f_9pyteomics_7cparser_18ModificationTarget__create(__pyx_v_target, __pyx_v_position_type)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_inst = ((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyteomics/cparser.pyx":1305
 *             target = string
 *         inst = ModificationTarget._create(target, position_type)
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_inst);
  __pyx_r = __pyx_v_inst;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":1278
 *         return ModificationTarget._from_string(string)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     cdef ModificationTarget _from_string(str string):
 *         cdef:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget._from_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_target);
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1243
 * cdef class ModificationTarget(object):
 *     cdef:
 *         public str target             # <<<<<<<<<<<<<<
 *         public PositionType position_type
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_6target_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_6target_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_6target___get__(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_6target___get__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->target);
  __pyx_r = __pyx_v_self->target;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_18ModificationTarget_6target_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_18ModificationTarget_6target_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_6target_2__set__(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_18ModificationTarget_6target_2__set__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 1243, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->target);
  __Pyx_DECREF(__pyx_v_self->target);
  __pyx_v_self->target = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.target.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_18ModificationTarget_6target_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_18ModificationTarget_6target_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_6target_4__del__(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_18ModificationTarget_6target_4__del__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->target);
  __Pyx_DECREF(__pyx_v_self->target);
  __pyx_v_self->target = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":1244
 *     cdef:
 *         public str target
 *         public PositionType position_type             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, target, position_type=PositionType.internal):
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_13position_type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_13position_type_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_13position_type___get__(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_13position_type___get__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(__pyx_v_self->position_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.position_type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9pyteomics_7cparser_18ModificationTarget_13position_type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9pyteomics_7cparser_18ModificationTarget_13position_type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_13position_type_2__set__(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9pyteomics_7cparser_18ModificationTarget_13position_type_2__set__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = ((enum __pyx_t_9pyteomics_7cparser_PositionType)__Pyx_PyLong_As_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_v_value)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1244, __pyx_L1_error)
  __pyx_v_self->position_type = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.position_type.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_11__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_18ModificationTarget_10__reduce_cython__, "ModificationTarget.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_18ModificationTarget_11__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_18ModificationTarget_11__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_18ModificationTarget_10__reduce_cython__};
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_11__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_10__reduce_cython__(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_10__reduce_cython__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.position_type, self.target)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
*/
  __pyx_t_1 = __Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(__pyx_v_self->position_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->target);
  __Pyx_GIVEREF(__pyx_v_self->target);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->target) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.position_type, self.target)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
*/
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self.position_type, self.target)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict) != (0)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.target is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.position_type, self.target)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.target is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, None), state
*/
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->target != ((PyObject*)Py_None));
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.target is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.target is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ModificationTarge); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_16113035);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_16113035);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_mstate_global->__pyx_int_16113035) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.target is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, None), state
 *     else:
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ModificationTarget__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ModificationTarge); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_16113035);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_16113035);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_mstate_global->__pyx_int_16113035) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ModificationTarget__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_13__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_18ModificationTarget_12__setstate_cython__, "ModificationTarget.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_18ModificationTarget_13__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_18ModificationTarget_13__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_18ModificationTarget_12__setstate_cython__};
static PyObject *__pyx_pw_9pyteomics_7cparser_18ModificationTarget_13__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_18ModificationTarget_12__setstate_cython__(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_18ModificationTarget_12__setstate_cython__(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ModificationTarget__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser___pyx_unpickle_ModificationTarget__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ModificationTarget__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.ModificationTarget.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_9pyteomics_7cparser_16generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyteomics/cparser.pyx":1308
 * 
 * 
 * def isoforms(str sequence, **kwargs):             # <<<<<<<<<<<<<<
 *     cdef:
 *         IsoformGenerator isoform_generator
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_15isoforms(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_14isoforms, "isoforms(str sequence, **kwargs)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_15isoforms = {"isoforms", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_15isoforms, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_14isoforms};
static PyObject *__pyx_pw_9pyteomics_7cparser_15isoforms(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_sequence = 0;
  PyObject *__pyx_v_kwargs = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isoforms (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_sequence,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1308, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, __pyx_v_kwargs, values, kwd_pos_args, __pyx_kwds_len, "isoforms", 1) < 0) __PYX_ERR(0, 1308, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("isoforms", 1, 1, 1, i); __PYX_ERR(0, 1308, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1308, __pyx_L3_error)
    }
    __pyx_v_sequence = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isoforms", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1308, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_CLEAR(__pyx_v_kwargs);
  __Pyx_AddTraceback("pyteomics.cparser.isoforms", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sequence), (&PyUnicode_Type), 1, "sequence", 1))) __PYX_ERR(0, 1308, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_14isoforms(__pyx_self, __pyx_v_sequence, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_DECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_14isoforms(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, PyObject *__pyx_v_kwargs) {
  struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isoforms", 0);
  __pyx_cur_scope = (struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *)__pyx_tp_new_9pyteomics_7cparser___pyx_scope_struct_1_isoforms(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1308, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_sequence = __pyx_v_sequence;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_sequence);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_sequence);
  __pyx_cur_scope->__pyx_v_kwargs = __pyx_v_kwargs;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_kwargs);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_kwargs);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_9pyteomics_7cparser_16generator1, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_isoforms, __pyx_mstate_global->__pyx_n_u_isoforms, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser); if (unlikely(!gen)) __PYX_ERR(0, 1308, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyteomics.cparser.isoforms", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_9pyteomics_7cparser_16generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *__pyx_cur_scope = ((struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  struct __pyx_opt_args_9pyteomics_7cparser__sequence_tostring __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isoforms", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L7_resume_from_yield;
    case 2: goto __pyx_L11_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(0, 1308, __pyx_L1_error)
  }

  /* "pyteomics/cparser.pyx":1314
 *         object gen
 * 
 *     variable_mods = kwargs.get("variable_mods", None)             # <<<<<<<<<<<<<<
 *     fixed_mods = kwargs.get("fixed_mods", None)
 *     labels = kwargs.get("labels", None)
*/
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_cur_scope->__pyx_v_kwargs, __pyx_mstate_global->__pyx_n_u_variable_mods, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_variable_mods = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1315
 * 
 *     variable_mods = kwargs.get("variable_mods", None)
 *     fixed_mods = kwargs.get("fixed_mods", None)             # <<<<<<<<<<<<<<
 *     labels = kwargs.get("labels", None)
 *     max_mods = kwargs.get("max_mods", None)
*/
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_cur_scope->__pyx_v_kwargs, __pyx_mstate_global->__pyx_n_u_fixed_mods, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_fixed_mods = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1316
 *     variable_mods = kwargs.get("variable_mods", None)
 *     fixed_mods = kwargs.get("fixed_mods", None)
 *     labels = kwargs.get("labels", None)             # <<<<<<<<<<<<<<
 *     max_mods = kwargs.get("max_mods", None)
 *     override = kwargs.get("override", False)
*/
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_cur_scope->__pyx_v_kwargs, __pyx_mstate_global->__pyx_n_u_labels, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_labels = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1317
 *     fixed_mods = kwargs.get("fixed_mods", None)
 *     labels = kwargs.get("labels", None)
 *     max_mods = kwargs.get("max_mods", None)             # <<<<<<<<<<<<<<
 *     override = kwargs.get("override", False)
 *     show_unmodified_termini = kwargs.get("show_unmodified_termini", False)
*/
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_cur_scope->__pyx_v_kwargs, __pyx_mstate_global->__pyx_n_u_max_mods, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_max_mods = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1318
 *     labels = kwargs.get("labels", None)
 *     max_mods = kwargs.get("max_mods", None)
 *     override = kwargs.get("override", False)             # <<<<<<<<<<<<<<
 *     show_unmodified_termini = kwargs.get("show_unmodified_termini", False)
 *     _format = kwargs.get("format", "str")
*/
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_cur_scope->__pyx_v_kwargs, __pyx_mstate_global->__pyx_n_u_override, Py_False); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_override = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1319
 *     max_mods = kwargs.get("max_mods", None)
 *     override = kwargs.get("override", False)
 *     show_unmodified_termini = kwargs.get("show_unmodified_termini", False)             # <<<<<<<<<<<<<<
 *     _format = kwargs.get("format", "str")
 * 
*/
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_cur_scope->__pyx_v_kwargs, __pyx_mstate_global->__pyx_n_u_show_unmodified_termini, Py_False); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_show_unmodified_termini = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1320
 *     override = kwargs.get("override", False)
 *     show_unmodified_termini = kwargs.get("show_unmodified_termini", False)
 *     _format = kwargs.get("format", "str")             # <<<<<<<<<<<<<<
 * 
 *     isoform_generator = IsoformGenerator(
*/
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_cur_scope->__pyx_v_kwargs, __pyx_mstate_global->__pyx_n_u_format, __pyx_mstate_global->__pyx_n_u_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v__format = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1322
 *     _format = kwargs.get("format", "str")
 * 
 *     isoform_generator = IsoformGenerator(             # <<<<<<<<<<<<<<
 *         variable_mods=variable_mods,
 *         fixed_mods=fixed_mods,
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF((PyObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator);
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator); 

  /* "pyteomics/cparser.pyx":1328
 *         max_mods=max_mods,
 *         override=override,
 *         show_unmodified_termini=show_unmodified_termini)             # <<<<<<<<<<<<<<
 *     gen = isoform_generator.generate(sequence)
 *     if _format == "str":
*/
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 6 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_variable_mods, __pyx_cur_scope->__pyx_v_variable_mods, __pyx_t_5, __pyx_callargs+1, 0) < 0) __PYX_ERR(0, 1322, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_fixed_mods, __pyx_cur_scope->__pyx_v_fixed_mods, __pyx_t_5, __pyx_callargs+1, 1) < 0) __PYX_ERR(0, 1322, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_labels, __pyx_cur_scope->__pyx_v_labels, __pyx_t_5, __pyx_callargs+1, 2) < 0) __PYX_ERR(0, 1322, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_max_mods, __pyx_cur_scope->__pyx_v_max_mods, __pyx_t_5, __pyx_callargs+1, 3) < 0) __PYX_ERR(0, 1322, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_override, __pyx_cur_scope->__pyx_v_override, __pyx_t_5, __pyx_callargs+1, 4) < 0) __PYX_ERR(0, 1322, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_show_unmodified_termini, __pyx_cur_scope->__pyx_v_show_unmodified_termini, __pyx_t_5, __pyx_callargs+1, 5) < 0) __PYX_ERR(0, 1322, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder(__pyx_t_3, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1322, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __Pyx_GIVEREF((PyObject *)__pyx_t_1);
  __pyx_cur_scope->__pyx_v_isoform_generator = ((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1329
 *         override=override,
 *         show_unmodified_termini=show_unmodified_termini)
 *     gen = isoform_generator.generate(sequence)             # <<<<<<<<<<<<<<
 *     if _format == "str":
 *         for form in gen:
*/
  __pyx_t_3 = ((PyObject *)__pyx_cur_scope->__pyx_v_isoform_generator);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_cur_scope->__pyx_v_sequence};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_generate, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_gen = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":1330
 *         show_unmodified_termini=show_unmodified_termini)
 *     gen = isoform_generator.generate(sequence)
 *     if _format == "str":             # <<<<<<<<<<<<<<
 *         for form in gen:
 *             yield _sequence_tostring(
*/
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_cur_scope->__pyx_v__format, __pyx_mstate_global->__pyx_n_u_str, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1330, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "pyteomics/cparser.pyx":1331
 *     gen = isoform_generator.generate(sequence)
 *     if _format == "str":
 *         for form in gen:             # <<<<<<<<<<<<<<
 *             yield _sequence_tostring(
 *                 form, show_unmodified_termini=show_unmodified_termini)
*/
    if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_gen)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_gen)) {
      __pyx_t_1 = __pyx_cur_scope->__pyx_v_gen; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_7 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_gen); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1331, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1331, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1331, __pyx_L1_error)
            #endif
            if (__pyx_t_7 >= __pyx_temp) break;
          }
          __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_7);
          ++__pyx_t_7;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1331, __pyx_L1_error)
            #endif
            if (__pyx_t_7 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_7));
          #else
          __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_7);
          #endif
          ++__pyx_t_7;
        }
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1331, __pyx_L1_error)
      } else {
        __pyx_t_3 = __pyx_t_8(__pyx_t_1);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1331, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_form);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_form, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":1333
 *         for form in gen:
 *             yield _sequence_tostring(
 *                 form, show_unmodified_termini=show_unmodified_termini)             # <<<<<<<<<<<<<<
 *     elif _format == "split":
 *         for form in gen:
*/
      if (!(likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_form))||((__pyx_cur_scope->__pyx_v_form) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_cur_scope->__pyx_v_form))) __PYX_ERR(0, 1333, __pyx_L1_error)
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_show_unmodified_termini); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1333, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":1332
 *     if _format == "str":
 *         for form in gen:
 *             yield _sequence_tostring(             # <<<<<<<<<<<<<<
 *                 form, show_unmodified_termini=show_unmodified_termini)
 *     elif _format == "split":
*/
      __pyx_t_9.__pyx_n = 1;
      __pyx_t_9.show_unmodified_termini = __pyx_t_6;
      __pyx_t_3 = __pyx_f_9pyteomics_7cparser__sequence_tostring(((PyObject*)__pyx_cur_scope->__pyx_v_form), 0, &__pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1332, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_r = __pyx_t_3;
      __pyx_t_3 = 0;
      __Pyx_XGIVEREF(__pyx_t_1);
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_7;
      __pyx_cur_scope->__pyx_t_2 = __pyx_t_8;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L7_resume_from_yield:;
      __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
      __pyx_cur_scope->__pyx_t_0 = 0;
      __Pyx_XGOTREF(__pyx_t_1);
      __pyx_t_7 = __pyx_cur_scope->__pyx_t_1;
      __pyx_t_8 = __pyx_cur_scope->__pyx_t_2;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1332, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":1331
 *     gen = isoform_generator.generate(sequence)
 *     if _format == "str":
 *         for form in gen:             # <<<<<<<<<<<<<<
 *             yield _sequence_tostring(
 *                 form, show_unmodified_termini=show_unmodified_termini)
*/
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":1330
 *         show_unmodified_termini=show_unmodified_termini)
 *     gen = isoform_generator.generate(sequence)
 *     if _format == "str":             # <<<<<<<<<<<<<<
 *         for form in gen:
 *             yield _sequence_tostring(
*/
    goto __pyx_L4;
  }

  /* "pyteomics/cparser.pyx":1334
 *             yield _sequence_tostring(
 *                 form, show_unmodified_termini=show_unmodified_termini)
 *     elif _format == "split":             # <<<<<<<<<<<<<<
 *         for form in gen:
 *             yield form
*/
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_cur_scope->__pyx_v__format, __pyx_mstate_global->__pyx_n_u_split, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1334, __pyx_L1_error)
  if (likely(__pyx_t_6)) {

    /* "pyteomics/cparser.pyx":1335
 *                 form, show_unmodified_termini=show_unmodified_termini)
 *     elif _format == "split":
 *         for form in gen:             # <<<<<<<<<<<<<<
 *             yield form
 *     else:
*/
    if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_gen)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_gen)) {
      __pyx_t_1 = __pyx_cur_scope->__pyx_v_gen; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_7 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_gen); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1335, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1335, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1335, __pyx_L1_error)
            #endif
            if (__pyx_t_7 >= __pyx_temp) break;
          }
          __pyx_t_3 = __Pyx_PyList_GetItemRef(__pyx_t_1, __pyx_t_7);
          ++__pyx_t_7;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1335, __pyx_L1_error)
            #endif
            if (__pyx_t_7 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_7));
          #else
          __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_7);
          #endif
          ++__pyx_t_7;
        }
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1335, __pyx_L1_error)
      } else {
        __pyx_t_3 = __pyx_t_8(__pyx_t_1);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1335, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_form);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_form, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyteomics/cparser.pyx":1336
 *     elif _format == "split":
 *         for form in gen:
 *             yield form             # <<<<<<<<<<<<<<
 *     else:
 *         raise PyteomicsError('Unsupported value of "format": {}'.format(_format))
*/
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_form);
      __pyx_r = __pyx_cur_scope->__pyx_v_form;
      __Pyx_XGIVEREF(__pyx_t_1);
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_7;
      __pyx_cur_scope->__pyx_t_2 = __pyx_t_8;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 2;
      return __pyx_r;
      __pyx_L11_resume_from_yield:;
      __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
      __pyx_cur_scope->__pyx_t_0 = 0;
      __Pyx_XGOTREF(__pyx_t_1);
      __pyx_t_7 = __pyx_cur_scope->__pyx_t_1;
      __pyx_t_8 = __pyx_cur_scope->__pyx_t_2;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1336, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":1335
 *                 form, show_unmodified_termini=show_unmodified_termini)
 *     elif _format == "split":
 *         for form in gen:             # <<<<<<<<<<<<<<
 *             yield form
 *     else:
*/
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":1334
 *             yield _sequence_tostring(
 *                 form, show_unmodified_termini=show_unmodified_termini)
 *     elif _format == "split":             # <<<<<<<<<<<<<<
 *         for form in gen:
 *             yield form
*/
    goto __pyx_L4;
  }

  /* "pyteomics/cparser.pyx":1338
 *             yield form
 *     else:
 *         raise PyteomicsError('Unsupported value of "format": {}'.format(_format))             # <<<<<<<<<<<<<<
*/
  /*else*/ {
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_PyteomicsError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_10 = __pyx_mstate_global->__pyx_kp_u_Unsupported_value_of_format;
    __Pyx_INCREF(__pyx_t_10);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_cur_scope->__pyx_v__format};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1338, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1338, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1338, __pyx_L1_error)
  }
  __pyx_L4:;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "pyteomics/cparser.pyx":1308
 * 
 * 
 * def isoforms(str sequence, **kwargs):             # <<<<<<<<<<<<<<
 *     cdef:
 *         IsoformGenerator isoform_generator
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("isoforms", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_SequencePosition(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_18__pyx_unpickle_SequencePosition(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_17__pyx_unpickle_SequencePosition, "__pyx_unpickle_SequencePosition(__pyx_type, long __pyx_checksum, __pyx_state)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_18__pyx_unpickle_SequencePosition = {"__pyx_unpickle_SequencePosition", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_18__pyx_unpickle_SequencePosition, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_17__pyx_unpickle_SequencePosition};
static PyObject *__pyx_pw_9pyteomics_7cparser_18__pyx_unpickle_SequencePosition(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_SequencePosition (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 1, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_SequencePosition", 0) < 0) __PYX_ERR(1, 1, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SequencePosition", 1, 3, 3, i); __PYX_ERR(1, 1, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 1, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SequencePosition", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_SequencePosition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_17__pyx_unpickle_SequencePosition(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_17__pyx_unpickle_SequencePosition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SequencePosition", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xe2ca0e1, 0xf7d828e, 0x96edc61):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe2ca0e1, 0xf7d828e, 0x96edc61) = (amino_acid, modification, position_type, terminal))" % __pyx_checksum
*/
  __pyx_t_1 = __Pyx_PyLong_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xe2ca0e1, 0xf7d828e, 0x96edc61):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe2ca0e1, 0xf7d828e, 0x96edc61) = (amino_acid, modification, position_type, terminal))" % __pyx_checksum
 *     __pyx_result = SequencePosition.__new__(__pyx_type)
*/
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_PickleError);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_PickleError);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_mstate_global->__pyx_n_u_PickleError) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
    __pyx_t_3 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0xe2ca0e1, 0xf7d828e, 0x96edc61):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe2ca0e1, 0xf7d828e, 0x96edc61) = (amino_acid, modification, position_type, terminal))" % __pyx_checksum             # <<<<<<<<<<<<<<
 *     __pyx_result = SequencePosition.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
    __pyx_t_3 = __Pyx_PyLong_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Incompatible_checksums_0x_x_vs_0, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_v___pyx_PickleError, __pyx_t_1, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xe2ca0e1, 0xf7d828e, 0x96edc61):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe2ca0e1, 0xf7d828e, 0x96edc61) = (amino_acid, modification, position_type, terminal))" % __pyx_checksum
*/
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe2ca0e1, 0xf7d828e, 0x96edc61) = (amino_acid, modification, position_type, terminal))" % __pyx_checksum
 *     __pyx_result = SequencePosition.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SequencePosition__set_state(<SequencePosition> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v___pyx_result = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe2ca0e1, 0xf7d828e, 0x96edc61) = (amino_acid, modification, position_type, terminal))" % __pyx_checksum
 *     __pyx_result = SequencePosition.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SequencePosition__set_state(<SequencePosition> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_2 = (__pyx_v___pyx_state != Py_None);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = SequencePosition.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SequencePosition__set_state(<SequencePosition> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_SequencePosition__set_state(SequencePosition __pyx_result, tuple __pyx_state):
*/
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser___pyx_unpickle_SequencePosition__set_state(((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe2ca0e1, 0xf7d828e, 0x96edc61) = (amino_acid, modification, position_type, terminal))" % __pyx_checksum
 *     __pyx_result = SequencePosition.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SequencePosition__set_state(<SequencePosition> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SequencePosition__set_state(<SequencePosition> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_SequencePosition__set_state(SequencePosition __pyx_result, tuple __pyx_state):
 *     __pyx_result.amino_acid = __pyx_state[0]; __pyx_result.modification = __pyx_state[1]; __pyx_result.position_type = __pyx_state[2]; __pyx_result.terminal = __pyx_state[3]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_SequencePosition(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_SequencePosition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_SequencePosition__set_state(<SequencePosition> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SequencePosition__set_state(SequencePosition __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.amino_acid = __pyx_state[0]; __pyx_result.modification = __pyx_state[1]; __pyx_result.position_type = __pyx_state[2]; __pyx_result.terminal = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
*/

static PyObject *__pyx_f_9pyteomics_7cparser___pyx_unpickle_SequencePosition__set_state(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SequencePosition__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_SequencePosition__set_state(SequencePosition __pyx_result, tuple __pyx_state):
 *     __pyx_result.amino_acid = __pyx_state[0]; __pyx_result.modification = __pyx_state[1]; __pyx_result.position_type = __pyx_state[2]; __pyx_result.terminal = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[4])
*/
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->amino_acid);
  __Pyx_DECREF(__pyx_v___pyx_result->amino_acid);
  __pyx_v___pyx_result->amino_acid = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->modification);
  __Pyx_DECREF(__pyx_v___pyx_result->modification);
  __pyx_v___pyx_result->modification = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = ((enum __pyx_t_9pyteomics_7cparser_PositionType)__Pyx_PyLong_As_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_t_1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->position_type = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->terminal);
  __Pyx_DECREF(__pyx_v___pyx_result->terminal);
  __pyx_v___pyx_result->terminal = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_SequencePosition__set_state(SequencePosition __pyx_result, tuple __pyx_state):
 *     __pyx_result.amino_acid = __pyx_state[0]; __pyx_result.modification = __pyx_state[1]; __pyx_result.position_type = __pyx_state[2]; __pyx_result.terminal = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[4])
*/
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 > 4);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_mstate_global->__pyx_n_u_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_3 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result.amino_acid = __pyx_state[0]; __pyx_result.modification = __pyx_state[1]; __pyx_result.position_type = __pyx_state[2]; __pyx_result.terminal = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[4])             # <<<<<<<<<<<<<<
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_mstate_global->__pyx_n_u_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __pyx_t_7;
    __Pyx_INCREF(__pyx_t_6);
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_8};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_update, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_SequencePosition__set_state(SequencePosition __pyx_result, tuple __pyx_state):
 *     __pyx_result.amino_acid = __pyx_state[0]; __pyx_result.modification = __pyx_state[1]; __pyx_result.position_type = __pyx_state[2]; __pyx_result.terminal = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[4])
*/
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_SequencePosition__set_state(<SequencePosition> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SequencePosition__set_state(SequencePosition __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.amino_acid = __pyx_state[0]; __pyx_result.modification = __pyx_state[1]; __pyx_result.position_type = __pyx_state[2]; __pyx_result.terminal = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_SequencePosition__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_IsoformGenerator(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_20__pyx_unpickle_IsoformGenerator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_19__pyx_unpickle_IsoformGenerator, "__pyx_unpickle_IsoformGenerator(__pyx_type, long __pyx_checksum, __pyx_state)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_20__pyx_unpickle_IsoformGenerator = {"__pyx_unpickle_IsoformGenerator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_20__pyx_unpickle_IsoformGenerator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_19__pyx_unpickle_IsoformGenerator};
static PyObject *__pyx_pw_9pyteomics_7cparser_20__pyx_unpickle_IsoformGenerator(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_IsoformGenerator (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 1, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_IsoformGenerator", 0) < 0) __PYX_ERR(1, 1, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IsoformGenerator", 1, 3, 3, i); __PYX_ERR(1, 1, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 1, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IsoformGenerator", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_IsoformGenerator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_19__pyx_unpickle_IsoformGenerator(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_19__pyx_unpickle_IsoformGenerator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_IsoformGenerator", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x09daa66, 0x1139f77, 0x2fc4213):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x09daa66, 0x1139f77, 0x2fc4213) = (fixed_mods, labels, last_indices, max_mods, original_sequence, override, persist, sequence, show_unmodified_termini, variable_mods))" % __pyx_checksum
*/
  __pyx_t_1 = __Pyx_PyLong_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[1], Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x09daa66, 0x1139f77, 0x2fc4213):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x09daa66, 0x1139f77, 0x2fc4213) = (fixed_mods, labels, last_indices, max_mods, original_sequence, override, persist, sequence, show_unmodified_termini, variable_mods))" % __pyx_checksum
 *     __pyx_result = IsoformGenerator.__new__(__pyx_type)
*/
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_PickleError);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_PickleError);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_mstate_global->__pyx_n_u_PickleError) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
    __pyx_t_3 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0x09daa66, 0x1139f77, 0x2fc4213):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x09daa66, 0x1139f77, 0x2fc4213) = (fixed_mods, labels, last_indices, max_mods, original_sequence, override, persist, sequence, show_unmodified_termini, variable_mods))" % __pyx_checksum             # <<<<<<<<<<<<<<
 *     __pyx_result = IsoformGenerator.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
    __pyx_t_3 = __Pyx_PyLong_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Incompatible_checksums_0x_x_vs_0_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_v___pyx_PickleError, __pyx_t_1, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x09daa66, 0x1139f77, 0x2fc4213):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x09daa66, 0x1139f77, 0x2fc4213) = (fixed_mods, labels, last_indices, max_mods, original_sequence, override, persist, sequence, show_unmodified_termini, variable_mods))" % __pyx_checksum
*/
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x09daa66, 0x1139f77, 0x2fc4213) = (fixed_mods, labels, last_indices, max_mods, original_sequence, override, persist, sequence, show_unmodified_termini, variable_mods))" % __pyx_checksum
 *     __pyx_result = IsoformGenerator.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IsoformGenerator__set_state(<IsoformGenerator> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v___pyx_result = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x09daa66, 0x1139f77, 0x2fc4213) = (fixed_mods, labels, last_indices, max_mods, original_sequence, override, persist, sequence, show_unmodified_termini, variable_mods))" % __pyx_checksum
 *     __pyx_result = IsoformGenerator.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_IsoformGenerator__set_state(<IsoformGenerator> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_2 = (__pyx_v___pyx_state != Py_None);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = IsoformGenerator.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IsoformGenerator__set_state(<IsoformGenerator> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_IsoformGenerator__set_state(IsoformGenerator __pyx_result, tuple __pyx_state):
*/
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser___pyx_unpickle_IsoformGenerator__set_state(((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x09daa66, 0x1139f77, 0x2fc4213) = (fixed_mods, labels, last_indices, max_mods, original_sequence, override, persist, sequence, show_unmodified_termini, variable_mods))" % __pyx_checksum
 *     __pyx_result = IsoformGenerator.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_IsoformGenerator__set_state(<IsoformGenerator> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IsoformGenerator__set_state(<IsoformGenerator> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_IsoformGenerator__set_state(IsoformGenerator __pyx_result, tuple __pyx_state):
 *     __pyx_result.fixed_mods = __pyx_state[0]; __pyx_result.labels = __pyx_state[1]; __pyx_result.last_indices = __pyx_state[2]; __pyx_result.max_mods = __pyx_state[3]; __pyx_result.original_sequence = __pyx_state[4]; __pyx_result.override = __pyx_state[5]; __pyx_result.persist = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.show_unmodified_termini = __pyx_state[8]; __pyx_result.variable_mods = __pyx_state[9]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_IsoformGenerator(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_IsoformGenerator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_IsoformGenerator__set_state(<IsoformGenerator> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_IsoformGenerator__set_state(IsoformGenerator __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.fixed_mods = __pyx_state[0]; __pyx_result.labels = __pyx_state[1]; __pyx_result.last_indices = __pyx_state[2]; __pyx_result.max_mods = __pyx_state[3]; __pyx_result.original_sequence = __pyx_state[4]; __pyx_result.override = __pyx_state[5]; __pyx_result.persist = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.show_unmodified_termini = __pyx_state[8]; __pyx_result.variable_mods = __pyx_state[9]
 *     if len(__pyx_state) > 10 and hasattr(__pyx_result, '__dict__'):
*/

static PyObject *__pyx_f_9pyteomics_7cparser___pyx_unpickle_IsoformGenerator__set_state(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_IsoformGenerator__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_IsoformGenerator__set_state(IsoformGenerator __pyx_result, tuple __pyx_state):
 *     __pyx_result.fixed_mods = __pyx_state[0]; __pyx_result.labels = __pyx_state[1]; __pyx_result.last_indices = __pyx_state[2]; __pyx_result.max_mods = __pyx_state[3]; __pyx_result.original_sequence = __pyx_state[4]; __pyx_result.override = __pyx_state[5]; __pyx_result.persist = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.show_unmodified_termini = __pyx_state[8]; __pyx_result.variable_mods = __pyx_state[9]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 10 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[10])
*/
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->fixed_mods);
  __Pyx_DECREF(__pyx_v___pyx_result->fixed_mods);
  __pyx_v___pyx_result->fixed_mods = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->labels);
  __Pyx_DECREF(__pyx_v___pyx_result->labels);
  __pyx_v___pyx_result->labels = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->last_indices);
  __Pyx_DECREF(__pyx_v___pyx_result->last_indices);
  __pyx_v___pyx_result->last_indices = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_As_long(__pyx_t_1); if (unlikely((__pyx_t_2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->max_mods = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->original_sequence);
  __Pyx_DECREF(__pyx_v___pyx_result->original_sequence);
  __pyx_v___pyx_result->original_sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->override = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->persist = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->sequence);
  __Pyx_DECREF(__pyx_v___pyx_result->sequence);
  __pyx_v___pyx_result->sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->show_unmodified_termini = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->variable_mods);
  __Pyx_DECREF(__pyx_v___pyx_result->variable_mods);
  __pyx_v___pyx_result->variable_mods = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_IsoformGenerator__set_state(IsoformGenerator __pyx_result, tuple __pyx_state):
 *     __pyx_result.fixed_mods = __pyx_state[0]; __pyx_result.labels = __pyx_state[1]; __pyx_result.last_indices = __pyx_state[2]; __pyx_result.max_mods = __pyx_state[3]; __pyx_result.original_sequence = __pyx_state[4]; __pyx_result.override = __pyx_state[5]; __pyx_result.persist = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.show_unmodified_termini = __pyx_state[8]; __pyx_result.variable_mods = __pyx_state[9]
 *     if len(__pyx_state) > 10 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[10])
*/
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 > 10);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_mstate_global->__pyx_n_u_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_3 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result.fixed_mods = __pyx_state[0]; __pyx_result.labels = __pyx_state[1]; __pyx_result.last_indices = __pyx_state[2]; __pyx_result.max_mods = __pyx_state[3]; __pyx_result.original_sequence = __pyx_state[4]; __pyx_result.override = __pyx_state[5]; __pyx_result.persist = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.show_unmodified_termini = __pyx_state[8]; __pyx_result.variable_mods = __pyx_state[9]
 *     if len(__pyx_state) > 10 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[10])             # <<<<<<<<<<<<<<
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_mstate_global->__pyx_n_u_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __pyx_t_7;
    __Pyx_INCREF(__pyx_t_6);
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_8};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_update, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_IsoformGenerator__set_state(IsoformGenerator __pyx_result, tuple __pyx_state):
 *     __pyx_result.fixed_mods = __pyx_state[0]; __pyx_result.labels = __pyx_state[1]; __pyx_result.last_indices = __pyx_state[2]; __pyx_result.max_mods = __pyx_state[3]; __pyx_result.original_sequence = __pyx_state[4]; __pyx_result.override = __pyx_state[5]; __pyx_result.persist = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.show_unmodified_termini = __pyx_state[8]; __pyx_result.variable_mods = __pyx_state[9]
 *     if len(__pyx_state) > 10 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[10])
*/
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_IsoformGenerator__set_state(<IsoformGenerator> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_IsoformGenerator__set_state(IsoformGenerator __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.fixed_mods = __pyx_state[0]; __pyx_result.labels = __pyx_state[1]; __pyx_result.last_indices = __pyx_state[2]; __pyx_result.max_mods = __pyx_state[3]; __pyx_result.original_sequence = __pyx_state[4]; __pyx_result.override = __pyx_state[5]; __pyx_result.persist = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.show_unmodified_termini = __pyx_state[8]; __pyx_result.variable_mods = __pyx_state[9]
 *     if len(__pyx_state) > 10 and hasattr(__pyx_result, '__dict__'):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_IsoformGenerator__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ModificationRule(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_22__pyx_unpickle_ModificationRule(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_21__pyx_unpickle_ModificationRule, "__pyx_unpickle_ModificationRule(__pyx_type, long __pyx_checksum, __pyx_state)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_22__pyx_unpickle_ModificationRule = {"__pyx_unpickle_ModificationRule", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_22__pyx_unpickle_ModificationRule, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_21__pyx_unpickle_ModificationRule};
static PyObject *__pyx_pw_9pyteomics_7cparser_22__pyx_unpickle_ModificationRule(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ModificationRule (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 1, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_ModificationRule", 0) < 0) __PYX_ERR(1, 1, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ModificationRule", 1, 3, 3, i); __PYX_ERR(1, 1, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 1, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ModificationRule", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_ModificationRule", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_21__pyx_unpickle_ModificationRule(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_21__pyx_unpickle_ModificationRule(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ModificationRule", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xccc7aa9, 0xe851a62, 0xc83ff59):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xccc7aa9, 0xe851a62, 0xc83ff59) = (modification, targets))" % __pyx_checksum
*/
  __pyx_t_1 = __Pyx_PyLong_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[2], Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xccc7aa9, 0xe851a62, 0xc83ff59):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xccc7aa9, 0xe851a62, 0xc83ff59) = (modification, targets))" % __pyx_checksum
 *     __pyx_result = ModificationRule.__new__(__pyx_type)
*/
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_PickleError);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_PickleError);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_mstate_global->__pyx_n_u_PickleError) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
    __pyx_t_3 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0xccc7aa9, 0xe851a62, 0xc83ff59):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xccc7aa9, 0xe851a62, 0xc83ff59) = (modification, targets))" % __pyx_checksum             # <<<<<<<<<<<<<<
 *     __pyx_result = ModificationRule.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
    __pyx_t_3 = __Pyx_PyLong_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Incompatible_checksums_0x_x_vs_0_3, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_v___pyx_PickleError, __pyx_t_1, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xccc7aa9, 0xe851a62, 0xc83ff59):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xccc7aa9, 0xe851a62, 0xc83ff59) = (modification, targets))" % __pyx_checksum
*/
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xccc7aa9, 0xe851a62, 0xc83ff59) = (modification, targets))" % __pyx_checksum
 *     __pyx_result = ModificationRule.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ModificationRule__set_state(<ModificationRule> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationRule);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v___pyx_result = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xccc7aa9, 0xe851a62, 0xc83ff59) = (modification, targets))" % __pyx_checksum
 *     __pyx_result = ModificationRule.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ModificationRule__set_state(<ModificationRule> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_2 = (__pyx_v___pyx_state != Py_None);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = ModificationRule.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ModificationRule__set_state(<ModificationRule> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationRule__set_state(ModificationRule __pyx_result, tuple __pyx_state):
*/
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser___pyx_unpickle_ModificationRule__set_state(((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xccc7aa9, 0xe851a62, 0xc83ff59) = (modification, targets))" % __pyx_checksum
 *     __pyx_result = ModificationRule.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ModificationRule__set_state(<ModificationRule> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ModificationRule__set_state(<ModificationRule> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ModificationRule__set_state(ModificationRule __pyx_result, tuple __pyx_state):
 *     __pyx_result.modification = __pyx_state[0]; __pyx_result.targets = __pyx_state[1]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ModificationRule(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_ModificationRule", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ModificationRule__set_state(<ModificationRule> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationRule__set_state(ModificationRule __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.modification = __pyx_state[0]; __pyx_result.targets = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
*/

static PyObject *__pyx_f_9pyteomics_7cparser___pyx_unpickle_ModificationRule__set_state(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ModificationRule__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationRule__set_state(ModificationRule __pyx_result, tuple __pyx_state):
 *     __pyx_result.modification = __pyx_state[0]; __pyx_result.targets = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])
*/
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->modification);
  __Pyx_DECREF(__pyx_v___pyx_result->modification);
  __pyx_v___pyx_result->modification = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->targets);
  __Pyx_DECREF(__pyx_v___pyx_result->targets);
  __pyx_v___pyx_result->targets = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ModificationRule__set_state(ModificationRule __pyx_result, tuple __pyx_state):
 *     __pyx_result.modification = __pyx_state[0]; __pyx_result.targets = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
*/
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = (__pyx_t_3 > 2);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_mstate_global->__pyx_n_u_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.modification = __pyx_state[0]; __pyx_result.targets = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])             # <<<<<<<<<<<<<<
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_mstate_global->__pyx_n_u_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __pyx_t_6;
    __Pyx_INCREF(__pyx_t_5);
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_update, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ModificationRule__set_state(ModificationRule __pyx_result, tuple __pyx_state):
 *     __pyx_result.modification = __pyx_state[0]; __pyx_result.targets = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
*/
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ModificationRule__set_state(<ModificationRule> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationRule__set_state(ModificationRule __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.modification = __pyx_state[0]; __pyx_result.targets = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_ModificationRule__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ModificationTarget(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_24__pyx_unpickle_ModificationTarget(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_9pyteomics_7cparser_23__pyx_unpickle_ModificationTarget, "__pyx_unpickle_ModificationTarget(__pyx_type, long __pyx_checksum, __pyx_state)");
static PyMethodDef __pyx_mdef_9pyteomics_7cparser_24__pyx_unpickle_ModificationTarget = {"__pyx_unpickle_ModificationTarget", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_24__pyx_unpickle_ModificationTarget, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_23__pyx_unpickle_ModificationTarget};
static PyObject *__pyx_pw_9pyteomics_7cparser_24__pyx_unpickle_ModificationTarget(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ModificationTarget (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 1, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_ModificationTarget", 0) < 0) __PYX_ERR(1, 1, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ModificationTarget", 1, 3, 3, i); __PYX_ERR(1, 1, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 1, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ModificationTarget", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_ModificationTarget", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_23__pyx_unpickle_ModificationTarget(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_23__pyx_unpickle_ModificationTarget(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ModificationTarget", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x0f5dd8b, 0xa1a722a, 0x5a11ef3):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x0f5dd8b, 0xa1a722a, 0x5a11ef3) = (position_type, target))" % __pyx_checksum
*/
  __pyx_t_1 = __Pyx_PyLong_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[3], Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x0f5dd8b, 0xa1a722a, 0x5a11ef3):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x0f5dd8b, 0xa1a722a, 0x5a11ef3) = (position_type, target))" % __pyx_checksum
 *     __pyx_result = ModificationTarget.__new__(__pyx_type)
*/
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_PickleError);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_PickleError);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_mstate_global->__pyx_n_u_PickleError) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
    __pyx_t_3 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0x0f5dd8b, 0xa1a722a, 0x5a11ef3):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x0f5dd8b, 0xa1a722a, 0x5a11ef3) = (position_type, target))" % __pyx_checksum             # <<<<<<<<<<<<<<
 *     __pyx_result = ModificationTarget.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
    __pyx_t_3 = __Pyx_PyLong_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Incompatible_checksums_0x_x_vs_0_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_v___pyx_PickleError, __pyx_t_1, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x0f5dd8b, 0xa1a722a, 0x5a11ef3):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x0f5dd8b, 0xa1a722a, 0x5a11ef3) = (position_type, target))" % __pyx_checksum
*/
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x0f5dd8b, 0xa1a722a, 0x5a11ef3) = (position_type, target))" % __pyx_checksum
 *     __pyx_result = ModificationTarget.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ModificationTarget__set_state(<ModificationTarget> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationTarget);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v___pyx_result = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x0f5dd8b, 0xa1a722a, 0x5a11ef3) = (position_type, target))" % __pyx_checksum
 *     __pyx_result = ModificationTarget.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ModificationTarget__set_state(<ModificationTarget> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_2 = (__pyx_v___pyx_state != Py_None);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = ModificationTarget.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ModificationTarget__set_state(<ModificationTarget> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationTarget__set_state(ModificationTarget __pyx_result, tuple __pyx_state):
*/
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser___pyx_unpickle_ModificationTarget__set_state(((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x0f5dd8b, 0xa1a722a, 0x5a11ef3) = (position_type, target))" % __pyx_checksum
 *     __pyx_result = ModificationTarget.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ModificationTarget__set_state(<ModificationTarget> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ModificationTarget__set_state(<ModificationTarget> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ModificationTarget__set_state(ModificationTarget __pyx_result, tuple __pyx_state):
 *     __pyx_result.position_type = __pyx_state[0]; __pyx_result.target = __pyx_state[1]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ModificationTarget(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_ModificationTarget", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ModificationTarget__set_state(<ModificationTarget> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationTarget__set_state(ModificationTarget __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.position_type = __pyx_state[0]; __pyx_result.target = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
*/

static PyObject *__pyx_f_9pyteomics_7cparser___pyx_unpickle_ModificationTarget__set_state(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  enum __pyx_t_9pyteomics_7cparser_PositionType __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ModificationTarget__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationTarget__set_state(ModificationTarget __pyx_result, tuple __pyx_state):
 *     __pyx_result.position_type = __pyx_state[0]; __pyx_result.target = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])
*/
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = ((enum __pyx_t_9pyteomics_7cparser_PositionType)__Pyx_PyLong_As_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_t_1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->position_type = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->target);
  __Pyx_DECREF(__pyx_v___pyx_result->target);
  __pyx_v___pyx_result->target = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ModificationTarget__set_state(ModificationTarget __pyx_result, tuple __pyx_state):
 *     __pyx_result.position_type = __pyx_state[0]; __pyx_result.target = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
*/
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 > 2);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_mstate_global->__pyx_n_u_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_3 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result.position_type = __pyx_state[0]; __pyx_result.target = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])             # <<<<<<<<<<<<<<
*/
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_mstate_global->__pyx_n_u_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __pyx_t_7;
    __Pyx_INCREF(__pyx_t_6);
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_8};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod(__pyx_mstate_global->__pyx_n_u_update, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ModificationTarget__set_state(ModificationTarget __pyx_result, tuple __pyx_state):
 *     __pyx_result.position_type = __pyx_state[0]; __pyx_result.target = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
*/
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ModificationTarget__set_state(<ModificationTarget> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationTarget__set_state(ModificationTarget __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.position_type = __pyx_state[0]; __pyx_result.target = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pyteomics.cparser.__pyx_unpickle_ModificationTarget__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
/* #### Code section: module_exttypes ### */
static struct __pyx_vtabstruct_9pyteomics_7cparser_SequencePosition __pyx_vtable_9pyteomics_7cparser_SequencePosition;

static PyObject *__pyx_tp_new_9pyteomics_7cparser_SequencePosition(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_SequencePosition > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser_SequencePosition)))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser_SequencePosition[--__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_SequencePosition];
    memset(o, 0, sizeof(struct __pyx_obj_9pyteomics_7cparser_SequencePosition));
    (void) PyObject_INIT(o, t);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  p = ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)o);
  p->__pyx_vtab = __pyx_vtabptr_9pyteomics_7cparser_SequencePosition;
  p->modification = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->amino_acid = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->terminal = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9pyteomics_7cparser_SequencePosition(PyObject *o) {
  struct __pyx_obj_9pyteomics_7cparser_SequencePosition *p = (struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)o;
  Py_CLEAR(p->modification);
  Py_CLEAR(p->amino_acid);
  Py_CLEAR(p->terminal);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_SequencePosition < 5000) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser_SequencePosition)))) {
    __pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser_SequencePosition[__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_SequencePosition++] = ((struct __pyx_obj_9pyteomics_7cparser_SequencePosition *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static PyObject *__pyx_sq_item_9pyteomics_7cparser_SequencePosition(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyLong_FromSsize_t(i); if(!x) return 0;
  #if CYTHON_USE_TYPE_SLOTS || (!CYTHON_USE_TYPE_SPECS && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  #else
  r = ((binaryfunc)PyType_GetSlot(Py_TYPE(o), Py_mp_subscript))(o, x);
  #endif
  Py_DECREF(x);
  return r;
}

static PyObject *__pyx_tp_richcompare_9pyteomics_7cparser_SequencePosition(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_9pyteomics_7cparser_16SequencePosition_21__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_9pyteomics_7cparser_16SequencePosition_23__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16SequencePosition_modification(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16SequencePosition_12modification_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16SequencePosition_modification(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16SequencePosition_12modification_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16SequencePosition_12modification_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16SequencePosition_position_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16SequencePosition_13position_type_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16SequencePosition_position_type(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16SequencePosition_13position_type_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16SequencePosition_amino_acid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16SequencePosition_10amino_acid_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16SequencePosition_amino_acid(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16SequencePosition_10amino_acid_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16SequencePosition_10amino_acid_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16SequencePosition_terminal(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16SequencePosition_8terminal_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16SequencePosition_terminal(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16SequencePosition_8terminal_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16SequencePosition_8terminal_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9pyteomics_7cparser_SequencePosition[] = {
  {"from_tuple", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_15from_tuple, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_14from_tuple},
  {"parse", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_17parse, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_16parse},
  {"from_iterable", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_19from_iterable, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_18from_iterable},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_25__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_24__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16SequencePosition_27__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16SequencePosition_26__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9pyteomics_7cparser_SequencePosition[] = {
  {"modification", __pyx_getprop_9pyteomics_7cparser_16SequencePosition_modification, __pyx_setprop_9pyteomics_7cparser_16SequencePosition_modification, PyDoc_STR("modification: str"), 0},
  {"position_type", __pyx_getprop_9pyteomics_7cparser_16SequencePosition_position_type, __pyx_setprop_9pyteomics_7cparser_16SequencePosition_position_type, PyDoc_STR("position_type: 'PositionType'"), 0},
  {"amino_acid", __pyx_getprop_9pyteomics_7cparser_16SequencePosition_amino_acid, __pyx_setprop_9pyteomics_7cparser_16SequencePosition_amino_acid, PyDoc_STR("amino_acid: str"), 0},
  {"terminal", __pyx_getprop_9pyteomics_7cparser_16SequencePosition_terminal, __pyx_setprop_9pyteomics_7cparser_16SequencePosition_terminal, PyDoc_STR("terminal: str"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_9pyteomics_7cparser_SequencePosition_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_9pyteomics_7cparser_SequencePosition},
  {Py_tp_repr, (void *)__pyx_pw_9pyteomics_7cparser_16SequencePosition_9__repr__},
  {Py_sq_length, (void *)__pyx_pw_9pyteomics_7cparser_16SequencePosition_11__len__},
  {Py_sq_item, (void *)__pyx_sq_item_9pyteomics_7cparser_SequencePosition},
  {Py_mp_length, (void *)__pyx_pw_9pyteomics_7cparser_16SequencePosition_11__len__},
  {Py_mp_subscript, (void *)__pyx_pw_9pyteomics_7cparser_16SequencePosition_3__getitem__},
  {Py_tp_doc, (void *)PyDoc_STR("SequencePosition(amino_acid, modification=None, terminal=None, position_type=None)")},
  {Py_tp_richcompare, (void *)__pyx_tp_richcompare_9pyteomics_7cparser_SequencePosition},
  {Py_tp_methods, (void *)__pyx_methods_9pyteomics_7cparser_SequencePosition},
  {Py_tp_getset, (void *)__pyx_getsets_9pyteomics_7cparser_SequencePosition},
  {Py_tp_init, (void *)__pyx_pw_9pyteomics_7cparser_16SequencePosition_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_9pyteomics_7cparser_SequencePosition},
  {0, 0},
};
static PyType_Spec __pyx_type_9pyteomics_7cparser_SequencePosition_spec = {
  "pyteomics.cparser.SequencePosition",
  sizeof(struct __pyx_obj_9pyteomics_7cparser_SequencePosition),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER,
  __pyx_type_9pyteomics_7cparser_SequencePosition_slots,
};
#else

static PySequenceMethods __pyx_tp_as_sequence_SequencePosition = {
  __pyx_pw_9pyteomics_7cparser_16SequencePosition_11__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_9pyteomics_7cparser_SequencePosition, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_SequencePosition = {
  __pyx_pw_9pyteomics_7cparser_16SequencePosition_11__len__, /*mp_length*/
  __pyx_pw_9pyteomics_7cparser_16SequencePosition_3__getitem__, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_9pyteomics_7cparser_SequencePosition = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyteomics.cparser.""SequencePosition", /*tp_name*/
  sizeof(struct __pyx_obj_9pyteomics_7cparser_SequencePosition), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9pyteomics_7cparser_SequencePosition, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_9pyteomics_7cparser_16SequencePosition_9__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_SequencePosition, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_SequencePosition, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  PyDoc_STR("SequencePosition(amino_acid, modification=None, terminal=None, position_type=None)"), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_tp_richcompare_9pyteomics_7cparser_SequencePosition, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9pyteomics_7cparser_SequencePosition, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9pyteomics_7cparser_SequencePosition, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_9pyteomics_7cparser_16SequencePosition_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9pyteomics_7cparser_SequencePosition, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_9pyteomics_7cparser_IsoformGenerator __pyx_vtable_9pyteomics_7cparser_IsoformGenerator;

static PyObject *__pyx_tp_new_9pyteomics_7cparser_IsoformGenerator(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_IsoformGenerator > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator)) & (int)(!__Pyx_PyType_HasFeature(t, (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE))))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser_IsoformGenerator[--__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_IsoformGenerator];
    memset(o, 0, sizeof(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  #endif
  p = ((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)o);
  p->__pyx_vtab = __pyx_vtabptr_9pyteomics_7cparser_IsoformGenerator;
  p->variable_mods = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->fixed_mods = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->labels = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->sequence = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->original_sequence = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->last_indices = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9pyteomics_7cparser_IsoformGenerator(PyObject *o) {
  struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *p = (struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_9pyteomics_7cparser_IsoformGenerator) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->variable_mods);
  Py_CLEAR(p->fixed_mods);
  Py_CLEAR(p->labels);
  Py_CLEAR(p->sequence);
  Py_CLEAR(p->original_sequence);
  Py_CLEAR(p->last_indices);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_IsoformGenerator < 100) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator)) & (int)(!__Pyx_PyType_HasFeature(Py_TYPE(o), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE))))) {
    __pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser_IsoformGenerator[__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_IsoformGenerator++] = ((struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_9pyteomics_7cparser_IsoformGenerator(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *p = (struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->variable_mods) {
    e = (*v)(p->variable_mods, a); if (e) return e;
  }
  if (p->fixed_mods) {
    e = (*v)(p->fixed_mods, a); if (e) return e;
  }
  if (p->labels) {
    e = (*v)(p->labels, a); if (e) return e;
  }
  if (p->sequence) {
    e = (*v)(p->sequence, a); if (e) return e;
  }
  if (p->original_sequence) {
    e = (*v)(p->original_sequence, a); if (e) return e;
  }
  if (p->last_indices) {
    e = (*v)(p->last_indices, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9pyteomics_7cparser_IsoformGenerator(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *p = (struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *)o;
  tmp = ((PyObject*)p->variable_mods);
  p->variable_mods = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->fixed_mods);
  p->fixed_mods = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->labels);
  p->labels = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->sequence);
  p->sequence = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->original_sequence);
  p->original_sequence = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->last_indices);
  p->last_indices = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_variable_mods(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_variable_mods(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13variable_mods_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_fixed_mods(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_fixed_mods(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_10fixed_mods_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_labels(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_6labels_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_labels(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_6labels_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_6labels_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_max_mods(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8max_mods_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_max_mods(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8max_mods_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_override(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8override_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_override(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8override_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_show_unmodified_termini(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_show_unmodified_termini(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_23show_unmodified_termini_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_persist(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7persist_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_persist(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_7persist_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_sequence(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8sequence_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_sequence(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8sequence_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_8sequence_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_original_sequence(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_original_sequence(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_17original_sequence_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_last_indices(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_12last_indices_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_last_indices(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_12last_indices_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_12last_indices_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9pyteomics_7cparser_IsoformGenerator[] = {
  {"generate", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_13generate, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_12generate},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_18__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_17__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_20__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16IsoformGenerator_19__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9pyteomics_7cparser_IsoformGenerator[] = {
  {"variable_mods", __pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_variable_mods, __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_variable_mods, PyDoc_STR("variable_mods: dict"), 0},
  {"fixed_mods", __pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_fixed_mods, __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_fixed_mods, PyDoc_STR("fixed_mods: dict"), 0},
  {"labels", __pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_labels, __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_labels, PyDoc_STR("labels: list"), 0},
  {"max_mods", __pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_max_mods, __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_max_mods, PyDoc_STR("max_mods: 'long'"), 0},
  {"override", __pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_override, __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_override, PyDoc_STR("override: 'bool'"), 0},
  {"show_unmodified_termini", __pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_show_unmodified_termini, __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_show_unmodified_termini, PyDoc_STR("show_unmodified_termini: 'bool'"), 0},
  {"persist", __pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_persist, __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_persist, PyDoc_STR("persist: 'bool'"), 0},
  {"sequence", __pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_sequence, __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_sequence, PyDoc_STR("sequence: list"), 0},
  {"original_sequence", __pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_original_sequence, __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_original_sequence, PyDoc_STR("original_sequence: list"), 0},
  {"last_indices", __pyx_getprop_9pyteomics_7cparser_16IsoformGenerator_last_indices, __pyx_setprop_9pyteomics_7cparser_16IsoformGenerator_last_indices, PyDoc_STR("last_indices: list"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_9pyteomics_7cparser_IsoformGenerator_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_9pyteomics_7cparser_IsoformGenerator},
  {Py_tp_call, (void *)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_16__call__},
  {Py_tp_doc, (void *)PyDoc_STR("IsoformGenerator(variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False, show_unmodified_termini=False, persist=True)")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_9pyteomics_7cparser_IsoformGenerator},
  {Py_tp_clear, (void *)__pyx_tp_clear_9pyteomics_7cparser_IsoformGenerator},
  {Py_tp_methods, (void *)__pyx_methods_9pyteomics_7cparser_IsoformGenerator},
  {Py_tp_getset, (void *)__pyx_getsets_9pyteomics_7cparser_IsoformGenerator},
  {Py_tp_init, (void *)__pyx_pw_9pyteomics_7cparser_16IsoformGenerator_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_9pyteomics_7cparser_IsoformGenerator},
  {0, 0},
};
static PyType_Spec __pyx_type_9pyteomics_7cparser_IsoformGenerator_spec = {
  "pyteomics.cparser.IsoformGenerator",
  sizeof(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_9pyteomics_7cparser_IsoformGenerator_slots,
};
#else

static PyTypeObject __pyx_type_9pyteomics_7cparser_IsoformGenerator = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyteomics.cparser.""IsoformGenerator", /*tp_name*/
  sizeof(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9pyteomics_7cparser_IsoformGenerator, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_16__call__, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("IsoformGenerator(variable_mods=None, fixed_mods=None, labels=None, max_mods=None, override=False, show_unmodified_termini=False, persist=True)"), /*tp_doc*/
  __pyx_tp_traverse_9pyteomics_7cparser_IsoformGenerator, /*tp_traverse*/
  __pyx_tp_clear_9pyteomics_7cparser_IsoformGenerator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9pyteomics_7cparser_IsoformGenerator, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9pyteomics_7cparser_IsoformGenerator, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_9pyteomics_7cparser_16IsoformGenerator_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9pyteomics_7cparser_IsoformGenerator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationRule __pyx_vtable_9pyteomics_7cparser_ModificationRule;

static PyObject *__pyx_tp_new_9pyteomics_7cparser_ModificationRule(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_9pyteomics_7cparser_ModificationRule *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_ModificationRule > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser_ModificationRule)) & (int)(!__Pyx_PyType_HasFeature(t, (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE))))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser_ModificationRule[--__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_ModificationRule];
    memset(o, 0, sizeof(struct __pyx_obj_9pyteomics_7cparser_ModificationRule));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  #endif
  p = ((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)o);
  p->__pyx_vtab = __pyx_vtabptr_9pyteomics_7cparser_ModificationRule;
  p->modification = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->targets = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9pyteomics_7cparser_ModificationRule(PyObject *o) {
  struct __pyx_obj_9pyteomics_7cparser_ModificationRule *p = (struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_9pyteomics_7cparser_ModificationRule) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->modification);
  Py_CLEAR(p->targets);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_ModificationRule < 100) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser_ModificationRule)) & (int)(!__Pyx_PyType_HasFeature(Py_TYPE(o), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE))))) {
    __pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser_ModificationRule[__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_ModificationRule++] = ((struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_9pyteomics_7cparser_ModificationRule(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9pyteomics_7cparser_ModificationRule *p = (struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->targets) {
    e = (*v)(p->targets, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9pyteomics_7cparser_ModificationRule(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9pyteomics_7cparser_ModificationRule *p = (struct __pyx_obj_9pyteomics_7cparser_ModificationRule *)o;
  tmp = ((PyObject*)p->targets);
  p->targets = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16ModificationRule_modification(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16ModificationRule_12modification_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16ModificationRule_modification(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16ModificationRule_12modification_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16ModificationRule_12modification_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_16ModificationRule_targets(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_16ModificationRule_7targets_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_16ModificationRule_targets(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_16ModificationRule_7targets_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_16ModificationRule_7targets_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_9pyteomics_7cparser_ModificationRule[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16ModificationRule_9__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16ModificationRule_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_16ModificationRule_11__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_16ModificationRule_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9pyteomics_7cparser_ModificationRule[] = {
  {"modification", __pyx_getprop_9pyteomics_7cparser_16ModificationRule_modification, __pyx_setprop_9pyteomics_7cparser_16ModificationRule_modification, PyDoc_STR("modification: str"), 0},
  {"targets", __pyx_getprop_9pyteomics_7cparser_16ModificationRule_targets, __pyx_setprop_9pyteomics_7cparser_16ModificationRule_targets, PyDoc_STR("targets: list"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_9pyteomics_7cparser_ModificationRule_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_9pyteomics_7cparser_ModificationRule},
  {Py_tp_repr, (void *)__pyx_pw_9pyteomics_7cparser_16ModificationRule_7__repr__},
  {Py_tp_call, (void *)__pyx_pw_9pyteomics_7cparser_16ModificationRule_5__call__},
  {Py_tp_doc, (void *)PyDoc_STR("ModificationRule(str label, targets)")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_9pyteomics_7cparser_ModificationRule},
  {Py_tp_clear, (void *)__pyx_tp_clear_9pyteomics_7cparser_ModificationRule},
  {Py_tp_methods, (void *)__pyx_methods_9pyteomics_7cparser_ModificationRule},
  {Py_tp_getset, (void *)__pyx_getsets_9pyteomics_7cparser_ModificationRule},
  {Py_tp_init, (void *)__pyx_pw_9pyteomics_7cparser_16ModificationRule_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_9pyteomics_7cparser_ModificationRule},
  {0, 0},
};
static PyType_Spec __pyx_type_9pyteomics_7cparser_ModificationRule_spec = {
  "pyteomics.cparser.ModificationRule",
  sizeof(struct __pyx_obj_9pyteomics_7cparser_ModificationRule),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_9pyteomics_7cparser_ModificationRule_slots,
};
#else

static PyTypeObject __pyx_type_9pyteomics_7cparser_ModificationRule = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyteomics.cparser.""ModificationRule", /*tp_name*/
  sizeof(struct __pyx_obj_9pyteomics_7cparser_ModificationRule), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9pyteomics_7cparser_ModificationRule, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_9pyteomics_7cparser_16ModificationRule_7__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  __pyx_pw_9pyteomics_7cparser_16ModificationRule_5__call__, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("ModificationRule(str label, targets)"), /*tp_doc*/
  __pyx_tp_traverse_9pyteomics_7cparser_ModificationRule, /*tp_traverse*/
  __pyx_tp_clear_9pyteomics_7cparser_ModificationRule, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9pyteomics_7cparser_ModificationRule, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9pyteomics_7cparser_ModificationRule, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_9pyteomics_7cparser_16ModificationRule_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9pyteomics_7cparser_ModificationRule, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_9pyteomics_7cparser_ModificationTarget __pyx_vtable_9pyteomics_7cparser_ModificationTarget;

static PyObject *__pyx_tp_new_9pyteomics_7cparser_ModificationTarget(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_ModificationTarget > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget)) & (int)(!__Pyx_PyType_HasFeature(t, (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE))))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser_ModificationTarget[--__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_ModificationTarget];
    memset(o, 0, sizeof(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget));
    (void) PyObject_INIT(o, t);
  } else
  #endif
  {
    if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  #endif
  p = ((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)o);
  p->__pyx_vtab = __pyx_vtabptr_9pyteomics_7cparser_ModificationTarget;
  p->target = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9pyteomics_7cparser_ModificationTarget(PyObject *o) {
  struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *p = (struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_9pyteomics_7cparser_ModificationTarget) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  Py_CLEAR(p->target);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_ModificationTarget < 1000) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget)) & (int)(!__Pyx_PyType_HasFeature(Py_TYPE(o), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE))))) {
    __pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser_ModificationTarget[__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser_ModificationTarget++] = ((struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_18ModificationTarget_target(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_18ModificationTarget_6target_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_18ModificationTarget_target(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_18ModificationTarget_6target_3__set__(o, v);
  }
  else {
    return __pyx_pw_9pyteomics_7cparser_18ModificationTarget_6target_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9pyteomics_7cparser_18ModificationTarget_position_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9pyteomics_7cparser_18ModificationTarget_13position_type_1__get__(o);
}

static int __pyx_setprop_9pyteomics_7cparser_18ModificationTarget_position_type(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9pyteomics_7cparser_18ModificationTarget_13position_type_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_9pyteomics_7cparser_ModificationTarget[] = {
  {"from_string", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_18ModificationTarget_9from_string, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_18ModificationTarget_8from_string},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_18ModificationTarget_11__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_18ModificationTarget_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_9pyteomics_7cparser_18ModificationTarget_13__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_18ModificationTarget_12__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9pyteomics_7cparser_ModificationTarget[] = {
  {"target", __pyx_getprop_9pyteomics_7cparser_18ModificationTarget_target, __pyx_setprop_9pyteomics_7cparser_18ModificationTarget_target, PyDoc_STR("target: str"), 0},
  {"position_type", __pyx_getprop_9pyteomics_7cparser_18ModificationTarget_position_type, __pyx_setprop_9pyteomics_7cparser_18ModificationTarget_position_type, PyDoc_STR("position_type: 'PositionType'"), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_9pyteomics_7cparser_ModificationTarget_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_9pyteomics_7cparser_ModificationTarget},
  {Py_tp_repr, (void *)__pyx_pw_9pyteomics_7cparser_18ModificationTarget_7__repr__},
  {Py_tp_call, (void *)__pyx_pw_9pyteomics_7cparser_18ModificationTarget_5__call__},
  {Py_tp_doc, (void *)PyDoc_STR("ModificationTarget(target, position_type=PositionType.internal)")},
  {Py_tp_methods, (void *)__pyx_methods_9pyteomics_7cparser_ModificationTarget},
  {Py_tp_getset, (void *)__pyx_getsets_9pyteomics_7cparser_ModificationTarget},
  {Py_tp_init, (void *)__pyx_pw_9pyteomics_7cparser_18ModificationTarget_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_9pyteomics_7cparser_ModificationTarget},
  {0, 0},
};
static PyType_Spec __pyx_type_9pyteomics_7cparser_ModificationTarget_spec = {
  "pyteomics.cparser.ModificationTarget",
  sizeof(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_9pyteomics_7cparser_ModificationTarget_slots,
};
#else

static PyTypeObject __pyx_type_9pyteomics_7cparser_ModificationTarget = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyteomics.cparser.""ModificationTarget", /*tp_name*/
  sizeof(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9pyteomics_7cparser_ModificationTarget, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_9pyteomics_7cparser_18ModificationTarget_7__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  __pyx_pw_9pyteomics_7cparser_18ModificationTarget_5__call__, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("ModificationTarget(target, position_type=PositionType.internal)"), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9pyteomics_7cparser_ModificationTarget, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9pyteomics_7cparser_ModificationTarget, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_9pyteomics_7cparser_18ModificationTarget_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9pyteomics_7cparser_ModificationTarget, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_9pyteomics_7cparser___pyx_scope_struct__generate(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser___pyx_scope_struct__generate > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate)))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser___pyx_scope_struct__generate[--__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser___pyx_scope_struct__generate];
    memset(o, 0, sizeof(struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_9pyteomics_7cparser___pyx_scope_struct__generate(PyObject *o) {
  struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *p = (struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_9pyteomics_7cparser___pyx_scope_struct__generate) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_a);
  Py_CLEAR(p->__pyx_v_assignment);
  Py_CLEAR(p->__pyx_v_assignments);
  Py_CLEAR(p->__pyx_v_assignments_product);
  Py_CLEAR(p->__pyx_v_indices);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_sequence);
  Py_CLEAR(p->__pyx_v_sequence_spec);
  Py_CLEAR(p->__pyx_v_subassignments);
  Py_CLEAR(p->__pyx_v_subindices);
  Py_CLEAR(p->__pyx_v_temp1);
  Py_CLEAR(p->__pyx_v_temp2);
  Py_CLEAR(p->__pyx_v_variable_mod_sites);
  Py_CLEAR(p->__pyx_t_1);
  Py_CLEAR(p->__pyx_t_6);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser___pyx_scope_struct__generate < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate)))) {
    __pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser___pyx_scope_struct__generate[__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser___pyx_scope_struct__generate++] = ((struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_9pyteomics_7cparser___pyx_scope_struct__generate(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *p = (struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_a) {
    e = (*v)(p->__pyx_v_a, a); if (e) return e;
  }
  if (p->__pyx_v_assignment) {
    e = (*v)(p->__pyx_v_assignment, a); if (e) return e;
  }
  if (p->__pyx_v_assignments) {
    e = (*v)(p->__pyx_v_assignments, a); if (e) return e;
  }
  if (p->__pyx_v_assignments_product) {
    e = (*v)(p->__pyx_v_assignments_product, a); if (e) return e;
  }
  if (p->__pyx_v_indices) {
    e = (*v)(p->__pyx_v_indices, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_sequence) {
    e = (*v)(p->__pyx_v_sequence, a); if (e) return e;
  }
  if (p->__pyx_v_sequence_spec) {
    e = (*v)(p->__pyx_v_sequence_spec, a); if (e) return e;
  }
  if (p->__pyx_v_subassignments) {
    e = (*v)(p->__pyx_v_subassignments, a); if (e) return e;
  }
  if (p->__pyx_v_subindices) {
    e = (*v)(p->__pyx_v_subindices, a); if (e) return e;
  }
  if (p->__pyx_v_temp1) {
    e = (*v)(p->__pyx_v_temp1, a); if (e) return e;
  }
  if (p->__pyx_v_temp2) {
    e = (*v)(p->__pyx_v_temp2, a); if (e) return e;
  }
  if (p->__pyx_v_variable_mod_sites) {
    e = (*v)(p->__pyx_v_variable_mod_sites, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  if (p->__pyx_t_6) {
    e = (*v)(p->__pyx_t_6, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_9pyteomics_7cparser___pyx_scope_struct__generate_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_9pyteomics_7cparser___pyx_scope_struct__generate},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_9pyteomics_7cparser___pyx_scope_struct__generate},
  {Py_tp_new, (void *)__pyx_tp_new_9pyteomics_7cparser___pyx_scope_struct__generate},
  {0, 0},
};
static PyType_Spec __pyx_type_9pyteomics_7cparser___pyx_scope_struct__generate_spec = {
  "pyteomics.cparser.__pyx_scope_struct__generate",
  sizeof(struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_9pyteomics_7cparser___pyx_scope_struct__generate_slots,
};
#else

static PyTypeObject __pyx_type_9pyteomics_7cparser___pyx_scope_struct__generate = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyteomics.cparser.""__pyx_scope_struct__generate", /*tp_name*/
  sizeof(struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct__generate), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9pyteomics_7cparser___pyx_scope_struct__generate, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_9pyteomics_7cparser___pyx_scope_struct__generate, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9pyteomics_7cparser___pyx_scope_struct__generate, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_9pyteomics_7cparser___pyx_scope_struct_1_isoforms(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser___pyx_scope_struct_1_isoforms > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms)))) {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser___pyx_scope_struct_1_isoforms[--__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser___pyx_scope_struct_1_isoforms];
    memset(o, 0, sizeof(struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_9pyteomics_7cparser___pyx_scope_struct_1_isoforms(PyObject *o) {
  struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *p = (struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_9pyteomics_7cparser___pyx_scope_struct_1_isoforms) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v__format);
  Py_CLEAR(p->__pyx_v_fixed_mods);
  Py_CLEAR(p->__pyx_v_form);
  Py_CLEAR(p->__pyx_v_gen);
  Py_CLEAR(p->__pyx_v_isoform_generator);
  Py_CLEAR(p->__pyx_v_kwargs);
  Py_CLEAR(p->__pyx_v_labels);
  Py_CLEAR(p->__pyx_v_max_mods);
  Py_CLEAR(p->__pyx_v_override);
  Py_CLEAR(p->__pyx_v_sequence);
  Py_CLEAR(p->__pyx_v_show_unmodified_termini);
  Py_CLEAR(p->__pyx_v_variable_mods);
  Py_CLEAR(p->__pyx_t_0);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser___pyx_scope_struct_1_isoforms < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms)))) {
    __pyx_mstate_global->__pyx_freelist_9pyteomics_7cparser___pyx_scope_struct_1_isoforms[__pyx_mstate_global->__pyx_freecount_9pyteomics_7cparser___pyx_scope_struct_1_isoforms++] = ((struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_9pyteomics_7cparser___pyx_scope_struct_1_isoforms(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *p = (struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_fixed_mods) {
    e = (*v)(p->__pyx_v_fixed_mods, a); if (e) return e;
  }
  if (p->__pyx_v_form) {
    e = (*v)(p->__pyx_v_form, a); if (e) return e;
  }
  if (p->__pyx_v_gen) {
    e = (*v)(p->__pyx_v_gen, a); if (e) return e;
  }
  if (p->__pyx_v_isoform_generator) {
    e = (*v)(((PyObject *)p->__pyx_v_isoform_generator), a); if (e) return e;
  }
  if (p->__pyx_v_kwargs) {
    e = (*v)(p->__pyx_v_kwargs, a); if (e) return e;
  }
  if (p->__pyx_v_labels) {
    e = (*v)(p->__pyx_v_labels, a); if (e) return e;
  }
  if (p->__pyx_v_max_mods) {
    e = (*v)(p->__pyx_v_max_mods, a); if (e) return e;
  }
  if (p->__pyx_v_override) {
    e = (*v)(p->__pyx_v_override, a); if (e) return e;
  }
  if (p->__pyx_v_show_unmodified_termini) {
    e = (*v)(p->__pyx_v_show_unmodified_termini, a); if (e) return e;
  }
  if (p->__pyx_v_variable_mods) {
    e = (*v)(p->__pyx_v_variable_mods, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_9pyteomics_7cparser___pyx_scope_struct_1_isoforms_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_9pyteomics_7cparser___pyx_scope_struct_1_isoforms},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_9pyteomics_7cparser___pyx_scope_struct_1_isoforms},
  {Py_tp_new, (void *)__pyx_tp_new_9pyteomics_7cparser___pyx_scope_struct_1_isoforms},
  {0, 0},
};
static PyType_Spec __pyx_type_9pyteomics_7cparser___pyx_scope_struct_1_isoforms_spec = {
  "pyteomics.cparser.__pyx_scope_struct_1_isoforms",
  sizeof(struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_9pyteomics_7cparser___pyx_scope_struct_1_isoforms_slots,
};
#else

static PyTypeObject __pyx_type_9pyteomics_7cparser___pyx_scope_struct_1_isoforms = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyteomics.cparser.""__pyx_scope_struct_1_isoforms", /*tp_name*/
  sizeof(struct __pyx_obj_9pyteomics_7cparser___pyx_scope_struct_1_isoforms), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9pyteomics_7cparser___pyx_scope_struct_1_isoforms, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_9pyteomics_7cparser___pyx_scope_struct_1_isoforms, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9pyteomics_7cparser___pyx_scope_struct_1_isoforms, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
/* #### Code section: initfunc_declarations ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate); /*proto*/
/* #### Code section: init_module ### */

static int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_9pyteomics_7cparser_std_amino_acids = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser_std_labels = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser_std_nterm = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser_std_cterm = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser__modX_sequence = Py_None; Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser__modX_group = Py_None; Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser__modX_split = Py_None; Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser__modX_split_findall = Py_None; Py_INCREF(Py_None);
  __Pyx_EnumBase = Py_None; Py_INCREF(Py_None);
  __Pyx_FlagBase = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("parse", (void (*)(void))__pyx_f_9pyteomics_7cparser_parse, "PyObject *(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_parse *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("amino_acid_composition", (void (*)(void))__pyx_f_9pyteomics_7cparser_amino_acid_composition, "PyObject *(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_split_label", (void (*)(void))__pyx_f_9pyteomics_7cparser__split_label, "PyObject *(PyObject *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("tostring", (void (*)(void))__pyx_f_9pyteomics_7cparser_tostring, "PyObject *(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_tostring *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_9pyteomics_7cparser_SequencePosition = &__pyx_vtable_9pyteomics_7cparser_SequencePosition;
  __pyx_vtable_9pyteomics_7cparser_SequencePosition._create = (struct __pyx_obj_9pyteomics_7cparser_SequencePosition *(*)(PyObject *, PyObject *, PyObject *, enum __pyx_t_9pyteomics_7cparser_PositionType))__pyx_f_9pyteomics_7cparser_16SequencePosition__create;
  __pyx_vtable_9pyteomics_7cparser_SequencePosition.get_index = (PyObject *(*)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, long, int __pyx_skip_dispatch))__pyx_f_9pyteomics_7cparser_16SequencePosition_get_index;
  __pyx_vtable_9pyteomics_7cparser_SequencePosition.get_size = (size_t (*)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *))__pyx_f_9pyteomics_7cparser_16SequencePosition_get_size;
  __pyx_vtable_9pyteomics_7cparser_SequencePosition.is_modified = (int (*)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *))__pyx_f_9pyteomics_7cparser_16SequencePosition_is_modified;
  __pyx_vtable_9pyteomics_7cparser_SequencePosition.add_modification = (PyObject *(*)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9pyteomics_7cparser_16SequencePosition_add_modification;
  __pyx_vtable_9pyteomics_7cparser_SequencePosition.as_tuple = (PyObject *(*)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, int __pyx_skip_dispatch))__pyx_f_9pyteomics_7cparser_16SequencePosition_as_tuple;
  __pyx_vtable_9pyteomics_7cparser_SequencePosition.copy = (struct __pyx_obj_9pyteomics_7cparser_SequencePosition *(*)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *))__pyx_f_9pyteomics_7cparser_16SequencePosition_copy;
  __pyx_vtable_9pyteomics_7cparser_SequencePosition._eq = (int (*)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *))__pyx_f_9pyteomics_7cparser_16SequencePosition__eq;
  __pyx_vtable_9pyteomics_7cparser_SequencePosition._eq_tuple = (int (*)(struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, PyObject *))__pyx_f_9pyteomics_7cparser_16SequencePosition__eq_tuple;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_9pyteomics_7cparser_SequencePosition_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition)) __PYX_ERR(0, 697, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_9pyteomics_7cparser_SequencePosition_spec, __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition) < 0) __PYX_ERR(0, 697, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition = &__pyx_type_9pyteomics_7cparser_SequencePosition;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition) < 0) __PYX_ERR(0, 697, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition->tp_dictoffset && __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_vtabptr_9pyteomics_7cparser_SequencePosition) < 0) __PYX_ERR(0, 697, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition) < 0) __PYX_ERR(0, 697, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_SequencePosition, (PyObject *) __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition) < 0) __PYX_ERR(0, 697, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_SequencePosition) < 0) __PYX_ERR(0, 697, __pyx_L1_error)
  __pyx_vtabptr_9pyteomics_7cparser_IsoformGenerator = &__pyx_vtable_9pyteomics_7cparser_IsoformGenerator;
  __pyx_vtable_9pyteomics_7cparser_IsoformGenerator.coerce_modification_dict = (PyObject *(*)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9pyteomics_7cparser_16IsoformGenerator_coerce_modification_dict;
  __pyx_vtable_9pyteomics_7cparser_IsoformGenerator.initialize_from_str = (PyObject *(*)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9pyteomics_7cparser_16IsoformGenerator_initialize_from_str;
  __pyx_vtable_9pyteomics_7cparser_IsoformGenerator.initialize_from_iterable = (PyObject *(*)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9pyteomics_7cparser_16IsoformGenerator_initialize_from_iterable;
  __pyx_vtable_9pyteomics_7cparser_IsoformGenerator.apply_fixed_mods = (PyObject *(*)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_9pyteomics_7cparser_16IsoformGenerator_apply_fixed_mods;
  __pyx_vtable_9pyteomics_7cparser_IsoformGenerator.build_site_map = (PyObject *(*)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_16IsoformGenerator_build_site_map *__pyx_optional_args))__pyx_f_9pyteomics_7cparser_16IsoformGenerator_build_site_map;
  __pyx_vtable_9pyteomics_7cparser_IsoformGenerator._reset_sequence = (void (*)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *))__pyx_f_9pyteomics_7cparser_16IsoformGenerator__reset_sequence;
  __pyx_vtable_9pyteomics_7cparser_IsoformGenerator.__pyx_fuse_0_apply_variable_assignment = (PyObject *(*)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, long *, PyObject *, size_t))__pyx_fuse_0__pyx_f_9pyteomics_7cparser_16IsoformGenerator__apply_variable_assignment;
  __pyx_vtable_9pyteomics_7cparser_IsoformGenerator.__pyx_fuse_1_apply_variable_assignment = (PyObject *(*)(struct __pyx_obj_9pyteomics_7cparser_IsoformGenerator *, PyObject *, long *, PyObject *, size_t))__pyx_fuse_1__pyx_f_9pyteomics_7cparser_16IsoformGenerator__apply_variable_assignment;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_9pyteomics_7cparser_IsoformGenerator_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator)) __PYX_ERR(0, 987, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_9pyteomics_7cparser_IsoformGenerator_spec, __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator) < 0) __PYX_ERR(0, 987, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator = &__pyx_type_9pyteomics_7cparser_IsoformGenerator;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator) < 0) __PYX_ERR(0, 987, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator->tp_dictoffset && __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator, __pyx_vtabptr_9pyteomics_7cparser_IsoformGenerator) < 0) __PYX_ERR(0, 987, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator) < 0) __PYX_ERR(0, 987, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_IsoformGenerator, (PyObject *) __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator) < 0) __PYX_ERR(0, 987, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator) < 0) __PYX_ERR(0, 987, __pyx_L1_error)
  __pyx_vtabptr_9pyteomics_7cparser_ModificationRule = &__pyx_vtable_9pyteomics_7cparser_ModificationRule;
  __pyx_vtable_9pyteomics_7cparser_ModificationRule.test = (int (*)(struct __pyx_obj_9pyteomics_7cparser_ModificationRule *, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, int __pyx_skip_dispatch))__pyx_f_9pyteomics_7cparser_16ModificationRule_test;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_9pyteomics_7cparser_ModificationRule_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule)) __PYX_ERR(0, 1209, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_9pyteomics_7cparser_ModificationRule_spec, __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule) < 0) __PYX_ERR(0, 1209, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule = &__pyx_type_9pyteomics_7cparser_ModificationRule;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule) < 0) __PYX_ERR(0, 1209, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule->tp_dictoffset && __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule, __pyx_vtabptr_9pyteomics_7cparser_ModificationRule) < 0) __PYX_ERR(0, 1209, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule) < 0) __PYX_ERR(0, 1209, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_ModificationRule, (PyObject *) __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule) < 0) __PYX_ERR(0, 1209, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationRule) < 0) __PYX_ERR(0, 1209, __pyx_L1_error)
  __pyx_vtabptr_9pyteomics_7cparser_ModificationTarget = &__pyx_vtable_9pyteomics_7cparser_ModificationTarget;
  __pyx_vtable_9pyteomics_7cparser_ModificationTarget.test = (int (*)(struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *, struct __pyx_obj_9pyteomics_7cparser_SequencePosition *, int __pyx_skip_dispatch))__pyx_f_9pyteomics_7cparser_18ModificationTarget_test;
  __pyx_vtable_9pyteomics_7cparser_ModificationTarget._create = (struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *(*)(PyObject *, enum __pyx_t_9pyteomics_7cparser_PositionType))__pyx_f_9pyteomics_7cparser_18ModificationTarget__create;
  __pyx_vtable_9pyteomics_7cparser_ModificationTarget._from_string = (struct __pyx_obj_9pyteomics_7cparser_ModificationTarget *(*)(PyObject *))__pyx_f_9pyteomics_7cparser_18ModificationTarget__from_string;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_9pyteomics_7cparser_ModificationTarget_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget)) __PYX_ERR(0, 1240, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_9pyteomics_7cparser_ModificationTarget_spec, __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget) < 0) __PYX_ERR(0, 1240, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget = &__pyx_type_9pyteomics_7cparser_ModificationTarget;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget) < 0) __PYX_ERR(0, 1240, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget->tp_dictoffset && __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget, __pyx_vtabptr_9pyteomics_7cparser_ModificationTarget) < 0) __PYX_ERR(0, 1240, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget) < 0) __PYX_ERR(0, 1240, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_ModificationTarget, (PyObject *) __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget) < 0) __PYX_ERR(0, 1240, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_9pyteomics_7cparser_ModificationTarget) < 0) __PYX_ERR(0, 1240, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_9pyteomics_7cparser___pyx_scope_struct__generate_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate)) __PYX_ERR(0, 1105, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_9pyteomics_7cparser___pyx_scope_struct__generate_spec, __pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate) < 0) __PYX_ERR(0, 1105, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate = &__pyx_type_9pyteomics_7cparser___pyx_scope_struct__generate;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate) < 0) __PYX_ERR(0, 1105, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate->tp_dictoffset && __pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct__generate->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_9pyteomics_7cparser___pyx_scope_struct_1_isoforms_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms)) __PYX_ERR(0, 1308, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_9pyteomics_7cparser___pyx_scope_struct_1_isoforms_spec, __pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms) < 0) __PYX_ERR(0, 1308, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms = &__pyx_type_9pyteomics_7cparser___pyx_scope_struct_1_isoforms;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms) < 0) __PYX_ERR(0, 1308, __pyx_L1_error)
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms->tp_dictoffset && __pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_9pyteomics_7cparser___pyx_scope_struct_1_isoforms->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7cpython_4type_type = __Pyx_ImportType_3_1_1(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_1_1(PyTypeObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  0, 0,
  #else
  sizeof(PyHeapTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_1_1(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_1_1); if (!__pyx_mstate->__pyx_ptype_7cpython_4type_type) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_cparser(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_cparser},
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  {Py_mod_gil, Py_MOD_GIL_USED},
  #endif
  #if PY_VERSION_HEX >= 0x030C0000 && CYTHON_USE_MODULE_STATE
  {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED},
  #endif
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "cparser",
      0, /* m_doc */
    #if CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstatetype), /* m_size */
    #else
      (CYTHON_PEP489_MULTI_PHASE_INIT) ? 0 : -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif

/* PyModInitFuncType */
#ifndef CYTHON_NO_PYINIT_EXPORT
  #define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#else
  #ifdef __cplusplus
  #define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
  #else
  #define __Pyx_PyMODINIT_FUNC PyObject *
  #endif
#endif

__Pyx_PyMODINIT_FUNC PyInit_cparser(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_cparser(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
/* ModuleCreationPEP489 */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
static PY_INT64_T __Pyx_GetCurrentInterpreterId(void) {
    {
        PyObject *module = PyImport_ImportModule("_interpreters"); // 3.13+ I think
        if (!module) {
            PyErr_Clear(); // just try the 3.8-3.12 version
            module = PyImport_ImportModule("_xxsubinterpreters");
            if (!module) goto bad;
        }
        PyObject *current = PyObject_CallMethod(module, "get_current", NULL);
        Py_DECREF(module);
        if (!current) goto bad;
        if (PyTuple_Check(current)) {
            PyObject *new_current = PySequence_GetItem(current, 0);
            Py_DECREF(current);
            current = new_current;
            if (!new_current) goto bad;
        }
        long long as_c_int = PyLong_AsLongLong(current);
        Py_DECREF(current);
        return as_c_int;
    }
  bad:
    PySys_WriteStderr("__Pyx_GetCurrentInterpreterId failed. Try setting the C define CYTHON_PEP489_MULTI_PHASE_INIT=0\n");
    return -1;
}
#endif
#if !CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    static PY_INT64_T main_interpreter_id = -1;
#if CYTHON_COMPILING_IN_GRAAL
    PY_INT64_T current_id = PyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX >= 0x03090000
    PY_INT64_T current_id = PyInterpreterState_GetID(PyInterpreterState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API
    PY_INT64_T current_id = __Pyx_GetCurrentInterpreterId();
#else
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
#endif
    if (unlikely(current_id == -1)) {
        return -1;
    }
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return 0;
    } else if (unlikely(main_interpreter_id != current_id)) {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#endif
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    #if !CYTHON_USE_MODULE_STATE
    if (__Pyx_check_single_interpreter())
        return NULL;
    #endif
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_cparser(PyObject *__pyx_pyinit_module)
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  __pyx_mstatetype *__pyx_mstate = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'cparser' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #else
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_t_1 = __pyx_pyinit_module;
  Py_INCREF(__pyx_t_1);
  #else
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #if CYTHON_USE_MODULE_STATE
  {
    int add_module_result = __Pyx_State_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "cparser" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = __pyx_t_1;
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  PyUnstable_Module_SetGIL(__pyx_m, Py_MOD_GIL_USED);
  #endif
  __pyx_mstate = __pyx_mstate_global;
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_mstate->__pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_mstate->__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_mstate->__pyx_d);
  __pyx_mstate->__pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_mstate->__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_cython_runtime = __Pyx_PyImport_AddModuleRef("cython_runtime"); if (unlikely(!__pyx_mstate->__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_mstate->__pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /* ImportRefnannyAPI */
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif

__Pyx_RefNannySetupContext("PyInit_cparser", 0);
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_mstate->__pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_mstate->__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants(__pyx_mstate) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__pyx_module_is_main_pyteomics__cparser) {
    if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_name, __pyx_mstate_global->__pyx_n_u_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pyteomics.cparser")) {
      if (unlikely((PyDict_SetItemString(modules, "pyteomics.cparser", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins(__pyx_mstate) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants(__pyx_mstate) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_CreateCodeObjects(__pyx_mstate) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code(__pyx_mstate);
  (void)__Pyx_modinit_variable_export_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_function_export_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_init_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code(__pyx_mstate);
  (void)__Pyx_modinit_function_import_code(__pyx_mstate);
  /*--- Execution code ---*/

  /* "EnumBase":10
 * 
 * cdef object __Pyx_EnumBase
 * from enum import IntEnum as __Pyx_EnumBase             # <<<<<<<<<<<<<<
 * 
 * cdef object __Pyx_FlagBase
*/
  __pyx_t_2 = __Pyx_PyList_Pack(1, __pyx_mstate_global->__pyx_n_u_IntEnum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_enum, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_IntEnum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_XGOTREF(__Pyx_EnumBase);
  __Pyx_DECREF_SET(__Pyx_EnumBase, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "EnumBase":13
 * 
 * cdef object __Pyx_FlagBase
 * from enum import IntFlag as __Pyx_FlagBase             # <<<<<<<<<<<<<<
 * 
*/
  __pyx_t_3 = __Pyx_PyList_Pack(1, __pyx_mstate_global->__pyx_n_u_IntFlag); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_enum, __pyx_t_3, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_IntFlag); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_XGOTREF(__Pyx_FlagBase);
  __Pyx_DECREF_SET(__Pyx_FlagBase, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "EnumType":23
 * 
 * 
 * PositionType = __Pyx_FlagBase('PositionType', [             # <<<<<<<<<<<<<<
 *     ('n_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(n_term)),
 *     ('c_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(c_term)),
*/
  __pyx_t_3 = NULL;
  __Pyx_INCREF(__Pyx_FlagBase);
  __pyx_t_4 = __Pyx_FlagBase; 

  /* "EnumType":24
 * 
 * PositionType = __Pyx_FlagBase('PositionType', [
 *     ('n_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(n_term)),             # <<<<<<<<<<<<<<
 *     ('c_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(c_term)),
 *     ('internal', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(internal)),
*/
  __pyx_t_5 = __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_e_9pyteomics_7cparser_n_term); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_Pack(2, __pyx_mstate_global->__pyx_n_u_n_term, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "EnumType":25
 * PositionType = __Pyx_FlagBase('PositionType', [
 *     ('n_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(n_term)),
 *     ('c_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(c_term)),             # <<<<<<<<<<<<<<
 *     ('internal', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(internal)),
 *     ('universal', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(universal)),
*/
  __pyx_t_5 = __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_e_9pyteomics_7cparser_c_term); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = PyTuple_Pack(2, __pyx_mstate_global->__pyx_n_u_c_term, __pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "EnumType":26
 *     ('n_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(n_term)),
 *     ('c_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(c_term)),
 *     ('internal', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(internal)),             # <<<<<<<<<<<<<<
 *     ('universal', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(universal)),
 * 
*/
  __pyx_t_5 = __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_e_9pyteomics_7cparser_internal); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_8 = PyTuple_Pack(2, __pyx_mstate_global->__pyx_n_u_internal, __pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "EnumType":27
 *     ('c_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(c_term)),
 *     ('internal', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(internal)),
 *     ('universal', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(universal)),             # <<<<<<<<<<<<<<
 * 
 * ], module=globals().get("__module__", 'pyteomics.cparser'))
*/
  __pyx_t_5 = __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(__pyx_e_9pyteomics_7cparser_universal); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_9 = PyTuple_Pack(2, __pyx_mstate_global->__pyx_n_u_universal, __pyx_t_5); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "EnumType":23
 * 
 * 
 * PositionType = __Pyx_FlagBase('PositionType', [             # <<<<<<<<<<<<<<
 *     ('n_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(n_term)),
 *     ('c_term', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(c_term)),
*/
  __pyx_t_5 = __Pyx_PyList_Pack(4, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "EnumType":29
 *     ('universal', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(universal)),
 * 
 * ], module=globals().get("__module__", 'pyteomics.cparser'))             # <<<<<<<<<<<<<<
 * 
 * if PY_VERSION_HEX >= 0x030B0000:
*/
  __pyx_t_9 = __Pyx_Globals(); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_mstate_global->__pyx_tuple[4], NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_10 = 1;
  {
    PyObject *__pyx_callargs[3 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PositionType, __pyx_t_5};
    __pyx_t_8 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 23, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_module, __pyx_t_9, __pyx_t_8, __pyx_callargs+3, 0) < 0) __PYX_ERR(1, 23, __pyx_L1_error)
    __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder(__pyx_t_4, __pyx_callargs+__pyx_t_10, (3-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 23, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_PositionType, __pyx_t_2) < 0) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "EnumType":31
 * ], module=globals().get("__module__", 'pyteomics.cparser'))
 * 
 * if PY_VERSION_HEX >= 0x030B0000:             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_11 = (PY_VERSION_HEX >= 0x030B0000);
  if (__pyx_t_11) {

    /* "EnumType":36
 * 
 * 
 *     PositionType._member_names_ = list(PositionType.__members__)             # <<<<<<<<<<<<<<
 * 
*/
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_PositionType); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_members); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PySequence_ListKeepNew(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_PositionType); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_member_names, __pyx_t_2) < 0) __PYX_ERR(1, 36, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "EnumType":31
 * ], module=globals().get("__module__", 'pyteomics.cparser'))
 * 
 * if PY_VERSION_HEX >= 0x030B0000:             # <<<<<<<<<<<<<<
 * 
 * 
*/
  }

  /* "pyteomics/cparser.pyx":41
 * 
 * 
 * import re             # <<<<<<<<<<<<<<
 * from collections import deque
 * import itertools as it
*/
  __pyx_t_4 = __Pyx_ImportDottedModule(__pyx_mstate_global->__pyx_n_u_re, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_re, __pyx_t_4) < 0) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":42
 * 
 * import re
 * from collections import deque             # <<<<<<<<<<<<<<
 * import itertools as it
 * from pyteomics.auxiliary import PyteomicsError, memoize, BasicComposition
*/
  __pyx_t_4 = __Pyx_PyList_Pack(1, __pyx_mstate_global->__pyx_n_u_deque); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_collections, __pyx_t_4, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_deque); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_deque, __pyx_t_4) < 0) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":43
 * import re
 * from collections import deque
 * import itertools as it             # <<<<<<<<<<<<<<
 * from pyteomics.auxiliary import PyteomicsError, memoize, BasicComposition
 * 
*/
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_mstate_global->__pyx_n_u_itertools, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_it, __pyx_t_2) < 0) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":44
 * from collections import deque
 * import itertools as it
 * from pyteomics.auxiliary import PyteomicsError, memoize, BasicComposition             # <<<<<<<<<<<<<<
 * 
 * cdef:
*/
  __pyx_t_2 = __Pyx_PyList_Pack(3, __pyx_mstate_global->__pyx_n_u_PyteomicsError, __pyx_mstate_global->__pyx_n_u_memoize, __pyx_mstate_global->__pyx_n_u_BasicComposition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pyteomics_auxiliary, __pyx_t_2, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_PyteomicsError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_PyteomicsError, __pyx_t_2) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_memoize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_memoize, __pyx_t_2) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_BasicComposition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_BasicComposition, __pyx_t_2) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":53
 * DEF TOSTRING_STACK_BUFFER_SIZE = 150
 * 
 * std_amino_acids = ['Q','W','E','R','T','Y','I','P','A','S',             # <<<<<<<<<<<<<<
 *                    'D','F','G','H','K','L','C','V','N','M']
 * """modX labels for the 20 standard amino acids."""
*/
  __pyx_t_4 = __Pyx_PyList_Pack(20, __pyx_mstate_global->__pyx_n_u_Q, __pyx_mstate_global->__pyx_n_u_W, __pyx_mstate_global->__pyx_n_u_E, __pyx_mstate_global->__pyx_n_u_R, __pyx_mstate_global->__pyx_n_u_T, __pyx_mstate_global->__pyx_n_u_Y, __pyx_mstate_global->__pyx_n_u_I, __pyx_mstate_global->__pyx_n_u_P, __pyx_mstate_global->__pyx_n_u_A, __pyx_mstate_global->__pyx_n_u_S, __pyx_mstate_global->__pyx_n_u_D, __pyx_mstate_global->__pyx_n_u_F, __pyx_mstate_global->__pyx_n_u_G, __pyx_mstate_global->__pyx_n_u_H, __pyx_mstate_global->__pyx_n_u_K, __pyx_mstate_global->__pyx_n_u_L, __pyx_mstate_global->__pyx_n_u_C, __pyx_mstate_global->__pyx_n_u_V, __pyx_mstate_global->__pyx_n_u_N, __pyx_mstate_global->__pyx_n_u_M); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser_std_amino_acids);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser_std_amino_acids, ((PyObject*)__pyx_t_4));
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":57
 * """modX labels for the 20 standard amino acids."""
 * 
 * std_nterm = 'H-'             # <<<<<<<<<<<<<<
 * """modX label for the unmodified N-terminus."""
 * 
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_H_2);
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser_std_nterm);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser_std_nterm, __pyx_mstate_global->__pyx_kp_u_H_2);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u_H_2);

  /* "pyteomics/cparser.pyx":60
 * """modX label for the unmodified N-terminus."""
 * 
 * std_cterm = '-OH'             # <<<<<<<<<<<<<<
 * """modX label for the unmodified C-terminus."""
 * 
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_OH);
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser_std_cterm);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser_std_cterm, __pyx_mstate_global->__pyx_kp_u_OH);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u_OH);

  /* "pyteomics/cparser.pyx":63
 * """modX label for the unmodified C-terminus."""
 * 
 * std_labels = std_amino_acids + [std_nterm, std_cterm]             # <<<<<<<<<<<<<<
 * """modX labels for the standard amino acids and unmodified termini."""
 * 
*/
  __pyx_t_4 = __Pyx_PyList_Pack(2, __pyx_v_9pyteomics_7cparser_std_nterm, __pyx_v_9pyteomics_7cparser_std_cterm); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyNumber_Add(__pyx_v_9pyteomics_7cparser_std_amino_acids, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser_std_labels);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser_std_labels, ((PyObject*)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":66
 * """modX labels for the standard amino acids and unmodified termini."""
 * 
 * _modX_sequence = re.compile(r'^([^-]+-)?((?:[^A-Z-]*[A-Z])+)(-[^-]+)?$')             # <<<<<<<<<<<<<<
 * _modX_group = re.compile(r'[^A-Z-]*[A-Z]')
 * _modX_split = re.compile(r'([^A-Z-]*)([A-Z])')
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_re); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_compile); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_10 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_A_Z_A_Z};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 66, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser__modX_sequence);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser__modX_sequence, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":67
 * 
 * _modX_sequence = re.compile(r'^([^-]+-)?((?:[^A-Z-]*[A-Z])+)(-[^-]+)?$')
 * _modX_group = re.compile(r'[^A-Z-]*[A-Z]')             # <<<<<<<<<<<<<<
 * _modX_split = re.compile(r'([^A-Z-]*)([A-Z])')
 * 
*/
  __pyx_t_9 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_re); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_compile); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_10 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_A_Z_A_Z_2};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser__modX_group);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser__modX_group, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":68
 * _modX_sequence = re.compile(r'^([^-]+-)?((?:[^A-Z-]*[A-Z])+)(-[^-]+)?$')
 * _modX_group = re.compile(r'[^A-Z-]*[A-Z]')
 * _modX_split = re.compile(r'([^A-Z-]*)([A-Z])')             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_re); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_compile); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_10 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_mstate_global->__pyx_kp_u_A_Z_A_Z_3};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser__modX_split);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser__modX_split, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":120
 * cpdef int length(object sequence,
 *                  int show_unmodified_termini=0, int split=0,
 *                  int allow_unknown_modifications=0, object labels=std_labels):             # <<<<<<<<<<<<<<
 *     """Calculate the number of amino acid residues in a polypeptide
 *     written in modX notation.
*/
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_labels);
  __pyx_mstate_global->__pyx_k__2 = __pyx_v_9pyteomics_7cparser_std_labels;
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_labels);

  /* "pyteomics/cparser.pyx":118
 *     return bool(match_modX(label))
 * 
 * cpdef int length(object sequence,             # <<<<<<<<<<<<<<
 *                  int show_unmodified_termini=0, int split=0,
 *                  int allow_unknown_modifications=0, object labels=std_labels):
*/
  __pyx_t_2 = PyTuple_Pack(4, __pyx_mstate_global->__pyx_int_0, __pyx_mstate_global->__pyx_int_0, __pyx_mstate_global->__pyx_int_0, __pyx_v_9pyteomics_7cparser_std_labels); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_1length, 0, __pyx_mstate_global->__pyx_n_u_length, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_length, __pyx_t_4) < 0) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":170
 * 
 * 
 * cpdef inline tuple _split_label(str label):             # <<<<<<<<<<<<<<
 *     cdef:
 *         str mod, X
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_3_split_label, 0, __pyx_mstate_global->__pyx_n_u_split_label, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[3])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_split_label, __pyx_t_4) < 0) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":187
 * 
 * 
 * cdef object _modX_split_findall = _modX_split.findall             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_9pyteomics_7cparser__modX_split, __pyx_mstate_global->__pyx_n_u_findall); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser__modX_split_findall);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser__modX_split_findall, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":211
 * 
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,             # <<<<<<<<<<<<<<
 *                  bint allow_unknown_modifications=0, object labels=None):
 *     """Parse a sequence string written in modX notation into a list of
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_5parse, 0, __pyx_mstate_global->__pyx_n_u_parse, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[4])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_mstate_global->__pyx_tuple[5]);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_parse, __pyx_t_4) < 0) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":435
 * 
 * 
 * cpdef str tostring(object parsed_sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     """Create a string from a parsed sequence.
 * 
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_7tostring, 0, __pyx_mstate_global->__pyx_n_u_tostring, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[5])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_mstate_global->__pyx_tuple[6]);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_tostring, __pyx_t_4) < 0) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":565
 * 
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=0, bint term_aa=0,             # <<<<<<<<<<<<<<
 *                                   bint allow_unknown_modifications=0, object labels=None):
 *     """Calculate amino acid composition of a polypeptide.
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_9amino_acid_composition, 0, __pyx_mstate_global->__pyx_n_u_amino_acid_composition, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[6])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_mstate_global->__pyx_tuple[5]);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_amino_acid_composition, __pyx_t_4) < 0) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":728
 *         return self.get_index(i)
 * 
 *     cpdef str get_index(self, long i):             # <<<<<<<<<<<<<<
 *         cdef:
 *             long size, n_mods
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16SequencePosition_5get_index, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SequencePosition_get_index, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[7])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_get_index, __pyx_t_4) < 0) __PYX_ERR(0, 728, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":772
 *         return self.modification is not None
 * 
 *     cpdef add_modification(self, str mod):             # <<<<<<<<<<<<<<
 *         self.modification = mod
 * 
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16SequencePosition_7add_modification, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SequencePosition_add_modificatio, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[8])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 772, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_add_modification, __pyx_t_4) < 0) __PYX_ERR(0, 772, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":781
 *         return self.get_size()
 * 
 *     cpdef tuple as_tuple(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t size, i, n_mods, j
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16SequencePosition_13as_tuple, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SequencePosition_as_tuple, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[9])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 781, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_as_tuple, __pyx_t_4) < 0) __PYX_ERR(0, 781, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":811
 *             self.amino_acid, (self.modification), self.terminal, self.position_type)
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_tuple(cls, tuple source):
 *         cdef:
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16SequencePosition_15from_tuple, __Pyx_CYFUNCTION_CLASSMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SequencePosition_from_tuple, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[10])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 811, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_from_tuple, __pyx_t_4) < 0) __PYX_ERR(0, 811, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_from_tuple); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 811, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_Method_ClassMethod(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 811, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_from_tuple, __pyx_t_2) < 0) __PYX_ERR(0, 811, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":850
 *         return cls(amino_acid, modification, terminal, position_type)
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def parse(cls, str sequence, **kwargs):
 *         cdef:
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16SequencePosition_17parse, __Pyx_CYFUNCTION_CLASSMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SequencePosition_parse, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[11])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 850, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_parse, __pyx_t_2) < 0) __PYX_ERR(0, 850, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_parse); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 850, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_Method_ClassMethod(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 850, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_parse, __pyx_t_4) < 0) __PYX_ERR(0, 850, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyteomics/cparser.pyx":864
 *         return result
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_iterable(cls, object iterable):
 *         cdef:
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16SequencePosition_19from_iterable, __Pyx_CYFUNCTION_CLASSMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SequencePosition_from_iterable, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[12])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_from_iterable, __pyx_t_4) < 0) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_from_iterable); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_Method_ClassMethod(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_from_iterable, __pyx_t_2) < 0) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16SequencePosition_25__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SequencePosition___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[13])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SequencePosition, (type(self), 0xe2ca0e1, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SequencePosition__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16SequencePosition_27__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SequencePosition___setstate_cyth, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[14])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_SequencePosition, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":906
 * 
 * 
 * cpdef str _sequence_tostring(list sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n, acc, j
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_11_sequence_tostring, 0, __pyx_mstate_global->__pyx_n_u_sequence_tostring, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[15])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 906, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[7]);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_sequence_tostring, __pyx_t_2) < 0) __PYX_ERR(0, 906, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":972
 * 
 * 
 * cpdef list copy_sequence(list sequence):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list result
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_13copy_sequence, 0, __pyx_mstate_global->__pyx_n_u_copy_sequence, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[16])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_copy_sequence, __pyx_t_2) < 0) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1026
 *         self.last_indices = []
 * 
 *     cpdef dict coerce_modification_dict(self, dict source):             # <<<<<<<<<<<<<<
 *         cdef:
 *             str label
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_3coerce_modification_dict, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_IsoformGenerator_coerce_modifica, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[17])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator, __pyx_mstate_global->__pyx_n_u_coerce_modification_dict, __pyx_t_2) < 0) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1038
 *         return result
 * 
 *     cpdef list initialize_from_str(self, str sequence):             # <<<<<<<<<<<<<<
 *         labels = self.labels + list(self.fixed_mods)
 *         self.sequence = SequencePosition.parse(sequence, labels=labels)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_5initialize_from_str, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_IsoformGenerator_initialize_from, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[18])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1038, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator, __pyx_mstate_global->__pyx_n_u_initialize_from_str, __pyx_t_2) < 0) __PYX_ERR(0, 1038, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1043
 *         return self.sequence
 * 
 *     cpdef list initialize_from_iterable(self, object sequence):             # <<<<<<<<<<<<<<
 *         self.sequence = SequencePosition.from_iterable(sequence)
 *         return self.sequence
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_7initialize_from_iterable, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_IsoformGenerator_initialize_from_2, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[19])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1043, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator, __pyx_mstate_global->__pyx_n_u_initialize_from_iterable, __pyx_t_2) < 0) __PYX_ERR(0, 1043, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1047
 *         return self.sequence
 * 
 *     cpdef list apply_fixed_mods(self, list sequence):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_9apply_fixed_mods, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_IsoformGenerator_apply_fixed_mod, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[20])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1047, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator, __pyx_mstate_global->__pyx_n_u_apply_fixed_mods, __pyx_t_2) < 0) __PYX_ERR(0, 1047, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1067
 *         return sequence
 * 
 *     cpdef dict build_site_map(self, list sequence, bint include_unmodified=True):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_11build_site_map, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_IsoformGenerator_build_site_map, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[21])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1067, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[8]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator, __pyx_mstate_global->__pyx_n_u_build_site_map, __pyx_t_2) < 0) __PYX_ERR(0, 1067, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1105
 *         return site_map
 * 
 *     @cython.boundscheck(False)             # <<<<<<<<<<<<<<
 *     def generate(self, sequence_spec):
 *         cdef:
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_13generate, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_IsoformGenerator_generate, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator, __pyx_mstate_global->__pyx_n_u_generate, __pyx_t_2) < 0) __PYX_ERR(0, 1105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_18__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_IsoformGenerator___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[22])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IsoformGenerator, (type(self), 0x09daa66, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IsoformGenerator__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16IsoformGenerator_20__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_IsoformGenerator___setstate_cyth, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[23])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_IsoformGenerator, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1221
 *                         for x in targets]
 * 
 *     cpdef bint test(self, SequencePosition position):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16ModificationRule_3test, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_ModificationRule_test, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[24])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationRule, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_2) < 0) __PYX_ERR(0, 1221, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16ModificationRule_9__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_ModificationRule___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[25])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationRule, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ModificationRule, (type(self), 0xccc7aa9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ModificationRule__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_16ModificationRule_11__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_ModificationRule___setstate_cyth, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[26])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationRule, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1246
 *         public PositionType position_type
 * 
 *     def __init__(self, target, position_type=PositionType.internal):             # <<<<<<<<<<<<<<
 *         self.target = target
 *         self.position_type = position_type
*/
  __pyx_t_2 = __Pyx_Enum_0bf71e__9pyteomics_7cparser_enum__dunderpyx_t_9pyteomi__etc_to_py(__pyx_e_9pyteomics_7cparser_internal); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_mstate_global->__pyx_k__4 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1250
 *         self.position_type = position_type
 * 
 *     cpdef bint test(self, SequencePosition position):             # <<<<<<<<<<<<<<
 *         if self.position_type == PositionType.universal:
 *             return True
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_18ModificationTarget_3test, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_ModificationTarget_test, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[27])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationTarget, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_2) < 0) __PYX_ERR(0, 1250, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1274
 *         return inst
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def from_string(str string):
 *         return ModificationTarget._from_string(string)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_18ModificationTarget_9from_string, __Pyx_CYFUNCTION_STATICMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_ModificationTarget_from_string, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[28])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationTarget, __pyx_mstate_global->__pyx_n_u_from_string, __pyx_t_2) < 0) __PYX_ERR(0, 1274, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = NULL;
  __Pyx_INCREF(__pyx_builtin_staticmethod);
  __pyx_t_8 = __pyx_builtin_staticmethod; 
  __Pyx_GetNameInClass(__pyx_t_9, (PyObject*)__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationTarget, __pyx_mstate_global->__pyx_n_u_from_string); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_9};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+__pyx_t_10, (2-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationTarget, __pyx_mstate_global->__pyx_n_u_from_string, __pyx_t_2) < 0) __PYX_ERR(0, 1274, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_18ModificationTarget_11__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_ModificationTarget___reduce_cyth, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[29])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationTarget, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ModificationTarget, (type(self), 0x0f5dd8b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ModificationTarget__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_18ModificationTarget_13__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_ModificationTarget___setstate_cy, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[30])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_9pyteomics_7cparser_ModificationTarget, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1308
 * 
 * 
 * def isoforms(str sequence, **kwargs):             # <<<<<<<<<<<<<<
 *     cdef:
 *         IsoformGenerator isoform_generator
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_15isoforms, 0, __pyx_mstate_global->__pyx_n_u_isoforms, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_isoforms, __pyx_t_2) < 0) __PYX_ERR(0, 1308, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_SequencePosition(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_18__pyx_unpickle_SequencePosition, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_SequencePosition, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[31])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_SequencePosition, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_SequencePosition__set_state(<SequencePosition> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SequencePosition__set_state(SequencePosition __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.amino_acid = __pyx_state[0]; __pyx_result.modification = __pyx_state[1]; __pyx_result.position_type = __pyx_state[2]; __pyx_result.terminal = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_20__pyx_unpickle_IsoformGenerator, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_IsoformGenerator, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[32])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_IsoformGenerator, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ModificationRule(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_22__pyx_unpickle_ModificationRule, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ModificationRule, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[33])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ModificationRule, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_ModificationRule__set_state(<ModificationRule> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationRule__set_state(ModificationRule __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.modification = __pyx_state[0]; __pyx_result.targets = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_9pyteomics_7cparser_24__pyx_unpickle_ModificationTarget, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ModificationTarge, NULL, __pyx_mstate_global->__pyx_n_u_pyteomics_cparser, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[34])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ModificationTarge, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":1
 * # cython: embedsignature=True             # <<<<<<<<<<<<<<
 * # cython: profile=False
 * 
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_length_line_118, __pyx_mstate_global->__pyx_kp_u_Calculate_the_number_of_amino_ac) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_parse_line_211, __pyx_mstate_global->__pyx_kp_u_Parse_a_sequence_string_written) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_amino_acid_composition_line_565, __pyx_mstate_global->__pyx_kp_u_Calculate_amino_acid_composition) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_test_2, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  if (__pyx_m) {
    if (__pyx_mstate->__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init pyteomics.cparser", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pyteomics.cparser");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #else
  return __pyx_m;
  #endif
}
/* #### Code section: pystring_table ### */

typedef struct {
    const char *s;
#if 2551 <= 65535
    const unsigned short n;
#elif 2551 / 2 < INT_MAX
    const unsigned int n;
#elif 2551 / 2 < LONG_MAX
    const unsigned long n;
#else
    const Py_ssize_t n;
#endif
#if 1 <= 31
    const unsigned int encoding : 5;
#elif 1 <= 255
    const unsigned char encoding;
#elif 1 <= 65535
    const unsigned short encoding;
#else
    const Py_ssize_t encoding;
#endif
    const unsigned int is_unicode : 1;
    const unsigned int intern : 1;
} __Pyx_StringTabEntry;
static const char * const __pyx_string_tab_encodings[] = { 0 };
static const __Pyx_StringTabEntry __pyx_string_tab[] = {
  {__pyx_k_, sizeof(__pyx_k_), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_ */
  {__pyx_k_A, sizeof(__pyx_k_A), 0, 1, 1}, /* PyObject cname: __pyx_n_u_A */
  {__pyx_k_A_Z_A_Z, sizeof(__pyx_k_A_Z_A_Z), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_A_Z_A_Z */
  {__pyx_k_A_Z_A_Z_2, sizeof(__pyx_k_A_Z_A_Z_2), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_A_Z_A_Z_2 */
  {__pyx_k_A_Z_A_Z_3, sizeof(__pyx_k_A_Z_A_Z_3), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_A_Z_A_Z_3 */
  {__pyx_k_BasicComposition, sizeof(__pyx_k_BasicComposition), 0, 1, 1}, /* PyObject cname: __pyx_n_u_BasicComposition */
  {__pyx_k_C, sizeof(__pyx_k_C), 0, 1, 1}, /* PyObject cname: __pyx_n_u_C */
  {__pyx_k_Calculate_amino_acid_composition, sizeof(__pyx_k_Calculate_amino_acid_composition), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Calculate_amino_acid_composition */
  {__pyx_k_Calculate_the_number_of_amino_ac, sizeof(__pyx_k_Calculate_the_number_of_amino_ac), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Calculate_the_number_of_amino_ac */
  {__pyx_k_Cannot_split_a_non_modX_label_s, sizeof(__pyx_k_Cannot_split_a_non_modX_label_s), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Cannot_split_a_non_modX_label_s */
  {__pyx_k_D, sizeof(__pyx_k_D), 0, 1, 1}, /* PyObject cname: __pyx_n_u_D */
  {__pyx_k_E, sizeof(__pyx_k_E), 0, 1, 1}, /* PyObject cname: __pyx_n_u_E */
  {__pyx_k_F, sizeof(__pyx_k_F), 0, 1, 1}, /* PyObject cname: __pyx_n_u_F */
  {__pyx_k_G, sizeof(__pyx_k_G), 0, 1, 1}, /* PyObject cname: __pyx_n_u_G */
  {__pyx_k_H, sizeof(__pyx_k_H), 0, 1, 1}, /* PyObject cname: __pyx_n_u_H */
  {__pyx_k_H_2, sizeof(__pyx_k_H_2), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_H_2 */
  {__pyx_k_I, sizeof(__pyx_k_I), 0, 1, 1}, /* PyObject cname: __pyx_n_u_I */
  {__pyx_k_Incompatible_checksums_0x_x_vs_0, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Incompatible_checksums_0x_x_vs_0 */
  {__pyx_k_Incompatible_checksums_0x_x_vs_0_2, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0_2), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Incompatible_checksums_0x_x_vs_0_2 */
  {__pyx_k_Incompatible_checksums_0x_x_vs_0_3, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0_3), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Incompatible_checksums_0x_x_vs_0_3 */
  {__pyx_k_Incompatible_checksums_0x_x_vs_0_4, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0_4), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Incompatible_checksums_0x_x_vs_0_4 */
  {__pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IndexError */
  {__pyx_k_IntEnum, sizeof(__pyx_k_IntEnum), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IntEnum */
  {__pyx_k_IntFlag, sizeof(__pyx_k_IntFlag), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IntFlag */
  {__pyx_k_IsoformGenerator, sizeof(__pyx_k_IsoformGenerator), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IsoformGenerator */
  {__pyx_k_IsoformGenerator___reduce_cython, sizeof(__pyx_k_IsoformGenerator___reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IsoformGenerator___reduce_cython */
  {__pyx_k_IsoformGenerator___setstate_cyth, sizeof(__pyx_k_IsoformGenerator___setstate_cyth), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IsoformGenerator___setstate_cyth */
  {__pyx_k_IsoformGenerator_apply_fixed_mod, sizeof(__pyx_k_IsoformGenerator_apply_fixed_mod), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IsoformGenerator_apply_fixed_mod */
  {__pyx_k_IsoformGenerator_build_site_map, sizeof(__pyx_k_IsoformGenerator_build_site_map), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IsoformGenerator_build_site_map */
  {__pyx_k_IsoformGenerator_coerce_modifica, sizeof(__pyx_k_IsoformGenerator_coerce_modifica), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IsoformGenerator_coerce_modifica */
  {__pyx_k_IsoformGenerator_generate, sizeof(__pyx_k_IsoformGenerator_generate), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IsoformGenerator_generate */
  {__pyx_k_IsoformGenerator_initialize_from, sizeof(__pyx_k_IsoformGenerator_initialize_from), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IsoformGenerator_initialize_from */
  {__pyx_k_IsoformGenerator_initialize_from_2, sizeof(__pyx_k_IsoformGenerator_initialize_from_2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_IsoformGenerator_initialize_from_2 */
  {__pyx_k_K, sizeof(__pyx_k_K), 0, 1, 1}, /* PyObject cname: __pyx_n_u_K */
  {__pyx_k_L, sizeof(__pyx_k_L), 0, 1, 1}, /* PyObject cname: __pyx_n_u_L */
  {__pyx_k_M, sizeof(__pyx_k_M), 0, 1, 1}, /* PyObject cname: __pyx_n_u_M */
  {__pyx_k_ModificationRule, sizeof(__pyx_k_ModificationRule), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ModificationRule */
  {__pyx_k_ModificationRule___reduce_cython, sizeof(__pyx_k_ModificationRule___reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ModificationRule___reduce_cython */
  {__pyx_k_ModificationRule___setstate_cyth, sizeof(__pyx_k_ModificationRule___setstate_cyth), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ModificationRule___setstate_cyth */
  {__pyx_k_ModificationRule_test, sizeof(__pyx_k_ModificationRule_test), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ModificationRule_test */
  {__pyx_k_ModificationTarget, sizeof(__pyx_k_ModificationTarget), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ModificationTarget */
  {__pyx_k_ModificationTarget___reduce_cyth, sizeof(__pyx_k_ModificationTarget___reduce_cyth), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ModificationTarget___reduce_cyth */
  {__pyx_k_ModificationTarget___setstate_cy, sizeof(__pyx_k_ModificationTarget___setstate_cy), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ModificationTarget___setstate_cy */
  {__pyx_k_ModificationTarget_from_string, sizeof(__pyx_k_ModificationTarget_from_string), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ModificationTarget_from_string */
  {__pyx_k_ModificationTarget_test, sizeof(__pyx_k_ModificationTarget_test), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ModificationTarget_test */
  {__pyx_k_N, sizeof(__pyx_k_N), 0, 1, 1}, /* PyObject cname: __pyx_n_u_N */
  {__pyx_k_None, sizeof(__pyx_k_None), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_None */
  {__pyx_k_Not_a_valid_modX_sequence, sizeof(__pyx_k_Not_a_valid_modX_sequence), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Not_a_valid_modX_sequence */
  {__pyx_k_Note_that_Cython_is_deliberately, sizeof(__pyx_k_Note_that_Cython_is_deliberately), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Note_that_Cython_is_deliberately */
  {__pyx_k_OH, sizeof(__pyx_k_OH), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_OH */
  {__pyx_k_P, sizeof(__pyx_k_P), 0, 1, 1}, /* PyObject cname: __pyx_n_u_P */
  {__pyx_k_Parse_a_sequence_string_written, sizeof(__pyx_k_Parse_a_sequence_string_written), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Parse_a_sequence_string_written */
  {__pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_PickleError */
  {__pyx_k_PositionType, sizeof(__pyx_k_PositionType), 0, 1, 1}, /* PyObject cname: __pyx_n_u_PositionType */
  {__pyx_k_Position_already_has_position_ty, sizeof(__pyx_k_Position_already_has_position_ty), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Position_already_has_position_ty */
  {__pyx_k_PyteomicsError, sizeof(__pyx_k_PyteomicsError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_PyteomicsError */
  {__pyx_k_Q, sizeof(__pyx_k_Q), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Q */
  {__pyx_k_R, sizeof(__pyx_k_R), 0, 1, 1}, /* PyObject cname: __pyx_n_u_R */
  {__pyx_k_S, sizeof(__pyx_k_S), 0, 1, 1}, /* PyObject cname: __pyx_n_u_S */
  {__pyx_k_SequencePosition, sizeof(__pyx_k_SequencePosition), 0, 1, 1}, /* PyObject cname: __pyx_n_u_SequencePosition */
  {__pyx_k_SequencePosition___reduce_cython, sizeof(__pyx_k_SequencePosition___reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_SequencePosition___reduce_cython */
  {__pyx_k_SequencePosition___setstate_cyth, sizeof(__pyx_k_SequencePosition___setstate_cyth), 0, 1, 1}, /* PyObject cname: __pyx_n_u_SequencePosition___setstate_cyth */
  {__pyx_k_SequencePosition_add_modificatio, sizeof(__pyx_k_SequencePosition_add_modificatio), 0, 1, 1}, /* PyObject cname: __pyx_n_u_SequencePosition_add_modificatio */
  {__pyx_k_SequencePosition_as_tuple, sizeof(__pyx_k_SequencePosition_as_tuple), 0, 1, 1}, /* PyObject cname: __pyx_n_u_SequencePosition_as_tuple */
  {__pyx_k_SequencePosition_from_iterable, sizeof(__pyx_k_SequencePosition_from_iterable), 0, 1, 1}, /* PyObject cname: __pyx_n_u_SequencePosition_from_iterable */
  {__pyx_k_SequencePosition_from_tuple, sizeof(__pyx_k_SequencePosition_from_tuple), 0, 1, 1}, /* PyObject cname: __pyx_n_u_SequencePosition_from_tuple */
  {__pyx_k_SequencePosition_get_index, sizeof(__pyx_k_SequencePosition_get_index), 0, 1, 1}, /* PyObject cname: __pyx_n_u_SequencePosition_get_index */
  {__pyx_k_SequencePosition_parse, sizeof(__pyx_k_SequencePosition_parse), 0, 1, 1}, /* PyObject cname: __pyx_n_u_SequencePosition_parse */
  {__pyx_k_T, sizeof(__pyx_k_T), 0, 1, 1}, /* PyObject cname: __pyx_n_u_T */
  {__pyx_k_Target_must_be_a_single_Amino_Ac, sizeof(__pyx_k_Target_must_be_a_single_Amino_Ac), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Target_must_be_a_single_Amino_Ac */
  {__pyx_k_Terminal_Modifications_with_Targ, sizeof(__pyx_k_Terminal_Modifications_with_Targ), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Terminal_Modifications_with_Targ */
  {__pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_TypeError */
  {__pyx_k_Unknown_label, sizeof(__pyx_k_Unknown_label), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Unknown_label */
  {__pyx_k_Unsupported_type_of_a_sequence_M, sizeof(__pyx_k_Unsupported_type_of_a_sequence_M), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Unsupported_type_of_a_sequence_M */
  {__pyx_k_Unsupported_type_of_sequence, sizeof(__pyx_k_Unsupported_type_of_sequence), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Unsupported_type_of_sequence */
  {__pyx_k_Unsupported_value_of_format, sizeof(__pyx_k_Unsupported_value_of_format), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_Unsupported_value_of_format */
  {__pyx_k_V, sizeof(__pyx_k_V), 0, 1, 1}, /* PyObject cname: __pyx_n_u_V */
  {__pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_ValueError */
  {__pyx_k_W, sizeof(__pyx_k_W), 0, 1, 1}, /* PyObject cname: __pyx_n_u_W */
  {__pyx_k_Y, sizeof(__pyx_k_Y), 0, 1, 1}, /* PyObject cname: __pyx_n_u_Y */
  {__pyx_k__3, sizeof(__pyx_k__3), 0, 1, 0}, /* PyObject cname: __pyx_kp_u__3 */
  {__pyx_k__5, sizeof(__pyx_k__5), 0, 1, 0}, /* PyObject cname: __pyx_kp_u__5 */
  {__pyx_k__6, sizeof(__pyx_k__6), 0, 1, 0}, /* PyObject cname: __pyx_kp_u__6 */
  {__pyx_k_a, sizeof(__pyx_k_a), 0, 1, 1}, /* PyObject cname: __pyx_n_u_a */
  {__pyx_k_add_modification, sizeof(__pyx_k_add_modification), 0, 1, 1}, /* PyObject cname: __pyx_n_u_add_modification */
  {__pyx_k_add_note, sizeof(__pyx_k_add_note), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_add_note */
  {__pyx_k_allow_unknown_modifications, sizeof(__pyx_k_allow_unknown_modifications), 0, 1, 1}, /* PyObject cname: __pyx_n_u_allow_unknown_modifications */
  {__pyx_k_amino_acid, sizeof(__pyx_k_amino_acid), 0, 1, 1}, /* PyObject cname: __pyx_n_u_amino_acid */
  {__pyx_k_amino_acid_composition, sizeof(__pyx_k_amino_acid_composition), 0, 1, 1}, /* PyObject cname: __pyx_n_u_amino_acid_composition */
  {__pyx_k_amino_acid_composition_line_565, sizeof(__pyx_k_amino_acid_composition_line_565), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_amino_acid_composition_line_565 */
  {__pyx_k_apply_fixed_mods, sizeof(__pyx_k_apply_fixed_mods), 0, 1, 1}, /* PyObject cname: __pyx_n_u_apply_fixed_mods */
  {__pyx_k_as_tuple, sizeof(__pyx_k_as_tuple), 0, 1, 1}, /* PyObject cname: __pyx_n_u_as_tuple */
  {__pyx_k_assignment, sizeof(__pyx_k_assignment), 0, 1, 1}, /* PyObject cname: __pyx_n_u_assignment */
  {__pyx_k_assignments, sizeof(__pyx_k_assignments), 0, 1, 1}, /* PyObject cname: __pyx_n_u_assignments */
  {__pyx_k_assignments_product, sizeof(__pyx_k_assignments_product), 0, 1, 1}, /* PyObject cname: __pyx_n_u_assignments_product */
  {__pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 1, 1}, /* PyObject cname: __pyx_n_u_asyncio_coroutines */
  {__pyx_k_build_site_map, sizeof(__pyx_k_build_site_map), 0, 1, 1}, /* PyObject cname: __pyx_n_u_build_site_map */
  {__pyx_k_c_term, sizeof(__pyx_k_c_term), 0, 1, 1}, /* PyObject cname: __pyx_n_u_c_term */
  {__pyx_k_class_getitem, sizeof(__pyx_k_class_getitem), 0, 1, 1}, /* PyObject cname: __pyx_n_u_class_getitem */
  {__pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 1, 1}, /* PyObject cname: __pyx_n_u_cline_in_traceback */
  {__pyx_k_close, sizeof(__pyx_k_close), 0, 1, 1}, /* PyObject cname: __pyx_n_u_close */
  {__pyx_k_cls, sizeof(__pyx_k_cls), 0, 1, 1}, /* PyObject cname: __pyx_n_u_cls */
  {__pyx_k_coerce_modification_dict, sizeof(__pyx_k_coerce_modification_dict), 0, 1, 1}, /* PyObject cname: __pyx_n_u_coerce_modification_dict */
  {__pyx_k_collections, sizeof(__pyx_k_collections), 0, 1, 1}, /* PyObject cname: __pyx_n_u_collections */
  {__pyx_k_combinations, sizeof(__pyx_k_combinations), 0, 1, 1}, /* PyObject cname: __pyx_n_u_combinations */
  {__pyx_k_compile, sizeof(__pyx_k_compile), 0, 1, 1}, /* PyObject cname: __pyx_n_u_compile */
  {__pyx_k_converter, sizeof(__pyx_k_converter), 0, 1, 1}, /* PyObject cname: __pyx_n_u_converter */
  {__pyx_k_copy_sequence, sizeof(__pyx_k_copy_sequence), 0, 1, 1}, /* PyObject cname: __pyx_n_u_copy_sequence */
  {__pyx_k_cterm, sizeof(__pyx_k_cterm), 0, 1, 1}, /* PyObject cname: __pyx_n_u_cterm */
  {__pyx_k_deque, sizeof(__pyx_k_deque), 0, 1, 1}, /* PyObject cname: __pyx_n_u_deque */
  {__pyx_k_dict, sizeof(__pyx_k_dict), 0, 1, 1}, /* PyObject cname: __pyx_n_u_dict */
  {__pyx_k_dict_2, sizeof(__pyx_k_dict_2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_dict_2 */
  {__pyx_k_disable, sizeof(__pyx_k_disable), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_disable */
  {__pyx_k_enable, sizeof(__pyx_k_enable), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_enable */
  {__pyx_k_entry, sizeof(__pyx_k_entry), 0, 1, 1}, /* PyObject cname: __pyx_n_u_entry */
  {__pyx_k_enum, sizeof(__pyx_k_enum), 0, 1, 1}, /* PyObject cname: __pyx_n_u_enum */
  {__pyx_k_findall, sizeof(__pyx_k_findall), 0, 1, 1}, /* PyObject cname: __pyx_n_u_findall */
  {__pyx_k_fixed_mods, sizeof(__pyx_k_fixed_mods), 0, 1, 1}, /* PyObject cname: __pyx_n_u_fixed_mods */
  {__pyx_k_form, sizeof(__pyx_k_form), 0, 1, 1}, /* PyObject cname: __pyx_n_u_form */
  {__pyx_k_format, sizeof(__pyx_k_format), 0, 1, 1}, /* PyObject cname: __pyx_n_u_format */
  {__pyx_k_format_2, sizeof(__pyx_k_format_2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_format_2 */
  {__pyx_k_from_iterable, sizeof(__pyx_k_from_iterable), 0, 1, 1}, /* PyObject cname: __pyx_n_u_from_iterable */
  {__pyx_k_from_string, sizeof(__pyx_k_from_string), 0, 1, 1}, /* PyObject cname: __pyx_n_u_from_string */
  {__pyx_k_from_tuple, sizeof(__pyx_k_from_tuple), 0, 1, 1}, /* PyObject cname: __pyx_n_u_from_tuple */
  {__pyx_k_func, sizeof(__pyx_k_func), 0, 1, 1}, /* PyObject cname: __pyx_n_u_func */
  {__pyx_k_gc, sizeof(__pyx_k_gc), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_gc */
  {__pyx_k_gen, sizeof(__pyx_k_gen), 0, 1, 1}, /* PyObject cname: __pyx_n_u_gen */
  {__pyx_k_generate, sizeof(__pyx_k_generate), 0, 1, 1}, /* PyObject cname: __pyx_n_u_generate */
  {__pyx_k_get, sizeof(__pyx_k_get), 0, 1, 1}, /* PyObject cname: __pyx_n_u_get */
  {__pyx_k_get_index, sizeof(__pyx_k_get_index), 0, 1, 1}, /* PyObject cname: __pyx_n_u_get_index */
  {__pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 1, 1}, /* PyObject cname: __pyx_n_u_getstate */
  {__pyx_k_groups, sizeof(__pyx_k_groups), 0, 1, 1}, /* PyObject cname: __pyx_n_u_groups */
  {__pyx_k_i, sizeof(__pyx_k_i), 0, 1, 1}, /* PyObject cname: __pyx_n_u_i */
  {__pyx_k_include_unmodified, sizeof(__pyx_k_include_unmodified), 0, 1, 1}, /* PyObject cname: __pyx_n_u_include_unmodified */
  {__pyx_k_indices, sizeof(__pyx_k_indices), 0, 1, 1}, /* PyObject cname: __pyx_n_u_indices */
  {__pyx_k_initialize_from_iterable, sizeof(__pyx_k_initialize_from_iterable), 0, 1, 1}, /* PyObject cname: __pyx_n_u_initialize_from_iterable */
  {__pyx_k_initialize_from_str, sizeof(__pyx_k_initialize_from_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_initialize_from_str */
  {__pyx_k_initializing, sizeof(__pyx_k_initializing), 0, 1, 1}, /* PyObject cname: __pyx_n_u_initializing */
  {__pyx_k_internal, sizeof(__pyx_k_internal), 0, 1, 1}, /* PyObject cname: __pyx_n_u_internal */
  {__pyx_k_is_coroutine, sizeof(__pyx_k_is_coroutine), 0, 1, 1}, /* PyObject cname: __pyx_n_u_is_coroutine */
  {__pyx_k_isenabled, sizeof(__pyx_k_isenabled), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_isenabled */
  {__pyx_k_isoform_generator, sizeof(__pyx_k_isoform_generator), 0, 1, 1}, /* PyObject cname: __pyx_n_u_isoform_generator */
  {__pyx_k_isoforms, sizeof(__pyx_k_isoforms), 0, 1, 1}, /* PyObject cname: __pyx_n_u_isoforms */
  {__pyx_k_it, sizeof(__pyx_k_it), 0, 1, 1}, /* PyObject cname: __pyx_n_u_it */
  {__pyx_k_items, sizeof(__pyx_k_items), 0, 1, 1}, /* PyObject cname: __pyx_n_u_items */
  {__pyx_k_iterable, sizeof(__pyx_k_iterable), 0, 1, 1}, /* PyObject cname: __pyx_n_u_iterable */
  {__pyx_k_itertools, sizeof(__pyx_k_itertools), 0, 1, 1}, /* PyObject cname: __pyx_n_u_itertools */
  {__pyx_k_j, sizeof(__pyx_k_j), 0, 1, 1}, /* PyObject cname: __pyx_n_u_j */
  {__pyx_k_k, sizeof(__pyx_k_k), 0, 1, 1}, /* PyObject cname: __pyx_n_u_k */
  {__pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 1, 1}, /* PyObject cname: __pyx_n_u_kwargs */
  {__pyx_k_label, sizeof(__pyx_k_label), 0, 1, 1}, /* PyObject cname: __pyx_n_u_label */
  {__pyx_k_labels, sizeof(__pyx_k_labels), 0, 1, 1}, /* PyObject cname: __pyx_n_u_labels */
  {__pyx_k_last_entry, sizeof(__pyx_k_last_entry), 0, 1, 1}, /* PyObject cname: __pyx_n_u_last_entry */
  {__pyx_k_length, sizeof(__pyx_k_length), 0, 1, 1}, /* PyObject cname: __pyx_n_u_length */
  {__pyx_k_length_line_118, sizeof(__pyx_k_length_line_118), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_length_line_118 */
  {__pyx_k_main, sizeof(__pyx_k_main), 0, 1, 1}, /* PyObject cname: __pyx_n_u_main */
  {__pyx_k_match, sizeof(__pyx_k_match), 0, 1, 1}, /* PyObject cname: __pyx_n_u_match */
  {__pyx_k_max_mods, sizeof(__pyx_k_max_mods), 0, 1, 1}, /* PyObject cname: __pyx_n_u_max_mods */
  {__pyx_k_member_names, sizeof(__pyx_k_member_names), 0, 1, 1}, /* PyObject cname: __pyx_n_u_member_names */
  {__pyx_k_members, sizeof(__pyx_k_members), 0, 1, 1}, /* PyObject cname: __pyx_n_u_members */
  {__pyx_k_memoize, sizeof(__pyx_k_memoize), 0, 1, 1}, /* PyObject cname: __pyx_n_u_memoize */
  {__pyx_k_mod, sizeof(__pyx_k_mod), 0, 1, 1}, /* PyObject cname: __pyx_n_u_mod */
  {__pyx_k_modification, sizeof(__pyx_k_modification), 0, 1, 1}, /* PyObject cname: __pyx_n_u_modification */
  {__pyx_k_module, sizeof(__pyx_k_module), 0, 1, 1}, /* PyObject cname: __pyx_n_u_module */
  {__pyx_k_module_2, sizeof(__pyx_k_module_2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_module_2 */
  {__pyx_k_n, sizeof(__pyx_k_n), 0, 1, 1}, /* PyObject cname: __pyx_n_u_n */
  {__pyx_k_n_mods, sizeof(__pyx_k_n_mods), 0, 1, 1}, /* PyObject cname: __pyx_n_u_n_mods */
  {__pyx_k_n_term, sizeof(__pyx_k_n_term), 0, 1, 1}, /* PyObject cname: __pyx_n_u_n_term */
  {__pyx_k_name, sizeof(__pyx_k_name), 0, 1, 1}, /* PyObject cname: __pyx_n_u_name */
  {__pyx_k_new, sizeof(__pyx_k_new), 0, 1, 1}, /* PyObject cname: __pyx_n_u_new */
  {__pyx_k_next, sizeof(__pyx_k_next), 0, 1, 1}, /* PyObject cname: __pyx_n_u_next */
  {__pyx_k_nterm, sizeof(__pyx_k_nterm), 0, 1, 1}, /* PyObject cname: __pyx_n_u_nterm */
  {__pyx_k_obj, sizeof(__pyx_k_obj), 0, 1, 1}, /* PyObject cname: __pyx_n_u_obj */
  {__pyx_k_override, sizeof(__pyx_k_override), 0, 1, 1}, /* PyObject cname: __pyx_n_u_override */
  {__pyx_k_parse, sizeof(__pyx_k_parse), 0, 1, 1}, /* PyObject cname: __pyx_n_u_parse */
  {__pyx_k_parse_line_211, sizeof(__pyx_k_parse_line_211), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_parse_line_211 */
  {__pyx_k_parsed_sequence, sizeof(__pyx_k_parsed_sequence), 0, 1, 1}, /* PyObject cname: __pyx_n_u_parsed_sequence */
  {__pyx_k_persist, sizeof(__pyx_k_persist), 0, 1, 1}, /* PyObject cname: __pyx_n_u_persist */
  {__pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pickle */
  {__pyx_k_pindices, sizeof(__pyx_k_pindices), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pindices */
  {__pyx_k_pop, sizeof(__pyx_k_pop), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pop */
  {__pyx_k_position, sizeof(__pyx_k_position), 0, 1, 1}, /* PyObject cname: __pyx_n_u_position */
  {__pyx_k_position_type, sizeof(__pyx_k_position_type), 0, 1, 1}, /* PyObject cname: __pyx_n_u_position_type */
  {__pyx_k_product, sizeof(__pyx_k_product), 0, 1, 1}, /* PyObject cname: __pyx_n_u_product */
  {__pyx_k_pyteomics_auxiliary, sizeof(__pyx_k_pyteomics_auxiliary), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyteomics_auxiliary */
  {__pyx_k_pyteomics_cparser, sizeof(__pyx_k_pyteomics_cparser), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyteomics_cparser */
  {__pyx_k_pyteomics_cparser, sizeof(__pyx_k_pyteomics_cparser), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_pyteomics_cparser */
  {__pyx_k_pyteomics_cparser_pyx, sizeof(__pyx_k_pyteomics_cparser_pyx), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_pyteomics_cparser_pyx */
  {__pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_PickleError */
  {__pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_checksum */
  {__pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_result */
  {__pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_state */
  {__pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_type */
  {__pyx_k_pyx_unpickle_IsoformGenerator, sizeof(__pyx_k_pyx_unpickle_IsoformGenerator), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_unpickle_IsoformGenerator */
  {__pyx_k_pyx_unpickle_ModificationRule, sizeof(__pyx_k_pyx_unpickle_ModificationRule), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_unpickle_ModificationRule */
  {__pyx_k_pyx_unpickle_ModificationTarge, sizeof(__pyx_k_pyx_unpickle_ModificationTarge), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_unpickle_ModificationTarge */
  {__pyx_k_pyx_unpickle_SequencePosition, sizeof(__pyx_k_pyx_unpickle_SequencePosition), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_unpickle_SequencePosition */
  {__pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 1, 1}, /* PyObject cname: __pyx_n_u_pyx_vtable */
  {__pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 1, 1}, /* PyObject cname: __pyx_n_u_qualname */
  {__pyx_k_range, sizeof(__pyx_k_range), 0, 1, 1}, /* PyObject cname: __pyx_n_u_range */
  {__pyx_k_re, sizeof(__pyx_k_re), 0, 1, 1}, /* PyObject cname: __pyx_n_u_re */
  {__pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 1, 1}, /* PyObject cname: __pyx_n_u_reduce */
  {__pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_reduce_cython */
  {__pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 1, 1}, /* PyObject cname: __pyx_n_u_reduce_ex */
  {__pyx_k_result, sizeof(__pyx_k_result), 0, 1, 1}, /* PyObject cname: __pyx_n_u_result */
  {__pyx_k_second_type_provided, sizeof(__pyx_k_second_type_provided), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_second_type_provided */
  {__pyx_k_self, sizeof(__pyx_k_self), 0, 1, 1}, /* PyObject cname: __pyx_n_u_self */
  {__pyx_k_self___class_____name___self_mo, sizeof(__pyx_k_self___class_____name___self_mo), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_self___class_____name___self_mo */
  {__pyx_k_self___class_____name___self_ta, sizeof(__pyx_k_self___class_____name___self_ta), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_self___class_____name___self_ta */
  {__pyx_k_send, sizeof(__pyx_k_send), 0, 1, 1}, /* PyObject cname: __pyx_n_u_send */
  {__pyx_k_sequence, sizeof(__pyx_k_sequence), 0, 1, 1}, /* PyObject cname: __pyx_n_u_sequence */
  {__pyx_k_sequence_spec, sizeof(__pyx_k_sequence_spec), 0, 1, 1}, /* PyObject cname: __pyx_n_u_sequence_spec */
  {__pyx_k_sequence_tostring, sizeof(__pyx_k_sequence_tostring), 0, 1, 1}, /* PyObject cname: __pyx_n_u_sequence_tostring */
  {__pyx_k_set_name, sizeof(__pyx_k_set_name), 0, 1, 1}, /* PyObject cname: __pyx_n_u_set_name */
  {__pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 1, 1}, /* PyObject cname: __pyx_n_u_setstate */
  {__pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 1, 1}, /* PyObject cname: __pyx_n_u_setstate_cython */
  {__pyx_k_show_unmodified_termini, sizeof(__pyx_k_show_unmodified_termini), 0, 1, 1}, /* PyObject cname: __pyx_n_u_show_unmodified_termini */
  {__pyx_k_size, sizeof(__pyx_k_size), 0, 1, 1}, /* PyObject cname: __pyx_n_u_size */
  {__pyx_k_source, sizeof(__pyx_k_source), 0, 1, 1}, /* PyObject cname: __pyx_n_u_source */
  {__pyx_k_spec, sizeof(__pyx_k_spec), 0, 1, 1}, /* PyObject cname: __pyx_n_u_spec */
  {__pyx_k_split, sizeof(__pyx_k_split), 0, 1, 1}, /* PyObject cname: __pyx_n_u_split */
  {__pyx_k_split_label, sizeof(__pyx_k_split_label), 0, 1, 1}, /* PyObject cname: __pyx_n_u_split_label */
  {__pyx_k_state, sizeof(__pyx_k_state), 0, 1, 1}, /* PyObject cname: __pyx_n_u_state */
  {__pyx_k_staticmethod, sizeof(__pyx_k_staticmethod), 0, 1, 1}, /* PyObject cname: __pyx_n_u_staticmethod */
  {__pyx_k_str, sizeof(__pyx_k_str), 0, 1, 1}, /* PyObject cname: __pyx_n_u_str */
  {__pyx_k_string, sizeof(__pyx_k_string), 0, 1, 1}, /* PyObject cname: __pyx_n_u_string */
  {__pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 1, 0}, /* PyObject cname: __pyx_kp_u_stringsource */
  {__pyx_k_subassignments, sizeof(__pyx_k_subassignments), 0, 1, 1}, /* PyObject cname: __pyx_n_u_subassignments */
  {__pyx_k_subindices, sizeof(__pyx_k_subindices), 0, 1, 1}, /* PyObject cname: __pyx_n_u_subindices */
  {__pyx_k_sum, sizeof(__pyx_k_sum), 0, 1, 1}, /* PyObject cname: __pyx_n_u_sum */
  {__pyx_k_target, sizeof(__pyx_k_target), 0, 1, 1}, /* PyObject cname: __pyx_n_u_target */
  {__pyx_k_targets, sizeof(__pyx_k_targets), 0, 1, 1}, /* PyObject cname: __pyx_n_u_targets */
  {__pyx_k_temp1, sizeof(__pyx_k_temp1), 0, 1, 1}, /* PyObject cname: __pyx_n_u_temp1 */
  {__pyx_k_temp2, sizeof(__pyx_k_temp2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_temp2 */
  {__pyx_k_term_aa, sizeof(__pyx_k_term_aa), 0, 1, 1}, /* PyObject cname: __pyx_n_u_term_aa */
  {__pyx_k_terminal, sizeof(__pyx_k_terminal), 0, 1, 1}, /* PyObject cname: __pyx_n_u_terminal */
  {__pyx_k_test, sizeof(__pyx_k_test), 0, 1, 1}, /* PyObject cname: __pyx_n_u_test */
  {__pyx_k_test_2, sizeof(__pyx_k_test_2), 0, 1, 1}, /* PyObject cname: __pyx_n_u_test_2 */
  {__pyx_k_throw, sizeof(__pyx_k_throw), 0, 1, 1}, /* PyObject cname: __pyx_n_u_throw */
  {__pyx_k_tokens, sizeof(__pyx_k_tokens), 0, 1, 1}, /* PyObject cname: __pyx_n_u_tokens */
  {__pyx_k_tostring, sizeof(__pyx_k_tostring), 0, 1, 1}, /* PyObject cname: __pyx_n_u_tostring */
  {__pyx_k_universal, sizeof(__pyx_k_universal), 0, 1, 1}, /* PyObject cname: __pyx_n_u_universal */
  {__pyx_k_update, sizeof(__pyx_k_update), 0, 1, 1}, /* PyObject cname: __pyx_n_u_update */
  {__pyx_k_use_setstate, sizeof(__pyx_k_use_setstate), 0, 1, 1}, /* PyObject cname: __pyx_n_u_use_setstate */
  {__pyx_k_value, sizeof(__pyx_k_value), 0, 1, 1}, /* PyObject cname: __pyx_n_u_value */
  {__pyx_k_variable_mod_sites, sizeof(__pyx_k_variable_mod_sites), 0, 1, 1}, /* PyObject cname: __pyx_n_u_variable_mod_sites */
  {__pyx_k_variable_mods, sizeof(__pyx_k_variable_mods), 0, 1, 1}, /* PyObject cname: __pyx_n_u_variable_mods */
  {0, 0, 0, 0, 0}
};
/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry const *t, PyObject **target, const char* const* encoding_names);

/* #### Code section: cached_builtins ### */

static int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_builtin_staticmethod = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_staticmethod); if (!__pyx_builtin_staticmethod) __PYX_ERR(0, 1274, __pyx_L1_error)
  __pyx_builtin_sum = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_sum); if (!__pyx_builtin_sum) __PYX_ERR(0, 154, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_range); if (!__pyx_builtin_range) __PYX_ERR(0, 201, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 495, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(0, 734, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 831, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xe2ca0e1, 0xf7d828e, 0x96edc61):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe2ca0e1, 0xf7d828e, 0x96edc61) = (amino_acid, modification, position_type, terminal))" % __pyx_checksum
*/
  __pyx_mstate_global->__pyx_tuple[0] = PyTuple_Pack(3, __pyx_mstate_global->__pyx_int_237805793, __pyx_mstate_global->__pyx_int_259883662, __pyx_mstate_global->__pyx_int_158260321); if (unlikely(!__pyx_mstate_global->__pyx_tuple[0])) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[0]);
  __pyx_mstate_global->__pyx_tuple[1] = PyTuple_Pack(3, __pyx_mstate_global->__pyx_int_10332774, __pyx_mstate_global->__pyx_int_18063223, __pyx_mstate_global->__pyx_int_50086419); if (unlikely(!__pyx_mstate_global->__pyx_tuple[1])) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[1]);
  __pyx_mstate_global->__pyx_tuple[2] = PyTuple_Pack(3, __pyx_mstate_global->__pyx_int_214727337, __pyx_mstate_global->__pyx_int_243604066, __pyx_mstate_global->__pyx_int_209977177); if (unlikely(!__pyx_mstate_global->__pyx_tuple[2])) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[2]);
  __pyx_mstate_global->__pyx_tuple[3] = PyTuple_Pack(3, __pyx_mstate_global->__pyx_int_16113035, __pyx_mstate_global->__pyx_int_169505322, __pyx_mstate_global->__pyx_int_94445299); if (unlikely(!__pyx_mstate_global->__pyx_tuple[3])) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[3]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[3]);

  /* "EnumType":29
 *     ('universal', __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(universal)),
 * 
 * ], module=globals().get("__module__", 'pyteomics.cparser'))             # <<<<<<<<<<<<<<
 * 
 * if PY_VERSION_HEX >= 0x030B0000:
*/
  __pyx_mstate_global->__pyx_tuple[4] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_n_u_module_2, __pyx_mstate_global->__pyx_kp_u_pyteomics_cparser); if (unlikely(!__pyx_mstate_global->__pyx_tuple[4])) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[4]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[4]);

  /* "pyteomics/cparser.pyx":211
 * 
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,             # <<<<<<<<<<<<<<
 *                  bint allow_unknown_modifications=0, object labels=None):
 *     """Parse a sequence string written in modX notation into a list of
*/
  __pyx_mstate_global->__pyx_tuple[5] = PyTuple_Pack(4, __pyx_mstate_global->__pyx_int_0, __pyx_mstate_global->__pyx_int_0, __pyx_mstate_global->__pyx_int_0, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[5])) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[5]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[5]);

  /* "pyteomics/cparser.pyx":435
 * 
 * 
 * cpdef str tostring(object parsed_sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     """Create a string from a parsed sequence.
 * 
*/
  __pyx_mstate_global->__pyx_tuple[6] = PyTuple_Pack(1, Py_True); if (unlikely(!__pyx_mstate_global->__pyx_tuple[6])) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[6]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[6]);

  /* "pyteomics/cparser.pyx":906
 * 
 * 
 * cpdef str _sequence_tostring(list sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     cdef:
 *         size_t i, n, acc, j
*/
  __pyx_mstate_global->__pyx_tuple[7] = PyTuple_Pack(1, Py_True); if (unlikely(!__pyx_mstate_global->__pyx_tuple[7])) __PYX_ERR(0, 906, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[7]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[7]);

  /* "pyteomics/cparser.pyx":1067
 *         return sequence
 * 
 *     cpdef dict build_site_map(self, list sequence, bint include_unmodified=True):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
*/
  __pyx_mstate_global->__pyx_tuple[8] = PyTuple_Pack(1, Py_True); if (unlikely(!__pyx_mstate_global->__pyx_tuple[8])) __PYX_ERR(0, 1067, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[8]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[8]);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.method_name = &__pyx_mstate->__pyx_n_u_get;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  if (__Pyx_InitStrings(__pyx_string_tab, __pyx_mstate->__pyx_string_tab, __pyx_string_tab_encodings) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_mstate->__pyx_int_0 = PyLong_FromLong(0); if (unlikely(!__pyx_mstate->__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_1 = PyLong_FromLong(1); if (unlikely(!__pyx_mstate->__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_10332774 = PyLong_FromLong(10332774L); if (unlikely(!__pyx_mstate->__pyx_int_10332774)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_16113035 = PyLong_FromLong(16113035L); if (unlikely(!__pyx_mstate->__pyx_int_16113035)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_18063223 = PyLong_FromLong(18063223L); if (unlikely(!__pyx_mstate->__pyx_int_18063223)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_50086419 = PyLong_FromLong(50086419L); if (unlikely(!__pyx_mstate->__pyx_int_50086419)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_94445299 = PyLong_FromLong(94445299L); if (unlikely(!__pyx_mstate->__pyx_int_94445299)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_158260321 = PyLong_FromLong(158260321L); if (unlikely(!__pyx_mstate->__pyx_int_158260321)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_169505322 = PyLong_FromLong(169505322L); if (unlikely(!__pyx_mstate->__pyx_int_169505322)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_209977177 = PyLong_FromLong(209977177L); if (unlikely(!__pyx_mstate->__pyx_int_209977177)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_214727337 = PyLong_FromLong(214727337L); if (unlikely(!__pyx_mstate->__pyx_int_214727337)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_237805793 = PyLong_FromLong(237805793L); if (unlikely(!__pyx_mstate->__pyx_int_237805793)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_243604066 = PyLong_FromLong(243604066L); if (unlikely(!__pyx_mstate->__pyx_int_243604066)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_259883662 = PyLong_FromLong(259883662L); if (unlikely(!__pyx_mstate->__pyx_int_259883662)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_int_neg_1 = PyLong_FromLong(-1); if (unlikely(!__pyx_mstate->__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_codeobjects ### */
\
        typedef struct {
            unsigned int argcount : 3;
            unsigned int num_posonly_args : 1;
            unsigned int num_kwonly_args : 1;
            unsigned int nlocals : 5;
            unsigned int flags : 10;
            unsigned int first_line : 11;
            unsigned int line_table_length : 15;
        } __Pyx_PyCode_New_function_description;
/* NewCodeObj.proto */
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        const char *line_table,
        PyObject *tuple_dedup_map
);


static int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate) {
  PyObject* tuple_dedup_map = PyDict_New();
  if (unlikely(!tuple_dedup_map)) return -1;
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 19, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 1105, 2};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_sequence_spec, __pyx_mstate->__pyx_n_u_sequence, __pyx_mstate->__pyx_n_u_variable_mod_sites, __pyx_mstate->__pyx_n_u_indices, __pyx_mstate->__pyx_n_u_assignments, __pyx_mstate->__pyx_n_u_assignment, __pyx_mstate->__pyx_n_u_subindices, __pyx_mstate->__pyx_n_u_subassignments, __pyx_mstate->__pyx_n_u_temp1, __pyx_mstate->__pyx_n_u_temp2, __pyx_mstate->__pyx_n_u_assignments_product, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_n, __pyx_mstate->__pyx_n_u_j, __pyx_mstate->__pyx_n_u_k, __pyx_mstate->__pyx_n_u_n_mods, __pyx_mstate->__pyx_n_u_pindices, __pyx_mstate->__pyx_n_u_a};
    __pyx_mstate_global->__pyx_codeobj_tab[0] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_generate, __pyx_k_A_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[0])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 12, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_VARKEYWORDS|CO_GENERATOR), 1308, 2};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_sequence, __pyx_mstate->__pyx_n_u_kwargs, __pyx_mstate->__pyx_n_u_isoform_generator, __pyx_mstate->__pyx_n_u_format_2, __pyx_mstate->__pyx_n_u_gen, __pyx_mstate->__pyx_n_u_variable_mods, __pyx_mstate->__pyx_n_u_fixed_mods, __pyx_mstate->__pyx_n_u_labels, __pyx_mstate->__pyx_n_u_max_mods, __pyx_mstate->__pyx_n_u_override, __pyx_mstate->__pyx_n_u_show_unmodified_termini, __pyx_mstate->__pyx_n_u_form};
    __pyx_mstate_global->__pyx_codeobj_tab[1] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_isoforms, __pyx_k__7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[1])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 118, 191};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_sequence, __pyx_mstate->__pyx_n_u_show_unmodified_termini, __pyx_mstate->__pyx_n_u_split, __pyx_mstate->__pyx_n_u_allow_unknown_modifications, __pyx_mstate->__pyx_n_u_labels};
    __pyx_mstate_global->__pyx_codeobj_tab[2] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_length, __pyx_k_1B_t_WA_z_E_Jaz_Qj_e1Ja_H_a_a_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[2])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 170, 98};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_label};
    __pyx_mstate_global->__pyx_codeobj_tab[3] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_split_label, __pyx_k_7_AZq_s_nA_A_1_q_vQ_QgV1_t1_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[3])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 211, 831};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_sequence, __pyx_mstate->__pyx_n_u_show_unmodified_termini, __pyx_mstate->__pyx_n_u_split, __pyx_mstate->__pyx_n_u_allow_unknown_modifications, __pyx_mstate->__pyx_n_u_labels};
    __pyx_mstate_global->__pyx_codeobj_tab[4] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_parse, __pyx_k_N_2C1P_wc_a_a_7y_UV_vS_nA_2Q_7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[4])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 435, 878};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_parsed_sequence, __pyx_mstate->__pyx_n_u_show_unmodified_termini};
    __pyx_mstate_global->__pyx_codeobj_tab[5] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_tostring, __pyx_k_H_O1_5Q_U_1_0_QgQ_AQ_U_1_awa_1I, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[5])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 565, 397};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_sequence, __pyx_mstate->__pyx_n_u_show_unmodified_termini, __pyx_mstate->__pyx_n_u_term_aa, __pyx_mstate->__pyx_n_u_allow_unknown_modifications, __pyx_mstate->__pyx_n_u_labels};
    __pyx_mstate_global->__pyx_codeobj_tab[6] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_amino_acid_composition, __pyx_k_PP_aCTTU_wc_a_z_A_q_Z_Na_1Ja_9D, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[6])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 728, 209};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_i};
    __pyx_mstate_global->__pyx_codeobj_tab[7] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_get_index, __pyx_k_A_t9A_7_2S_AQ_r_uBb_j_Rq_4_oQ_r, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[7])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 772, 10};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_mod};
    __pyx_mstate_global->__pyx_codeobj_tab[8] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_add_modification, __pyx_k_A_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[8])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 781, 174};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[9] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_as_tuple, __pyx_k_A_t9A_AQ_A_4_oQ_Qd_AXS_A_4_1_a_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[9])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 10, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 811, 185};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_cls, __pyx_mstate->__pyx_n_u_source, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_entry, __pyx_mstate->__pyx_n_u_last_entry, __pyx_mstate->__pyx_n_u_terminal, __pyx_mstate->__pyx_n_u_amino_acid, __pyx_mstate->__pyx_n_u_modification, __pyx_mstate->__pyx_n_u_position_type};
    __pyx_mstate_global->__pyx_codeobj_tab[10] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_from_tuple, __pyx_k_A_A_q_1A_Q_1_E_aq_E_uIQa_A_7q_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[10])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_VARKEYWORDS), 850, 82};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_cls, __pyx_mstate->__pyx_n_u_sequence, __pyx_mstate->__pyx_n_u_kwargs, __pyx_mstate->__pyx_n_u_n, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_tokens, __pyx_mstate->__pyx_n_u_result, __pyx_mstate->__pyx_n_u_converter};
    __pyx_mstate_global->__pyx_codeobj_tab[11] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_parse, __pyx_k_A_Cq_az_9_vXQ_Kq_E_aq_1G_81_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[11])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 864, 70};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_cls, __pyx_mstate->__pyx_n_u_iterable, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_n, __pyx_mstate->__pyx_n_u_result, __pyx_mstate->__pyx_n_u_converter, __pyx_mstate->__pyx_n_u_obj};
    __pyx_mstate_global->__pyx_codeobj_tab[12] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_from_iterable, __pyx_k_A_Cq_G1_z_q_gR_1_U_gQiq_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[12])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 131};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[13] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_k_T_d_5ET_G1F_a_vWA_q_t_we3d_uTWW, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[13])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16, 11};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[14] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_k_avQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[14])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 906, 491};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_sequence, __pyx_mstate->__pyx_n_u_show_unmodified_termini};
    __pyx_mstate_global->__pyx_codeobj_tab[15] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_sequence_tostring, __pyx_k_I_1A_U_1_1_3j_q_as_Qa_1_3nG1_as, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[15])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 972, 79};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_sequence};
    __pyx_mstate_global->__pyx_codeobj_tab[16] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_copy_sequence, __pyx_k_1A_Zq_U_1__AZq_85_q_1_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[16])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1026, 54};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_source};
    __pyx_mstate_global->__pyx_codeobj_tab[17] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_coerce_modification_dict, __pyx_k_A_G_fF_1G4q_87_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[17])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1038, 44};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_sequence};
    __pyx_mstate_global->__pyx_codeobj_tab[18] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_initialize_from_str, __pyx_k_A_XRt1D_L_az_t1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[18])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1043, 23};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_sequence};
    __pyx_mstate_global->__pyx_codeobj_tab[19] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_initialize_from_iterable, __pyx_k_A_L_aq_t1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[19])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1047, 113};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_sequence};
    __pyx_mstate_global->__pyx_codeobj_tab[20] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_apply_fixed_mods, __pyx_k_A_Kq_E_aq_1_Qd_q_1F_1_4uAQ_t8_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[20])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1067, 255};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_sequence, __pyx_mstate->__pyx_n_u_include_unmodified};
    __pyx_mstate_global->__pyx_codeobj_tab[21] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_build_site_map, __pyx_k_K1_q_Kq_E_aq_1_Qd_2_3avQa_4uAQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[21])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 289};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[22] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_k_T_d_4_d_TQeeiittx_y_C_C_G_G_R_R, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[22])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16, 11};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[23] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_k_avQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[23])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1221, 63};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_position};
    __pyx_mstate_global->__pyx_codeobj_tab[24] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_test, __pyx_k_A_Kq_A_E_aq_O1D_q_Qa_q_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[24])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 107};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[25] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_k_T_A_G1F_a_vWA_q_t_S_IWA_q_1_Qg, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[25])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16, 11};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[26] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_k_avQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[26])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1250, 81};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_position};
    __pyx_mstate_global->__pyx_codeobj_tab[27] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_test, __pyx_k_A_4_oQ_1_4_oQ_x_c_Q_q_4xwa_t83ha, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[27])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1274, 11};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_string};
    __pyx_mstate_global->__pyx_codeobj_tab[28] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyteomics_cparser_pyx, __pyx_mstate->__pyx_n_u_from_string, __pyx_k_A_aq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[28])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 99};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[29] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_k_T_Q_G1F_a_vWA_q_t87_q_34q_QR_34, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[29])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16, 11};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[30] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_k_0_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[30])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 89};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_PickleError, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[31] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_pyx_unpickle_SequencePosition, __pyx_k_hk_A_1_____81A_7_2_3FnTU_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[31])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 89};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_PickleError, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[32] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_pyx_unpickle_IsoformGenerator, __pyx_k_hk_A_1_n_n_p_p_q_81A_7_2_3FnTU, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[32])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 89};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_PickleError, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[33] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_pyx_unpickle_ModificationRule, __pyx_k_hk_A_1_A_A_C_C_D_81A_7_2_3FnTU, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[33])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1, 89};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_PickleError, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[34] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_pyx_unpickle_ModificationTarge, __pyx_k_hk_A_1_A_A_C_C_D_XQa_7_4A5J_XY, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[34])) goto bad;
  }
  Py_DECREF(tuple_dedup_map);
  return 0;
  bad:
  Py_DECREF(tuple_dedup_map);
  return -1;
}
/* #### Code section: init_globals ### */

static int __Pyx_InitGlobals(void) {
  /* PythonCompatibility.init */
  if (likely(__Pyx_init_co_variables() == 0)); else

if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CachedMethodType.init */
  #if CYTHON_COMPILING_IN_LIMITED_API
{
    PyObject *typesModule=NULL;
    typesModule = PyImport_ImportModule("types");
    if (typesModule) {
        __pyx_mstate_global->__Pyx_CachedMethodType = PyObject_GetAttrString(typesModule, "MethodType");
        Py_DECREF(typesModule);
    }
} // error handling follows
#endif

if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
            "name '%U' is not defined", name);
    }
    return result;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        if (!PyErr_Occurred())
            PyErr_SetNone(PyExc_NameError);
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
    PyErr_Clear();
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    if (unlikely(__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, name, &result) == -1)) PyErr_Clear();
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return result;
    }
#else
    result = _PyDict_GetItem_KnownHash(__pyx_mstate_global->__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL && !CYTHON_VECTORCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject *const *args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject *const *args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
    PyObject *kwdefs;
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object"))) {
        return NULL;
    }
    if (
            co->co_kwonlyargcount == 0 &&
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) != (0)) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
#if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x03090000
    #define __Pyx_PyVectorcall_Function(callable) _PyVectorcall_Function(callable)
  #elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE vectorcallfunc __Pyx_PyVectorcall_Function(PyObject *callable) {
    PyTypeObject *tp = Py_TYPE(callable);
    #if defined(__Pyx_CyFunction_USED)
    if (__Pyx_CyFunction_CheckExact(callable)) {
        return __Pyx_CyFunction_func_vectorcall(callable);
    }
    #endif
    if (!PyType_HasFeature(tp, Py_TPFLAGS_HAVE_VECTORCALL)) {
        return NULL;
    }
    assert(PyCallable_Check(callable));
    Py_ssize_t offset = tp->tp_vectorcall_offset;
    assert(offset > 0);
    vectorcallfunc ptr;
    memcpy(&ptr, (char *) callable + offset, sizeof(ptr));
    return ptr;
}
  #else
    #define __Pyx_PyVectorcall_Function(callable) PyVectorcall_Function(callable)
  #endif
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject *const *args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    #if PY_VERSION_HEX < 0x030800B1
    #if CYTHON_FAST_PYCCALL
    if (PyCFunction_Check(func)) {
        if (kwargs) {
            return _PyCFunction_FastCallDict(func, args, nargs, kwargs);
        } else {
            return _PyCFunction_FastCallKeywords(func, args, nargs, NULL);
        }
    }
    if (!kwargs && __Pyx_IS_TYPE(func, &PyMethodDescr_Type)) {
        return _PyMethodDescr_FastCallKeywords(func, args, nargs, NULL);
    }
    #endif
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs);
    }
    #endif
    #endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
        vectorcallfunc f = __Pyx_PyVectorcall_Function(func);
        if (f) {
            return f(func, args, _nargs, NULL);
        }
        #elif defined(__Pyx_CyFunction_USED) && CYTHON_BACKPORT_VECTORCALL
        if (__Pyx_CyFunction_CheckExact(func)) {
            __pyx_vectorcallfunc f = __Pyx_CyFunction_func_vectorcall(func);
            if (f) return f(func, args, _nargs, NULL);
        }
        #elif CYTHON_COMPILING_IN_LIMITED_API && CYTHON_VECTORCALL
        return PyObject_Vectorcall(func, args, _nargs, NULL);
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_mstate_global->__pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* unicode_tailmatch */
static int __Pyx_PyUnicode_TailmatchTuple(PyObject* s, PyObject* substrings,
                                          Py_ssize_t start, Py_ssize_t end, int direction) {
    Py_ssize_t i, count = __Pyx_PyTuple_GET_SIZE(substrings);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(count < 0)) return -1;
    #endif
    for (i = 0; i < count; i++) {
        Py_ssize_t result;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        result = PyUnicode_Tailmatch(s, PyTuple_GET_ITEM(substrings, i),
                                     start, end, direction);
#else
        PyObject* sub = __Pyx_PySequence_ITEM(substrings, i);
        if (unlikely(!sub)) return -1;
        result = PyUnicode_Tailmatch(s, sub, start, end, direction);
        Py_DECREF(sub);
#endif
        if (result) {
            return (int) result;
        }
    }
    return 0;
}
static int __Pyx_PyUnicode_Tailmatch(PyObject* s, PyObject* substr,
                                     Py_ssize_t start, Py_ssize_t end, int direction) {
    if (unlikely(PyTuple_Check(substr))) {
        return __Pyx_PyUnicode_TailmatchTuple(s, substr, start, end, direction);
    }
    return (int) PyUnicode_Tailmatch(s, substr, start, end, direction);
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE && !CYTHON_AVOID_BORROWED_REFS && !CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            return __Pyx_PyList_GetItemRef(o, n);
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_subscript) {
            PyObject *r, *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
}

/* PyLongBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_Fallback___Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, int inplace) {
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject* __Pyx_Unpacked___Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    long a, x;
#ifdef HAVE_LONG_LONG
    const PY_LONG_LONG llb = intval;
    PY_LONG_LONG lla, llx;
#endif
    if (unlikely(__Pyx_PyLong_IsZero(op1))) {
        return __Pyx_NewRef(op2);
    }
    if (likely(__Pyx_PyLong_IsCompact(op1))) {
        a = __Pyx_PyLong_CompactValue(op1);
    } else {
        const digit* digits = __Pyx_PyLong_Digits(op1);
        const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(op1);
        switch (size) {
            case -2:
                if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                    a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    break;
                #ifdef HAVE_LONG_LONG
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                    lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    goto long_long;
                #endif
                }
                CYTHON_FALLTHROUGH;
            case 2:
                if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                    a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    break;
                #ifdef HAVE_LONG_LONG
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    goto long_long;
                #endif
                }
                CYTHON_FALLTHROUGH;
            case -3:
                if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                    a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    break;
                #ifdef HAVE_LONG_LONG
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                    lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    goto long_long;
                #endif
                }
                CYTHON_FALLTHROUGH;
            case 3:
                if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                    a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    break;
                #ifdef HAVE_LONG_LONG
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    goto long_long;
                #endif
                }
                CYTHON_FALLTHROUGH;
            case -4:
                if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                    a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    break;
                #ifdef HAVE_LONG_LONG
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                    lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    goto long_long;
                #endif
                }
                CYTHON_FALLTHROUGH;
            case 4:
                if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                    a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    break;
                #ifdef HAVE_LONG_LONG
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    goto long_long;
                #endif
                }
                CYTHON_FALLTHROUGH;
            default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
        }
    }
            x = a + b;
        return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
    long_long:
            llx = lla + llb;
        return PyLong_FromLongLong(llx);
#endif
    return __Pyx_Fallback___Pyx_PyLong_AddObjC(op1, op2, inplace);
    
    
}
#endif
static PyObject* __Pyx_Float___Pyx_PyLong_AddObjC(PyObject *float_val, long intval, int zerodivision_check) {
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    double a = __Pyx_PyFloat_AS_DOUBLE(float_val);
        double result;
        
        result = ((double)a) + (double)b;
        return PyFloat_FromDouble(result);
}
static CYTHON_INLINE PyObject* __Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        return __Pyx_Unpacked___Pyx_PyLong_AddObjC(op1, op2, intval, inplace, zerodivision_check);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        return __Pyx_Float___Pyx_PyLong_AddObjC(op1, intval, zerodivision_check);
    }
    return __Pyx_Fallback___Pyx_PyLong_AddObjC(op1, op2, inplace);
}
#endif

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* PyObjectCallNoArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectCallOneArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(tp);
    PyErr_Format(PyExc_AttributeError,
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else {
      Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(t);
 #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely(size < 0)) return;
 #endif
      if (size < index) {
        __Pyx_RaiseNeedMoreValuesError(size);
      } else {
        __Pyx_RaiseTooManyValuesError(index);
      }
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
        PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple) {
    if (likely(is_tuple || PyTuple_Check(tuple))) {
        Py_ssize_t size;
        if (has_known_size) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        size = __Pyx_PyTuple_GET_SIZE(tuple);
        if (likely(size == 2)) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        if (size >= 0) {
            __Pyx_UnpackTupleError(tuple, 2);
        }
        return -1;
    } else {
        return __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple);
    }
}
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
    value1 = __Pyx_PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = __Pyx_PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = __Pyx_PyObject_GetIterNextFunc(iter);
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
#if CYTHON_COMPILING_IN_PYPY
#include <string.h>
#endif
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#else
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE int __Pyx_dict_iter_next_source_is_dict(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem) {
    PyObject *key, *value;
    if (unlikely(orig_length != PyDict_Size(iter_obj))) {
        PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
        return -1;
    }
    if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
        return 0;
    }
    if (pitem) {
        PyObject* tuple = PyTuple_New(2);
        if (unlikely(!tuple)) {
            return -1;
        }
        Py_INCREF(key);
        Py_INCREF(value);
        #if CYTHON_ASSUME_SAFE_MACROS
        PyTuple_SET_ITEM(tuple, 0, key);
        PyTuple_SET_ITEM(tuple, 1, value);
        #else
        if (unlikely(PyTuple_SetItem(tuple, 0, key) < 0)) {
            Py_DECREF(value);
            Py_DECREF(tuple);
            return -1;
        }
        if (unlikely(PyTuple_SetItem(tuple, 1, value) < 0)) {
            Py_DECREF(tuple);
            return -1;
        }
        #endif
        *pitem = tuple;
    } else {
        if (pkey) {
            Py_INCREF(key);
            *pkey = key;
        }
        if (pvalue) {
            Py_INCREF(value);
            *pvalue = value;
        }
    }
    return 1;
}
#endif
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        int result;
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_BEGIN_CRITICAL_SECTION(iter_obj);
#endif
        result = __Pyx_dict_iter_next_source_is_dict(iter_obj, orig_length, ppos, pkey, pvalue, pitem);
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_END_CRITICAL_SECTION();
#endif
        return result;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(tuple_size < 0)) return -1;
        #endif
        if (unlikely(pos >= tuple_size)) return 0;
        *ppos = pos + 1;
        #if CYTHON_ASSUME_SAFE_MACROS
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        #else
        next_item = PyTuple_GetItem(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
        #endif
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t list_size = __Pyx_PyList_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(list_size < 0)) return -1;
        #endif
        if (unlikely(pos >= list_size)) return 0;
        *ppos = pos + 1;
        #if CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
        next_item = PyList_GetItemRef(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
        #elif CYTHON_ASSUME_SAFE_MACROS
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
        #else
        next_item = PyList_GetItem(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
        Py_INCREF(next_item);
        #endif
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* RaiseException */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
#elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}

/* TupleAndListFromArray */
#if !CYTHON_COMPILING_IN_CPYTHON && CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    Py_ssize_t i;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    for (i = 0; i < n; i++) {
        if (unlikely(__Pyx_PyTuple_SET_ITEM(res, i, src[i]) < 0)) {
            Py_DECREF(res);
            return NULL;
        }
        Py_INCREF(src[i]);
    }
    return res;
}
#elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL ||\
        !(CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS)
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length, length2;
        int kind;
        void *data1, *data2;
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        #endif
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return -1;
        #endif
        length2 = __Pyx_PyUnicode_GET_LENGTH(s2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length2 < 0)) return -1;
        #endif
        if (length != length2) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    return (equals == Py_EQ);
return_ne:
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = __Pyx_PyTuple_GET_SIZE(kwnames);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(n == -1)) return NULL;
    #endif
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        if (s == namei) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        int eq = __Pyx_PyUnicode_Equals(s, namei, Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs;
    PyObject *dict;
#if !CYTHON_ASSUME_SAFE_SIZE
    nkwargs = PyTuple_Size(kwnames);
    if (unlikely(nkwargs < 0)) return NULL;
#else
    nkwargs = PyTuple_GET_SIZE(kwnames);
#endif
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
#if !CYTHON_ASSUME_SAFE_MACROS
        PyObject *key = PyTuple_GetItem(kwnames, i);
        if (!key) goto bad;
#else
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
#endif
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
}

/* UnpackUnboundCMethod */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
#elif CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject **args, Py_ssize_t nargs, PyObject *kwnames) {
        return _PyObject_Vectorcall
            (method, args ? args+1 : NULL, nargs ? nargs-1 : 0, kwnames);
}
#else
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames) {
    return
#if PY_VERSION_HEX < 0x03090000
    _PyObject_Vectorcall
#else
    PyObject_Vectorcall
#endif
        (method, args ? args+1 : NULL, nargs ? (size_t) nargs-1 : 0, kwnames);
}
#endif
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
     METH_VARARGS | METH_KEYWORDS,
#else
     METH_FASTCALL | METH_KEYWORDS,
#endif
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method, *result=NULL;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    result = method;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            result = unbound_method;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    if (unlikely(target->method)) {
        Py_DECREF(result);
    } else
#endif
    target->method = result;
    return 0;
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, args, 2);
        }
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, 2, NULL);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod2(&tmp_cfunc, self, arg1, arg2);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *result = NULL;
        PyObject *args = PyTuple_New(2);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
        return result;
    }
#endif
    {
        PyObject *args[4] = {NULL, self, arg1, arg2};
        return __Pyx_PyObject_FastCall(cfunc->method, args+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
    }
}

/* ParseKeywords */
static int __Pyx_ValidateDuplicatePosArgs(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char* function_name)
{
    PyObject ** const *name = argnames;
    while (name != first_kw_arg) {
        PyObject *key = **name;
        int found = PyDict_Contains(kwds, key);
        if (unlikely(found)) {
            if (found == 1) __Pyx_RaiseDoubleKeywordsError(function_name, key);
            goto bad;
        }
        name++;
    }
    return 0;
bad:
    return -1;
}
#if CYTHON_USE_UNICODE_INTERNALS
static CYTHON_INLINE int __Pyx_UnicodeKeywordsEqual(PyObject *s1, PyObject *s2) {
    int kind;
    Py_ssize_t len = PyUnicode_GET_LENGTH(s1);
    if (len != PyUnicode_GET_LENGTH(s2)) return 0;
    kind = PyUnicode_KIND(s1);
    if (kind != PyUnicode_KIND(s2)) return 0;
    const void *data1 = PyUnicode_DATA(s1);
    const void *data2 = PyUnicode_DATA(s2);
    return (memcmp(data1, data2, (size_t) len * (size_t) kind) == 0);
}
#endif
static int __Pyx_MatchKeywordArg_str(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    #if CYTHON_USE_UNICODE_INTERNALS
    Py_hash_t key_hash = ((PyASCIIObject*)key)->hash;
    if (unlikely(key_hash == -1)) {
        key_hash = PyObject_Hash(key);
        if (unlikely(key_hash == -1))
            goto bad;
    }
    #endif
    name = first_kw_arg;
    while (*name) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (key_hash == ((PyASCIIObject*)name_str)->hash && __Pyx_UnicodeKeywordsEqual(name_str, key)) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) {
                *index_found = (size_t) (name - argnames);
                return 1;
            }
        }
        #endif
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (unlikely(key_hash == ((PyASCIIObject*)name_str)->hash)) {
            if (__Pyx_UnicodeKeywordsEqual(name_str, key))
                goto arg_passed_twice;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            if (unlikely(name_str == key)) goto arg_passed_twice;
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) goto arg_passed_twice;
        }
        #endif
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
bad:
    return -1;
}
static int __Pyx_MatchKeywordArg_nostr(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    if (unlikely(!PyUnicode_Check(key))) goto invalid_keyword_type;
    name = first_kw_arg;
    while (*name) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (cmp == 1) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        if (unlikely(cmp == -1)) goto bad;
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (unlikely(cmp != 0)) {
            if (cmp == 1) goto arg_passed_twice;
            else goto bad;
        }
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
bad:
    return -1;
}
static CYTHON_INLINE int __Pyx_MatchKeywordArg(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    return likely(PyUnicode_CheckExact(key)) ?
        __Pyx_MatchKeywordArg_str(key, argnames, first_kw_arg, index_found, function_name) :
        __Pyx_MatchKeywordArg_nostr(key, argnames, first_kw_arg, index_found, function_name);
}
static void __Pyx_RejectUnknownKeyword(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char *function_name)
{
    Py_ssize_t pos = 0;
    PyObject *key = NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(kwds);
    while (PyDict_Next(kwds, &pos, &key, NULL)) {
        PyObject** const *name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (!*name) {
            #if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(key);
            #endif
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp != 1) {
                if (cmp == 0) {
                    PyErr_Format(PyExc_TypeError,
                        "%s() got an unexpected keyword argument '%U'",
                        function_name, key);
                }
                #if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(key);
                #endif
                break;
            }
            #if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(key);
            #endif
        }
    }
    __Pyx_END_CRITICAL_SECTION();
    assert(PyErr_Occurred());
}
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t extracted = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    name = first_kw_arg;
    while (*name && num_kwargs > extracted) {
        PyObject * key = **name;
        PyObject *value;
        int found = 0;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        found = PyDict_GetItemRef(kwds, key, &value);
        #else
        value = PyDict_GetItemWithError(kwds, key);
        if (value) {
            Py_INCREF(value);
            found = 1;
        } else {
            if (unlikely(PyErr_Occurred())) goto bad;
        }
        #endif
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            extracted++;
        }
        name++;
    }
    if (num_kwargs > extracted) {
        if (ignore_unknown_kwargs) {
            if (unlikely(__Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name) == -1))
                goto bad;
        } else {
            __Pyx_RejectUnknownKeyword(kwds, argnames, first_kw_arg, function_name);
            goto bad;
        }
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t len;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    if (PyDict_Update(kwds2, kwds) < 0) goto bad;
    name = first_kw_arg;
    while (*name) {
        PyObject *key = **name;
        PyObject *value;
#if !CYTHON_COMPILING_IN_LIMITED_API && (PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop))
        int found = PyDict_Pop(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
        }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        int found = PyDict_GetItemRef(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            if (unlikely(PyDict_DelItem(kwds2, key) < 0)) goto bad;
        }
#else
    #if CYTHON_COMPILING_IN_CPYTHON
        value = _PyDict_Pop(kwds2, key, kwds2);
    #else
        value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, kwds2, key, kwds2);
    #endif
        if (value == kwds2) {
            Py_DECREF(value);
        } else {
            if (unlikely(!value)) goto bad;
            values[name-argnames] = value;
        }
#endif
        name++;
    }
    len = PyDict_Size(kwds2);
    if (len > 0) {
        return __Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name);
    } else if (unlikely(len == -1)) {
        goto bad;
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject *key = NULL;
    PyObject** const * name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    for (Py_ssize_t pos = 0; pos < num_kwargs; pos++) {
#if CYTHON_AVOID_BORROWED_REFS
        key = __Pyx_PySequence_ITEM(kwds, pos);
#else
        key = __Pyx_PyTuple_GET_ITEM(kwds, pos);
#endif
#if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!key)) goto bad;
#endif
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            PyObject *value = kwvalues[pos];
            values[name-argnames] = __Pyx_NewRef(value);
        } else {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp == 1) {
                PyObject *value = kwvalues[pos];
                values[index_found] = __Pyx_NewRef(value);
            } else {
                if (unlikely(cmp == -1)) goto bad;
                if (kwds2) {
                    PyObject *value = kwvalues[pos];
                    if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
                } else if (!ignore_unknown_kwargs) {
                    goto invalid_keyword;
                }
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        key = NULL;
        #endif
    }
    return 0;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    goto bad;
bad:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(key);
    #endif
    return -1;
}
static int __Pyx_ParseKeywords(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds)))
        return __Pyx_ParseKeywordsTuple(kwds, kwvalues, argnames, kwds2, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
    else if (kwds2)
        return __Pyx_ParseKeywordDictToDict(kwds, argnames, kwds2, values, num_pos_args, function_name);
    else
        return __Pyx_ParseKeywordDict(kwds, argnames, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    PyObject *extra_info = __pyx_mstate_global->__pyx_empty_unicode;
    int from_annotation_subclass = 0;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (!exact) {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    } else if (exact == 2) {
        if (__Pyx_TypeCheck(obj, type)) {
            from_annotation_subclass = 1;
            extra_info = __pyx_mstate_global->__pyx_kp_u_Note_that_Cython_is_deliberately;
        }
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")"
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        "%s%U"
#endif
        , name, type_name, obj_type_name
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        , (from_annotation_subclass ? ". " : ""), extra_info
#endif
        );
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (exact == 2 && from_annotation_subclass) {
        PyObject *res;
        PyObject *vargs[2];
        vargs[0] = PyErr_GetRaisedException();
        vargs[1] = extra_info;
        res = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_kp_u_add_note, vargs, 2, NULL);
        Py_XDECREF(res);
        PyErr_SetRaisedException(vargs[0]);
    }
#endif
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyUnicode_Substring */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
            PyObject* text, Py_ssize_t start, Py_ssize_t stop) {
    Py_ssize_t length;
    #if !CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(__Pyx_PyUnicode_READY(text) == -1)) return NULL;
    #endif
    length = __Pyx_PyUnicode_GET_LENGTH(text);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(length < 0)) return NULL;
    #endif
    if (start < 0) {
        start += length;
        if (start < 0)
            start = 0;
    }
    if (stop < 0)
        stop += length;
    else if (stop > length)
        stop = length;
    if (stop <= start)
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_unicode);
    if (start == 0 && stop == length)
        return __Pyx_NewRef(text);
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyUnicode_Substring(text, start, stop);
#else
    return PyUnicode_FromKindAndData(PyUnicode_KIND(text),
        PyUnicode_1BYTE_DATA(text) + start*PyUnicode_KIND(text), stop-start);
#endif
}

/* PyObjectFastCallMethod */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf) {
    PyObject *result;
    PyObject *attr = PyObject_GetAttr(args[0], name);
    if (unlikely(!attr))
        return NULL;
    result = __Pyx_PyObject_FastCall(attr, args+1, nargsf - 1);
    Py_DECREF(attr);
    return result;
}
#endif

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (unlikely(!j)) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            Py_INCREF(v);
#if CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
            PyList_SetItem(o, n, v);
#else
            PyObject* old = PyList_GET_ITEM(o, n);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
#endif
            return 1;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_ass_subscript) {
            int r;
            PyObject *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return -1;
            r = mm->mp_ass_subscript(o, key, v);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return sm->sq_ass_item(o, i, v);
        }
    }
#else
    if (is_list || !PyMapping_Check(o))
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyLong_FromSsize_t(i), v);
}

/* PyObjectCall2Args */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectCallMethod1 */
#if !(CYTHON_VECTORCALL && __PYX_LIMITED_VERSION_HEX >= 0x030C0000)
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
#endif
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
#if CYTHON_VECTORCALL && __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    PyObject *args[2] = {obj, arg};
    (void) __Pyx_PyObject_GetMethod;
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_Call2Args;
    return PyObject_VectorcallMethod(method_name, args, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
#endif
}

/* pop_index */
static PyObject* __Pyx__PyObject_PopNewIndex(PyObject* L, PyObject* py_ix) {
    PyObject *r;
    if (unlikely(!py_ix)) return NULL;
    r = __Pyx__PyObject_PopIndex(L, py_ix);
    Py_DECREF(py_ix);
    return r;
}
static PyObject* __Pyx__PyObject_PopIndex(PyObject* L, PyObject* py_ix) {
    return __Pyx_PyObject_CallMethod1(L, __pyx_mstate_global->__pyx_n_u_pop, py_ix);
}
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
static PyObject* __Pyx__PyList_PopIndex(PyObject* L, PyObject* py_ix, Py_ssize_t ix) {
    Py_ssize_t size = PyList_GET_SIZE(L);
    if (likely(size > (((PyListObject*)L)->allocated >> 1))) {
        Py_ssize_t cix = ix;
        if (cix < 0) {
            cix += size;
        }
        if (likely(__Pyx_is_valid_index(cix, size))) {
            PyObject* v = PyList_GET_ITEM(L, cix);
            __Pyx_SET_SIZE(L, Py_SIZE(L) - 1);
            size -= 1;
            memmove(&PyList_GET_ITEM(L, cix), &PyList_GET_ITEM(L, cix+1), (size_t)(size-cix)*sizeof(PyObject*));
            return v;
        }
    }
    if (py_ix == Py_None) {
        return __Pyx__PyObject_PopNewIndex(L, PyLong_FromSsize_t(ix));
    } else {
        return __Pyx__PyObject_PopIndex(L, py_ix);
    }
}
#endif

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_mstate_global->__pyx_n_u_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* RejectKeywords */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds) {
    PyObject *key = NULL;
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds))) {
        key = __Pyx_PySequence_ITEM(kwds, 0);
    } else {
        Py_ssize_t pos = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
        if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return;
#endif
        PyDict_Next(kwds, &pos, &key, NULL);
        Py_INCREF(key);
    }
    if (likely(key)) {
        PyErr_Format(PyExc_TypeError,
            "%s() got an unexpected keyword argument '%U'",
            function_name, key);
        Py_DECREF(key);
    }
}

/* CIntToDigits */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp = PySequence_Repeat(padding, uoffset - prepend_sign);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyUnicode_Concat(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyUnicode_Concat(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* COrdinalToPyUnicode */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value) {
    return value <= 1114111;
}
static PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t ulength, char padding_char) {
    if (likely(ulength <= 250)) {
        char chars[256];
        if (value <= 255) {
            memset(chars, padding_char, (size_t) (ulength - 1));
            chars[ulength-1] = (char) value;
            return PyUnicode_DecodeLatin1(chars, ulength, NULL);
        }
        char *cpos = chars + sizeof(chars);
        if (value < 0x800) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xc0 | (value & 0x1f));
        } else if (value < 0x10000) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xe0 | (value & 0x0f));
        } else {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xf0 | (value & 0x07));
        }
        cpos -= ulength;
        memset(cpos, padding_char, (size_t) (ulength - 1));
        return PyUnicode_DecodeUTF8(cpos, chars + sizeof(chars) - cpos, NULL);
    }
    if (value <= 127 && CYTHON_USE_UNICODE_INTERNALS) {
        const char chars[1] = {(char) value};
        return __Pyx_PyUnicode_BuildFromAscii(ulength, chars, 1, 0, padding_char);
    }
    {
        PyObject *uchar, *padding_uchar, *padding, *result;
        padding_uchar = PyUnicode_FromOrdinal(padding_char);
        if (unlikely(!padding_uchar)) return NULL;
        padding = PySequence_Repeat(padding_uchar, ulength - 1);
        Py_DECREF(padding_uchar);
        if (unlikely(!padding)) return NULL;
        uchar = PyUnicode_FromOrdinal(value);
        if (unlikely(!uchar)) {
            Py_DECREF(padding);
            return NULL;
        }
        result = PyUnicode_Concat(padding, uchar);
        Py_DECREF(padding);
        Py_DECREF(uchar);
        return result;
    }
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_enum____pyx_t_9pyteomics_7cparser_PositionType(enum __pyx_t_9pyteomics_7cparser_PositionType value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType)*3+2];
    char *dpos, *end = digits + sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    enum __pyx_t_9pyteomics_7cparser_PositionType remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum __pyx_t_9pyteomics_7cparser_PositionType neg_one = (enum __pyx_t_9pyteomics_7cparser_PositionType) -1, const_zero = (enum __pyx_t_9pyteomics_7cparser_PositionType) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'c') {
        if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                     !(sizeof(value) <= 2 || value & ~ (enum __pyx_t_9pyteomics_7cparser_PositionType) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
            PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
            return NULL;
        }
        if (width <= 1) {
            return PyUnicode_FromOrdinal((int) value);
        }
        return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
    }
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (enum __pyx_t_9pyteomics_7cparser_PositionType) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (enum __pyx_t_9pyteomics_7cparser_PositionType) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (enum __pyx_t_9pyteomics_7cparser_PositionType) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    if (max_char > 1114111) max_char = 1114111;
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - result_ulength < 0))
        goto overflow;
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = values[i];
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (__Pyx_PyUnicode_READY(uval) == (-1))
            goto bad;
        #endif
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(ulength < 0)) goto bad;
        #endif
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    Py_ssize_t i;
    PyObject *result = NULL;
    PyObject *value_tuple = PyTuple_New(value_count);
    if (unlikely(!value_tuple)) return NULL;
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    for (i=0; i<value_count; i++) {
        if (__Pyx_PyTuple_SET_ITEM(value_tuple, i, values[i]) != (0)) goto bad;
        Py_INCREF(values[i]);
    }
    result = PyUnicode_Join(__pyx_mstate_global->__pyx_empty_unicode, value_tuple);
bad:
    Py_DECREF(value_tuple);
    return result;
#endif
}

/* MergeKeywords */
static int __Pyx_MergeKeywords_dict(PyObject *kwdict, PyObject *source_dict) {
    Py_ssize_t len1, len2;
    len2 = PyDict_Size(source_dict);
    if (unlikely(len2 == -1)) return -1;
    if (len2 == 0) {
        return 0;
    }
    len1 = PyDict_Size(kwdict);
    if (unlikely(len1 == -1)) return -1;
    if (len1 > 0) {
        PyObject *key, *smaller_dict, *larger_dict;
        Py_ssize_t ppos = 0;
        int duplicates_found = 0;
        if (len1 <= len2) {
            smaller_dict = kwdict;
            larger_dict = source_dict;
        } else {
            smaller_dict = source_dict;
            larger_dict = kwdict;
        }
        __Pyx_BEGIN_CRITICAL_SECTION(smaller_dict);
        while (PyDict_Next(smaller_dict, &ppos, &key, NULL)) {
            #if CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
            Py_INCREF(key);
            #endif
            if (unlikely(PyDict_Contains(larger_dict, key))) {
                __Pyx_RaiseDoubleKeywordsError("function", key);
                #if CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
                Py_DECREF(key);
                #endif
                duplicates_found = 1;
                break;
            };
            #if CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
            Py_DECREF(key);
            #endif
        }
        __Pyx_END_CRITICAL_SECTION();
        if (unlikely(duplicates_found))
            return -1;
    }
    return PyDict_Update(kwdict, source_dict);
}
static int __Pyx_MergeKeywords_any(PyObject *kwdict, PyObject *source_mapping) {
    PyObject *iter, *key = NULL, *value = NULL;
    int source_is_dict, result;
    Py_ssize_t orig_length, ppos = 0;
    iter = __Pyx_dict_iterator(source_mapping, 0, __pyx_mstate_global->__pyx_n_u_items, &orig_length, &source_is_dict);
    if (unlikely(!iter)) {
        PyObject *args;
        if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError))) goto bad;
        PyErr_Clear();
        args = PyTuple_Pack(1, source_mapping);
        if (likely(args)) {
            PyObject *fallback = PyObject_Call((PyObject*)&PyDict_Type, args, NULL);
            Py_DECREF(args);
            if (likely(fallback)) {
                result = __Pyx_MergeKeywords_dict(kwdict, fallback);
                Py_DECREF(fallback);
                return result;
            }
        }
        if (unlikely(!iter)) goto bad;
    }
    while (1) {
        result = __Pyx_dict_iter_next(iter, orig_length, &ppos, &key, &value, NULL, source_is_dict);
        if (unlikely(result < 0)) goto bad;
        if (!result) break;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        {
            int inserted = PyDict_SetDefaultRef(kwdict, key, value, NULL);
            if (unlikely(inserted != 0)) {
                if (inserted == 1) __Pyx_RaiseDoubleKeywordsError("function", key);
                result = -1;
            }
        }
    #else
        if (unlikely(PyDict_Contains(kwdict, key))) {
            __Pyx_RaiseDoubleKeywordsError("function", key);
            result = -1;
        } else {
            result = PyDict_SetItem(kwdict, key, value);
        }
    #endif
        Py_DECREF(key);
        Py_DECREF(value);
        if (unlikely(result < 0)) goto bad;
    }
    Py_XDECREF(iter);
    return 0;
bad:
    Py_XDECREF(iter);
    return -1;
}
static CYTHON_INLINE int __Pyx_MergeKeywords(PyObject *kwdict, PyObject *source_mapping) {
    assert(PyDict_Check(kwdict));
    if (likely(PyDict_Check(source_mapping))) {
        return __Pyx_MergeKeywords_dict(kwdict, source_mapping);
    } else {
        return __Pyx_MergeKeywords_any(kwdict, source_mapping);
    }
}

/* GetAttr3 */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    int res = PyObject_GetOptionalAttr(o, n, &r);
    return (res != 0) ? r : __Pyx_NewRef(d);
#else
  #if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
  #endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
#endif
}

/* PyObjectVectorCallKwBuilder */
#if CYTHON_VECTORCALL
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_PyObject_FastCallDict;
    if (__Pyx_PyTuple_SET_ITEM(builder, n, key) != (0)) return -1;
    Py_INCREF(key);
    args[n] = value;
    return 0;
}
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_VectorcallBuilder_AddArgStr;
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n);
}
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    PyObject *pyKey = PyUnicode_FromString(key);
    if (!pyKey) return -1;
    return __Pyx_VectorcallBuilder_AddArg(pyKey, value, builder, args, n);
}
#else // CYTHON_VECTORCALL
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, CYTHON_UNUSED PyObject **args, CYTHON_UNUSED int n) {
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return PyDict_SetItem(builder, key, value);
}
#endif

/* PyObjectVectorCallMethodKwBuilder */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames) {
    PyObject *result;
    PyObject *obj = PyObject_GetAttr(args[0], name);
    if (unlikely(!obj))
        return NULL;
    result = __Pyx_Object_Vectorcall_CallFromBuilder(obj, args+1, nargsf-1, kwnames);
    Py_DECREF(obj);
    return result;
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C0000
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#elif __PYX_LIMITED_VERSION_HEX > 0x030C0000
    local_value = PyErr_GetRaisedException();
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
#if __PYX_LIMITED_VERSION_HEX > 0x030C0000
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
#else
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
#endif // __PYX_LIMITED_VERSION_HEX > 0x030C0000
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#elif __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    PyErr_SetHandledException(local_value);
    Py_XDECREF(local_value);
    Py_XDECREF(local_type);
    Py_XDECREF(local_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
#if __PYX_LIMITED_VERSION_HEX <= 0x030C0000
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
#endif
}

/* pep479 */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen) {
    PyObject *exc, *val, *tb, *cur_exc, *new_exc;
    __Pyx_PyThreadState_declare
    int is_async_stopiteration = 0;
    CYTHON_MAYBE_UNUSED_VAR(in_async_gen);
    __Pyx_PyThreadState_assign
    cur_exc = __Pyx_PyErr_CurrentExceptionType();
    if (likely(!__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopIteration))) {
        if (in_async_gen && unlikely(__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopAsyncIteration))) {
            is_async_stopiteration = 1;
        } else {
            return;
        }
    }
    __Pyx_GetException(&exc, &val, &tb);
    Py_XDECREF(exc);
    Py_XDECREF(tb);
    new_exc = PyObject_CallFunction(PyExc_RuntimeError, "s",
        is_async_stopiteration ? "async generator raised StopAsyncIteration" :
        in_async_gen ? "async generator raised StopIteration" :
        "generator raised StopIteration");
    if (!new_exc) {
        Py_XDECREF(val);
        return;
    }
    PyException_SetCause(new_exc, val); // steals ref to val
    PyErr_SetObject(PyExc_RuntimeError, new_exc);
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* PyUnicode_Unicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj) {
    if (unlikely(obj == Py_None))
        obj = __pyx_mstate_global->__pyx_kp_u_None;
    return __Pyx_NewRef(obj);
}

/* GetItemIntUnicode */
static CYTHON_INLINE Py_UCS4 __Pyx_GetItemInt_Unicode_Fast(PyObject* ustring, Py_ssize_t i,
                                                           int wraparound, int boundscheck) {
    Py_ssize_t length;
    if (unlikely(__Pyx_PyUnicode_READY(ustring) < 0)) return (Py_UCS4)-1;
    if (wraparound | boundscheck) {
        length = __Pyx_PyUnicode_GET_LENGTH(ustring);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return (Py_UCS4)-1;
        #endif
        if (wraparound & unlikely(i < 0)) i += length;
        if ((!boundscheck) || likely(__Pyx_is_valid_index(i, length))) {
            return __Pyx_PyUnicode_READ_CHAR(ustring, i);
        } else {
            PyErr_SetString(PyExc_IndexError, "string index out of range");
            return (Py_UCS4)-1;
        }
    } else {
        return __Pyx_PyUnicode_READ_CHAR(ustring, i);
    }
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    int was_initialized =  __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return __Pyx_CallCFunction(cfunc, self, arg);
        } else if (flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, &arg, 1);
        } else if (flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, &arg, 1, NULL);
        }
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod1(&tmp_cfunc, self, arg);
    }
#endif
    PyObject* result = __Pyx__CallUnboundCMethod1(cfunc, self, arg);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *args = PyTuple_New(1);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
    } else
#endif
    {
        result = __Pyx_PyObject_Call2Args(cfunc->method, self, arg);
    }
    return result;
}

/* dict_getitem_default */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyBytes_CheckExact(key) || PyUnicode_CheckExact(key) || PyLong_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *empty_list = 0;
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    if (level == -1) {
        const char* package_sep = strchr(__Pyx_MODULE_NAME, '.');
        if (package_sep != (0)) {
            module = PyImport_ImportModuleLevelObject(
                name, __pyx_mstate_global->__pyx_d, empty_dict, from_list, 1);
            if (unlikely(!module)) {
                if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                    goto bad;
                PyErr_Clear();
            }
        }
        level = 0;
    }
    if (!module) {
        module = PyImport_ImportModuleLevelObject(
            name, __pyx_mstate_global->__pyx_d, empty_dict, from_list, level);
    }
bad:
    Py_XDECREF(empty_dict);
    Py_XDECREF(empty_list);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_mstate_global->__pyx_kp_u__5);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
                CYTHON_COMPILING_IN_GRAAL
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
    }
    return value;
}

/* HasAttr */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!PyUnicode_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_PyObject_GetAttrStrNoError(o, n);
    if (!r) {
        return (unlikely(PyErr_Occurred())) ? -1 : 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}
#endif

/* CallTypeTraverse */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg) {
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
    if (__Pyx_get_runtime_version() < 0x03090000) return 0;
    #endif
    if (!always_call) {
        PyTypeObject *base = __Pyx_PyObject_GetSlot(o, tp_base, PyTypeObject*);
        unsigned long flags = PyType_GetFlags(base);
        if (flags & Py_TPFLAGS_HEAPTYPE) {
            return 0;
        }
    }
    Py_VISIT((PyObject*)Py_TYPE(o));
    return 0;
}
#endif

/* FixUpExtensionType */
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if PY_VERSION_HEX > 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
#else
    const PyType_Slot *slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
        int changed = 0;
#if !(PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON)
        const
#endif
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
#if PY_VERSION_HEX < 0x030900b1
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
#if PY_VERSION_HEX >= 0x030800b4
                    type->tp_vectorcall_offset = memb->offset;
#else
                    type->tp_print = (printfunc) memb->offset;
#endif
                    changed = 1;
                }
#endif
#else
                if ((0));
#endif
#if PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    if (unlikely(PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr) < 0)) {
                        Py_DECREF(descr);
                        return -1;
                    }
                    Py_DECREF(descr);
                    changed = 1;
                }
#endif
            }
            memb++;
        }
        if (changed)
            PyType_Modified(type);
    }
#endif
    return 0;
}

/* ValidateBasesTuple */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_SIZE
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (unlikely(n < 0)) return -1;
#endif
    for (i = 1; i < n; i++)
    {
        PyTypeObject *b;
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !CYTHON_USE_TYPE_SLOTS
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
CYTHON_UNUSED static int __Pyx_PyType_HasMultipleInheritance(PyTypeObject *t) {
    while (t) {
        PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
        if (bases) {
            return 1;
        }
        t = __Pyx_PyType_GetSlot(t, tp_base, PyTypeObject*);
    }
    return 0;
}
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !CYTHON_COMPILING_IN_CPYTHON || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    if (!__Pyx_PyType_HasMultipleInheritance(t)) {
        return PyType_Ready(t);
    }
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400) &&\
                !CYTHON_COMPILING_IN_GRAAL
        gc = PyImport_GetModule(__pyx_mstate_global->__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_mstate_global->__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i=0;
    Py_ssize_t size;
    void** base_vtables;
    __Pyx_TypeName tp_base_name = NULL;
    __Pyx_TypeName base_name = NULL;
    void* unknown = (void*)-1;
    PyObject* bases = __Pyx_PyType_GetSlot(type, tp_bases, PyObject*);
    int base_depth = 0;
    {
        PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        while (base) {
            base_depth += 1;
            base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
        }
    }
    base_vtables = (void**) PyMem_Malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
#if CYTHON_COMPILING_IN_LIMITED_API
    size = PyTuple_Size(bases);
    if (size < 0) goto other_failure;
#else
    size = PyTuple_GET_SIZE(bases);
#endif
    for (i = 1; i < size; i++) {
        PyObject *basei;
        void* base_vtable;
#if CYTHON_AVOID_BORROWED_REFS
        basei = PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#else
        basei = PyTuple_GET_ITEM(bases, i);
#endif
        base_vtable = __Pyx_GetVtable((PyTypeObject*)basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
            }
        }
    }
    PyErr_Clear();
    PyMem_Free(base_vtables);
    return 0;
bad:
    {
        PyTypeObject* basei = NULL;
        PyTypeObject* tp_base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        tp_base_name = __Pyx_PyType_GetFullyQualifiedName(tp_base);
#if CYTHON_AVOID_BORROWED_REFS
        basei = (PyTypeObject*)PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = (PyTypeObject*)PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#else
        basei = (PyTypeObject*)PyTuple_GET_ITEM(bases, i);
#endif
        base_name = __Pyx_PyType_GetFullyQualifiedName(basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
    }
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
really_bad: // bad has failed!
#endif
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
other_failure:
#endif
    PyMem_Free(base_vtables);
    return -1;
}

/* LimitedApiGetTypeDict */
#if CYTHON_COMPILING_IN_LIMITED_API
static Py_ssize_t __Pyx_GetTypeDictOffset(void) {
    PyObject *tp_dictoffset_o;
    Py_ssize_t tp_dictoffset;
    tp_dictoffset_o = PyObject_GetAttrString((PyObject*)(&PyType_Type), "__dictoffset__");
    if (unlikely(!tp_dictoffset_o)) return -1;
    tp_dictoffset = PyLong_AsSsize_t(tp_dictoffset_o);
    Py_DECREF(tp_dictoffset_o);
    if (unlikely(tp_dictoffset == 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' doesn't have a dictoffset");
        return -1;
    } else if (unlikely(tp_dictoffset < 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' has an unexpected negative dictoffset. "
            "Please report this as Cython bug");
        return -1;
    }
    return tp_dictoffset;
}
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp) {
    static Py_ssize_t tp_dictoffset = 0;
    if (unlikely(tp_dictoffset == 0)) {
        tp_dictoffset = __Pyx_GetTypeDictOffset();
        if (unlikely(tp_dictoffset == -1 && PyErr_Occurred())) {
            tp_dictoffset = 0; // try again next time?
            return NULL;
        }
    }
    return *(PyObject**)((char*)tp + tp_dictoffset);
}
#endif

/* DelItemOnTypeDict */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_DelItem(tp_dict, k);
    if (likely(!result)) PyType_Modified(tp);
    return result;
}

/* SetItemOnTypeDict */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_SetItem(tp_dict, k, v);
    if (likely(!result)) {
        PyType_Modified(tp);
        if (unlikely(PyObject_HasAttr(v, __pyx_mstate_global->__pyx_n_u_set_name))) {
            PyObject *setNameResult = PyObject_CallMethodObjArgs(v, __pyx_mstate_global->__pyx_n_u_set_name,  (PyObject *) tp, k, NULL);
            if (!setNameResult) return -1;
            Py_DECREF(setNameResult);
        }
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_mstate_global->__pyx_n_u_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_mstate_global->__pyx_n_u_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython);
            if (likely(reduce_cython)) {
                ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_mstate_global->__pyx_n_u_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetFullyQualifiedName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_1_1
#define __PYX_HAVE_RT_ImportType_3_1_1
static PyTypeObject *__Pyx_ImportType_3_1_1(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_1_1 check_size)
{
    PyObject *result = 0;
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !CYTHON_COMPILING_IN_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    if (size == 0) {
        return (PyTypeObject *)result;
    }
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_1_1 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_1_1 && (size_t)basicsize > size) {
        if (PyErr_WarnFormat(NULL, 0,
                "%.200s.%.200s size changed, may indicate binary incompatibility. "
                "Expected %zd from C header, got %zd from PyObject",
                module_name, class_name, size, basicsize) < 0) {
            goto bad;
        }
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* ListPack */
static PyObject *__Pyx_PyList_Pack(Py_ssize_t n, ...) {
    va_list va;
    PyObject *l = PyList_New(n);
    va_start(va, n);
    if (unlikely(!l)) goto end;
    for (Py_ssize_t i=0; i<n; ++i) {
        PyObject *arg = va_arg(va, PyObject*);
        Py_INCREF(arg);
        if (__Pyx_PyList_SET_ITEM(l, i, arg) != (0)) {
            Py_CLEAR(l);
            goto end;
        }
    }
    end:
    va_end(va);
    return l;
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n)))
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* Globals */
static PyObject* __Pyx_Globals(void) {
    return __Pyx_NewRef(__pyx_mstate_global->__pyx_d);
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* ImportDottedModule */
static PyObject *__Pyx__ImportDottedModule_Error(PyObject *name, PyObject *parts_tuple, Py_ssize_t count) {
    PyObject *partial_name = NULL, *slice = NULL, *sep = NULL;
    Py_ssize_t size;
    if (unlikely(PyErr_Occurred())) {
        PyErr_Clear();
    }
#if CYTHON_ASSUME_SAFE_SIZE
    size = PyTuple_GET_SIZE(parts_tuple);
#else
    size = PyTuple_Size(parts_tuple);
    if (size < 0) goto bad;
#endif
    if (likely(size == count)) {
        partial_name = name;
    } else {
        slice = PySequence_GetSlice(parts_tuple, 0, count);
        if (unlikely(!slice))
            goto bad;
        sep = PyUnicode_FromStringAndSize(".", 1);
        if (unlikely(!sep))
            goto bad;
        partial_name = PyUnicode_Join(sep, slice);
    }
    PyErr_Format(
        PyExc_ModuleNotFoundError,
        "No module named '%U'", partial_name);
bad:
    Py_XDECREF(sep);
    Py_XDECREF(slice);
    Py_XDECREF(partial_name);
    return NULL;
}
static PyObject *__Pyx__ImportDottedModule_Lookup(PyObject *name) {
    PyObject *imported_module;
#if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
        CYTHON_COMPILING_IN_GRAAL
    PyObject *modules = PyImport_GetModuleDict();
    if (unlikely(!modules))
        return NULL;
    imported_module = __Pyx_PyDict_GetItemStr(modules, name);
    Py_XINCREF(imported_module);
#else
    imported_module = PyImport_GetModule(name);
#endif
    return imported_module;
}
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple) {
    Py_ssize_t i, nparts;
#if CYTHON_ASSUME_SAFE_SIZE
    nparts = PyTuple_GET_SIZE(parts_tuple);
#else
    nparts = PyTuple_Size(parts_tuple);
    if (nparts < 0) return NULL;
#endif
    for (i=1; i < nparts && module; i++) {
        PyObject *part, *submodule;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        part = PyTuple_GET_ITEM(parts_tuple, i);
#else
        part = __Pyx_PySequence_ITEM(parts_tuple, i);
        if (!part) return NULL;
#endif
        submodule = __Pyx_PyObject_GetAttrStrNoError(module, part);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
        Py_DECREF(part);
#endif
        Py_DECREF(module);
        module = submodule;
    }
    if (unlikely(!module)) {
        return __Pyx__ImportDottedModule_Error(name, parts_tuple, i);
    }
    return module;
}
static PyObject *__Pyx__ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
    PyObject *imported_module;
    PyObject *module = __Pyx_Import(name, NULL, 0);
    if (!parts_tuple || unlikely(!module))
        return module;
    imported_module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(imported_module)) {
        Py_DECREF(module);
        return imported_module;
    }
    PyErr_Clear();
    return __Pyx_ImportDottedModule_WalkParts(module, name, parts_tuple);
}
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(module)) {
        PyObject *spec = __Pyx_PyObject_GetAttrStrNoError(module, __pyx_mstate_global->__pyx_n_u_spec);
        if (likely(spec)) {
            PyObject *unsafe = __Pyx_PyObject_GetAttrStrNoError(spec, __pyx_mstate_global->__pyx_n_u_initializing);
            if (likely(!unsafe || !__Pyx_PyObject_IsTrue(unsafe))) {
                Py_DECREF(spec);
                spec = NULL;
            }
            Py_XDECREF(unsafe);
        }
        if (likely(!spec)) {
            PyErr_Clear();
            return module;
        }
        Py_DECREF(spec);
        Py_DECREF(module);
    } else if (PyErr_Occurred()) {
        PyErr_Clear();
    }
#endif
    return __Pyx__ImportDottedModule(name, parts_tuple);
}

/* FetchSharedCythonModule */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef(__PYX_ABI_MODULE_NAME);
}

/* dict_setdefault */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value,
                                                       int is_safe_type) {
    PyObject* value;
    CYTHON_MAYBE_UNUSED_VAR(is_safe_type);
#if CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_CallMethod(d, "setdefault", "OO", key, default_value);
#elif PY_VERSION_HEX >= 0x030d0000
    PyDict_SetDefaultRef(d, key, default_value, &value);
#else
    value = PyDict_SetDefault(d, key, default_value);
    if (unlikely(!value)) return NULL;
    Py_INCREF(value);
#endif
    return value;
}

/* FetchCommonType */
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t expected_basicsize) {
    Py_ssize_t basicsize;
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
    if (unlikely(!py_basicsize)) return -1;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = NULL;
    if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) return -1;
#else
    basicsize = ((PyTypeObject*) cached_type)->tp_basicsize;
#endif
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module = NULL, *cached_type = NULL, *abi_module_dict, *new_cached_type, *py_object_name;
    int get_item_ref_result;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    py_object_name = PyUnicode_FromString(object_name);
    if (!py_object_name) return NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) goto done;
    abi_module_dict = PyModule_GetDict(abi_module);
    if (!abi_module_dict) goto done;
    get_item_ref_result = __Pyx_PyDict_GetItemRef(abi_module_dict, py_object_name, &cached_type);
    if (get_item_ref_result == 1) {
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else if (unlikely(get_item_ref_result == -1)) {
        goto bad;
    }
    CYTHON_UNUSED_VAR(module);
    cached_type = __Pyx_PyType_FromModuleAndSpec(abi_module, spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    new_cached_type = __Pyx_PyDict_SetDefault(abi_module_dict, py_object_name, cached_type, 1);
    if (unlikely(new_cached_type != cached_type)) {
        if (unlikely(!new_cached_type)) goto bad;
        Py_DECREF(cached_type);
        cached_type = new_cached_type;
        if (__Pyx_VerifyCachedType(
                cached_type,
                object_name,
                spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else {
        Py_DECREF(new_cached_type);
    }
done:
    Py_XDECREF(abi_module);
    Py_DECREF(py_object_name);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* PyMethodNew */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *result;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    #if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    {
        PyObject *args[] = {func, self};
        result = PyObject_Vectorcall(__pyx_mstate_global->__Pyx_CachedMethodType, args, 2, NULL);
    }
    #else
    result = PyObject_CallFunctionObjArgs(__pyx_mstate_global->__Pyx_CachedMethodType, func, self, NULL);
    #endif
    return result;
}
#else
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#endif

/* PyVectorcallFastCallDict */
#if CYTHON_METH_FASTCALL && (CYTHON_VECTORCALL || CYTHON_BACKPORT_VECTORCALL)
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i, pos;
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    #if !CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t nkw = PyDict_Size(kw);
    if (unlikely(nkw == -1)) return NULL;
    #else
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    #endif
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (PyDict_Next(kw, &pos, &key, &value)) {
        keys_are_strings &=
        #if CYTHON_COMPILING_IN_LIMITED_API
            PyType_GetFlags(Py_TYPE(key));
        #else
            Py_TYPE(key)->tp_flags;
        #endif
        Py_INCREF(key);
        Py_INCREF(value);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(PyTuple_SetItem(kwnames, i, key) < 0)) goto cleanup;
        #else
        PyTuple_SET_ITEM(kwnames, i, key);
        #endif
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    Py_ssize_t kw_size =
        likely(kw == NULL) ?
        0 :
#if !CYTHON_ASSUME_SAFE_SIZE
        PyDict_Size(kw);
#else
        PyDict_GET_SIZE(kw);
#endif
    if (kw_size == 0) {
        return vc(func, args, nargs, NULL);
    }
#if !CYTHON_ASSUME_SAFE_SIZE
    else if (unlikely(kw_size == -1)) {
        return NULL;
    }
#endif
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunctionNoMethod(PyObject *func, void (*cfunc)(void)) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if ((PyObject*)Py_TYPE(func) == __pyx_mstate_global->__Pyx_CachedMethodType) {
        int result;
        PyObject *newFunc = PyObject_GetAttr(func, __pyx_mstate_global->__pyx_n_u_func);
        if (unlikely(!newFunc)) {
            PyErr_Clear(); // It's only an optimization, so don't throw an error
            return 0;
        }
        result = __Pyx__IsSameCyOrCFunctionNoMethod(newFunc, cfunc);
        Py_DECREF(newFunc);
        return result;
    }
    return __Pyx__IsSameCyOrCFunctionNoMethod(func, cfunc);
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if (PyMethod_Check(func)) {
        func = PyMethod_GET_FUNCTION(func);
    }
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure) {
    PyObject *result;
    CYTHON_UNUSED_VAR(closure);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_doc_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#else
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_name_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    PyObject *result;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    Py_INCREF(op->func_qualname);
    result = op->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_dict_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    PyObject *result;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_dict_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_dict, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_defaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_kwdefaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_annotations_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine_value(__pyx_CyFunctionObject *op) {
    int is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
    if (is_coroutine) {
        PyObject *is_coroutine_value, *module, *fromlist, *marker = __pyx_mstate_global->__pyx_n_u_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(marker);
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_mstate_global->__pyx_n_u_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        is_coroutine_value = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(is_coroutine_value)) {
            return is_coroutine_value;
        }
ignore:
        PyErr_Clear();
    }
    return __Pyx_PyBool_FromLong(is_coroutine);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    result = __Pyx_CyFunction_get_is_coroutine_value(op);
    if (unlikely(!result))
        return NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    if (op->func_is_coroutine) {
        Py_DECREF(result);
        result = __Pyx_NewRef(op->func_is_coroutine);
    } else {
        op->func_is_coroutine = __Pyx_NewRef(result);
    }
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static void __Pyx_CyFunction_raise_argument_count_error(__pyx_CyFunctionObject *func, const char* message, Py_ssize_t size) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        py_name, message, size);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        name, message, size);
#endif
}
static void __Pyx_CyFunction_raise_type_error(__pyx_CyFunctionObject *func, const char* message) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s",
        py_name, message);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s",
        name, message);
#endif
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {"func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {"func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {"__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {"func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {"__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {"_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
    {"__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#if CYTHON_METH_FASTCALL
#if CYTHON_BACKPORT_VECTORCALL || CYTHON_COMPILING_IN_LIMITED_API
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {"__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(args);
    __Pyx_BEGIN_CRITICAL_SECTION(m);
    Py_INCREF(m->func_qualname);
    result = m->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
#if PY_VERSION_HEX < 0x030900B1
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    Py_CLEAR(m->defaults);
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    {
        int e = __Pyx_call_type_traverse((PyObject*)m, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
    Py_VISIT(m->func_dict);
    __Pyx_VISIT_CONST(m->func_name);
    __Pyx_VISIT_CONST(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    __Pyx_VISIT_CONST(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
#endif
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_is_coroutine);
    Py_VISIT(m->defaults);
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
    PyObject *repr;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    repr = PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
    __Pyx_END_CRITICAL_SECTION();
    return repr;
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void(*)(void))meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes no arguments", size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes exactly one argument", size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
    __Pyx_CyFunction_raise_type_error(
        (__pyx_CyFunctionObject*)func, "takes no keyword arguments");
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL && (CYTHON_VECTORCALL || CYTHON_BACKPORT_VECTORCALL)
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_SIZE
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(argc < 0)) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL && (CYTHON_VECTORCALL || CYTHON_BACKPORT_VECTORCALL)
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            __Pyx_CyFunction_raise_type_error(
                cyfunc, "needs an argument");
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(__Pyx_PyTuple_GET_SIZE(kwnames))) {
        __Pyx_CyFunction_raise_type_error(
            cyfunc, "takes no keyword arguments");
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes no arguments", nargs);
        return NULL;
    }
    return meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes exactly one argument", nargs);
        return NULL;
    }
    return meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCMethod)(void(*)(void))meth)(self, cls, args, (size_t)nargs, kwnames);
}
#endif
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if CYTHON_METH_FASTCALL
#if defined(Py_TPFLAGS_HAVE_VECTORCALL)
    Py_TPFLAGS_HAVE_VECTORCALL |
#elif defined(_Py_TPFLAGS_HAVE_VECTORCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
#endif // CYTHON_METH_FASTCALL
#if PY_VERSION_HEX >= 0x030A0000
    Py_TPFLAGS_IMMUTABLETYPE |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
static int __pyx_CyFunction_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_CyFunctionType_spec, NULL);
    if (unlikely(mstate->__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func, PyTypeObject *defaults_type) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_CallObject((PyObject*)defaults_type, NULL); // _PyObject_New(defaults_type);
    if (unlikely(!m->defaults))
        return NULL;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_mstate_global->__pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* ClassMethod */
static PyObject* __Pyx_Method_ClassMethod(PyObject *method) {
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM <= 0x05080000
    if (PyObject_TypeCheck(method, &PyWrapperDescr_Type)) {
        return PyClassMethod_New(method);
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (PyMethodDescr_Check(method))
#else
    if (__Pyx_TypeCheck(method, &PyMethodDescr_Type))
#endif
    {
#if CYTHON_COMPILING_IN_LIMITED_API
        return PyErr_Format(
            PyExc_SystemError,
            "Cython cannot yet handle classmethod on a MethodDescriptorType (%S) in limited API mode. "
            "This is most likely a classmethod in a cdef class method with binding=False. "
            "Try setting 'binding' to True.",
            method);
#elif CYTHON_COMPILING_IN_GRAAL
        PyTypeObject *d_type = PyDescrObject_GetType(method);
        return PyDescr_NewClassMethod(d_type, PyMethodDescrObject_GetMethod(method));
#else
        PyMethodDescrObject *descr = (PyMethodDescrObject *)method;
        PyTypeObject *d_type = descr->d_common.d_type;
        return PyDescr_NewClassMethod(d_type, descr->d_method);
#endif
    }
#endif
#if !CYTHON_COMPILING_IN_LIMITED_API
    else if (PyMethod_Check(method)) {
        return PyClassMethod_New(PyMethod_GET_FUNCTION(method));
    }
    else {
        return PyClassMethod_New(method);
    }
#else
    {
        PyObject *func=NULL;
        PyObject *builtins, *classmethod, *classmethod_str, *result=NULL;
        if (__Pyx_TypeCheck(method, __pyx_mstate_global->__Pyx_CachedMethodType)) {
            func = PyObject_GetAttrString(method, "__func__");
            if (!func) goto bad;
        } else {
            func = method;
            Py_INCREF(func);
        }
        builtins = PyEval_GetBuiltins(); // borrowed
        if (unlikely(!builtins)) goto bad;
        classmethod_str = PyUnicode_FromString("classmethod");
        if (unlikely(!classmethod_str)) goto bad;
        classmethod = PyObject_GetItem(builtins, classmethod_str);
        Py_DECREF(classmethod_str);
        if (unlikely(!classmethod)) goto bad;
        result = PyObject_CallFunctionObjArgs(classmethod, func, NULL);
        Py_DECREF(classmethod);
        bad:
        Py_XDECREF(func);
        return result;
    }
#endif
}

/* GetNameInClass */
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    PyObject *dict;
    assert(PyType_Check(nmspace));
#if CYTHON_USE_TYPE_SLOTS
    dict = ((PyTypeObject*)nmspace)->tp_dict;
    Py_XINCREF(dict);
#else
    dict = PyObject_GetAttr(nmspace, __pyx_mstate_global->__pyx_n_u_dict);
#endif
    if (likely(dict)) {
        result = PyObject_GetItem(dict, name);
        Py_DECREF(dict);
        if (result) {
            return result;
        }
    }
    PyErr_Clear();
    __Pyx_GetModuleGlobalNameUncached(result, name);
    return result;
}

/* CLineInTraceback */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_mstate_global->__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_mstate_global->__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __Pyx_BEGIN_CRITICAL_SECTION(*cython_runtime_dict);
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_mstate_global->__pyx_n_u_cline_in_traceback))
        Py_XINCREF(use_cline);
        __Pyx_END_CRITICAL_SECTION();
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_cython_runtime, __pyx_mstate_global->__pyx_n_u_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_INCREF(use_cline);
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_mstate_global->__pyx_cython_runtime, __pyx_mstate_global->__pyx_n_u_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    Py_XDECREF(use_cline);
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static __Pyx_CachedCodeObjectType *__pyx__find_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line) {
    __Pyx_CachedCodeObjectType* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!code_cache->entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if (unlikely(pos >= code_cache->count) || unlikely(code_cache->entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = code_cache->entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__find_code_object;
    return NULL; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just miss.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type old_count = __pyx_atomic_incr_acq_rel(&code_cache->accessor_count);
    if (old_count < 0) {
        __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
        return NULL;
    }
#endif
    __Pyx_CachedCodeObjectType *result = __pyx__find_code_object(code_cache, code_line);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
#endif
    return result;
#endif
}
static void __pyx__insert_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line, __Pyx_CachedCodeObjectType* code_object)
{
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = code_cache->entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            code_cache->entries = entries;
            code_cache->max_count = 64;
            code_cache->count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if ((pos < code_cache->count) && unlikely(code_cache->entries[pos].code_line == code_line)) {
        __Pyx_CachedCodeObjectType* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_INCREF(code_object);
        Py_DECREF(tmp);
        return;
    }
    if (code_cache->count == code_cache->max_count) {
        int new_max = code_cache->max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            code_cache->entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        code_cache->entries = entries;
        code_cache->max_count = new_max;
    }
    for (i=code_cache->count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    code_cache->count++;
    Py_INCREF(code_object);
}
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__insert_code_object;
    return; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just fail.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type expected = 0;
    if (!__pyx_atomic_int_cmp_exchange(&code_cache->accessor_count, &expected, INT_MIN)) {
        return;
    }
#endif
    __pyx__insert_code_object(code_cache, code_line, code_object);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_sub(&code_cache->accessor_count, INT_MIN);
#endif
#endif
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result = PyObject_Call(replace, __pyx_mstate_global->__pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        (void) __pyx_cfilenm;
        (void) __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!code_object) {
        code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
        if (unlikely(!code_object)) goto bad;
        py_py_line = PyLong_FromLong(py_line);
        if (unlikely(!py_py_line)) goto bad;
        py_funcname = PyUnicode_FromString(funcname);
        if (unlikely(!py_funcname)) goto bad;
        dict = PyDict_New();
        if (unlikely(!dict)) goto bad;
        {
            PyObject *old_code_object = code_object;
            code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
            Py_DECREF(old_code_object);
        }
        if (unlikely(!code_object)) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, code_object);
    } else {
        dict = PyDict_New();
    }
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    if (c_line) {
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
    }
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_mstate_global->__pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE enum __pyx_t_9pyteomics_7cparser_PositionType __Pyx_PyLong_As_enum____pyx_t_9pyteomics_7cparser_PositionType(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum __pyx_t_9pyteomics_7cparser_PositionType neg_one = (enum __pyx_t_9pyteomics_7cparser_PositionType) -1, const_zero = (enum __pyx_t_9pyteomics_7cparser_PositionType) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        enum __pyx_t_9pyteomics_7cparser_PositionType val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (enum __pyx_t_9pyteomics_7cparser_PositionType) -1;
        val = __Pyx_PyLong_As_enum____pyx_t_9pyteomics_7cparser_PositionType(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) >= 2 * PyLong_SHIFT)) {
                            return (enum __pyx_t_9pyteomics_7cparser_PositionType) (((((enum __pyx_t_9pyteomics_7cparser_PositionType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) >= 3 * PyLong_SHIFT)) {
                            return (enum __pyx_t_9pyteomics_7cparser_PositionType) (((((((enum __pyx_t_9pyteomics_7cparser_PositionType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) >= 4 * PyLong_SHIFT)) {
                            return (enum __pyx_t_9pyteomics_7cparser_PositionType) (((((((((enum __pyx_t_9pyteomics_7cparser_PositionType)digits[3]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (enum __pyx_t_9pyteomics_7cparser_PositionType) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(enum __pyx_t_9pyteomics_7cparser_PositionType, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(enum __pyx_t_9pyteomics_7cparser_PositionType, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) - 1 > 2 * PyLong_SHIFT)) {
                            return (enum __pyx_t_9pyteomics_7cparser_PositionType) (((enum __pyx_t_9pyteomics_7cparser_PositionType)-1)*(((((enum __pyx_t_9pyteomics_7cparser_PositionType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) - 1 > 2 * PyLong_SHIFT)) {
                            return (enum __pyx_t_9pyteomics_7cparser_PositionType) ((((((enum __pyx_t_9pyteomics_7cparser_PositionType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) - 1 > 3 * PyLong_SHIFT)) {
                            return (enum __pyx_t_9pyteomics_7cparser_PositionType) (((enum __pyx_t_9pyteomics_7cparser_PositionType)-1)*(((((((enum __pyx_t_9pyteomics_7cparser_PositionType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) - 1 > 3 * PyLong_SHIFT)) {
                            return (enum __pyx_t_9pyteomics_7cparser_PositionType) ((((((((enum __pyx_t_9pyteomics_7cparser_PositionType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) - 1 > 4 * PyLong_SHIFT)) {
                            return (enum __pyx_t_9pyteomics_7cparser_PositionType) (((enum __pyx_t_9pyteomics_7cparser_PositionType)-1)*(((((((((enum __pyx_t_9pyteomics_7cparser_PositionType)digits[3]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(enum __pyx_t_9pyteomics_7cparser_PositionType, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) - 1 > 4 * PyLong_SHIFT)) {
                            return (enum __pyx_t_9pyteomics_7cparser_PositionType) ((((((((((enum __pyx_t_9pyteomics_7cparser_PositionType)digits[3]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[2]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[1]) << PyLong_SHIFT) | (enum __pyx_t_9pyteomics_7cparser_PositionType)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(enum __pyx_t_9pyteomics_7cparser_PositionType, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(enum __pyx_t_9pyteomics_7cparser_PositionType, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        enum __pyx_t_9pyteomics_7cparser_PositionType val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() or PyLong_AsNativeBytes() not available, cannot convert large enums");
        val = (enum __pyx_t_9pyteomics_7cparser_PositionType) -1;
#endif
        if (unlikely(ret))
            return (enum __pyx_t_9pyteomics_7cparser_PositionType) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to enum __pyx_t_9pyteomics_7cparser_PositionType");
    return (enum __pyx_t_9pyteomics_7cparser_PositionType) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to enum __pyx_t_9pyteomics_7cparser_PositionType");
    return (enum __pyx_t_9pyteomics_7cparser_PositionType) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if defined(HAVE_LONG_LONG) && !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyLong_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyLong_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if defined(HAVE_LONG_LONG) && !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyLong_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 2 * PyLong_SHIFT)) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 3 * PyLong_SHIFT)) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 4 * PyLong_SHIFT)) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (size_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(size_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(size_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(size_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        size_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (size_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (size_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (size_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (size_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(size_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((size_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(size_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((size_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((size_t) 1) << (sizeof(size_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (size_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_enum____pyx_t_9pyteomics_7cparser_PositionType(enum __pyx_t_9pyteomics_7cparser_PositionType value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const enum __pyx_t_9pyteomics_7cparser_PositionType neg_one = (enum __pyx_t_9pyteomics_7cparser_PositionType) -1, const_zero = (enum __pyx_t_9pyteomics_7cparser_PositionType) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if defined(HAVE_LONG_LONG) && !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(enum __pyx_t_9pyteomics_7cparser_PositionType));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
static __Pyx_TypeName
__Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp)
{
    PyObject *module = NULL, *name = NULL, *result = NULL;
    #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
    name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_qualname);
    #else
    name = PyType_GetQualName(tp);
    #endif
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) goto bad;
    module = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_module_2);
    if (unlikely(module == NULL) || unlikely(!PyUnicode_Check(module))) goto bad;
    if (PyUnicode_CompareWithASCIIString(module, "builtins") == 0) {
        result = name;
        name = NULL;
        goto done;
    }
    result = PyUnicode_FromFormat("%U.%U", module, name);
    if (unlikely(result == NULL)) goto bad;
  done:
    Py_XDECREF(name);
    Py_XDECREF(module);
    return result;
  bad:
    PyErr_Clear();
    if (name) {
        result = name;
        name = NULL;
    } else {
        result = __Pyx_NewRef(__pyx_mstate_global->__pyx_kp_u__6);
    }
    goto done;
}
#endif

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* IterNextPlain */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void) {
    if (unlikely(!__pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache))
        __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache = __Pyx_GetBuiltinName(__pyx_mstate_global->__pyx_n_u_next);
    return __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator) {
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    PyObject *result;
    PyObject *next = __Pyx_GetBuiltinNext_LimitedAPI();
    if (unlikely(!next)) return NULL;
    result = PyObject_CallFunctionObjArgs(next, iterator, NULL);
    return result;
#else
    (void)__Pyx_GetBuiltinName; // only for early limited API
    iternextfunc iternext = __Pyx_PyObject_GetIterNextFunc(iterator);
    assert(iternext);
    return iternext(iterator);
#endif
}

/* ReturnWithStopIteration */
static void __Pyx__ReturnWithStopIteration(PyObject* value, int async);
static CYTHON_INLINE void __Pyx_ReturnWithStopIteration(PyObject* value, int async, int iternext) {
    if (value == Py_None) {
        if (async || !iternext)
            PyErr_SetNone(async ? PyExc_StopAsyncIteration : PyExc_StopIteration);
        return;
    }
    __Pyx__ReturnWithStopIteration(value, async);
}
static void __Pyx__ReturnWithStopIteration(PyObject* value, int async) {
#if CYTHON_COMPILING_IN_CPYTHON
    __Pyx_PyThreadState_declare
#endif
    PyObject *exc;
    PyObject *exc_type = async ? PyExc_StopAsyncIteration : PyExc_StopIteration;
#if CYTHON_COMPILING_IN_CPYTHON
    if ((PY_VERSION_HEX >= (0x030C00A6)) || unlikely(PyTuple_Check(value) || PyExceptionInstance_Check(value))) {
        if (PY_VERSION_HEX >= (0x030e00A1)) {
            exc = __Pyx_PyObject_CallOneArg(exc_type, value);
        } else {
            PyObject *args_tuple = PyTuple_New(1);
            if (unlikely(!args_tuple)) return;
            Py_INCREF(value);
            PyTuple_SET_ITEM(args_tuple, 0, value);
            exc = PyObject_Call(exc_type, args_tuple, NULL);
            Py_DECREF(args_tuple);
        }
        if (unlikely(!exc)) return;
    } else {
        Py_INCREF(value);
        exc = value;
    }
    #if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    #if CYTHON_USE_EXC_INFO_STACK
    if (!__pyx_tstate->exc_info->exc_value)
    #else
    if (!__pyx_tstate->exc_type)
    #endif
    {
        Py_INCREF(exc_type);
        __Pyx_ErrRestore(exc_type, exc, NULL);
        return;
    }
    #endif
#else
    exc = __Pyx_PyObject_CallOneArg(exc_type, value);
    if (unlikely(!exc)) return;
#endif
    PyErr_SetObject(exc_type, exc);
    Py_DECREF(exc);
}

/* CoroutineBase */
#if !CYTHON_COMPILING_IN_LIMITED_API
#include <frameobject.h>
#if PY_VERSION_HEX >= 0x030b00a6 && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#endif // CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void
__Pyx_Coroutine_Undelegate(__pyx_CoroutineObject *gen) {
#if CYTHON_USE_AM_SEND
    gen->yieldfrom_am_send = NULL;
#endif
    Py_CLEAR(gen->yieldfrom);
}
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    CYTHON_UNUSED_VAR(__pyx_tstate);
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
        else if (likely(__Pyx_IS_TYPE(ev, (PyTypeObject*)PyExc_StopIteration))) {
            #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
            value = PyObject_GetAttr(ev, __pyx_mstate_global->__pyx_n_u_value);
            if (unlikely(!value)) goto limited_api_failure;
            #else
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            #endif
            Py_DECREF(ev);
        }
        else if (unlikely(PyTuple_Check(ev))) {
            Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(ev);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely(tuple_size < 0)) {
                Py_XDECREF(tb);
                Py_DECREF(ev);
                Py_DECREF(et);
                return -1;
            }
            #endif
            if (tuple_size >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#elif CYTHON_ASSUME_SAFE_MACROS
                value = PySequence_ITEM(ev, 0);
#else
                value = PySequence_GetItem(ev, 0);
                if (!value) goto limited_api_failure;
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_GetAttr(ev, __pyx_mstate_global->__pyx_n_u_value);
#else
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
#endif
    Py_DECREF(ev);
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!value)) return -1;
#endif
    *pvalue = value;
    return 0;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL || !CYTHON_ASSUME_SAFE_MACROS
  limited_api_failure:
    Py_XDECREF(et);
    Py_XDECREF(tb);
    Py_XDECREF(ev);
    return -1;
#endif
}
static CYTHON_INLINE
__Pyx_PySendResult __Pyx_Coroutine_status_from_result(PyObject **retval) {
    if (*retval) {
        return PYGEN_NEXT;
    } else if (likely(__Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, retval) == 0)) {
        return PYGEN_RETURN;
    } else {
        return PYGEN_ERROR;
    }
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_CLEAR(exc_state->exc_value);
#else
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
#endif
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(__pyx_CoroutineObject *gen) {
    const char *msg;
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
static void __Pyx_Coroutine_AlreadyTerminatedError(PyObject *gen, PyObject *value, int closing) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    CYTHON_MAYBE_UNUSED_VAR(closing);
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
__Pyx_PySendResult __Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, PyObject **result, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(__Pyx_Coroutine_get_is_running(self));  // Callers should ensure is_running
    if (unlikely(self->resume_label == -1)) {
        __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
        return PYGEN_ERROR;
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_value) {
        #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        #else
        PyObject *exc_tb;
        #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
        exc_tb = PyException_GetTraceback(exc_state->exc_value);
        #elif PY_VERSION_HEX >= 0x030B00a4
        exc_tb = ((PyBaseExceptionObject*) exc_state->exc_value)->traceback;
        #else
        exc_tb = exc_state->exc_traceback;
        #endif
        if (exc_tb) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
            #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
            Py_DECREF(exc_tb);
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    retval = self->body(self, tstate, value);
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    *result = retval;
    if (self->resume_label == -1) {
        return likely(retval) ? PYGEN_RETURN : PYGEN_ERROR;
    }
    return PYGEN_NEXT;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(exc_state);
#else
    PyObject *exc_tb;
    #if PY_VERSION_HEX >= 0x030B00a4
    if (!exc_state->exc_value) return;
    exc_tb = PyException_GetTraceback(exc_state->exc_value);
    #else
    exc_tb = exc_state->exc_traceback;
    #endif
    if (likely(exc_tb)) {
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
        #if PY_VERSION_HEX >= 0x030B00a4
        Py_DECREF(exc_tb);
        #endif
    }
#endif
}
#define __Pyx_Coroutine_MethodReturnFromResult(gen, result, retval, iternext)\
    ((result) == PYGEN_NEXT ? (retval) : __Pyx__Coroutine_MethodReturnFromResult(gen, result, retval, iternext))
static PyObject *
__Pyx__Coroutine_MethodReturnFromResult(PyObject* gen, __Pyx_PySendResult result, PyObject *retval, int iternext) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if (likely(result == PYGEN_RETURN)) {
        int is_async = 0;
        #ifdef __Pyx_AsyncGen_USED
        is_async = __Pyx_AsyncGen_CheckExact(gen);
        #endif
        __Pyx_ReturnWithStopIteration(retval, is_async, iternext);
        Py_XDECREF(retval);
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
#if PY_VERSION_HEX < 0x030d00A1
            _PyGen_SetStopIterationValue(result);
#else
            if (!PyTuple_Check(result) && !PyExceptionInstance_Check(result)) {
                PyErr_SetObject(PyExc_StopIteration, result);
            } else {
                PyObject *exc = __Pyx_PyObject_CallOneArg(PyExc_StopIteration, result);
                if (likely(exc != NULL)) {
                    PyErr_SetObject(PyExc_StopIteration, exc);
                    Py_DECREF(exc);
                }
            }
#endif
        }
        Py_DECREF(result);
        result = NULL;
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE __Pyx_PySendResult
__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen, PyObject** retval) {
    __Pyx_PySendResult result;
    PyObject *val = NULL;
    assert(__Pyx_Coroutine_get_is_running(gen));
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    result = __Pyx_Coroutine_SendEx(gen, val, retval, 0);
    Py_XDECREF(val);
    return result;
}
#if CYTHON_USE_AM_SEND
static __Pyx_PySendResult
__Pyx_Coroutine_SendToDelegate(__pyx_CoroutineObject *gen, __Pyx_pyiter_sendfunc gen_am_send, PyObject *value, PyObject **retval) {
    PyObject *ret = NULL;
    __Pyx_PySendResult delegate_result, result;
    assert(__Pyx_Coroutine_get_is_running(gen));
    delegate_result = gen_am_send(gen->yieldfrom, value, &ret);
    if (delegate_result == PYGEN_NEXT) {
        assert (ret != NULL);
        *retval = ret;
        return PYGEN_NEXT;
    }
    assert (delegate_result != PYGEN_ERROR || ret == NULL);
    __Pyx_Coroutine_Undelegate(gen);
    result = __Pyx_Coroutine_SendEx(gen, ret, retval, 0);
    Py_XDECREF(ret);
    return result;
}
#endif
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval = NULL;
    __Pyx_PySendResult result = __Pyx_Coroutine_AmSend(self, value, &retval);
    return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 0);
}
static __Pyx_PySendResult
__Pyx_Coroutine_AmSend(PyObject *self, PyObject *value, PyObject **retval) {
    __Pyx_PySendResult result;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        *retval = __Pyx_Coroutine_AlreadyRunningError(gen);
        return PYGEN_ERROR;
    }
    #if CYTHON_USE_AM_SEND
    if (gen->yieldfrom_am_send) {
        result = __Pyx_Coroutine_SendToDelegate(gen, gen->yieldfrom_am_send, value, retval);
    } else
    #endif
    if (gen->yieldfrom) {
        PyObject *yf = gen->yieldfrom;
        PyObject *ret;
      #if !CYTHON_USE_AM_SEND
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
      #endif
        {
            #if !CYTHON_COMPILING_IN_LIMITED_API || __PYX_LIMITED_VERSION_HEX >= 0x03080000
            if (value == Py_None && PyIter_Check(yf))
                ret = __Pyx_PyIter_Next_Plain(yf);
            else
            #endif
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_mstate_global->__pyx_n_u_send, value);
        }
        if (likely(ret)) {
            __Pyx_Coroutine_unset_is_running(gen);
            *retval = ret;
            return PYGEN_NEXT;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, retval);
    } else {
        result = __Pyx_Coroutine_SendEx(gen, value, retval, 0);
    }
    __Pyx_Coroutine_unset_is_running(gen);
    return result;
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    __Pyx_PySendResult result;
    PyObject *retval = NULL;
    CYTHON_UNUSED_VAR(gen);
    assert(__Pyx_Coroutine_get_is_running(gen));
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        result = __Pyx_Coroutine_Close(yf, &retval);
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        result = __Pyx_Coroutine_Close(yf, &retval);
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        result = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf);
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
        result = PYGEN_RETURN;
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
        result = PYGEN_RETURN;
    } else
    #endif
    {
        PyObject *meth;
        result = PYGEN_RETURN;
        meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_mstate_global->__pyx_n_u_close);
        if (unlikely(!meth)) {
            if (unlikely(PyErr_Occurred())) {
                PyErr_WriteUnraisable(yf);
            }
        } else {
            retval = __Pyx_PyObject_CallNoArg(meth);
            Py_DECREF(meth);
            if (unlikely(!retval)) {
                result = PYGEN_ERROR;
            }
        }
    }
    Py_XDECREF(retval);
    return result == PYGEN_ERROR ? -1 : 0;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __Pyx_PySendResult result;
    PyObject *retval = NULL;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    }
    #if CYTHON_USE_AM_SEND
    if (gen->yieldfrom_am_send) {
        result = __Pyx_Coroutine_SendToDelegate(gen, gen->yieldfrom_am_send, Py_None, &retval);
    } else
    #endif
    if (gen->yieldfrom) {
        PyObject *yf = gen->yieldfrom;
        PyObject *ret;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && (PY_VERSION_HEX < 0x030A00A3 || !CYTHON_USE_AM_SEND)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
            ret = __Pyx_PyIter_Next_Plain(yf);
        if (likely(ret)) {
            __Pyx_Coroutine_unset_is_running(gen);
            return ret;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, &retval);
    } else {
        result = __Pyx_Coroutine_SendEx(gen, Py_None, &retval, 0);
    }
    __Pyx_Coroutine_unset_is_running(gen);
    return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 1);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, PyObject *arg) {
    PyObject *retval = NULL;
    __Pyx_PySendResult result;
    CYTHON_UNUSED_VAR(arg);
    result = __Pyx_Coroutine_Close(self, &retval);
    if (unlikely(result == PYGEN_ERROR))
        return NULL;
    Py_XDECREF(retval);
    Py_RETURN_NONE;
}
static __Pyx_PySendResult
__Pyx_Coroutine_Close(PyObject *self, PyObject **retval) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PySendResult result;
    PyObject *yf;
    int err = 0;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        *retval = __Pyx_Coroutine_AlreadyRunningError(gen);
        return PYGEN_ERROR;
    }
    yf = gen->yieldfrom;
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    result = __Pyx_Coroutine_SendEx(gen, NULL, retval, 1);
    if (result == PYGEN_ERROR) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_Coroutine_unset_is_running(gen);
        if (!__Pyx_PyErr_Occurred()) {
            return PYGEN_RETURN;
        } else if (likely(__Pyx_PyErr_ExceptionMatches2(PyExc_GeneratorExit, PyExc_StopIteration))) {
            __Pyx_PyErr_Clear();
            return PYGEN_RETURN;
        }
        return PYGEN_ERROR;
    } else if (likely(result == PYGEN_RETURN && *retval == Py_None)) {
        __Pyx_Coroutine_unset_is_running(gen);
        return PYGEN_RETURN;
    } else {
        const char *msg;
        Py_DECREF(*retval);
        *retval = NULL;
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
            msg = "async generator ignored GeneratorExit";
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        __Pyx_Coroutine_unset_is_running(gen);
        return PYGEN_ERROR;
    }
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen)))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    yf = gen->yieldfrom;
    if (yf) {
        __Pyx_PySendResult result;
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                goto propagate_exception;
            goto throw_here;
        }
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_mstate_global->__pyx_n_u_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (unlikely(PyErr_Occurred())) {
                    __Pyx_Coroutine_unset_is_running(gen);
                    return NULL;
                }
                __Pyx_Coroutine_Undelegate(gen);
                goto throw_here;
            }
            if (likely(args)) {
                ret = __Pyx_PyObject_Call(meth, args, NULL);
            } else {
                PyObject *cargs[4] = {NULL, typ, val, tb};
                ret = __Pyx_PyObject_FastCall(meth, cargs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
            }
            Py_DECREF(meth);
        }
        Py_DECREF(yf);
        if (ret) {
            __Pyx_Coroutine_unset_is_running(gen);
            return ret;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, &ret);
        __Pyx_Coroutine_unset_is_running(gen);
        return __Pyx_Coroutine_MethodReturnFromResult(self, result, ret, 0);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
propagate_exception:
    {
        PyObject *retval = NULL;
        __Pyx_PySendResult result = __Pyx_Coroutine_SendEx(gen, NULL, &retval, 0);
        __Pyx_Coroutine_unset_is_running(gen);
        return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 0);
    }
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (unlikely(!PyArg_UnpackTuple(args, "throw", 1, 3, &typ, &val, &tb)))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_VISIT(exc_state->exc_value);
#else
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
#endif
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    {
        int e = __Pyx_call_type_traverse((PyObject*)gen, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if CYTHON_USE_TP_FINALIZE
        if (unlikely(PyObject_CallFinalizerFromDealloc(self)))
#else
        {
            destructor del = __Pyx_PyObject_GetSlot(gen, tp_del, destructor);
            if (del) del(self);
        }
        if (unlikely(Py_REFCNT(self) > 0))
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    __Pyx_PyHeapTypeObject_GC_Del(gen);
}
#if CYTHON_USE_TP_FINALIZE
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *retval = NULL;
        __Pyx_PySendResult result = __Pyx_Coroutine_Close(self, &retval);
        if (result == PYGEN_ERROR) {
            PyErr_WriteUnraisable(self);
        } else {
            Py_XDECREF(retval);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
}
#endif
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_name;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_name, value);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_qualname;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_qualname, value);
    return 0;
}
static PyObject *
__Pyx__Coroutine_get_frame(__pyx_CoroutineObject *self)
{
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyObject *frame;
    #if PY_VERSION_HEX >= 0x030d0000
    Py_BEGIN_CRITICAL_SECTION(self);
    #endif
    frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        PyObject *globals = PyDict_New();
        if (unlikely(!globals)) return NULL;
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            globals,                        /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        Py_DECREF(globals);
        if (unlikely(!frame))
            return NULL;
        if (unlikely(self->gi_frame)) {
            Py_DECREF(frame);
            frame = self->gi_frame;
        } else {
            self->gi_frame = frame;
        }
    }
    Py_INCREF(frame);
    #if PY_VERSION_HEX >= 0x030d0000
    Py_END_CRITICAL_SECTION();
    #endif
    return frame;
#else
    CYTHON_UNUSED_VAR(self);
    Py_RETURN_NONE;
#endif
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, void *context) {
    CYTHON_UNUSED_VAR(context);
    PyObject *frame = self->gi_frame;
    if (frame)
        return __Pyx_NewRef(frame);
    return __Pyx__Coroutine_get_frame(self);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->yieldfrom_am_send = NULL;
    #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_LIMITED_API
    gen->gi_exc_state.exc_value = NULL;
    #else
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
    #endif
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}
static char __Pyx_Coroutine_test_and_set_is_running(__pyx_CoroutineObject *gen) {
    char result;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_BEGIN_CRITICAL_SECTION(gen);
    #endif
    result = gen->is_running;
    gen->is_running = 1;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_END_CRITICAL_SECTION();
    #endif
    return result;
}
static void __Pyx_Coroutine_unset_is_running(__pyx_CoroutineObject *gen) {
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_BEGIN_CRITICAL_SECTION(gen);
    #endif
    assert(gen->is_running);
    gen->is_running = 0;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_END_CRITICAL_SECTION();
    #endif
}
static char __Pyx_Coroutine_get_is_running(__pyx_CoroutineObject *gen) {
    char result;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_BEGIN_CRITICAL_SECTION(gen);
    #endif
    result = gen->is_running;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_END_CRITICAL_SECTION();
    #endif
    return result;
}
static PyObject *__Pyx_Coroutine_get_is_running_getter(PyObject *gen, void *closure) {
    CYTHON_UNUSED_VAR(closure);
    char result = __Pyx_Coroutine_get_is_running((__pyx_CoroutineObject*)gen);
    if (result) Py_RETURN_TRUE;
    else Py_RETURN_FALSE;
}
#if __PYX_HAS_PY_AM_SEND == 2
static void __Pyx_SetBackportTypeAmSend(PyTypeObject *type, __Pyx_PyAsyncMethodsStruct *static_amsend_methods, __Pyx_pyiter_sendfunc am_send) {
    Py_ssize_t ptr_offset = (char*)(type->tp_as_async) - (char*)type;
    if (ptr_offset < 0 || ptr_offset > type->tp_basicsize) {
        return;
    }
    memcpy((void*)static_amsend_methods, (void*)(type->tp_as_async), sizeof(*type->tp_as_async));
    static_amsend_methods->am_send = am_send;
    type->tp_as_async = __Pyx_SlotTpAsAsync(static_amsend_methods);
}
#endif

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {"gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     PyDoc_STR("object being iterated by 'yield from', or None")},
    {"gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {"__module__", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_modulename), 0, 0},
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CoroutineObject, gi_weakreflist), READONLY, 0},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {"__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     PyDoc_STR("name of the generator"), 0},
    {"__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     PyDoc_STR("qualified name of the generator"), 0},
    {"gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     PyDoc_STR("Frame of the generator"), 0},
    {"gi_running", __Pyx_Coroutine_get_is_running_getter, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_GeneratorType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_Coroutine_dealloc},
    {Py_tp_traverse, (void *)__Pyx_Coroutine_traverse},
    {Py_tp_iter, (void *)PyObject_SelfIter},
    {Py_tp_iternext, (void *)__Pyx_Generator_Next},
    {Py_tp_methods, (void *)__pyx_Generator_methods},
    {Py_tp_members, (void *)__pyx_Generator_memberlist},
    {Py_tp_getset, (void *)__pyx_Generator_getsets},
    {Py_tp_getattro, (void *) PyObject_GenericGetAttr},
#if CYTHON_USE_TP_FINALIZE
    {Py_tp_finalize, (void *)__Pyx_Coroutine_del},
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    {Py_am_send, (void *)__Pyx_Coroutine_AmSend},
#endif
    {0, 0},
};
static PyType_Spec __pyx_GeneratorType_spec = {
    __PYX_TYPE_MODULE_PREFIX "generator",
    sizeof(__pyx_CoroutineObject),
    0,
#if PY_VERSION_HEX >= 0x030A0000
    Py_TPFLAGS_IMMUTABLETYPE |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE | __Pyx_TPFLAGS_HAVE_AM_SEND,
    __pyx_GeneratorType_slots
};
#if __PYX_HAS_PY_AM_SEND == 2
static __Pyx_PyAsyncMethodsStruct __pyx_Generator_as_async;
#endif
static int __pyx_Generator_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_GeneratorType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_GeneratorType_spec, NULL);
    if (unlikely(!mstate->__pyx_GeneratorType)) {
        return -1;
    }
#if __PYX_HAS_PY_AM_SEND == 2
    __Pyx_SetBackportTypeAmSend(mstate->__pyx_GeneratorType, &__pyx_Generator_as_async, &__Pyx_Coroutine_AmSend);
#endif
    return 0;
}
static PyObject *__Pyx_Generator_GetInlinedResult(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *retval = NULL;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    }
    __Pyx_PySendResult result = __Pyx_Coroutine_SendEx(gen, Py_None, &retval, 0);
    __Pyx_Coroutine_unset_is_running(gen);
    (void) result;
    assert (result == PYGEN_RETURN || result == PYGEN_ERROR);
    assert ((result == PYGEN_RETURN && retval != NULL) || (result == PYGEN_ERROR && retval == NULL));
    return retval;
}

/* GetRuntimeVersion */
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    return Py_Version & ~0xFFUL;
#else
    static unsigned long __Pyx_cached_runtime_version = 0;
    if (__Pyx_cached_runtime_version == 0) {
        const char* rt_version = Py_GetVersion();
        unsigned long version = 0;
        unsigned long factor = 0x01000000UL;
        unsigned int digit = 0;
        int i = 0;
        while (factor) {
            while ('0' <= rt_version[i] && rt_version[i] <= '9') {
                digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
                ++i;
            }
            version += factor * digit;
            if (rt_version[i] != '.')
                break;
            digit = 0;
            factor >>= 8;
            ++i;
        }
        __Pyx_cached_runtime_version = version;
    }
    return __Pyx_cached_runtime_version;
#endif
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* FunctionExport */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, "__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, "__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
    cobj = PyCapsule_New(tmp.p, sig, 0);
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* NewCodeObj */
#if CYTHON_COMPILING_IN_LIMITED_API
    static PyObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                       PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                       PyObject *fv, PyObject *cell, PyObject* fn,
                                       PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
#elif PY_VERSION_HEX >= 0x030B0000
  static PyCodeObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                         PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                         PyObject *fv, PyObject *cell, PyObject* fn,
                                         PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, __pyx_mstate_global->__pyx_empty_bytes);
    return result;
  }
#elif PY_VERSION_HEX >= 0x030800B2 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        const char *line_table,
        PyObject *tuple_dedup_map
) {
    PyObject *code_obj = NULL, *varnames_tuple_dedup = NULL, *code_bytes = NULL, *line_table_bytes = NULL;
    Py_ssize_t var_count = (Py_ssize_t) descr.nlocals;
    PyObject *varnames_tuple = PyTuple_New(var_count);
    if (unlikely(!varnames_tuple)) return NULL;
    for (Py_ssize_t i=0; i < var_count; i++) {
        Py_INCREF(varnames[i]);
        if (__Pyx_PyTuple_SET_ITEM(varnames_tuple, i, varnames[i]) != (0)) goto done;
    }
    #if CYTHON_COMPILING_IN_LIMITED_API
    varnames_tuple_dedup = PyDict_GetItem(tuple_dedup_map, varnames_tuple);
    if (!varnames_tuple_dedup) {
        if (unlikely(PyDict_SetItem(tuple_dedup_map, varnames_tuple, varnames_tuple) < 0)) goto done;
        varnames_tuple_dedup = varnames_tuple;
    }
    #else
    varnames_tuple_dedup = PyDict_SetDefault(tuple_dedup_map, varnames_tuple, varnames_tuple);
    if (unlikely(!varnames_tuple_dedup)) goto done;
    #endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(varnames_tuple_dedup);
    #endif
    if (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table != NULL
        && !CYTHON_COMPILING_IN_GRAAL) {
        line_table_bytes = PyBytes_FromStringAndSize(line_table, descr.line_table_length);
        if (unlikely(!line_table_bytes)) goto done;
        Py_ssize_t code_len = (descr.line_table_length * 2 + 4) & ~3;
        code_bytes = PyBytes_FromStringAndSize(NULL, code_len);
        if (unlikely(!code_bytes)) goto done;
        char* c_code_bytes = PyBytes_AsString(code_bytes);
        if (unlikely(!c_code_bytes)) goto done;
        memset(c_code_bytes, 0, (size_t) code_len);
    }
    code_obj = (PyObject*) __Pyx__PyCode_New(
        (int) descr.argcount,
        (int) descr.num_posonly_args,
        (int) descr.num_kwonly_args,
        (int) descr.nlocals,
        0,
        (int) descr.flags,
        code_bytes ? code_bytes : __pyx_mstate_global->__pyx_empty_bytes,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        varnames_tuple_dedup,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        filename,
        funcname,
        (int) descr.first_line,
        (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table_bytes) ? line_table_bytes : __pyx_mstate_global->__pyx_empty_bytes
    );
done:
    Py_XDECREF(code_bytes);
    Py_XDECREF(line_table_bytes);
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(varnames_tuple_dedup);
    #endif
    Py_DECREF(varnames_tuple);
    return code_obj;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry const *t, PyObject **target, const char* const* encoding_names) {
    while (t->s) {
        PyObject *str;
        if (t->is_unicode) {
            if (t->intern) {
                str = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                str = PyUnicode_Decode(t->s, t->n - 1, encoding_names[t->encoding], NULL);
            } else {
                str = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            str = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        if (!str)
            return -1;
        *target = str;
        if (PyObject_Hash(str) == -1)
            return -1;
        ++t;
        ++target;
    }
    return 0;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    {
        const char* result;
        Py_ssize_t unicode_length;
        CYTHON_MAYBE_UNUSED_VAR(unicode_length); // only for __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (unlikely(PyArg_Parse(o, "s#", &result, length) < 0)) return NULL;
        #else
        result = PyUnicode_AsUTF8AndSize(o, length);
        #endif
        #if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        unicode_length = PyUnicode_GetLength(o);
        if (unlikely(unicode_length < 0)) return NULL;
        if (unlikely(unicode_length != *length)) {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
        #endif
        return result;
    }
#else
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
}
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
    if (PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
    if (PyByteArray_Check(o)) {
#if (CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS) || (CYTHON_COMPILING_IN_PYPY && (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE)))
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
#else
        *length = PyByteArray_Size(o);
        if (*length == -1) return NULL;
        return PyByteArray_AsString(o);
#endif
    } else
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_LongWrongResultType(PyObject* result) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(result));
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
    PyErr_Format(PyExc_TypeError,
                 "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ")",
                 result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  PyObject *res = NULL;
  if (likely(PyLong_Check(x)))
      return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  if (likely(m && m->nb_int)) {
      res = m->nb_int(x);
  }
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
      res = PyNumber_Long(x);
  }
#endif
  if (likely(res)) {
      if (unlikely(!PyLong_CheckExact(res))) {
          return __Pyx_PyNumber_LongWrongResultType(res);
      }
  }
  else if (!PyErr_Occurred()) {
      PyErr_SetString(PyExc_TypeError,
                      "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyLong_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyLong_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b) {
    CYTHON_UNUSED_VAR(b);
    return __Pyx_NewRef(Py_None);
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t ival) {
    return PyLong_FromSize_t(ival);
}


/* MultiPhaseInitModuleState */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
#ifndef CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#if (CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX >= 0x030C0000)
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 1
#else
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 0
#endif
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE && !CYTHON_ATOMICS
#error "Module state with PEP489 requires atomics. Currently that's one of\
 C11, C++11, gcc atomic intrinsics or MSVC atomic intrinsics"
#endif
#if !CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#define __Pyx_ModuleStateLookup_Lock()
#define __Pyx_ModuleStateLookup_Unlock()
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
static PyMutex __Pyx_ModuleStateLookup_mutex = {0};
#define __Pyx_ModuleStateLookup_Lock() PyMutex_Lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() PyMutex_Unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(__cplusplus) && __cplusplus >= 201103L
#include <mutex>
static std::mutex __Pyx_ModuleStateLookup_mutex;
#define __Pyx_ModuleStateLookup_Lock() __Pyx_ModuleStateLookup_mutex.lock()
#define __Pyx_ModuleStateLookup_Unlock() __Pyx_ModuleStateLookup_mutex.unlock()
#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ > 201112L) && !defined(__STDC_NO_THREADS__)
#include <threads.h>
static mtx_t __Pyx_ModuleStateLookup_mutex;
static once_flag __Pyx_ModuleStateLookup_mutex_once_flag = ONCE_FLAG_INIT;
static void __Pyx_ModuleStateLookup_initialize_mutex(void) {
    mtx_init(&__Pyx_ModuleStateLookup_mutex, mtx_plain);
}
#define __Pyx_ModuleStateLookup_Lock()\
  call_once(&__Pyx_ModuleStateLookup_mutex_once_flag, __Pyx_ModuleStateLookup_initialize_mutex);\
  mtx_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() mtx_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(HAVE_PTHREAD_H)
#include <pthread.h>
static pthread_mutex_t __Pyx_ModuleStateLookup_mutex = PTHREAD_MUTEX_INITIALIZER;
#define __Pyx_ModuleStateLookup_Lock() pthread_mutex_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() pthread_mutex_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(_WIN32)
#include <Windows.h>  // synchapi.h on its own doesn't work
static SRWLOCK __Pyx_ModuleStateLookup_mutex = SRWLOCK_INIT;
#define __Pyx_ModuleStateLookup_Lock() AcquireSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() ReleaseSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#else
#error "No suitable lock available for CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE.\
 Requires C standard >= C11, or C++ standard >= C++11,\
 or pthreads, or the Windows 32 API, or Python >= 3.13."
#endif
typedef struct {
    int64_t id;
    PyObject *module;
} __Pyx_InterpreterIdAndModule;
typedef struct {
    char interpreter_id_as_index;
    Py_ssize_t count;
    Py_ssize_t allocated;
    __Pyx_InterpreterIdAndModule table[1];
} __Pyx_ModuleStateLookupData;
#define __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE 32
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_int_type __Pyx_ModuleStateLookup_read_counter = 0;
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_ptr_type __Pyx_ModuleStateLookup_data = 0;
#else
static __Pyx_ModuleStateLookupData* __Pyx_ModuleStateLookup_data = NULL;
#endif
static __Pyx_InterpreterIdAndModule* __Pyx_State_FindModuleStateLookupTableLowerBound(
        __Pyx_InterpreterIdAndModule* table,
        Py_ssize_t count,
        int64_t interpreterId) {
    __Pyx_InterpreterIdAndModule* begin = table;
    __Pyx_InterpreterIdAndModule* end = begin + count;
    if (begin->id == interpreterId) {
        return begin;
    }
    while ((end - begin) > __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
        __Pyx_InterpreterIdAndModule* halfway = begin + (end - begin)/2;
        if (halfway->id == interpreterId) {
            return halfway;
        }
        if (halfway->id < interpreterId) {
            begin = halfway;
        } else {
            end = halfway;
        }
    }
    for (; begin < end; ++begin) {
        if (begin->id >= interpreterId) return begin;
    }
    return begin;
}
static PyObject *__Pyx_State_FindModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return NULL;
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData* data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
    {
        __pyx_atomic_incr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        if (likely(data)) {
            __Pyx_ModuleStateLookupData* new_data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_acquire(&__Pyx_ModuleStateLookup_data);
            if (likely(data == new_data)) {
                goto read_finished;
            }
        }
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        __Pyx_ModuleStateLookup_Lock();
        __pyx_atomic_incr_relaxed(&__Pyx_ModuleStateLookup_read_counter);
        data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
        __Pyx_ModuleStateLookup_Unlock();
    }
  read_finished:;
#else
    __Pyx_ModuleStateLookupData* data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_InterpreterIdAndModule* found = NULL;
    if (unlikely(!data)) goto end;
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            found = data->table+interpreter_id;
        }
    } else {
        found = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
    }
  end:
    {
        PyObject *result=NULL;
        if (found && found->id == interpreter_id) {
            result = found->module;
        }
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
#endif
        return result;
    }
}
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static void __Pyx_ModuleStateLookup_wait_until_no_readers(void) {
    while (__pyx_atomic_load(&__Pyx_ModuleStateLookup_read_counter) != 0);
}
#else
#define __Pyx_ModuleStateLookup_wait_until_no_readers()
#endif
static int __Pyx_State_AddModuleInterpIdAsIndex(__Pyx_ModuleStateLookupData **old_data, PyObject* module, int64_t interpreter_id) {
    Py_ssize_t to_allocate = (*old_data)->allocated;
    while (to_allocate <= interpreter_id) {
        if (to_allocate == 0) to_allocate = 1;
        else to_allocate *= 2;
    }
    __Pyx_ModuleStateLookupData *new_data = *old_data;
    if (to_allocate != (*old_data)->allocated) {
         new_data = (__Pyx_ModuleStateLookupData *)realloc(
            *old_data,
            sizeof(__Pyx_ModuleStateLookupData)+(to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
        if (!new_data) {
            PyErr_NoMemory();
            return -1;
        }
        for (Py_ssize_t i = new_data->allocated; i < to_allocate; ++i) {
            new_data->table[i].id = i;
            new_data->table[i].module = NULL;
        }
        new_data->allocated = to_allocate;
    }
    new_data->table[interpreter_id].module = module;
    if (new_data->count < interpreter_id+1) {
        new_data->count = interpreter_id+1;
    }
    *old_data = new_data;
    return 0;
}
static void __Pyx_State_ConvertFromInterpIdAsIndex(__Pyx_ModuleStateLookupData *data) {
    __Pyx_InterpreterIdAndModule *read = data->table;
    __Pyx_InterpreterIdAndModule *write = data->table;
    __Pyx_InterpreterIdAndModule *end = read + data->count;
    for (; read<end; ++read) {
        if (read->module) {
            write->id = read->id;
            write->module = read->module;
            ++write;
        }
    }
    data->count = write - data->table;
    for (; write<end; ++write) {
        write->id = 0;
        write->module = NULL;
    }
    data->interpreter_id_as_index = 0;
}
static int __Pyx_State_AddModule(PyObject* module, CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    int result = 0;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *old_data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *old_data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_ModuleStateLookupData *new_data = old_data;
    if (!new_data) {
        new_data = (__Pyx_ModuleStateLookupData *)calloc(1, sizeof(__Pyx_ModuleStateLookupData));
        if (!new_data) {
            result = -1;
            PyErr_NoMemory();
            goto end;
        }
        new_data->allocated = 1;
        new_data->interpreter_id_as_index = 1;
    }
    __Pyx_ModuleStateLookup_wait_until_no_readers();
    if (new_data->interpreter_id_as_index) {
        if (interpreter_id < __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
            result = __Pyx_State_AddModuleInterpIdAsIndex(&new_data, module, interpreter_id);
            goto end;
        }
        __Pyx_State_ConvertFromInterpIdAsIndex(new_data);
    }
    {
        Py_ssize_t insert_at = 0;
        {
            __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
                new_data->table, new_data->count, interpreter_id);
            assert(lower_bound);
            insert_at = lower_bound - new_data->table;
            if (unlikely(insert_at < new_data->count && lower_bound->id == interpreter_id)) {
                lower_bound->module = module;
                goto end;  // already in table, nothing more to do
            }
        }
        if (new_data->count+1 >= new_data->allocated) {
            Py_ssize_t to_allocate = (new_data->count+1)*2;
            new_data =
                (__Pyx_ModuleStateLookupData*)realloc(
                    new_data,
                    sizeof(__Pyx_ModuleStateLookupData) +
                    (to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
            if (!new_data) {
                result = -1;
                new_data = old_data;
                PyErr_NoMemory();
                goto end;
            }
            new_data->allocated = to_allocate;
        }
        ++new_data->count;
        int64_t last_id = interpreter_id;
        PyObject *last_module = module;
        for (Py_ssize_t i=insert_at; i<new_data->count; ++i) {
            int64_t current_id = new_data->table[i].id;
            new_data->table[i].id = last_id;
            last_id = current_id;
            PyObject *current_module = new_data->table[i].module;
            new_data->table[i].module = last_module;
            last_module = current_module;
        }
    }
  end:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, new_data);
#else
    __Pyx_ModuleStateLookup_data = new_data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return result;
}
static int __Pyx_State_RemoveModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *data = __Pyx_ModuleStateLookup_data;
#endif
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            data->table[interpreter_id].module = NULL;
        }
        goto done;
    }
    {
        __Pyx_ModuleStateLookup_wait_until_no_readers();
        __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
        if (!lower_bound) goto done;
        if (lower_bound->id != interpreter_id) goto done;
        __Pyx_InterpreterIdAndModule *end = data->table+data->count;
        for (;lower_bound<end-1; ++lower_bound) {
            lower_bound->id = (lower_bound+1)->id;
            lower_bound->module = (lower_bound+1)->module;
        }
    }
    --data->count;
    if (data->count == 0) {
        free(data);
        data = NULL;
    }
  done:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, data);
#else
    __Pyx_ModuleStateLookup_data = data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return 0;
}
#endif

/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
